@@ -3,7 +3,7 @@
 -- portions copyright (c) 2007-2009 Judah Jacobson
 {-# OPTIONS_GHC -Wall #-}
 import Distribution.Simple
-         ( defaultMainWithHooks, UserHooks(..), simpleUserHooks, Args )
+         ( defaultMainWithHooks, UserHooks(..), simpleUserHooks )
 import Distribution.PackageDescription ( PackageDescription )
 import Distribution.Package ( packageVersion )
 import Distribution.Version( Version )
@@ -11,8 +11,8 @@ import Distribution.Simple.LocalBuildInfo
          ( LocalBuildInfo(..), absoluteInstallDirs )
 import Distribution.Simple.InstallDirs (mandir, CopyDest (NoCopyDest))
 import Distribution.Simple.Setup
-    (configVerbosity, copyDest, copyVerbosity, fromFlag, ConfigFlags,
-     installVerbosity)
+    (buildVerbosity, copyDest, copyVerbosity, fromFlag,
+     haddockVerbosity, installVerbosity, replVerbosity )
 import Distribution.Simple.BuildPaths ( autogenPackageModulesDir )
 import Distribution.Simple.Utils
     (copyFiles, createDirectoryIfMissingVerbose, rawSystemStdout,
@@ -23,7 +23,7 @@ import Distribution.Text ( display )
 import Control.Monad ( unless, when, void )
 import System.Directory ( doesDirectoryExist, doesFileExist )
 import System.IO ( openFile, IOMode(..) )
-import System.Process (runProcess)
+import System.Process (callProcess, runProcess)
 import Data.List( isInfixOf )
 import System.FilePath ( (</>) )
 
@@ -33,25 +33,39 @@ catchAny :: IO a -> (Exception.SomeException -> IO a) -> IO a
 catchAny f h = Exception.catch f (\e -> h (e :: Exception.SomeException))
 
 main :: IO ()
-main =
-  defaultMainWithHooks $
-  simpleUserHooks
-    { postConf = postConfHook
-    , postBuild = \_ _ _ lbi -> buildManpage lbi
-    , postCopy =
-        \_ flags pkg lbi ->
-          installManpage pkg lbi
-            (fromFlag $ copyVerbosity flags) (fromFlag $ copyDest flags)
-    , postInst =
-        \_ flags pkg lbi ->
-          installManpage pkg lbi (fromFlag $ installVerbosity flags) NoCopyDest
-    }
+main = defaultMainWithHooks $ simpleUserHooks {
 
-postConfHook :: Args -> ConfigFlags -> PackageDescription -> LocalBuildInfo -> IO ()
-postConfHook _args flags pkg lbi = do
-  let verbosity = fromFlag $ configVerbosity flags
+  buildHook = \ pkg lbi hooks flags ->
+              let verb = fromFlag $ buildVerbosity flags
+               in commonBuildHook buildHook pkg lbi hooks verb >>= ($ flags),
+
+  haddockHook = \ pkg lbi hooks flags ->
+                let verb = fromFlag $ haddockVerbosity flags
+                 in commonBuildHook haddockHook pkg lbi hooks verb >>= ($ flags) ,
+  replHook = \pkg lbi hooks flags args ->
+                let verb = fromFlag $ replVerbosity flags
+                 in commonBuildHook replHook pkg lbi hooks verb >>= (\f -> f flags args) ,
+  postBuild = \ _ _ _ lbi -> buildManpage lbi,
+  postCopy = \ _ flags pkg lbi ->
+             installManpage pkg lbi (fromFlag $ copyVerbosity flags) (fromFlag $ copyDest flags),
+  postInst = \ _ flags pkg lbi ->
+             installManpage pkg lbi (fromFlag $ installVerbosity flags) NoCopyDest
+
+}
+
+-- | For @./Setup build@ and @./Setup haddock@, do some unusual
+-- things, then invoke the base behaviour ("simple hook").
+commonBuildHook :: (UserHooks -> PackageDescription -> LocalBuildInfo -> t -> a)
+                -> PackageDescription -> LocalBuildInfo -> t -> Verbosity -> IO a
+commonBuildHook runHook pkg lbi hooks verbosity = do
+  versionInfoExists <-
+    (&&) <$> doesFileExist "release/distributed-context"
+         <*> doesFileExist "release/distributed-version"
+  unless versionInfoExists $
+    callProcess "runghc" ["./release/gen-version-info.hs"]
   (version, state) <- determineVersion verbosity pkg
   generateVersionModule verbosity lbi version state
+  return $ runHook simpleUserHooks pkg lbi hooks
 
 -- ---------------------------------------------------------------------
 -- man page
