@@ -41,7 +41,7 @@ die "hmmm.. we couldn't parse your XML. The error was: $@"  if $@;
 #     'some other patch' => { ... }, 
 
 for my $patch_name (keys %{ $xml->{patch} }) {
-    my $issue_re = qr/resolved? \s+ (issue\d+)/msxi;
+    my $issue_re = qr/resolved? \s+ (issue ?\d+)/msxi;
 
     next unless ($patch_name =~ $issue_re);
     my $issue = $1;
@@ -63,12 +63,13 @@ for my $patch_name (keys %{ $xml->{patch} }) {
 
      # Each patches can potentially update the status of a different issue, so generates a different e-mail
     my $msg = MIME::Lite->new(
-         From     => $email, 
+         From     => 'noreply@darcs.net',
          To      =>'bugs@lists.osuosl.org',
          #To       =>'mark@stosberg.com',
          Subject  =>"[$issue] [$UPDATE_STRING]",
          Type     =>'text/plain',
-         Data     => qq!The following patch updated issue $issue with $UPDATE_STRING
+         Data     => qq!The following patch sent by $email updated issue $issue with
+$UPDATE_STRING
 
 * $patch_name $comment
 
