@@ -15,7 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE CPP, ScopedTypeVariables #-}
 
 -- |
 -- Module      : Main
@@ -27,12 +27,12 @@
 
 module Main ( main ) where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Control.Exception ( AssertionFailed(..), handle )
+import Control.Exception ( handle, ErrorCall )
 import System.Exit ( exitWith, ExitCode(..) )
 import System.Environment ( getArgs )
+import System.IO ( hPutStrLn, stderr )
 
 import Darcs.UI.RunCommand ( runTheCommand )
 import Darcs.UI.Commands.Help ( helpCmd, listAvailableCommands, printVersion,
@@ -56,7 +56,7 @@ execExceptionHandler (ExecException cmd args redirects reason) = do
     exitWith $ ExitFailure 3
 
 main :: IO ()
-main = withAtexit . withSignalsHandled . handleExecFail . handleAssertFail $ do
+main = handleErrors . withAtexit . withSignalsHandled . handleExecFail $ do
     atexit reportBadSources
     setDarcsEncodings
     argv <- getArgs
@@ -74,8 +74,14 @@ main = withAtexit . withSignalsHandled . handleExecFail . handleAssertFail $ do
         ["--exact-version"] -> printExactVersion
         _ -> runTheCommand commandControlList (head argv) (tail argv)
   where
+    handleErrors =
+      handle (\(e::ErrorCall) -> do
+        hPutStrLn stderr $
+          "This is a bug! Please report it at http://bugs.darcs.net " ++
+          "or via email to bugs@darcs.net:\n" ++
+          show e
+        exitWith $ ExitFailure 4)
     handleExecFail = handle execExceptionHandler
-    handleAssertFail = handle $ \(AssertionFailed e) -> bug e
     printExactVersion =  do
         putStrLn $ "darcs compiled on " ++ __DATE__ ++ ", at " ++ __TIME__ ++ "\n"
         putStrLn $ "Weak Hash: " ++ weakhash
