@@ -149,7 +149,7 @@ index = [ testCase "index versioning" check_index_versions
                exist <- doesFileExist "_darcs/index"
                performGC -- required in win32 to trigger file close
                when exist $ removeFile "_darcs/index"
-               idx <- updateIndex =<< updateIndexFrom "_darcs/index" darcsTreeHash x
+               idx <- treeFromIndex =<< updateIndexFrom "_darcs/index" darcsTreeHash x
                return (x, idx)
           check_index = extractRepoAndRun $
             do (pris, idx) <- build_index
@@ -280,7 +280,7 @@ tree = [ testCase "modifyTree" check_modify
           check_diffTrees = extractRepoAndRun $
                  do Prelude.writeFile "foo_dir/foo_a" "b\n"
                     working_plain <- filter nondarcs `fmap` readPlainTree "."
-                    working <- updateIndex =<<
+                    working <- treeFromIndex =<<
                                  updateIndexFrom "_darcs/index" darcsTreeHash working_plain
                     pristine <- readDarcsPristine "."
                     (working', pristine') <- diffTrees working pristine
