@@ -17,6 +17,8 @@
 
 module Darcs.Test.Misc ( testSuite ) where
 
+import Darcs.Prelude
+
 import Darcs.Util.ByteString
     ( unpackPSFromUTF8, fromHex2PS, fromPS2Hex
     , propHexConversion
@@ -30,8 +32,10 @@ import Darcs.Util.Diff.Myers ( shiftBoundaries )
 
 import Darcs.Test.Misc.CommandLine ( commandLineTestSuite )
 import qualified Darcs.Test.Misc.Encoding as Encoding
+import qualified Darcs.Test.Misc.Graph as Graph
+import qualified Darcs.Test.Misc.URL as URL
 
-import qualified Data.ByteString.Char8 as BC ( unpack, pack, last )
+import qualified Data.ByteString.Char8 as BC ( unpack, pack, snoc )
 import qualified Data.ByteString as B ( ByteString, pack, empty, null )
 import Data.Char ( ord )
 import Data.Array.Base
@@ -49,6 +53,8 @@ testSuite = testGroup ""
  , lcsTestSuite
  , commandLineTestSuite
  , Encoding.testSuite
+ , Graph.testSuite
+ , URL.testSuite
  ]
 
 
@@ -81,9 +87,12 @@ instance Arbitrary B.ByteString where
 -- if certain conditions are met
 prop_betweenLinesPS :: B.ByteString -> B.ByteString -> B.ByteString -> Property
 prop_betweenLinesPS start end ps =
-  not (B.null start) && not (B.null end) &&
-  (B.null ps || BC.last ps == '\n') ==>
-  betweenLinesPS start end ps == spec_betweenLinesPS start end ps
+  not (B.null start) && not (B.null end)
+  ==> betweenLinesPS start end (twist ps) == spec_betweenLinesPS start end (twist ps)
+  where
+    twist s
+      | B.null s = s
+      | otherwise = s `BC.snoc` '\n'
 
 -- ----------------------------------------------------------------------
 -- * LCS
