@@ -1,5 +1,7 @@
 module Darcs.Test.Misc.Encoding ( testSuite ) where
 
+import Darcs.Prelude
+
 import qualified Data.ByteString as B
 import Control.Monad
 import Data.Word
@@ -25,7 +27,12 @@ newtype MyByteString = MBS { _mbsBytes :: [Word8] }
  deriving Show
 
 instance Arbitrary MyByteString where
-  arbitrary = MBS <$> sized (\n -> vectorOf (100*n) arbitrary)
+  arbitrary =
+    MBS <$> frequency
+      -- make sure we test some very long ByteStrings
+      [ (1, sized (\n -> vectorOf (100*n) arbitrary))
+      , (9, sized (\n -> vectorOf n arbitrary))
+      ]
   shrink (MBS ws) = MBS <$> shrinkList shrink ws
 
 toBS :: MyByteString -> B.ByteString
