@@ -15,22 +15,22 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE FlexibleInstances, TypeSynonymInstances #-}
-
-module Darcs.Test.Patch.Arbitrary.RepoPatchV1 () where
+{-# LANGUAGE UndecidableInstances #-}
+module Darcs.Test.Patch.Arbitrary.RepoPatchV1 (Patch) where
 
 import Prelude ()
 import Darcs.Prelude
 
 import Test.QuickCheck
+import Control.Exception ( try, evaluate, SomeException )
 import Control.Monad ( liftM, liftM2, liftM3, liftM4, replicateM )
+import System.IO.Unsafe
 
 import qualified Data.ByteString as B ( ByteString )
 import qualified Data.ByteString.Char8 as BC ( pack )
 
-import Darcs.Patch ( addfile, adddir, move,
-                     hunk, tokreplace, binary,
-                     changepref, invert, merge )
+import Darcs.Patch
+import Darcs.Patch.Annotate
 import Darcs.Patch.V1 ()
 import Darcs.Patch.V1.Core ( RepoPatchV1(..) )
 import qualified Darcs.Patch.V1.Prim as V1 ( Prim(..) )
@@ -39,11 +39,17 @@ import Darcs.Patch.Witnesses.Ordered
 import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed), unseal, mapSeal, Sealed2(..) )
 import Darcs.Patch.Witnesses.Unsafe
 
+import Darcs.Util.Path  ( AnchoredPath, floatPath, catPaths )
+
 -- This definitely feels a bit weird to be importing Properties here, and
 -- probably means we want to move this elsewhere, but Darcs.Test.Patch.Check is
 -- already taken with something apparently only semi-related
 import Darcs.Test.Patch.Properties.Check( checkAPatch )
-import Darcs.Test.Patch.Arbitrary.Generic ( MightHaveDuplicate )
+import Darcs.Test.Patch.Arbitrary.Generic ( MightHaveDuplicate, ArbitraryPrim,PrimBased(..) )
+import Darcs.Test.Patch.Arbitrary.RepoPatch
+import Darcs.Test.Patch.Merge.Checked ( CheckedMerge(..) )
+import Darcs.Test.Patch.RepoModel ( RepoState, ModelOf )
+import Darcs.Test.Patch.WithState ( PropagateShrink(..) )
 
 type Patch = RepoPatchV1 V1.Prim
 
@@ -53,6 +59,19 @@ pp = PP . V1.Prim
 class ArbitraryP p where
     arbitraryP :: Gen (Sealed (p wX))
 
+instance
+  (Annotate prim, ArbitraryPrim prim, PrimPatch prim, ApplyState prim ~ RepoState (ModelOf prim))
+  => ArbitraryRepoPatch (RepoPatchV1 prim)
+  where
+
+    notRepoPatchV1 = Nothing
+
+instance PrimPatch prim => CheckedMerge (RepoPatchV1 prim) where
+  validateMerge v =
+    case unsafePerformIO (try (evaluate v)) of
+      Left (_ :: SomeException) -> Nothing
+      Right x -> Just x
+
 {-
 TODO: there is a lot of overlap in testing between between this module
 and Darcs.Test.Patch.QuickCheck
@@ -74,11 +93,14 @@ Arbitrary instances in this module.)
 instance Arbitrary (Sealed (Prim wX)) where
     arbitrary = arbitraryP
 
+instance Arbitrary (Sealed ((Prim :> Prim) wX)) where
+    arbitrary = arbitraryP
+
 instance Arbitrary (Sealed (FL Patch wX)) where
     arbitrary = arbitraryP
 
--- instance Arbitrary (Sealed2 (Prim :> Prim)) where
-    -- arbitrary = unseal Sealed2 <$> arbitraryP
+instance Arbitrary (Sealed2 (Prim :> Prim)) where
+    arbitrary = unseal Sealed2 <$> arbitraryP
 
 instance Arbitrary (Sealed2 (FL Patch)) where
     arbitrary = unseal Sealed2 <$> arbitraryP
@@ -111,6 +133,13 @@ instance ArbitraryP Prim where
 
 instance MightHaveDuplicate (RepoPatchV1 prim)
 
+type instance ModelOf (RepoPatchV1 prim) = ModelOf prim
+
+instance (PrimPatch prim, ArbitraryPrim prim, PropagateShrink prim prim) => PrimBased (RepoPatchV1 prim) where
+  type OnlyPrim (RepoPatchV1 prim) = prim
+  primEffect prim = prim :>: NilFL
+  liftFromPrim = PP
+
 hunkgen :: Gen (Sealed (Prim wX))
 hunkgen = do
   i <- frequency [(1,choose (0,5)),(1,choose (0,35)),
@@ -132,7 +161,7 @@ tokreplacegen = do
      then return $ Sealed $ tokreplace f "A-Za-z" "old" "new"
      else return $ Sealed $ tokreplace f "A-Za-z_" o n
 
-twofilegen :: (forall wY . FilePath -> FilePath -> Prim wX wY) -> Gen (Sealed (Prim wX))
+twofilegen :: (forall wY . AnchoredPath -> AnchoredPath -> Prim wX wY) -> Gen (Sealed (Prim wX))
 twofilegen p = do
   n1 <- filepathgen
   n2 <- filepathgen
@@ -194,7 +223,7 @@ mergegen n = do
           _ :/\: p2' ->
               if checkAPatch (p1+>+p2')
               then return $ Sealed $ p1+>+p2'
-              else impossible
+              else error "impossible case"
      else mergegen n
   where len = if n < 15 then n`div`3 else 3
 
@@ -234,18 +263,6 @@ filelinegen = liftM BC.pack $
               frequency [(1,map fromSafeChar `fmap` arbitrary),(5,toklinegen),
                          (1,return ""), (1,return "{"), (1,return "}") ]
 
-filepathgen :: Gen String
-filepathgen = liftM fixpath badfpgen
-
-fixpath :: String -> String
-fixpath "" = "test"
-fixpath p = fpth p
-
-fpth :: String -> String
-fpth ('/':'/':cs) = fpth ('/':cs)
-fpth (c:cs) = c : fpth cs
-fpth [] = []
-
 newtype SafeChar = SS Char
 instance Arbitrary SafeChar where
     arbitrary = oneof $ map (return . SS) (['a'..'z']++['A'..'Z']++['1'..'9']++"0")
@@ -253,10 +270,15 @@ instance Arbitrary SafeChar where
 fromSafeChar :: SafeChar -> Char
 fromSafeChar (SS s) = s
 
-badfpgen :: Gen String
-badfpgen =  frequency [(1,return "test"), (1,return "hello"), (1,return "world"),
-                       (1,map fromSafeChar `fmap` arbitrary),
-                       (1,liftM2 (\a b-> a++"/"++b) filepathgen filepathgen) ]
+filepathgen :: Gen AnchoredPath
+filepathgen =
+  frequency
+    [ (1, return $ floatPath "test")
+    , (1, return $ floatPath "hello")
+    , (1, return $ floatPath "world")
+    , (1, floatPath `fmap` map fromSafeChar `fmap` arbitrary)
+    , (1, liftM2 catPaths filepathgen filepathgen)
+    ]
 
 regularizePatches :: FL Patch wX wY -> FL Patch wX wY
 regularizePatches patches = rpint (unsafeCoerceP NilFL) patches
