@@ -15,6 +15,7 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
+{-# LANGUAGE OverloadedStrings #-}
 module Darcs.Test.Patch.Examples.Set1
        ( knownCommutes, knownCantCommutes, knownMerges
        , knownMergeEquivs, knownCanons, mergePairs2
@@ -24,25 +25,34 @@ module Darcs.Test.Patch.Examples.Set1
 
 import Prelude ()
 import Darcs.Prelude
-import Data.Maybe ( fromJust )
 import System.IO.Unsafe ( unsafePerformIO )
 import qualified Data.ByteString.Char8 as BC ( pack )
 import qualified Data.ByteString as B ( empty )
+import Data.String ( IsString(..) )
+
 import Darcs.Patch
      ( commute, invert, merge
-     , Named, namepatch
-     , readPatch, fromPrim
+     , Named, infopatch
+     , readPatch
      , adddir, addfile, hunk, binary, rmdir, rmfile, tokreplace )
-import Darcs.Patch.Prim ( PrimOf, FromPrim )
+import Darcs.Patch.Info ( patchinfo )
+import Darcs.Patch.FromPrim ( PrimOf, FromPrim(..) )
 import qualified Darcs.Patch.V1 as V1 ( RepoPatchV1 )
 import qualified Darcs.Patch.V1.Prim as V1 ( Prim(..) )
 import Darcs.Test.Patch.Properties.Check( checkAPatch )
 import Darcs.Patch.Witnesses.Ordered
-import Darcs.Patch.Witnesses.Sealed ( unsafeUnseal )
+import Darcs.Patch.Witnesses.Sealed ( unseal )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP, unsafeCoercePEnd )
+import Darcs.Util.Path ( AnchoredPath, floatPath )
+
+instance IsString AnchoredPath where
+  fromString = floatPath
 
 type Patch = V1.RepoPatchV1 V1.Prim
 
+flFromPrim :: FromPrim p => PrimOf p wX wY -> FL p wX wY
+flFromPrim p = fromAnonymousPrim p :>: NilFL
+
 -- The unit tester function is really just a glorified map for functions that
 -- return lists, in which the lists get concatenated (where map would end up
 -- with a list of lists).
@@ -85,9 +95,12 @@ knownCanons =
      (quickhunk 1 "abcde" "acde" :>: NilFL, quickhunk 2 "b" "" :>: NilFL)]
 
 quickhunk :: (FromPrim p, PrimOf p ~ V1.Prim) => Int -> String -> String -> p wX wY
-quickhunk l o n = fromPrim $ hunk "test" l (map (\c -> BC.pack [c]) o)
+quickhunk l o n = fromAnonymousPrim $ hunk "test" l (map (\c -> BC.pack [c]) o)
                                              (map (\c -> BC.pack [c]) n)
 
+quickhunkFL :: (FromPrim p, PrimOf p ~ V1.Prim) => Int -> String -> String -> FL p wX wY
+quickhunkFL l o n = quickhunk l o n :>: NilFL
+
 -- ----------------------------------------------------------------------
 -- * Merge/unmgerge tests
 -- ----------------------------------------------------------------------
@@ -139,8 +152,8 @@ knownCommutes = [
                   (testhunk 2
                    ["hello world all that is old is good old_"]
                    ["I don't like old things"]:>
-                   fromPrim (tokreplace "test" "A-Za-z_" "old" "new"),
-                   fromPrim (tokreplace "test" "A-Za-z_" "old" "new"):>
+                   flFromPrim (tokreplace "test" "A-Za-z_" "old" "new"),
+                   flFromPrim (tokreplace "test" "A-Za-z_" "old" "new"):>
                    testhunk 2
                    ["hello world all that is new is good old_"]
                    ["I don't like new things"]),
@@ -148,10 +161,10 @@ knownCommutes = [
                    testhunk 1 ["A"] ["B"],
                    testhunk 1 ["A"] ["B"]:>
                    testhunk 2 ["C"] ["D"]),
-                  ((quickmerge (fromPrim (addfile "hello"):\/:fromPrim (addfile "hello"))):>
-                   fromPrim (rmfile "NwNSO"),
-                   fromPrim (rmfile "NwNSO"):>
-                   (quickmerge (fromPrim (addfile "hello"):\/:fromPrim (addfile "hello")))),
+                  ((quickmerge (flFromPrim (addfile "hello"):\/:flFromPrim (addfile "hello"))):>
+                   flFromPrim (rmfile "NwNSO"),
+                   flFromPrim (rmfile "NwNSO"):>
+                   (quickmerge (flFromPrim (addfile "hello"):\/:flFromPrim (addfile "hello")))),
 
                   (testhunk 1 [] ["a"]:>
                    quickmerge (testhunk 3 ["o"] ["n"]:\/:
@@ -179,7 +192,7 @@ knownCommutes = [
                    testhunk 1 ["A"] ["B","C"],
                    testhunk 1 ["A"] ["B","C"]:>
                    testhunk 3 ["B"] ["C","D"])]
-  where testhunk l o n = fromPrim $ hunk "test" l (map BC.pack o) (map BC.pack n)
+  where testhunk l o n = flFromPrim $ hunk "test" l (map BC.pack o) (map BC.pack n)
 
 knownCantCommutes :: [(FL Patch :> FL Patch) wX wY]
 knownCantCommutes = [
@@ -190,9 +203,9 @@ knownCantCommutes = [
                       (testhunk 1 [] ["a"]:>
                        quickmerge (testhunk 2 ["o"] ["n"]:\/:
                                    testhunk 2 ["o"] ["v"])),
-                      (fromPrim (addfile "test"):>
-                       fromPrim (hunk "test" 1 ([BC.pack "a"]) ([BC.pack "b"])))]
-  where testhunk l o n = fromPrim $ hunk "test" l (map BC.pack o) (map BC.pack n)
+                      (flFromPrim (addfile "test"):>
+                       flFromPrim (hunk "test" 1 ([BC.pack "a"]) ([BC.pack "b"])))]
+  where testhunk l o n = flFromPrim $ hunk "test" l (map BC.pack o) (map BC.pack n)
 
 -- ----------------------------------------------------------------------
 -- * Merge tests
@@ -210,9 +223,9 @@ knownMerges = [
                 (testhunk 3 [BC.pack "A"] []:\/:
                  testhunk 1 [BC.pack "B"] [],
                  testhunk 2 [BC.pack "A"] []),
-                (fromPrim (rmdir "./test/world"):\/:
-                 fromPrim (hunk "./world" 3 [BC.pack "A"] []),
-                 fromPrim (rmdir "./test/world")),
+                (flFromPrim (rmdir "./test/world"):\/:
+                 flFromPrim (hunk "./world" 3 [BC.pack "A"] []),
+                 flFromPrim (rmdir "./test/world")),
 
                 ((quickhunk 1 "a" "bc" :>:
                   quickhunk 6 "d" "ef" :>: NilFL):\/:
@@ -228,7 +241,7 @@ knownMerges = [
                 (testhunk 2 [BC.pack "A"] [BC.pack "B",BC.pack "C"]:\/:
                  testhunk 1 [BC.pack "B"] [BC.pack "C",BC.pack "D"],
                  testhunk 3 [BC.pack "A"] [BC.pack "B",BC.pack "C"])]
-  where testhunk l o n = fromPrim $ hunk "test" l o n
+  where testhunk l o n = flFromPrim $ hunk "test" l o n
 
 knownMergeEquivs :: [((FL Patch :\/: FL Patch) wX wY, FL Patch wY wZ)]
 knownMergeEquivs = [
@@ -252,20 +265,20 @@ knownMergeEquivs = [
                      -- move "old" "test",
                      -- joinPatches (move "a" "test" :>:
                      --              move "old" "test-conflict" :>: NilFL)),
-                     (fromPrim (hunk "test" 1 [] [BC.pack "A"]) :\/:
-                       fromPrim (hunk "test" 1 [] [BC.pack "B"]),
-                       fromPrim (hunk "test" 1 [] [BC.pack "A", BC.pack "B"])),
-                     (fromPrim (hunk "test" 1 [] [BC.pack "a"]):\/:
-                      fromPrim (hunk "test" 1 [BC.pack "b"] []),
+                     (flFromPrim (hunk "test" 1 [] [BC.pack "A"]) :\/:
+                       flFromPrim (hunk "test" 1 [] [BC.pack "B"]),
+                       flFromPrim (hunk "test" 1 [] [BC.pack "A", BC.pack "B"])),
+                     (flFromPrim (hunk "test" 1 [] [BC.pack "a"]):\/:
+                      flFromPrim (hunk "test" 1 [BC.pack "b"] []),
                       unsafeCoerceP NilFL),
                       --hunk "test" 1 [] [BC.pack "v v v v v v v",
                       --                  BC.pack "*************",
                       --                  BC.pack "a",
                       --                  BC.pack "b",
                       --                  BC.pack "^ ^ ^ ^ ^ ^ ^"]),
-                     (quickhunk 4 "a"  "" :\/:
-                      quickhunk 3 "a"  "",
-                      quickhunk 3 "aa" ""),
+                     (quickhunkFL 4 "a"  "" :\/:
+                      quickhunkFL 3 "a"  "",
+                      quickhunkFL 3 "aa" ""),
                      ((quickhunk 1 "a" "bc" :>:
                        quickhunk 6 "d" "ef" :>: NilFL) :\/:
                        (quickhunk 3 "a" "bc" :>:
@@ -274,9 +287,9 @@ knownMergeEquivs = [
                       quickhunk 8 "d" "ef" :>:
                       quickhunk 1 "a" "bc" :>:
                       quickhunk 7 "d" "ef" :>: NilFL),
-                     (quickmerge (quickhunk 2 "" "bd":\/:quickhunk 2 "" "a") :\/:
-                              quickmerge (quickhunk 2 "" "c":\/:quickhunk 2 "" "a"),
-                              quickhunk 2 "" "abdc")
+                     (quickmerge (quickhunkFL 2 "" "bd":\/:quickhunkFL 2 "" "a") :\/:
+                              quickmerge (quickhunkFL 2 "" "c":\/:quickhunkFL 2 "" "a"),
+                              quickhunkFL 2 "" "abdc")
                      ]
 
 
@@ -311,19 +324,24 @@ testPatchesCompositeFourHunks :: [FL Patch wX wY]
 testPatchesMerged :: [FL Patch wX wY]
 validPatches :: [FL Patch wX wY]
 
-testPatchesNamed = [unsafePerformIO $
-                      namepatch "date is" "patch name" "David Roundy" []
-                                (fromPrim $ addfile "test"),
-                      unsafePerformIO $
-                      namepatch "Sat Oct 19 08:31:13 EDT 2002"
-                                "This is another patch" "David Roundy"
-                                ["This log file has","two lines in it"]
-                                (fromPrim $ rmfile "test")]
-testPatchesAddfile = map fromPrim
+testPatchesNamed =
+  [ unsafePerformIO $ do
+      info <- patchinfo "date is" "patch name" "David Roundy" []
+      return $ infopatch info $ addfile "test" :>: NilFL
+  , unsafePerformIO $ do
+      info <-
+        patchinfo
+          "Sat Oct 19 08:31:13 EDT 2002"
+          "This is another patch"
+          "David Roundy"
+          ["This log file has", "two lines in it"]
+      return $ infopatch info $ rmfile "test" :>: NilFL
+  ]
+testPatchesAddfile = map flFromPrim
                        [addfile "test",adddir "test",addfile "test/test"]
 testPatchesRmfile = map invert testPatchesAddfile
 testPatchesHunk  =
-    [fromPrim (hunk file line old new) |
+    [flFromPrim (hunk file line old new) |
      file <- ["test"],
      line <- [1,2],
      old <- map (map BC.pack) partials,
@@ -332,17 +350,21 @@ testPatchesHunk  =
     ]
     where partials  = [["A"],["B"],[],["B","B2"]]
 
+fromRight :: Either a b -> b
+fromRight (Right x) = x
+fromRight _ = error "impossible"
+
 primitiveTestPatches = testPatchesAddfile ++
                          testPatchesRmfile ++
                          testPatchesHunk ++
-                         [unsafeUnseal.fromJust.readPatch $
+                         [unseal unsafeCoercePEnd.fromRight.readPatch $
                           BC.pack "move ./test/test ./hello",
-                          unsafeUnseal.fromJust.readPatch $
+                          unseal unsafeCoercePEnd.fromRight.readPatch $
                           BC.pack "move ./test ./hello"] ++
                          testPatchesBinary
 
 testPatchesBinary =
-    [fromPrim $ binary "./hello"
+    [flFromPrim $ binary "./hello"
      (BC.pack $ "agadshhdhdsa75745457574asdgg" ++
       "a326424677373735753246463gadshhdhdsaasdgg" ++
       "a326424677373735753246463gadshhdhdsaasdgg" ++
@@ -351,7 +373,7 @@ testPatchesBinary =
       "a326424677373735753246463gadshhdhdsaasdgg" ++
       "a326424677373735753246463gadshhdhdsaasdgg" ++
       "a326424677373735753246463gadshhdhdsaagg"),
-     fromPrim $ binary "./hello"
+     flFromPrim $ binary "./hello"
      B.empty
      (BC.pack "adafjttkykrere")]
 
