@@ -15,6 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
+{-# LANGUAGE OverloadedStrings #-}
 module Darcs.Test.Patch.Examples.Set2Unwitnessed
        ( primPermutables, primPatches
        , commutables, commutablesFL
@@ -22,16 +23,25 @@ module Darcs.Test.Patch.Examples.Set2Unwitnessed
        , repov2NonduplicateTriples, repov2Patches, repov2PatchLoopExamples
        ) where
 
+import Darcs.Prelude
+
 import Data.Maybe ( catMaybes )
 import qualified Data.ByteString.Char8 as BC ( pack )
+import Data.String ( IsString(..) )
+
 import Darcs.Patch.Witnesses.Sealed
-import Darcs.Patch.Invert ( Invert(..) )
-import Darcs.Patch.Prim ( PrimPatch, fromPrim )
+import Darcs.Patch ( invert, hunk )
+import Darcs.Patch.Commute ( Commute )
+import Darcs.Patch.Invert ( Invert )
+import Darcs.Patch.FromPrim ( fromAnonymousPrim )
+import Darcs.Patch.Prim ( PrimPatch )
 import Darcs.Patch.V2 ( RepoPatchV2 )
 -- import Darcs.Test.Patch.Test () -- for instance Eq Patch
 -- import Darcs.Test.Patch.Examples.Set2Unwitnessed
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
-import qualified Darcs.Test.Patch.Arbitrary.RepoPatchV2 as W ( notDuplicatestriple )
+import qualified Darcs.Test.Patch.Arbitrary.Generic as W ( notDuplicatestriple )
+import Darcs.Test.Patch.Arbitrary.RepoPatchV2 ()
+import Darcs.Test.Patch.Arbitrary.PrimV1 ()
 --import Darcs.Util.Printer ( greenText )
 --import Darcs.Util.Printer.Color ( traceDoc )
 --import Darcs.Util.Printer.Color ( errorDoc )
@@ -44,10 +54,10 @@ import qualified Data.ByteString as B ( ByteString )
 import Darcs.Test.Patch.V1Model ( V1Model, Content
                                 , makeRepo, makeFile)
 import Darcs.Test.Patch.WithState ( WithStartState(..) )
-import Darcs.Util.Path ( makeName )
-import Darcs.Patch.Prim ( PrimPatchBase(..), FromPrim, PrimConstruct(..) )
+import Darcs.Util.Path ( AnchoredPath, floatPath, makeName )
+import Darcs.Patch.FromPrim ( PrimPatchBase(..), FromPrim )
 import Darcs.Patch.Merge ( Merge )
-import Darcs.Test.Patch.Arbitrary.Generic
+import Darcs.Test.Patch.Arbitrary.PatchTree
     ( Tree(..)
     , TreeWithFlattenPos(..)
     , commutePairFromTree, commuteTripleFromTree
@@ -55,24 +65,32 @@ import Darcs.Test.Patch.Arbitrary.Generic
     , canonizeTree
     )
 
+instance IsString AnchoredPath where
+  fromString = floatPath
+
+
 -- import Debug.Trace
 
 type Patch = RepoPatchV2 Prim2
 
 makeSimpleRepo :: String -> Content -> V1Model wX
-makeSimpleRepo filename content = makeRepo [(makeName filename, makeFile content)]
+makeSimpleRepo filename content =
+    makeRepo [(either error id $ makeName filename, makeFile content)]
 
+withStartState :: s wX -> p wX -> Sealed (WithStartState s p)
+withStartState s p = seal (WithStartState s p)
 
-w_tripleExamples :: (FromPrim p, Merge p, Invert p, PrimPatchBase p, PrimConstruct (PrimOf p)) => [Sealed2 (p W.:> p W.:> p)]
-w_tripleExamples = [commuteTripleFromTree seal2 $
-                   WithStartState (makeSimpleRepo "file" [])
+w_tripleExamples :: (FromPrim p, Merge p, PrimPatchBase p)
+                 => [Sealed2 (p W.:> p W.:> p)]
+w_tripleExamples = catMaybes [commuteTripleFromTree seal2 $
+                   withStartState (makeSimpleRepo "file" [])
                    (ParTree
                     (SeqTree (hunk "file" 1 [] [BC.pack "g"])
                      (SeqTree (hunk "file" 2 [] [BC.pack "j"])
                       (SeqTree (hunk "file" 1 [] [BC.pack "s"]) NilTree)))
                     (SeqTree (hunk "file" 1 [] [BC.pack "e"]) NilTree))
                   ,commuteTripleFromTree seal2 $
-                   WithStartState (makeSimpleRepo "file" [BC.pack "j"])
+                   withStartState (makeSimpleRepo "file" [BC.pack "j"])
                    (ParTree
                     (SeqTree (hunk "file" 1 [] [BC.pack "s"])
                      (ParTree
@@ -82,13 +100,14 @@ w_tripleExamples = [commuteTripleFromTree seal2 $
                   ]
 
 
-w_mergeExamples :: (FromPrim p, Merge p, Invert p, PrimPatchBase p, PrimConstruct (PrimOf p)) => [Sealed2 (p W.:\/: p)]
+w_mergeExamples :: (FromPrim p, Commute p, Merge p, PrimPatchBase p)
+                => [Sealed2 (p W.:\/: p)]
 w_mergeExamples = map (unseal2 (mergePairFromCommutePair seal2)) w_commuteExamples
 
-w_commuteExamples :: (FromPrim p, Merge p, PrimPatchBase p, PrimConstruct (PrimOf p)) => [Sealed2 (p W.:> p)]
-w_commuteExamples = [
+w_commuteExamples :: (FromPrim p, Merge p, PrimPatchBase p) => [Sealed2 (p W.:> p)]
+w_commuteExamples = catMaybes [
                    commutePairFromTWFP seal2 $
-                   WithStartState (makeSimpleRepo "file" [])
+                   withStartState (makeSimpleRepo "file" [])
                    (TWFP 3
                     (ParTree
                      (SeqTree (hunk "file" 1 [] [BC.pack "h"]) NilTree)
@@ -97,7 +116,7 @@ w_commuteExamples = [
                          (SeqTree (hunk "file" 1 [] [BC.pack "v"])
                            (SeqTree (hunk "file" 2 [BC.pack "f"] []) NilTree)))))),
                    commutePairFromTWFP seal2 $
-                   WithStartState
+                   withStartState
                    (makeSimpleRepo "file" [BC.pack "f",BC.pack "s",BC.pack "d"])
                    (TWFP 3
                     (ParTree
@@ -108,7 +127,7 @@ w_commuteExamples = [
                         (SeqTree (hunk "file" 1 [BC.pack "s",BC.pack "d"] [])
                           (SeqTree (hunk "file" 1 [] [BC.pack "v"]) NilTree)))))),
 {-                   commutePairFromTWFP seal2 $
-                   WithStartState
+                   withStartState
                    (makeSimpleRepo "file" [BC.pack "f",BC.pack "u",
                                             BC.pack "s",BC.pack "d"])
                    (TWFP 5
@@ -122,7 +141,7 @@ w_commuteExamples = [
                         (SeqTree (hunk "file" 1 [] [BC.pack "a"])
                          (SeqTree (hunk "file" 1 [BC.pack "a"] []) NilTree))))))),-}
                    commutePairFromTree seal2 $
-                   WithStartState (makeSimpleRepo "file" [BC.pack "n",BC.pack "t",BC.pack "h"])
+                   withStartState (makeSimpleRepo "file" [BC.pack "n",BC.pack "t",BC.pack "h"])
                    (ParTree
                     (SeqTree (hunk "file" 1 [BC.pack "n",BC.pack "t",BC.pack "h"] [])
                      NilTree)
@@ -130,13 +149,13 @@ w_commuteExamples = [
                      (SeqTree (hunk "file" 1 [BC.pack "n"] [])
                       (SeqTree (hunk "file" 1 [BC.pack "t"] []) NilTree)))),
                   commutePairFromTree seal2 $
-                  WithStartState (makeSimpleRepo "file" [])
+                  withStartState (makeSimpleRepo "file" [])
                   (ParTree
                    (SeqTree (hunk "file" 1 [] [BC.pack "n"]) NilTree)
                    (SeqTree (hunk "file" 1 [] [BC.pack "i"])
                                 (SeqTree (hunk "file" 1 [] [BC.pack "i"]) NilTree))),
                   commutePairFromTree seal2 $
-                  WithStartState (makeSimpleRepo "file" [])
+                  withStartState (makeSimpleRepo "file" [])
                   (ParTree
                    (SeqTree (hunk "file" 1 [] [BC.pack "c"])
                      (ParTree
@@ -145,7 +164,7 @@ w_commuteExamples = [
                         (SeqTree (hunk "file" 1 [] [BC.pack "d"]) NilTree))))
                    (SeqTree (hunk "file" 1 [] [BC.pack "f"]) NilTree)),
                   commutePairFromTWFP seal2 $
-                  WithStartState (makeSimpleRepo "file" [])
+                  withStartState (makeSimpleRepo "file" [])
                   (TWFP 1
                   (ParTree
                    (ParTree
@@ -153,7 +172,7 @@ w_commuteExamples = [
                     (SeqTree (hunk "file" 1 [] [BC.pack "t"]) NilTree))
                    (SeqTree (hunk "file" 1 [] [BC.pack "f"]) NilTree))),
                    commutePairFromTWFP seal2 $
-                   WithStartState (makeSimpleRepo "file" [BC.pack "f",BC.pack " r",
+                   withStartState (makeSimpleRepo "file" [BC.pack "f",BC.pack " r",
                                                             BC.pack "c",BC.pack "v"])
                    (TWFP 4
                     (ParTree
@@ -165,7 +184,7 @@ w_commuteExamples = [
                           (SeqTree (hunk "file" 1 [] [BC.pack "y"]) NilTree))))
                      (SeqTree (hunk "file" 4 [BC.pack "v"] []) NilTree))),
                    commutePairFromTree seal2 $
-                   WithStartState (makeSimpleRepo "file" [])
+                   withStartState (makeSimpleRepo "file" [])
                    (ParTree
                     (SeqTree (hunk "file" 1 [] [BC.pack "z"]) NilTree)
                     (ParTree
@@ -174,7 +193,7 @@ w_commuteExamples = [
                       (SeqTree (hunk "file" 1 [] [BC.pack "r"]) NilTree)
                       (SeqTree (hunk "file" 1 [] [BC.pack "d"]) NilTree))))
                  , commutePairFromTree seal2 $
-                   WithStartState (makeSimpleRepo "file" [BC.pack "t",BC.pack "r",BC.pack "h"])
+                   withStartState (makeSimpleRepo "file" [BC.pack "t",BC.pack "r",BC.pack "h"])
                    (ParTree
                     (ParTree
                      (SeqTree (hunk "file" 1 [BC.pack "t",BC.pack "r",BC.pack "h"] [])
@@ -183,7 +202,7 @@ w_commuteExamples = [
                     (SeqTree (hunk "file" 1 [BC.pack "t"] [])
                      (SeqTree (hunk "file" 2 [BC.pack "h"] []) NilTree)))
                  , commutePairFromTWFP seal2 $
-                   WithStartState (makeSimpleRepo "file" []) $
+                   withStartState (makeSimpleRepo "file" []) $
                    TWFP 2
                    (ParTree
                     (SeqTree (hunk "file" 1 [] [BC.pack "h"]) NilTree)
@@ -191,26 +210,26 @@ w_commuteExamples = [
                      (SeqTree (hunk "file" 2 [] [BC.pack "m"])
                       (SeqTree (hunk "file" 1 [] [BC.pack "v"]) NilTree))))
                  , commutePairFromTree seal2 $
-                 WithStartState (makeSimpleRepo "file" [])
+                 withStartState (makeSimpleRepo "file" [])
                  (ParTree
                   (SeqTree (hunk "file" 1 [] [BC.pack "p"])
                    (SeqTree (hunk "file" 1 [BC.pack "p"] [])
                     (SeqTree (hunk "file" 1 [] [BC.pack "c"]) NilTree)))
                   (SeqTree (hunk "file" 1 [] [BC.pack "z"]) NilTree))
                  , commutePairFromTree seal2 $
-                 WithStartState (makeSimpleRepo "file" [])
+                 withStartState (makeSimpleRepo "file" [])
                  (ParTree
                   (SeqTree (hunk "file" 1 [] [BC.pack "j" ])
                    (SeqTree (hunk "file" 1 [BC.pack "j"] []) NilTree))
                   (SeqTree (hunk "file" 1 [] [BC.pack "v"]) NilTree))
                  , commutePairFromTree seal2 $
-                 WithStartState (makeSimpleRepo "file" [])
+                 withStartState (makeSimpleRepo "file" [])
                  (ParTree
                   (SeqTree (hunk "file" 1 [] [BC.pack "v"]) NilTree)
                   (SeqTree (hunk "file" 1 [] [BC.pack "j" ])
                    (SeqTree (hunk "file" 1 [BC.pack "j"] []) NilTree)))
                  , commutePairFromTree seal2 $
-                 WithStartState (makeSimpleRepo "file" [BC.pack "x",BC.pack "c"])
+                 withStartState (makeSimpleRepo "file" [BC.pack "x",BC.pack "c"])
                  (ParTree
                   (SeqTree (hunk "file" 1 [] [BC.pack "h"])
                    (ParTree
@@ -219,7 +238,7 @@ w_commuteExamples = [
                      (SeqTree (hunk "file" 1 [] [BC.pack "j"]) NilTree))))
                   (SeqTree (hunk "file" 1 [] [BC.pack "l"]) NilTree))
                  , commutePairFromTree seal2 $
-                 WithStartState (makeSimpleRepo "file" [])
+                 withStartState (makeSimpleRepo "file" [])
                  (ParTree
                   (SeqTree (hunk "file" 1 [] (packStringLetters "s")) NilTree)
                   (SeqTree (hunk "file" 1 [] (packStringLetters "k"))
@@ -291,7 +310,7 @@ w_repov2PatchLoopExamples =
              NilTree
              NilTree)))))))))]
   where
-      fx :: String
+      fx :: IsString a => a
       fx = "F"
 
 mergeExamples :: [Sealed2 (Patch :\/: Patch)]
@@ -352,12 +371,12 @@ repov2Patches = concatMap commutable2patches repov2Commutables
 repov2Triples :: [(Patch :> Patch :> Patch) wX wY]
 repov2Triples = [ob' :> oa2 :> a2'',
                  oa' :> oa2 :> a2'']
-               ++ map unsafeUnseal2 tripleExamples
-               ++ map unsafeUnseal2 (concatMap getTriples repov2FLs)
-    where oa = fromPrim $ quickhunk 1 "o" "aa"
+               ++ map (unseal2 unsafeCoerceP) tripleExamples
+               ++ map (unseal2 unsafeCoerceP) (concatMap getTriples repov2FLs)
+    where oa = fromAnonymousPrim $ quickhunk 1 "o" "aa"
           oa2 = oa
-          a2 = fromPrim $ quickhunk 2 "a34" "2xx"
-          ob = fromPrim $ quickhunk 1 "o" "bb"
+          a2 = fromAnonymousPrim $ quickhunk 2 "a34" "2xx"
+          ob = fromAnonymousPrim $ quickhunk 1 "o" "bb"
           ob' :/\: oa' = merge (oa :\/: ob)
           a2' :/\: _ = merge (ob' :\/: a2)
           a2'' :/\: _ = merge (oa2 :\/: a2')
@@ -367,22 +386,22 @@ repov2NonduplicateTriples = filter (notDuplicatestriple) repov2Triples
 
 repov2FLs :: [FL (Patch) wX wY]
 repov2FLs = [oa :>: invert oa :>: oa :>: invert oa :>: ps +>+ oa :>: invert oa :>: NilFL]
-    where oa = fromPrim $ quickhunk 1 "o" "a"
+    where oa = fromAnonymousPrim $ quickhunk 1 "o" "a"
           ps :/\: _ = merge (oa :>: invert oa :>: NilFL :\/: oa :>: invert oa :>: NilFL)
 
 repov2Commutables :: [(Patch :> Patch) wX wY]
-repov2Commutables = map unsafeUnseal2 commuteExamples++
+repov2Commutables = map (unseal2 unsafeCoerceP) commuteExamples++
                      map mergeable2commutable repov2Mergeables++
-                     [invert oa :> ob'] ++ map unsafeUnseal2 (concatMap getPairs repov2FLs)
-    where oa = fromPrim $ quickhunk 1 "o" "a"
-          ob = fromPrim $ quickhunk 1 "o" "b"
+                     [invert oa :> ob'] ++ map (unseal2 unsafeCoerceP) (concatMap getPairs repov2FLs)
+    where oa = fromAnonymousPrim $ quickhunk 1 "o" "a"
+          ob = fromAnonymousPrim $ quickhunk 1 "o" "b"
           _ :/\: ob' = mergeFL (ob :\/: oa :>: invert oa :>: NilFL)
 
 repov2Mergeables :: [(Patch :\/: Patch) wX wY]
-repov2Mergeables = map (\ (x :\/: y) -> fromPrim x :\/: fromPrim y) mergeables
+repov2Mergeables = map (\ (x :\/: y) -> fromAnonymousPrim x :\/: fromAnonymousPrim y) mergeables
                         ++ repov2IglooMergeables
                         ++ repov2QuickcheckMergeables
-                        ++ map unsafeUnseal2 mergeExamples
+                        ++ map (unseal2 unsafeCoerceP) mergeExamples
                         ++ catMaybes (map pair2m (concatMap getPairs repov2FLs))
                         ++ [(oa :\/: od),
                             (oa :\/: a2'),
@@ -400,16 +419,16 @@ repov2Mergeables = map (\ (x :\/: y) -> fromPrim x :\/: fromPrim y) mergeables
                             (ob'' :\/: og''),
                             (ob'' :\/: oc''),
                             (oc' :\/: od'')]
-    where oa = fromPrim $ quickhunk 1 "o" "aa"
-          a2 = fromPrim $ quickhunk 2 "a34" "2xx"
-          og = fromPrim $ quickhunk 3 "4" "g"
-          ob = fromPrim $ quickhunk 1 "o" "bb"
-          b2 = fromPrim $ quickhunk 2 "b" "2"
-          oc = fromPrim $ quickhunk 1 "o" "cc"
-          od = fromPrim $ quickhunk 7 "x" "d"
-          oe = fromPrim $ quickhunk 7 "x" "e"
-          pf = fromPrim $ quickhunk 7 "x" "f"
-          od'' = fromPrim $ quickhunk 8 "x" "d"
+    where oa = fromAnonymousPrim $ quickhunk 1 "o" "aa"
+          a2 = fromAnonymousPrim $ quickhunk 2 "a34" "2xx"
+          og = fromAnonymousPrim $ quickhunk 3 "4" "g"
+          ob = fromAnonymousPrim $ quickhunk 1 "o" "bb"
+          b2 = fromAnonymousPrim $ quickhunk 2 "b" "2"
+          oc = fromAnonymousPrim $ quickhunk 1 "o" "cc"
+          od = fromAnonymousPrim $ quickhunk 7 "x" "d"
+          oe = fromAnonymousPrim $ quickhunk 7 "x" "e"
+          pf = fromAnonymousPrim $ quickhunk 7 "x" "f"
+          od'' = fromAnonymousPrim $ quickhunk 8 "x" "d"
           ob' :>: b2' :>: NilFL :/\: _ = mergeFL (oa :\/: ob :>: b2 :>: NilFL)
           a2' :/\: _ = merge (ob' :\/: a2)
           ob'' :/\: _ = merge (a2 :\/: ob')
@@ -437,12 +456,12 @@ repov2IglooMergeables = [(a :\/: b),
                     (z' :\/: y'),
                     (x' :\/: z'),
                     (a :\/: a)]
-    where a = fromPrim $ quickhunk 1 "1" "A"
-          b = fromPrim $ quickhunk 2 "2" "B"
-          c = fromPrim $ quickhunk 3 "3" "C"
-          x = fromPrim $ quickhunk 1 "1BC" "xbc"
-          y = fromPrim $ quickhunk 1 "A2C" "ayc"
-          z = fromPrim $ quickhunk 1 "AB3" "abz"
+    where a = fromAnonymousPrim $ quickhunk 1 "1" "A"
+          b = fromAnonymousPrim $ quickhunk 2 "2" "B"
+          c = fromAnonymousPrim $ quickhunk 3 "3" "C"
+          x = fromAnonymousPrim $ quickhunk 1 "1BC" "xbc"
+          y = fromAnonymousPrim $ quickhunk 1 "A2C" "ayc"
+          z = fromAnonymousPrim $ quickhunk 1 "AB3" "abz"
           x' :/\: _ = merge (a :\/: x)
           y' :/\: _ = merge (b :\/: y)
           z' :/\: _ = merge (c :\/: z)
@@ -457,11 +476,11 @@ repov2QuickcheckMergeables = [-- invert k1 :\/: n1
                              , k' :\/: k'
                              , k3 :\/: k3
                              ] ++ catMaybes (map pair2m pairs)
-    where hb = fromPrim $ quickhunk 0 "" "hb"
-          k = fromPrim $ quickhunk 0 "" "k"
-          n = fromPrim $ quickhunk 0 "" "n"
-          b = fromPrim $ quickhunk 1 "b" ""
-          d = fromPrim $ quickhunk 2 "" "d"
+    where hb = fromAnonymousPrim $ quickhunk 0 "" "hb"
+          k = fromAnonymousPrim $ quickhunk 0 "" "k"
+          n = fromAnonymousPrim $ quickhunk 0 "" "n"
+          b = fromAnonymousPrim $ quickhunk 1 "b" ""
+          d = fromAnonymousPrim $ quickhunk 2 "" "d"
           d':/\:_ = merge (b :\/: d)
           --k1 :>: n1 :>: NilFL :/\: _ = mergeFL (hb :\/: k :>: n :>: NilFL)
           --k2 :>: n2 :>: NilFL :/\: _ =
@@ -473,9 +492,9 @@ repov2QuickcheckMergeables = [-- invert k1 :\/: n1
           pair2m (Sealed2 (xx :> y)) = do y' :> _ <- commute (xx :> y)
                                           return $ unsafeCoerceP (xx :\/: y')
 
-          i = fromPrim $ quickhunk 0 "" "i"
-          x = fromPrim $ quickhunk 0 "" "x"
-          xi = fromPrim $ quickhunk 0 "xi" ""
+          i = fromAnonymousPrim $ quickhunk 0 "" "i"
+          x = fromAnonymousPrim $ quickhunk 0 "" "x"
+          xi = fromAnonymousPrim $ quickhunk 0 "xi" ""
           d3 :/\: _ = merge (xi :\/: d)
           _ :/\: k3 = mergeFL (k :\/: i :>: x :>: xi :>: d3 :>: NilFL)
 
