@@ -20,15 +20,20 @@
 module Darcs.Test.Patch.Info ( testSuite ) where
 
 import Prelude hiding ( pi )
+
+import Control.Applicative ( (<|>) )
 import qualified Data.ByteString as B ( ByteString, pack )
-import qualified Data.ByteString.Char8 as BC ( unpack )
-import Data.List ( sort , isPrefixOf )
+import qualified Data.ByteString.Char8 as BC ( pack, unpack )
+import Data.List ( sort , isPrefixOf, partition )
 import Data.Maybe ( isNothing )
 import Data.Text as T ( find, any )
 import Data.Text.Encoding ( decodeUtf8With )
 import Data.Text.Encoding.Error ( lenientDecode )
+import Data.Word ( Word32 )
+import Numeric ( showHex )
 import Test.QuickCheck ( Arbitrary(arbitrary), oneof, listOf, choose, shrink
                        , Gen, suchThat, scale )
+import Test.QuickCheck.Gen ( chooseAny )
 import Test.Framework.Providers.QuickCheck2 ( testProperty )
 import Test.Framework (Test, testGroup)
 -- import Text.Show.Pretty ( ppShow )
@@ -36,9 +41,10 @@ import Test.Framework (Test, testGroup)
 import Darcs.Patch.Info
     ( PatchInfo(..), rawPatchInfo, showPatchInfo, readPatchInfo
     , piLog, piAuthor, piName, validDate, validLog, validAuthor
-    , validDatePS, validLogPS, validAuthorPS
+    , validDatePS, validLogPS, validAuthorPS, piDateString
     )
-import Darcs.Patch.ReadMonads ( parseStrictly )
+import Darcs.Test.TestOnly.Instance ()
+import Darcs.Util.Parser ( parse )
 import Darcs.Patch.Show ( ShowPatchFor(..) )
 import Darcs.Util.ByteString
     ( decodeLocale, packStringToUTF8, unpackPSFromUTF8, linesPS )
@@ -86,16 +92,19 @@ rawPatchInfoShow = {- ppShow -} show
 
 instance Arbitrary UTF8PatchInfo where
     arbitrary = UTF8PatchInfo `fmap` arbitraryUTF8PatchInfo
-    shrink upi = flip withUTF8PatchInfo upi $ \pi -> do
-        sn <- shrink (piName pi)
-        sa <- shrink (piAuthor pi)
-        sl <- shrink (filter (not . isPrefixOf "Ignore-this:") (piLog pi))
-        i <- shrink (isInverted pi)
-        return (UTF8PatchInfo (rawPatchInfo sn (BC.unpack (_piDate pi)) sa sl i))
+    shrink (UTF8PatchInfo pi) = map UTF8PatchInfo (shrinkPatchInfo pi)
 
 instance Arbitrary UTF8OrNotPatchInfo where
     arbitrary = UTF8OrNotPatchInfo `fmap` oneof ([arbitraryUTF8PatchInfo,
                                                   arbitraryUnencodedPatchInfo])
+    shrink (UTF8OrNotPatchInfo pi) = map UTF8OrNotPatchInfo (shrinkPatchInfo pi)
+
+-- Generate a random "Ignore-this:" line that makes sure that separately
+-- generated PatchInfos are not equal
+generateJunk :: Gen String
+generateJunk =
+  fmap (("Ignore-this: " ++) . concatMap (flip showHex "")) $
+  sequence $ replicate 5 (chooseAny :: Gen Word32) 
 
 -- | Generate arbitrary patch metadata.
 -- Note : We must NOT use 'patchinfo' from Darcs.Patch.Info
@@ -107,19 +116,22 @@ arbitraryUTF8PatchInfo = do
     n <- (asString `fmap` arbitrary) `suchThat` validLog
     a <- (asString `fmap` arbitrary) `suchThat` validAuthor
     l <- lines `fmap` scale (* 2) arbitrary
-    i <- return False
-    return $ rawPatchInfo d n a l i
+    junk <- generateJunk
+    i <- arbitrary
+    return $ rawPatchInfo d n a (l ++ [junk]) i
 
 -- | Generate arbitrary patch metadata that has totally arbitrary byte strings
---   as its name, date, author and log.
+--   as its name, date, author and log, as well as an arbitrary "legacy
+--   inverted" setting.
 arbitraryUnencodedPatchInfo :: Gen PatchInfo
 arbitraryUnencodedPatchInfo = do
     d <- arbitraryByteString `suchThat` validDatePS
     n <- arbitraryByteString `suchThat` validLogPS
     a <- arbitraryByteString `suchThat` validAuthorPS
     l <- linesPS `fmap` scale (* 2) arbitraryByteString
-    i <- return False
-    return (PatchInfo d n a l i)
+    junk <- generateJunk
+    i <- arbitrary
+    return (PatchInfo d n a (l ++ [BC.pack junk]) i)
 
 arbitraryByteString :: Gen B.ByteString
 arbitraryByteString = B.pack <$> listOf arbitrary
@@ -158,7 +170,7 @@ packUnpackTest :: Test
 packUnpackTest = testProperty "Testing UTF-8 packing and unpacking" $
     \uString -> asString uString == (unpackPSFromUTF8 . packStringToUTF8) (asString uString)
 
-superset :: (Eq a, Ord a) => [a] -> [a] -> Bool
+superset :: Ord a => [a] -> [a] -> Bool
 superset a b = sorted_superset (sort a) (sort b)
   where sorted_superset (x:xs) (y:ys) | x == y = sorted_superset xs ys
                                       | x <  y = sorted_superset xs (y:ys)
@@ -176,14 +188,35 @@ withUTF8OrNotPatchInfo f mpi = case mpi of
 parseUnparseTest :: Test
 parseUnparseTest = testProperty "parse . show == id" propParseUnparse
 
-parsePatchInfo :: B.ByteString -> Maybe PatchInfo
-parsePatchInfo = fmap fst . parseStrictly readPatchInfo
+parsePatchInfo :: B.ByteString -> Either String PatchInfo
+parsePatchInfo = fmap fst . parse readPatchInfo
 
 unparsePatchInfo :: PatchInfo -> B.ByteString
 unparsePatchInfo = renderPS . showPatchInfo ForStorage
 
+-- Once generated, we assume that shrinking will preserve UTF8ness etc,
+-- so we reuse this function for all the various Arbitrary instances
+shrinkPatchInfo :: PatchInfo -> [PatchInfo]
+shrinkPatchInfo pi =
+  go shrink return return return <|>
+  go return shrink return return <|>
+  go return return shrink return <|>
+  go return return return shrink
+  where
+    go f1 f2 f3 f4 = do
+      sn <- f1 (piName pi)
+      sa <- f2 (piAuthor pi)
+      sl <- f3 logLines
+      i <- f4 (_piLegacyIsInverted pi)
+      return $ rawPatchInfo (piDateString pi) sn sa (sl ++ junkLines) i
+    -- We need to be careful to preserve the junk lines to prevent creating
+    -- two identical PatchInfos from different ones, which would break darcs' invariants
+    -- and cause a genuine failure to be shrunk into a spurious one.
+    (junkLines, logLines) = partition (isPrefixOf "Ignore-this:") . map BC.unpack . _piLog $ pi
+
 instance Arbitrary PatchInfo where
     arbitrary = arbitraryUnencodedPatchInfo
+    shrink = shrinkPatchInfo
 
 propParseUnparse :: PatchInfo -> Bool
-propParseUnparse pi = Just pi == parsePatchInfo (unparsePatchInfo pi)
+propParseUnparse pi = Right pi == parsePatchInfo (unparsePatchInfo pi)
