@@ -16,21 +16,53 @@
 --  Boston, MA 02110-1301, USA.
 
 module Darcs.Test.Patch.Properties.Generic
-    ( invertSymmetry, inverseComposition, invertRollback,
-      recommute, commuteInverses, effectPreserving,
-      permutivity, partialPermutivity,
-      patchAndInverseCommute, mergeEitherWay,
-      show_read,
-      mergeCommute, mergeConsistent, mergeArgumentsConsistent,
-      coalesceEffectPreserving, coalesceCommute, propIsMergeable
+    ( invertInvolution
+    , inverseComposition
+    , invertRollback
+    , recommute
+    , commuteInverses
+    , effectPreserving
+    , inverseDoesntCommute
+    , permutivity
+    , squareCommuteLaw
+    , mergeEitherWay
+    , showRead
+    , mergeEitherWayValid
+    , mergeCommute
+    , mergeConsistent
+    , mergeArgumentsConsistent
+    , coalesceEffectPreserving
+    , coalesceCommute
+    , PatchProperty
+    , MergeProperty
+    , SequenceProperty
     ) where
 
-import Darcs.Test.Util.TestResult ( TestResult, succeeded, failed, rejected,
-                                    (<&&>), fromMaybe )
-import Darcs.Test.Patch.RepoModel ( RepoModel, RepoState, repoApply, eqModel, showModel
-                                  , maybeFail )
-import Darcs.Test.Patch.WithState ( WithState(..), WithStartState(..) )
-import Darcs.Test.Patch.Arbitrary.Generic ( Tree, flattenOne, MightBeEmptyHunk(..), MightHaveDuplicate(..) )
+import Darcs.Prelude
+
+import Darcs.Test.Patch.RepoModel
+    ( ModelOf
+    , RepoModel
+    , RepoState
+    , eqModel
+    , maybeFail
+    , repoApply
+    , showModel
+    )
+import Darcs.Test.Util.TestResult
+    ( TestResult
+    , failed
+    , maybeFailed
+    , rejected
+    , succeeded
+    )
+import Darcs.Test.Patch.WithState ( WithState(..) )
+import Darcs.Test.Patch.Arbitrary.Generic
+    ( MightBeEmptyHunk(..)
+    , MightHaveDuplicate(..)
+    , TestablePrim
+    )
+import Darcs.Test.Patch.Properties.Check ( checkAPatch, Check )
 
 import Control.Monad ( msum )
 
@@ -38,56 +70,91 @@ import Darcs.Patch.Witnesses.Show ( Show2(..), show2 )
 import Darcs.Patch.Read ( ReadPatch )
 import Darcs.Patch.Show
     ( ShowPatchBasic, displayPatch, showPatch, ShowPatchFor(ForStorage) )
-import Darcs.Patch.Prim.Class ( PrimPatch, PrimOf, FromPrim )
 import Darcs.Patch ()
 import Darcs.Patch.Apply ( Apply, ApplyState )
-import Darcs.Patch.Commute ( commute, commuteFL )
+import Darcs.Patch.Commute ( Commute, commute, commuteFL )
+import Darcs.Patch.CommuteFn ( CommuteFn )
 import Darcs.Patch.Merge ( Merge(merge) )
 import Darcs.Patch.Read ( readPatch )
-import Darcs.Patch.Invert ( Invert, invert, invertFL )
+import Darcs.Patch.Invert ( Invert(..) )
 import Darcs.Patch.Witnesses.Eq ( Eq2(..), EqCheck(..) )
-import Darcs.Patch.Witnesses.Ordered ( FL(..), (:>)(..), (:\/:)(..), (:/\:)(..), lengthFL, eqFL, reverseRL )
-import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed), seal2, Sealed2 )
-import Darcs.Util.Printer ( Doc, renderPS, redText, greenText, ($$), text )
+import Darcs.Patch.Witnesses.Ordered
+    ( (:/\:)(..)
+    , (:>)(..)
+    , (:\/:)(..)
+    , FL(..)
+    , RL(..)
+    , eqFL
+    , mapFL
+    )
+import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
+import Darcs.Util.Printer ( Doc, renderPS, redText, greenText, ($$), text, vcat )
 --import Darcs.ColorPrinter ( traceDoc )
 
-propIsMergeable :: forall model p wX . (FromPrim p, Merge p, RepoModel model)
-                  => Sealed (WithStartState model (Tree (PrimOf p)))
-                  -> Maybe (Tree p wX)
-propIsMergeable (Sealed (WithStartState _ t))
-   = case flattenOne t of
-        Sealed ps -> let _ = seal2 ps :: Sealed2 (FL p)
-                     in case lengthFL ps of
-                       _ -> Nothing
+type PatchProperty p = forall wA wB. p wA wB -> TestResult
+-- type PairProperty p = forall wA wB. (p :> p) wA wB -> TestResult
+type MergeProperty p = forall wA wB. (FL p :\/: FL p) wA wB -> TestResult
+type SequenceProperty p = forall wA wB. RL p wA wB -> TestResult
 
--- | invert symmetry   inv(inv(p)) = p
-invertSymmetry :: (Invert p, Eq2 p) => p wA wB -> TestResult
-invertSymmetry p = case invert (invert p) =\/= p of
-                        IsEq  -> succeeded
-                        NotEq -> failed $ redText "p /= inv(inv(p))"
+-- | @A^^=A@
+invertInvolution :: (Invert p, Eq2 p, ShowPatchBasic p) => p wA wB -> TestResult
+invertInvolution p =
+  let p' = invert (invert p)
+  in case p =\/= p' of
+    IsEq  -> succeeded
+    NotEq ->
+      failed $ redText "p /= p^^, where"
+      $$ text "##p=" $$ displayPatch p
+      $$ text "##p^^=" $$ displayPatch p'
 
-inverseComposition :: (Invert p, Eq2 p) => (p :> p) wX wY -> TestResult
+displayPatchFL :: ShowPatchBasic p => FL p wX wY -> Doc
+displayPatchFL = vcat . mapFL displayPatch
+
+-- | @(AB)^ = B^A^@
+inverseComposition :: (Invert p, Eq2 p, ShowPatchBasic p)
+                   => (p :> p) wX wY -> TestResult
 inverseComposition (a :> b) =
-    case eqFL (reverseRL (invertFL (a:>:b:>:NilFL))) (invert b:>:invert a:>:NilFL) of
-      IsEq -> succeeded
-      NotEq -> failed $ redText "inv(a :>: b :>: NilFL) /= inv(b) :>: inv(a) :>: NilFL"
+  let ab = a:>:b:>:NilFL
+      iab = invert ab
+      ibia = invert b:>:invert a:>:NilFL
+  in case eqFL iab ibia of
+    IsEq -> succeeded
+    NotEq ->
+      failed $ redText "ab^ /= b^a^, where"
+      $$ text "##ab=" $$ displayPatchFL ab
+      $$ text "##(ab)^=" $$ displayPatchFL iab
+      $$ text "##b^a^=" $$ displayPatchFL ibia
 
--- | invert rollback   if b = A(a) then a = A'(b)
-invertRollback :: (Invert p, Apply p, ApplyState p ~ RepoState model, ShowPatchBasic p, RepoModel model)
-                => WithState model p wA wB -> TestResult
-invertRollback (WithState a x b)
-  = case maybeFail $ repoApply b (invert x) of
-         Nothing -> failed $ redText "x' not applicable to b."
-         Just a1 -> if a1 `eqModel` a
-                       then succeeded
-                       else failed $ redText "a1: " $$ text (showModel a1)
-                                  $$ redText " ---- is not equals to a:" $$ text (showModel a)
-                                  $$ redText "where a was" $$ text (showModel b)
-                                  $$ redText "with (invert x) on top:" $$ displayPatch (invert x)
+-- | @ apply A x = y ==> apply A^ y = x@
+invertRollback
+  :: ( ApplyState p ~ RepoState model
+     , Invert p
+     , Apply p
+     , ShowPatchBasic p
+     , RepoModel model
+     , model ~ ModelOf p
+     )
+  => WithState p wA wB
+  -> TestResult
+invertRollback (WithState a x b) =
+  case maybeFail $ repoApply b (invert x) of
+    Nothing -> failed $ redText "x^ not applicable to b."
+    Just a' ->
+      if a' `eqModel` a
+        then
+          succeeded
+        else
+          failed $
+            redText "##original repo a:" $$ text (showModel a) $$
+            redText "##with patch x:" $$ displayPatch x $$
+            redText "##results in b:" $$ text (showModel b) $$
+            redText "##but (invert x):" $$ displayPatch (invert x) $$
+            redText "##applied to b is a':" $$ text (showModel a') $$
+            redText "##which is not equal to a."
 
 -- | recommute   AB ↔ B′A′ if and only if B′A′ ↔ AB
 recommute :: (ShowPatchBasic p, Eq2 p, MightHaveDuplicate p)
-          => (forall wX wY . ((p :> p) wX wY -> Maybe ((p :> p) wX wY)))
+          => CommuteFn p p
           -> (p :> p) wA wB -> TestResult
 recommute c (x :> y) =
     case c (x :> y) of
@@ -124,16 +191,25 @@ recommute c (x :> y) =
 
 -- | commuteInverses   AB ↔ B′A′ if and only if B⁻¹A⁻¹ ↔ A′⁻¹B′⁻¹
 commuteInverses :: (Invert p, ShowPatchBasic p, Eq2 p)
-                => (forall wX wY . (p :> p) wX wY -> Maybe ((p :> p) wX wY))
+                => CommuteFn p p
                 -> (p :> p) wA wB -> TestResult
 commuteInverses c (x :> y) =
     case c (x :> y) of
-    Nothing -> rejected
+    Nothing ->
+      -- check that inverse commute neither
+      case c (invert y :> invert x) of
+        Just _ -> failed $
+          redText "second commute did not fail"
+          $$ redText "x" $$ displayPatch x
+          $$ redText "y" $$ displayPatch y
+          $$ redText "invert y" $$ displayPatch (invert y)
+          $$ redText "invert x" $$ displayPatch (invert x)
+        Nothing -> succeeded
     Just (y' :> x') ->
         case c (invert y :> invert x) of
         Nothing -> failed $ redText "second commute failed" $$
                             redText "x" $$ displayPatch x $$ redText "y" $$ displayPatch y $$
-                            redText "y'" $$ displayPatch y' $$ redText "x'" $$ displayPatch x'
+                            redText "invert y" $$ displayPatch (invert y) $$ redText "invert x" $$ displayPatch (invert x)
         Just (ix' :> iy') ->
             case invert ix' =/\= x' of
             NotEq -> failed $ redText "invert ix' /= x'" $$
@@ -154,11 +230,12 @@ effectPreserving
   :: ( Apply p
      , MightBeEmptyHunk p
      , RepoModel model
+     , model ~ ModelOf p
      , ApplyState p ~ RepoState model
      , ShowPatchBasic p
      )
-  => (forall wX wY. (p :> p) wX wY -> Maybe ((p :> p) wX wY))
-  -> WithState model (p :> p) wA wB
+  => CommuteFn p p
+  -> WithState (p :> p) wA wB
   -> TestResult
 effectPreserving _ (WithState _ (x :> _) _)
   | isEmptyHunk x = rejected
@@ -201,13 +278,13 @@ effectPreserving c (WithState r (x :> y) r') =
   where
     displayModel = text . showModel
 
--- | patchAndInverseCommute   If AB ↔ B′A′ then A⁻¹B′ ↔ BA′⁻¹
-patchAndInverseCommute
+-- | squareCommuteLaw   If AB ↔ B′A′ then A⁻¹B′ ↔ BA′⁻¹
+squareCommuteLaw
   :: (Invert p, ShowPatchBasic p, Eq2 p)
-  => (forall wX wY. (p :> p) wX wY -> Maybe ((p :> p) wX wY))
+  => CommuteFn p p
   -> (p :> p) wA wB
   -> TestResult
-patchAndInverseCommute c (x :> y) =
+squareCommuteLaw c (x :> y) =
   case c (x :> y) of
     Nothing -> rejected
     Just (y' :> x') ->
@@ -243,85 +320,101 @@ patchAndInverseCommute c (x :> y) =
                   redText "invert ix'" $$ displayPatch (invert ix')
                 IsEq -> succeeded
 
-permutivity :: (ShowPatchBasic p, Eq2 p) => (forall wX wY . (p :> p) wX wY -> Maybe ((p :> p) wX wY))
+permutivity :: (ShowPatchBasic p, Eq2 p)
+            => CommuteFn p p
             -> (p :> p :> p) wA wB -> TestResult
-permutivity c (x:>y:>z) =
+permutivity c (x :> y :> z) =
   case c (x :> y) of
-  Nothing -> rejected
-  Just (y1 :> x1) ->
+   Nothing -> rejected
+   Just (y1 :> x1) ->
     case c (y :> z) of
     Nothing -> rejected
     Just (z2 :> y2) ->
       case c (x :> z2) of
-      Nothing -> rejected
+      Nothing ->
+        case c (x1 :> z) of
+          Just _ -> failed $ redText "##partial permutivity:" $$
+            redText "##x" $$ displayPatch x $$
+            redText "##y" $$ displayPatch y $$
+            redText "##z" $$ displayPatch z $$
+            redText "##y1" $$ displayPatch y1 $$
+            redText "##x1" $$ displayPatch x1 $$
+            redText "##z2" $$ displayPatch z2 $$
+            redText "##y2" $$ displayPatch y2 $$
+            redText "##x :> z2 does not commute, whereas x1 :> z does"
+          Nothing -> succeeded
       Just (z3 :> x3) ->
         case c (x1 :> z) of
-          Nothing -> failed $ redText "permutivity1"
+          Nothing ->
+            failed $ redText "##permutivity1:" $$
+              redText "##x" $$ displayPatch x $$
+              redText "##y" $$ displayPatch y $$
+              redText "##z" $$ displayPatch z $$
+              redText "##y1" $$ displayPatch y1 $$
+              redText "##y2" $$ displayPatch y2 $$
+              redText "##failed commute with z of" $$
+              redText "##x1" $$ displayPatch x1 $$
+              redText "##whereas x commutes with" $$
+              redText "##z2" $$ displayPatch z2
           Just (z4 :> x4) ->
             --traceDoc (greenText "third commuted" $$
             --          greenText "about to commute" $$
             --          greenText "y1" $$ displayPatch y1 $$
             --          greenText "z4" $$ displayPatch z4) $
             case c (y1 :> z4) of
-            Nothing -> failed $ redText "permutivity2"
+            Nothing ->
+              failed $ redText "##permutivity2:" $$
+                redText "##failed to commute y1 with z4, where" $$
+                redText "##x" $$ displayPatch x $$
+                redText "##y" $$ displayPatch y $$
+                redText "##z" $$ displayPatch z $$
+                redText "##y1" $$ displayPatch y1 $$
+                redText "##x1" $$ displayPatch x1 $$
+                redText "##z2" $$ displayPatch z2 $$
+                redText "##y2" $$ displayPatch y2 $$
+                redText "##z3" $$ displayPatch z3 $$
+                redText "##x3" $$ displayPatch x3 $$
+                redText "##z4" $$ displayPatch z4 $$
+                redText "##x4" $$ displayPatch x4
             Just (z3_ :> y4)
                 | IsEq <- z3_ =\/= z3 ->
                      --traceDoc (greenText "passed z3") $ error "foobar test" $
                      case c (y4 :> x4) of
-                     Nothing -> failed $ redText "permutivity5: input was" $$
-                                         redText "x" $$ displayPatch x $$
-                                         redText "y" $$ displayPatch y $$
-                                         redText "z" $$ displayPatch z $$
-                                         redText "z3" $$ displayPatch z3 $$
-                                         redText "failed commute of" $$
-                                         redText "y4" $$ displayPatch y4 $$
-                                         redText "x4" $$ displayPatch x4 $$
-                                         redText "whereas commute of x and y give" $$
-                                         redText "y1" $$ displayPatch y1 $$
-                                         redText "x1" $$ displayPatch x1
+                     Nothing -> failed $
+                        redText "##permutivity5: input was" $$
+                        redText "##x" $$ displayPatch x $$
+                        redText "##y" $$ displayPatch y $$
+                        redText "##z" $$ displayPatch z $$
+                        redText "##z3" $$ displayPatch z3 $$
+                        redText "##z4" $$ displayPatch z4 $$
+                        redText "##failed commute of" $$
+                        redText "##y4" $$ displayPatch y4 $$
+                        redText "##x4" $$ displayPatch x4 $$
+                        redText "##whereas commute of x and y give" $$
+                        redText "##y1" $$ displayPatch y1 $$
+                        redText "##x1" $$ displayPatch x1
                      Just (x3_ :> y2_)
-                          | NotEq <- x3_ =\/= x3 -> failed $ redText "permutivity6"
-                          | NotEq <- y2_ =/\= y2 -> failed $ redText "permutivity7"
+                          | NotEq <- x3_ =\/= x3 ->
+                              failed $
+                                redText "##permutivity6: x3_ /= x3" $$
+                                redText "##x3_" $$ displayPatch x3_ $$
+                                redText "##x3" $$ displayPatch x3
+                          | NotEq <- y2_ =/\= y2 ->
+                              failed $
+                                redText "##permutivity7: y2_ /= y2" $$
+                                redText "##y2_" $$ displayPatch y2_ $$
+                                redText "##y2" $$ displayPatch y2
                           | otherwise -> succeeded
                 | otherwise ->
-                    failed $ redText "permutivity failed" $$
-                             redText "z3" $$ displayPatch z3 $$
-                             redText "z3_" $$ displayPatch z3_
-
-partialPermutivity
-  :: (Invert p, ShowPatchBasic p)
-  => (forall wX wY. (p :> p) wX wY -> Maybe ((p :> p) wX wY))
-  -> (p :> p :> p) wA wB
-  -> TestResult
-partialPermutivity c (xx :> yy :> zz) =
-  pp (xx :> yy :> zz) <&&> pp (invert zz :> invert yy :> invert xx)
-  where
-    pp (x :> y :> z) =
-      case c (y :> z) of
-        Nothing -> rejected
-        Just (z1 :> y1) ->
-          case c (x :> z1) of
-            Nothing -> rejected
-            Just (_ :> x1) ->
-              case c (x :> y) of
-                Just _ -> rejected -- this is covered by full permutivity test above
-                Nothing ->
-                  case c (x1 :> y1) of
-                    Nothing -> succeeded
-                    Just _ ->
-                      failed $
-                      greenText "partialPermutivity error" $$ greenText "x" $$
-                      displayPatch x $$
-                      greenText "y" $$
-                      displayPatch y $$
-                      greenText "z" $$
-                      displayPatch z
+                    failed $ redText "##permutivity failed" $$
+                             redText "##z3" $$ displayPatch z3 $$
+                             redText "##z3_" $$ displayPatch z3_
 
 mergeArgumentsConsistent :: (ShowPatchBasic p) =>
                               (forall wX wY . p wX wY -> Maybe Doc)
                            -> (p :\/: p) wA wB -> TestResult
 mergeArgumentsConsistent isConsistent (x :\/: y) =
-  fromMaybe $
+  maybeFailed $
     msum [(\z -> redText "mergeArgumentsConsistent x" $$ displayPatch x $$ z) `fmap` isConsistent x,
           (\z -> redText "mergeArgumentsConsistent y" $$ displayPatch y $$ z) `fmap` isConsistent y]
 
@@ -331,7 +424,7 @@ mergeConsistent :: (ShowPatchBasic p, Merge p) =>
 mergeConsistent isConsistent (x :\/: y) =
     case merge (x :\/: y) of
     y' :/\: x' ->
-      fromMaybe $
+      maybeFailed $
         msum [(\z -> redText "mergeConsistent x" $$ displayPatch x $$ z) `fmap` isConsistent x,
               (\z -> redText "mergeConsistent y" $$ displayPatch y $$ z) `fmap` isConsistent y,
               (\z -> redText "mergeConsistent x'" $$ displayPatch x' $$ z $$
@@ -339,15 +432,33 @@ mergeConsistent isConsistent (x :\/: y) =
                      redText "and y" $$ displayPatch y) `fmap` isConsistent x',
               (\z -> redText "mergeConsistent y'" $$ displayPatch y' $$ z) `fmap` isConsistent y']
 
-mergeEitherWay :: (Eq2 p, Merge p) => (p :\/: p) wX wY -> TestResult
+-- merge (A\/B) = B'/\A' <==> merge (B\/A) = A'/\B'
+--  or, equivalently,
+-- merge . swap_par = swap_antipar . merge
+--  where swap_par  (A\/B) = B\/A and swap_antipar (A/\B) = B/\A
+-- It should not be needed to test this, since it follows from
+-- mergeCommute and recommute.
+mergeEitherWay :: (Eq2 p, ShowPatchBasic p, Merge p)
+               => (p :\/: p) wX wY -> TestResult
 mergeEitherWay (x :\/: y) =
-    case merge (x :\/: y) of
-    y' :/\: x' -> case merge (y :\/: x) of
-                  x'' :/\: y'' | IsEq <- x'' =\/= x',
-                                 IsEq <- y'' =\/= y' -> succeeded
-                               | otherwise -> failed $ redText "mergeEitherWay bug"
+  case merge (x :\/: y) of
+    y' :/\: x' ->
+      case merge (y :\/: x) of
+        x'' :/\: y''
+          | IsEq <- x'' =\/= x'
+          , IsEq <- y'' =\/= y' -> succeeded
+          | otherwise ->
+            failed $
+              redText "##x" $$ displayPatch x $$
+              redText "##y" $$ displayPatch y $$
+              redText "##y'" $$ displayPatch y' $$
+              redText "##x'" $$ displayPatch x' $$
+              redText "##x''" $$ displayPatch x'' $$
+              redText "##y''" $$ displayPatch y'' $$
+              redText "##x'' /= x' or y'' /= y'"
 
-mergeCommute :: (Eq2 p, ShowPatchBasic p, Merge p, MightHaveDuplicate p)
+-- merge (A\/B) = B'/\A' ==> AB' <--> BA'
+mergeCommute :: (Eq2 p, ShowPatchBasic p, Commute p, Merge p, MightHaveDuplicate p)
              => (p :\/: p) wX wY -> TestResult
 mergeCommute (x :\/: y) =
     case merge (x :\/: y) of
@@ -394,9 +505,9 @@ mergeCommute (x :\/: y) =
 
 -- | coalesce effect preserving
 coalesceEffectPreserving
-            :: (PrimPatch prim, RepoModel model, ApplyState prim ~ RepoState model )
+            :: TestablePrim prim
             => (forall wX wY . (prim :> prim) wX wY -> Maybe (FL prim wX wY))
-            -> WithState model (prim :> prim) wA wB -> TestResult
+            -> WithState (prim :> prim) wA wB -> TestResult
 coalesceEffectPreserving j (WithState r (a :> b) r') =
   case j (a :> b) of
        Nothing -> rejected
@@ -422,7 +533,7 @@ coalesceEffectPreserving j (WithState r (a :> b) r') =
                                         $$ text (showModel r_x)
 
 coalesceCommute
-          :: (PrimPatch prim, MightBeEmptyHunk prim)
+          :: (TestablePrim prim, MightBeEmptyHunk prim)
           => (forall wX wY . (prim :> prim) wX wY -> Maybe (FL prim wX wY))
           -> (prim :> prim :> prim) wA wB -> TestResult
 coalesceCommute _ (a :> _ :> _) | isEmptyHunk a = rejected
@@ -474,18 +585,62 @@ coalesceCommute j (a :> b :> c) =
 
 -- note: we would normally use displayPatch in the failure message
 -- but that would be very misleading here
-show_read :: (Show2 p, Eq2 p, ReadPatch p, ShowPatchBasic p) => p wA wB -> TestResult
-show_read p = let ps = renderPS (showPatch ForStorage p)
-              in case readPatch ps of
-                 Nothing -> failed (redText "unable to read " $$ showPatch ForStorage p)
-                 Just (Sealed p'  ) | IsEq <- p' =\/= p -> succeeded
-                                    | otherwise -> failed $ redText "trouble reading patch p" $$
-                                                            showPatch ForStorage p $$
-                                                            redText "reads as p'" $$
-                                                            showPatch ForStorage p' $$
-                                                            redText "aka" $$
-                                                            greenText (show2 p) $$
-                                                            redText "and" $$
-                                                            greenText (show2 p')
+showRead :: (Show2 p, Eq2 p, ReadPatch p, ShowPatchBasic p) => p wA wB -> TestResult
+showRead p =
+  let ps = renderPS (showPatch ForStorage p)
+   in case readPatch ps of
+        Left e -> failed (redText "unable to read " $$ showPatch ForStorage p $$ text e)
+        Right (Sealed p')
+          | IsEq <- p' =\/= p -> succeeded
+          | otherwise ->
+            failed $
+            redText "##trouble reading patch p" $$ showPatch ForStorage p $$
+            redText "##reads as p'" $$
+            showPatch ForStorage p' $$
+            redText "##aka" $$
+            greenText (show2 p) $$
+            redText "##and" $$
+            greenText (show2 p')
 
 -- vim: fileencoding=utf-8 :
+
+mergeEitherWayValid
+  :: (Check p, Merge p, Invert p, ShowPatchBasic p)
+  => (p :\/: p) wX wY
+  -> TestResult
+mergeEitherWayValid (p1 :\/: p2) =
+  case merge (p1 :\/: p2) of
+    _ :/\: p1' ->
+      case p2 :>: p1' :>: NilFL of
+        combo2 ->
+          case merge (p2 :\/: p1) of
+            _ :/\: p2' ->
+              case p1 :>: p2' :>: NilFL of
+                combo1
+                  | not $ checkAPatch combo1 ->
+                      failed $ text "combo1 invalid: p1="
+                      $$ displayPatch p1
+                      $$ text "p2="
+                      $$ displayPatch p2
+                      $$ text "combo1="
+                      $$ vcat (mapFL displayPatch combo1)
+                  | checkAPatch (invert combo1 :>: combo2 :>: NilFL) ->
+                      succeeded
+                  | otherwise ->
+                      failed $ text "merge both ways invalid: p1="
+                      $$ displayPatch p1
+                      $$ text "p2="
+                      $$ displayPatch p2
+                      $$ text "combo1="
+                      $$ vcat (mapFL displayPatch combo1)
+                      $$ text "combo2="
+                      $$ vcat (mapFL displayPatch combo2)
+
+inverseDoesntCommute :: (ShowPatchBasic p, Invert p, Commute p)
+                     => p wY1 wY2 -> TestResult
+inverseDoesntCommute x =
+  case commute (x :> invert x) of
+    Nothing -> succeeded
+    Just (ix' :> x') -> failed $ redText "x:" $$ displayPatch x
+      $$ redText "commutes with x^ to ix':" $$ displayPatch ix'
+      $$ redText "x':" $$ displayPatch x'
