@@ -1,6 +1,9 @@
 module Darcs.Test.Patch.RepoModel where
+
+import Darcs.Prelude
+
 import Darcs.Patch.Apply ( Apply, ApplyState )
-import Darcs.Patch.Witnesses.Ordered ( FL )
+import Darcs.Patch.Witnesses.Ordered ( FL, RL )
 
 import Test.QuickCheck ( Gen )
 
@@ -37,6 +40,7 @@ class RepoModel model where
   aSmallRepo :: Gen (model x)
   repoApply :: (Apply p, ApplyState p ~ RepoState model) => model x -> p x y -> Fail (model y)
 
-type family ModelOf (patch :: * -> * -> *) :: * -> *
+type family ModelOf (p :: * -> * -> *) :: * -> *
 
-type instance ModelOf (FL prim) = ModelOf prim
+type instance ModelOf (FL p) = ModelOf p
+type instance ModelOf (RL p) = ModelOf p
