@@ -2,6 +2,8 @@
 
 module Darcs.Test.Patch.Selection ( testSuite ) where
 
+import Darcs.Prelude
+
 import Test.Framework (Test, testGroup)
 import Test.Framework.Providers.HUnit ( testCase )
 
@@ -14,16 +16,18 @@ import Darcs.Patch.RepoType ( RepoType(..), RebaseType(..) )
 
 import Darcs.UI.SelectChanges
     ( PatchSelectionOptions(..)
-    , selectionContext
+    , selectionConfig
     , runSelection
     , WhichChanges(..)
     )
-import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd(..) )
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, patchInfoAndPatch )
 import Darcs.Patch.Info ( rawPatchInfo )
 import Darcs.UI.Options.All
-    ( Verbosity(..), Summary(..)
+    ( Verbosity(..), WithSummary(..)
     , WithContext(..), SelectDeps(..), MatchFlag(..) )
 
+import Darcs.Test.TestOnly.Instance ()
+
 -- A test module for interactive patch selection.
 
 type Patch = RepoPatchV2 V2.Prim
@@ -42,16 +46,16 @@ dontReadContents whch =
        lnmPatches [] = NilFL
        lnmPatches (n:names) = buildPatch n  :>: lnmPatches names
        buildPatch :: String -> PatchInfoAnd ('RepoType 'NoRebase) Patch wX wY
-       buildPatch name = PIAP (rawPatchInfo "1999" name "harness" [] False) (error "Patch content read!")
+       buildPatch name = patchInfoAndPatch (rawPatchInfo "1999" name "harness" [] False) (error "Patch content read!")
        pso = PatchSelectionOptions
            { verbosity = Quiet
            , matchFlags = [OnePatch "."]  -- match on every patch
            , interactive = False
            , selectDeps = AutoDeps
-           , summary = NoSummary
+           , withSummary = NoSummary
            , withContext = NoContext
            }
-       context = selectionContext whch "select" pso Nothing Nothing
+       context = selectionConfig whch "select" pso Nothing Nothing
    (unselected :> selected) <- runSelection launchNuclearMissilesPatches context
    -- Forcing selection to happen (at least to the point of knowing whether unselected
    -- and unselected are NilFL or not) should not evaluate the `undefined` inside of our
