@@ -91,6 +91,8 @@ runtest' (ShellTest fmt _ _ dp da) srcdir =
      let pathToUse = map (fromText . pack) $ takeDirectory dp:Native.splitSearchPath p
      let env =
           [ ("HOME", EnvFilePath wd)
+          -- in case someone has XDG_CACHE_HOME set:
+          , ("XDG_CACHE_HOME", EnvFilePath (wd </> ".cache"))
           , ("TESTDATA", EnvFilePath (srcdir </> "tests" </> "data"))
           , ("TESTBIN", EnvFilePath (srcdir </> "tests" </> "bin"))
           , ("DARCS_TESTING_PREFS_DIR", EnvFilePath $ wd </> ".darcs")
