@@ -1,7 +1,6 @@
 {-# OPTIONS_GHC -fno-warn-missing-methods -fno-warn-orphans #-}
-{-# LANGUAGE TypeSynonymInstances, MultiParamTypeClasses
-           , ConstraintKinds, UndecidableInstances
-           , UndecidableSuperClasses #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE UndecidableInstances, UndecidableSuperClasses #-}
 -- Copyright (C) 2010, 2011 Petr Rockai
 --
 -- Permission is hereby granted, free of charge, to any person
@@ -29,7 +28,6 @@ module Darcs.Patch.ApplyMonad
   , ApplyMonadTree(..)
   ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import qualified Data.ByteString      as B
@@ -38,54 +36,50 @@ import qualified Data.Map             as M
 import qualified Darcs.Util.Tree.Monad as TM
 import Darcs.Util.Tree ( Tree )
 import Data.Maybe ( fromMaybe )
-import Darcs.Util.Path
-    ( FileName, movedirfilename, fn2fp, isParentOrEqOf, floatPath, AnchoredPath )
+import Darcs.Util.Path ( AnchoredPath, movedirfilename, isPrefix )
 import Control.Monad.State.Strict
 import Control.Monad.Identity( Identity )
 import Darcs.Patch.MonadProgress
 
 import GHC.Exts ( Constraint )
 
-fn2ap :: FileName -> AnchoredPath
-fn2ap = floatPath . fn2fp
-
 class ToTree s where
   toTree :: s m -> Tree m
 
 instance ToTree Tree where
   toTree = id
 
-class (Functor m, Monad m, ApplyMonad state (ApplyMonadOver state m))
+class (Monad m, ApplyMonad state (ApplyMonadOver state m))
       => ApplyMonadTrans (state :: (* -> *) -> *) m where
   type ApplyMonadOver state m :: * -> *
   runApplyMonad :: (ApplyMonadOver state m) x -> state m -> m (x, state m)
 
-instance (Functor m, Monad m) => ApplyMonadTrans Tree m where
+instance Monad m => ApplyMonadTrans Tree m where
   type ApplyMonadOver Tree m = TM.TreeMonad m
   runApplyMonad = TM.virtualTreeMonad
 
 class ApplyMonadState (state :: (* -> *) -> *) where
   type ApplyMonadStateOperations state :: (* -> *) -> Constraint
 
-class (Functor m, Monad m) => ApplyMonadTree m where
+class Monad m => ApplyMonadTree m where
     -- a semantic, Tree-based interface for patch application
-    mDoesDirectoryExist ::  FileName -> m Bool
-    mDoesFileExist ::  FileName -> m Bool
-    mReadFilePS ::  FileName -> m B.ByteString
-    mCreateDirectory ::  FileName -> m ()
-    mRemoveDirectory ::  FileName -> m ()
-    mCreateFile ::  FileName -> m ()
+    mDoesDirectoryExist ::  AnchoredPath -> m Bool
+    mDoesFileExist ::  AnchoredPath -> m Bool
+    mReadFilePS ::  AnchoredPath -> m B.ByteString
+    mCreateDirectory ::  AnchoredPath -> m ()
+    mRemoveDirectory ::  AnchoredPath -> m ()
+    mCreateFile ::  AnchoredPath -> m ()
     mCreateFile f = mModifyFilePS f $ \_ -> return B.empty
-    mRemoveFile ::  FileName -> m ()
-    mRename ::  FileName -> FileName -> m ()
-    mModifyFilePS ::  FileName -> (B.ByteString -> m B.ByteString) -> m ()
+    mRemoveFile ::  AnchoredPath -> m ()
+    mRename ::  AnchoredPath -> AnchoredPath -> m ()
+    mModifyFilePS ::  AnchoredPath -> (B.ByteString -> m B.ByteString) -> m ()
     mChangePref ::  String -> String -> String -> m ()
     mChangePref _ _ _ = return ()
 
 instance ApplyMonadState Tree where
     type ApplyMonadStateOperations Tree = ApplyMonadTree
 
-class ( Functor m, Monad m, Functor (ApplyMonadBase m), Monad (ApplyMonadBase m)
+class ( Monad m, Monad (ApplyMonadBase m)
       , ApplyMonadStateOperations state m, ToTree state
       )
        -- ApplyMonadOver (ApplyMonadBase m) ~ m is *not* required in general,
@@ -99,48 +93,48 @@ class ( Functor m, Monad m, Functor (ApplyMonadBase m), Monad (ApplyMonadBase m)
 
     getApplyState :: m (state (ApplyMonadBase m))
 
-instance (Functor m, Monad m) => ApplyMonad Tree (TM.TreeMonad m) where
+instance Monad m => ApplyMonad Tree (TM.TreeMonad m) where
     type ApplyMonadBase (TM.TreeMonad m) = m
     getApplyState = gets TM.tree
     nestedApply a start = lift $ runApplyMonad a start
     liftApply a start = do x <- gets TM.tree
                            lift $ runApplyMonad (lift $ a x) start
 
-instance (Functor m, Monad m) => ApplyMonadTree (TM.TreeMonad m) where
+instance Monad m => ApplyMonadTree (TM.TreeMonad m) where
 
-    mDoesDirectoryExist d = TM.directoryExists (fn2ap d)
-    mDoesFileExist d = TM.fileExists (fn2ap d)
-    mReadFilePS p = B.concat `fmap` BL.toChunks `fmap` TM.readFile (fn2ap p)
-    mModifyFilePS p j = do have <- TM.fileExists (fn2ap p)
-                           x <- if have then B.concat `fmap` BL.toChunks `fmap` TM.readFile (fn2ap p)
+    mDoesDirectoryExist p = TM.directoryExists p
+    mDoesFileExist p = TM.fileExists p
+    mReadFilePS p = B.concat `fmap` BL.toChunks `fmap` TM.readFile p
+    mModifyFilePS p j = do have <- TM.fileExists p
+                           x <- if have then B.concat `fmap` BL.toChunks `fmap` TM.readFile p
                                         else return B.empty
-                           TM.writeFile (fn2ap p) . BL.fromChunks . (:[]) =<< j x
-    mCreateDirectory p = TM.createDirectory (fn2ap p)
-    mRename from to = TM.rename (fn2ap from) (fn2ap to)
-    mRemoveDirectory = TM.unlink . fn2ap
-    mRemoveFile = TM.unlink . fn2ap
+                           TM.writeFile p . BL.fromChunks . (:[]) =<< j x
+    mCreateDirectory p = TM.createDirectory p
+    mRename from to = TM.rename from to
+    mRemoveDirectory = TM.unlink
+    mRemoveFile = TM.unlink
 
 -- Latest name, current original name.
-type OrigFileNameOf = (FileName, FileName)
+type OrigFileNameOf = (AnchoredPath, AnchoredPath)
 -- Touched files, new file list (after removes etc.) and rename details
-type FilePathMonadState = ([FileName], [FileName], [OrigFileNameOf])
+type FilePathMonadState = ([AnchoredPath], [AnchoredPath], [OrigFileNameOf])
 type FilePathMonad = State FilePathMonadState
 
 -- |trackOrigRename takes an old and new name and attempts to apply the mapping
 -- to the OrigFileNameOf pair. If the old name is the most up-to-date name of
 -- the file in question, the first element of the OFNO will match, otherwise if
 -- the up-to-date name was originally old, the second element will match.
-trackOrigRename :: FileName -> FileName -> OrigFileNameOf -> OrigFileNameOf
+trackOrigRename :: AnchoredPath -> AnchoredPath -> OrigFileNameOf -> OrigFileNameOf
 trackOrigRename old new pair@(latest, from)
-    | old `isParentOrEqOf` latest = (latest, movedirfilename old new latest)
-    | old `isParentOrEqOf` from = (latest, movedirfilename old new from)
+    | old `isPrefix` latest = (latest, movedirfilename old new latest)
+    | old `isPrefix` from = (latest, movedirfilename old new from)
     | otherwise = pair
 
 -- |withFileNames takes a maybe list of existing rename-pairs, a list of
 -- filenames and an action, and returns the resulting triple of affected files,
 -- updated filename list and new rename details. If the rename-pairs are not
 -- present, a new list is generated from the filesnames.
-withFileNames :: Maybe [OrigFileNameOf] -> [FileName] -> FilePathMonad a
+withFileNames :: Maybe [OrigFileNameOf] -> [AnchoredPath] -> FilePathMonad a
     -> FilePathMonadState
 withFileNames mbofnos fps x = execState x ([], fps, ofnos) where
     ofnos = fromMaybe (map (\y -> (y, y)) fps) mbofnos
@@ -151,7 +145,7 @@ instance ApplyMonad Tree FilePathMonad where
 
 instance ApplyMonadTree FilePathMonad where
     -- We can't check it actually is a directory here
-    mDoesDirectoryExist d = gets $ \(_, fs, _) -> d `elem` fs
+    mDoesDirectoryExist p = gets $ \(_, fs, _) -> p `elem` fs
 
     mCreateDirectory = mCreateFile
     mCreateFile f = modify $ \(ms, fs, rns) -> (f : ms, fs, rns)
@@ -166,7 +160,7 @@ instance ApplyMonadTree FilePathMonad where
 instance MonadProgress FilePathMonad where
   runProgressActions = silentlyRunProgressActions
 
-type RestrictedApply = State (M.Map FileName B.ByteString)
+type RestrictedApply = State (M.Map AnchoredPath B.ByteString)
 
 instance ApplyMonad Tree RestrictedApply where
   type ApplyMonadBase RestrictedApply = Identity
@@ -187,5 +181,5 @@ instance ApplyMonadTree RestrictedApply where
 instance MonadProgress RestrictedApply where
   runProgressActions = silentlyRunProgressActions
 
-withFiles :: [(FileName, B.ByteString)] -> RestrictedApply a -> [(FileName, B.ByteString)]
+withFiles :: [(AnchoredPath, B.ByteString)] -> RestrictedApply a -> [(AnchoredPath, B.ByteString)]
 withFiles p x = M.toList $ execState x $ M.fromList p
