@@ -3,11 +3,11 @@ module Darcs.Patch.FileHunk
     )
     where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Darcs.Util.Path ( FileName )
+import Darcs.Util.Path ( AnchoredPath )
 import Darcs.Patch.Format ( FileNameFormat )
+import Darcs.Patch.Invert ( Invert(..) )
 import Darcs.Patch.Show ( formatFileName )
 
 import Darcs.Util.Printer
@@ -17,7 +17,9 @@ import Darcs.Util.Printer
 import qualified Data.ByteString as B ( ByteString )
 
 
-data FileHunk wX wY = FileHunk !FileName !Int [B.ByteString] [B.ByteString]
+data FileHunk wX wY = FileHunk !AnchoredPath !Int [B.ByteString] [B.ByteString]
+
+type role FileHunk nominal nominal
 
 class IsHunk p where
     isHunk :: p wX wY -> Maybe (FileHunk wX wY)
@@ -27,3 +29,6 @@ showFileHunk x (FileHunk f line old new) =
            blueText "hunk" <+> formatFileName x f <+> text (show line)
         $$ lineColor Magenta (prefix "-" (vcat $ map userchunkPS old))
         $$ lineColor Cyan    (prefix "+" (vcat $ map userchunkPS new))
+
+instance Invert FileHunk where
+    invert (FileHunk path line old new) = FileHunk path line new old
