@@ -71,20 +71,16 @@ instance ApplyMonadTree FileModMonad where
     mRemoveDirectory = remove
     mRename a b = do
       fns <- gets fst
-      if S.notMember a fns then
-         addMod (PInvalid a)  -- works around some old repo inconsistencies
-       else
-        do -- we have to account for directory moves
-           addMod (PRename a b)
-           modifyFps (S.delete a)
-           addFile b
-           forM_ (S.toList fns) $ \fn ->
-             when (a `isPrefix` fn && a /= fn) $ do
-               modifyFps (S.delete fn)
-               let newfn = movedirfilename a b fn
-               addFile newfn
-               addMod (PRename fn newfn)
-
+      -- we have to account for directory moves
+      addMod (PRename a b)
+      modifyFps (S.delete a)
+      addFile b
+      forM_ (S.toList fns) $ \fn ->
+        when (a `isPrefix` fn && a /= fn) $ do
+          modifyFps (S.delete fn)
+          let newfn = movedirfilename a b fn
+          addFile newfn
+          addMod (PRename fn newfn)
     mModifyFilePS f _ = addMod (PTouch f)
 
 -- ---------------------------------------------------------------------
