@@ -56,10 +56,6 @@ data PatchMod a
   | PRename a
             a
   | PRemove a
-  | PInvalid a
-    -- ^ This is an invalid patch
-    --   e.g. there is a patch 'Move Autoconf.lhs Autoconf.lhs.in'
-    --   where there is no Autoconf.lhs in the darcs repo
   | PDuplicateTouch a
     -- ^ this is used for duplicate patches that don't
     --   have any effect, but we still want to keep
