@@ -19,8 +19,8 @@ module Darcs.Patch.Info
     ( PatchInfo(..) -- constructor and fields exported *only for tests*
     , rawPatchInfo  -- exported *only for tests*
     , patchinfo
-    , invertName
     , addJunk
+    , replaceJunk
     , makePatchname
     , readPatchInfo
     , justName
@@ -30,7 +30,6 @@ module Darcs.Patch.Info
     , toXml
     , toXmlShort
     , piDate
-    , setPiDate
     , piDateString
     , piName
     , piRename
@@ -48,11 +47,10 @@ module Darcs.Patch.Info
     , validAuthorPS
     ) where
 
-import Prelude ( (^) )
 import Darcs.Prelude
 
 import Data.Char ( isAscii )
-import System.Random ( randomRIO )
+import Crypto.Random ( seedNew, seedToInteger )
 import Numeric ( showHex )
 import Control.Monad ( when, unless, void )
 
@@ -62,9 +60,9 @@ import Darcs.Util.ByteString
     , unlinesPS
     , unpackPSFromUTF8
     )
-import qualified Darcs.Patch.ReadMonads as RM ( take )
-import Darcs.Patch.ReadMonads as RM ( skipSpace, char,
-                                      takeTill, anyChar, ParserM,
+import qualified Darcs.Util.Parser as RM ( take )
+import Darcs.Util.Parser as RM ( skipSpace, char,
+                                      takeTill, anyChar, Parser,
                                       option,
                                       takeTillChar,
                                       linesStartingWithEndingWith)
@@ -87,20 +85,66 @@ import Darcs.Util.Hash ( sha1PS, SHA1 )
 import Darcs.Util.Prompt ( promptYorn )
 import Darcs.Util.Show ( appPrec )
 
--- | A PatchInfo value contains the metadata of a patch. The date, name, author
--- and log fields are UTF-8 encoded text in darcs 2.4 and later, and just
--- sequences of bytes (decoded with whatever is the locale when displayed) in
--- earlier darcs.
---
--- The members with names that start with '_' are not supposed to be used
--- directly in code that does not care how the patch info is stored.
-data PatchInfo = PatchInfo { _piDate    :: !B.ByteString
-                           , _piName    :: !B.ByteString
-                           , _piAuthor  :: !B.ByteString
-                           , _piLog     :: ![B.ByteString]
-                           , isInverted :: !Bool
-                           }
-                 deriving (Eq,Ord)
+import Darcs.Test.TestOnly ( TestOnly )
+
+{- |
+A PatchInfo value contains the metadata of a patch. The date, name, author
+and log fields are UTF-8 encoded text in darcs 2.4 and later, and just
+sequences of bytes (decoded with whatever is the locale when displayed) in
+earlier darcs.
+
+The members with names that start with '_' are not supposed to be used
+directly in code that does not care how the patch info is stored.
+
+@_piLegacyIsInverted@:
+
+Historically, the @isInverted@ flag was used to indicate that a Named patch
+was inverted.
+
+We no longer support direct inversion of 'Darcs.Patch.Named.Named' patches,
+except sometimes via the 'Darcs.Patch.Invertible.Invertible' wrapper which
+tracks inversion in the wrapper.
+
+However, going even further back in time, inverted patches could be written
+out by @darcs rollback@. This was changed in 2008 so any patches on disk
+with this flag set would have been written by a darcs from prior to then.
+As they still exist, including in the darcs repository itself, we need
+to support them.
+
+As far as current darcs is concerned, the flag should be treated like any
+other field in 'PatchInfo' apart from never being set freshly:
+
+ - There is no semantic relationship between a 'PatchInfo' with
+   @piLegacyIsInverted = False@ and the same 'PatchInfo' with
+   @piLegacyIsInverted = True@. For example they are not inverses of each
+   other.
+
+- New or amended patches should never be written out with
+  @_piLegacyIsInverted = True@.
+
+ - We do need to maintain backwards compatibility so we take care to
+   preserve things like the hash, on-disk format etc.
+
+- A patch with @_piLegacyIsInverted = True@ should work with all the
+  normal darcs operations.
+
+The flag is completely separate and orthogonal to the tracking of
+explicit inversion in the 'Darcs.Patch.Invertible.Invertible' wrapper.
+The 'Darcs.Patch.Invertible.Invertible' wrapper
+is only used in memory and never stored to disk so there should be no
+confusion when reading a patch from disk. Within the codebase they
+serve completely different purposes and should not interact at all.
+-}
+data PatchInfo =
+  PatchInfo { _piDate    :: !B.ByteString
+            , _piName    :: !B.ByteString
+            , _piAuthor  :: !B.ByteString
+            , _piLog     :: ![B.ByteString]
+              -- | See the long description of this field in the
+              -- docs above.
+            , _piLegacyIsInverted :: !Bool
+            }
+  deriving (Eq,Ord)
 
 instance Show PatchInfo where
     showsPrec d (PatchInfo date name author log inverted) =
@@ -138,13 +182,19 @@ validAuthor = notElem '*'
 validAuthorPS :: B.ByteString -> Bool
 validAuthorPS = BC.notElem '*'
 
-rawPatchInfo :: String -> String -> String -> [String] -> Bool -> PatchInfo
-rawPatchInfo date name author log inverted =
+rawPatchInfo
+  :: TestOnly
+  => String -> String -> String -> [String] -> Bool -> PatchInfo
+rawPatchInfo = rawPatchInfoInternal
+
+rawPatchInfoInternal :: String -> String -> String -> [String] -> Bool -> PatchInfo
+rawPatchInfoInternal date name author log inverted =
     PatchInfo { _piDate     = BC.pack $ validateDate date
               , _piName     = packStringToUTF8 $ validateName name
               , _piAuthor   = packStringToUTF8 $ validateAuthor author
               , _piLog      = map (packStringToUTF8 . validateLog) log
-              , isInverted  = inverted }
+              , _piLegacyIsInverted  = inverted
+              }
   where
     validateAuthor = validate validAuthor "author"
     validateName = validate validLog "patch name"
@@ -159,13 +209,14 @@ rawPatchInfo date name author log inverted =
 -- the date string's sanity.
 patchinfo :: String -> String -> String -> [String] -> IO PatchInfo
 patchinfo date name author log =
-    addJunk $ rawPatchInfo date name author log False
+    addJunk $ rawPatchInfoInternal date name author log False
 
 -- | addJunk adds a line that contains a random number to make the patch
 --   unique.
 addJunk :: PatchInfo -> IO PatchInfo
 addJunk pinf =
-    do x <- randomRIO (0,2^(128 ::Integer) :: Integer)
+    do x <- seedToInteger <$> seedNew
+       -- Note: this is now 40 bytes long compare to the 32 we had before
        when (_piLog pinf /= ignoreJunk (_piLog pinf)) $
             do putStrLn $ "Lines beginning with 'Ignore-this: ' " ++
                           "will not be shown when displaying a patch."
@@ -174,6 +225,9 @@ addJunk pinf =
        return $ pinf { _piLog = BC.pack (head ignored++showHex x ""):
                                  _piLog pinf }
 
+replaceJunk :: PatchInfo -> IO PatchInfo
+replaceJunk pi@(PatchInfo {_piLog=log}) = addJunk $ pi{_piLog = ignoreJunk log}
+
 ignored :: [String] -- this is a [String] so we can change the junk header.
 ignored = ["Ignore-this: "]
 
@@ -184,13 +238,14 @@ ignoreJunk = filter isnt_ignored
 
 
 -- * Patch info formatting
-invertName :: PatchInfo -> PatchInfo
-invertName pi = pi { isInverted = not (isInverted pi) }
 
--- | Get the name, including an "UNDO: " prefix if the patch is inverted.
+-- | Get the name, including an "UNDO: " prefix if the patch is
+-- a legacy inverted patch.
 justName :: PatchInfo -> String
-justName pinf = if isInverted pinf then "UNDO: " ++ nameString
-                                     else nameString
+justName pinf =
+  if _piLegacyIsInverted pinf
+    then "UNDO: " ++ nameString
+    else nameString
   where nameString = metadataToString (_piName pinf)
 
 -- | Returns the author of a patch.
@@ -210,10 +265,10 @@ displayPatchInfo pi =
   where hfn x = case piTag pi of
                 Nothing -> inverted <+> text x
                 Just t -> text "  tagged" <+> text t
-        inverted = if isInverted pi then text "  UNDO:" else text "  *"
+        inverted = if _piLegacyIsInverted pi then text "  UNDO:" else text "  *"
 
 -- | Returns the name of the patch. Unlike 'justName', it does not preprend
---   "UNDO: " to the name if the patch is inverted.
+--   "UNDO: " to the name if the patch has the legacy inverted flag set.
 piName :: PatchInfo -> String
 piName = metadataToString . _piName
 
@@ -240,9 +295,6 @@ piDate = readPatchDate . _piDate
 piDateString :: PatchInfo -> String
 piDateString = BC.unpack . _piDate
 
-setPiDate :: String -> PatchInfo -> PatchInfo
-setPiDate date pi = pi { _piDate = BC.pack date }
-
 -- | Get the log message of a patch.
 piLog :: PatchInfo -> [String]
 piLog = map metadataToString . ignoreJunk . _piLog
@@ -282,7 +334,7 @@ toXml' includeComments pi =
     <+> text "author='" <> escapeXMLByteString (_piAuthor pi) <> text "'"
     <+> text "date='" <> escapeXMLByteString (_piDate pi) <> text "'"
     <+> text "local_date='" <> escapeXML (friendlyD $ _piDate pi) <> text "'"
-    <+> text "inverted='" <> text (show $ isInverted pi) <> text "'"
+    <+> text "inverted='" <> text (show $ _piLegacyIsInverted pi) <> text "'"
     <+> text "hash='" <> text (show $ makePatchname pi) <> text "'>"
     $$  indent abstract
     $$  text "</patch>"
@@ -329,8 +381,9 @@ bstrReplace c s bs | B.null bs   = B.empty
                                      else B.cons (B.head bs)
                                                  (bstrReplace c s (B.tail bs))
 
--- | Hash on patch metadata (patch name, author, date, log, and \"inverted\"
--- flag. Robust against context changes but does not garantee patch contents.
+-- | Hash on patch metadata (patch name, author, date, log, and the legacy
+-- \"inverted\" flag.
+-- Robust against context changes but does not guarantee patch contents.
 -- Usually used as matcher or patch identifier (see Darcs.Patch.Match).
 makePatchname :: PatchInfo -> SHA1
 makePatchname pi = sha1PS sha1_me
@@ -340,7 +393,7 @@ makePatchname pi = sha1PS sha1_me
                                   _piAuthor pi,
                                   _piDate pi,
                                   B.concat $ _piLog pi,
-                                  b2ps $ isInverted pi]
+                                  b2ps $ _piLegacyIsInverted pi]
 
 
 showPatchInfo :: ShowPatchFor -> PatchInfo -> Doc
@@ -365,7 +418,7 @@ storePatchInfo pi =
     blueText "[" <> packedString (_piName pi)
  $$ packedString (_piAuthor pi) <> text inverted <> packedString (_piDate pi)
                                  <> myunlines (_piLog pi) <> blueText "] "
-    where inverted = if isInverted pi then "*-" else "**"
+    where inverted = if _piLegacyIsInverted pi then "*-" else "**"
           myunlines [] = empty
           myunlines xs =
               foldr (\s -> ((text "\n " <> packedString s) <>)) (text "\n") xs
@@ -380,7 +433,7 @@ storePatchInfo pi =
 -- > ]
 --
 -- See 'showPatchInfo' for the inverse operation.
-readPatchInfo :: ParserM m => m PatchInfo
+readPatchInfo :: Parser PatchInfo
 readPatchInfo = do
   skipSpace
   char '['
@@ -395,5 +448,5 @@ readPatchInfo = do
                    , _piName = name
                    , _piAuthor = author
                    , _piLog = log
-                   , isInverted = BC.index s2 1 /= '*'
+                   , _piLegacyIsInverted = BC.index s2 1 /= '*'
                    }
