@@ -90,7 +90,7 @@ import Text.ParserCombinators.Parsec.Expr
     , Operator ( Infix, Prefix )
     , buildExpressionParser
     )
-import Text.Regex ( mkRegex, matchRegex )
+import Darcs.Util.Regex ( mkRegex, matchRegex )
 
 import Control.Exception ( Exception, throw )
 import Data.Maybe ( isJust )
