@@ -21,84 +21,81 @@
 module Darcs.Patch.Permutations ( removeFL, removeRL, removeCommon,
                                   commuteWhatWeCanFL, commuteWhatWeCanRL,
                                   genCommuteWhatWeCanRL, genCommuteWhatWeCanFL,
-                                  partitionFL, partitionRL,
+                                  partitionFL, partitionRL, partitionFL',
                                   simpleHeadPermutationsFL, headPermutationsRL,
-                                  headPermutationsFL,
+                                  headPermutationsFL, permutationsRL,
                                   removeSubsequenceFL, removeSubsequenceRL,
-                                  partitionConflictingFL,
-                                  inverseCommuter
+                                  partitionConflictingFL
                                 ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Data.Maybe ( mapMaybe )
-import Darcs.Patch.Commute ( Commute, commute, commuteFLorComplain, commuteRL )
-import Darcs.Patch.CommuteFn ( CommuteFn )
-import Darcs.Patch.Invert ( Invert(..) )
+import Darcs.Patch.Commute ( Commute, commute, commuteFL, commuteRL )
+import Darcs.Patch.Merge ( CleanMerge(..), cleanMergeFL )
 import Darcs.Patch.Witnesses.Eq ( Eq2(..), EqCheck(..) )
 import Darcs.Patch.Witnesses.Ordered
-    ( FL(..), RL(..), (:>)(..), (+<+)
-    , reverseFL, (+>+), (:\/:)(..), lengthFL
-    , lengthRL, reverseRL )
+    ( FL(..), RL(..), (:>)(..), (:\/:)(..), (:/\:)(..)
+    , (+<+), (+>+)
+    , lengthFL, lengthRL
+    , reverseFL, reverseRL
+    )
 
--- |split an 'FL' into "left" and "right" lists according to a predicate @p@, using commutation as necessary.
--- If a patch does satisfy the predicate but cannot be commuted past one that does not satisfy
--- the predicate, it goes in the "middle" list; to sum up, we have: @all p left@ and @all (not.p) right@, while
--- midddle is mixed.
--- Note that @p@ should be invariant under commutation (i.e. if 'x1' can commute to 'x2' then 'p x1 <=> p x2').
+-- | Split an 'FL' according to a predicate, using commutation as necessary,
+-- into those that satisfy the predicate and can be commuted to the left, and
+-- those that do not satisfy it and can be commuted to the right. Whatever
+-- remains stays in the middle.
+--
+-- Note that the predicate @p@ should be invariant under commutation:
+-- if @commute(x:>y)==Just(y':>x')@ then @p x == p x' && p y == p y'@.
 partitionFL :: Commute p
             => (forall wU wV . p wU wV -> Bool)  -- ^predicate; if true we would like the patch in the "left" list
             -> FL p wX wY                        -- ^input 'FL'
             -> (FL p :> FL p :> FL p) wX wY      -- ^"left", "middle" and "right"
+partitionFL keepleft ps =
+    case partitionFL' keepleft NilRL NilRL ps of
+        left :> middle :> right -> left :> reverseRL middle :> reverseRL right
 
--- optimise by using an accumulating parameter to track all the "right" patches that we've found so far
+-- optimise by using an accumulating parameter to track all the "left"
+-- patches that we've found so far; also do not reverse the result lists
 partitionFL' :: Commute p
              => (forall wU wV . p wU wV -> Bool)
              -> RL p wA wB  -- the "middle" patches found so far
              -> RL p wB wC  -- the "right" patches found so far
              -> FL p wC wD
-             -> (FL p :> FL p :> FL p) wA wD
-
-partitionFL keepleft = partitionFL' keepleft NilRL NilRL
-
-partitionFL' _ middle right NilFL = NilFL :> reverseRL middle :> reverseRL right
+             -> (FL p :> RL p :> RL p) wA wD
+partitionFL' _ middle right NilFL = NilFL :> middle :> right
 partitionFL' keepleft middle right (p :>: ps)
-   | keepleft p = case commuteRL (right :> p) of
-     Just (p' :> right') -> case commuteRL (middle :> p') of
-       Just (p'' :> middle') -> case partitionFL' keepleft middle' right' ps of
-         (a :> b :> c) -> p'' :>: a :> b :> c
-       Nothing -> partitionFL' keepleft (middle :<: p') right' ps
-     Nothing -> case commuteWhatWeCanRL (right :> p) of
-       (tomiddle :> p' :> right') -> partitionFL' keepleft (middle +<+ tomiddle :<: p') right' ps
-   | otherwise = partitionFL' keepleft middle (right :<: p) ps
+    | keepleft p = case commuteWhatWeCanRL (right :> p) of
+        (NilRL :> p' :> right') -> case commuteRL (middle :> p') of
+            Just (p'' :> middle') -> case partitionFL' keepleft middle' right' ps of
+                (a :> b :> c) -> p'' :>: a :> b :> c
+            Nothing -> partitionFL' keepleft (middle :<: p') right' ps
+        (tomiddle :> p' :> right') ->
+            partitionFL' keepleft (middle +<+ tomiddle :<: p') right' ps
+    | otherwise = partitionFL' keepleft middle (right :<: p) ps
 
-
--- |split an 'RL' into "left" and "right" lists according to a predicate, using commutation as necessary.
--- If a patch does satisfy the predicate but cannot be commuted past one that does not satisfy
--- the predicate, it goes in the "left" list.
-partitionRL :: Commute p
-            => (forall wU wV . p wU wV -> Bool)    -- ^predicate; if true we would like the patch in the "right" list
+-- | Split an 'RL' according to a predicate, using commutation as necessary,
+-- into those that satisfy the predicate and can be commuted to the right, and
+-- those that don't, i.e. either do not satisfy the predicate or cannot be
+-- commuted to the right.
+--
+-- Note that the predicate @p@ should be invariant under commutation:
+-- if @commute(x:>y)==Just(y':>x')@ then @p x == p x' && p y == p y'@.
+partitionRL :: forall p wX wY. Commute p
+            => (forall wU wV . p wU wV -> Bool) -- ^predicate; if true we would like the patch in the "right" list
             -> RL p wX wY                       -- ^input 'RL'
             -> (RL p :> RL p) wX wY             -- ^"left" and "right" results
-
--- optimise by using an accumulating parameter to track all the "left" patches that we've found so far
-partitionRL' :: Commute p
-             => (forall wU wV . p wU wV -> Bool)
-             -> RL p wX wZ
-             -> FL p wZ wY   -- the "left" patches found so far
-             -> (RL p :> RL p) wX wY
-
-partitionRL keepright ps = partitionRL' keepright ps NilFL
-
-partitionRL' _ NilRL qs = reverseFL qs :> NilRL
-
-partitionRL' keepright (ps :<: p) qs
-   | keepright p,
-     Right (qs' :> p') <- commuteFLorComplain (p :> qs)
-       = case partitionRL' keepright ps qs' of
-         a :> b -> a :> b :<: p'
-   | otherwise = partitionRL' keepright ps (p :>: qs)
+partitionRL keepright = go . (:> NilFL)
+  where
+    go :: (RL p :> FL p) wA wB -> (RL p :> RL p) wA wB
+    go (NilRL :> qs) = (reverseFL qs :> NilRL)
+    go (ps :<: p :> qs)
+      | keepright p
+      , Just (qs' :> p') <- commuteFL (p :> qs) =
+          case go (ps :> qs') of
+            a :> b -> a :> b :<: p'
+      | otherwise = go (ps :> p :>: qs)
 
 commuteWhatWeCanFL :: Commute p => (p :> FL p) wX wY -> (FL p :> p :> FL p) wX wY
 commuteWhatWeCanFL = genCommuteWhatWeCanFL commute
@@ -139,7 +136,7 @@ removeCommon (xs :\/: ys) = rc xs (headPermutationsFL ys)
           rc nms ((n:>ns):_) | Just ms <- removeFL n nms = removeCommon (ms :\/: ns)
           rc ms [n:>ns] = ms :\/: n:>:ns
           rc ms (_:nss) = rc ms nss
-          rc _ [] = impossible -- because we already checked for NilFL case
+          rc _ [] = error "impossible case" -- because we already checked for NilFL case
 
 -- | 'removeFL' @x xs@ removes @x@ from @xs@ if @x@ can be commuted to its head.
 --   Otherwise it returns 'Nothing'
@@ -219,6 +216,11 @@ headPermutationsRL (ps:<:p) =
                                               Just $ xs:<:p1':<:p2'
               swapfirstRL _ = Nothing
 
+-- | All permutations of an 'RL'.
+permutationsRL :: Commute p => RL p wX wY -> [RL p wX wY]
+permutationsRL ps =
+  [qs' :<: q | qs :<: q <- headPermutationsRL ps, qs' <- permutationsRL qs]
+
 instance (Eq2 p, Commute p) => Eq2 (FL p) where
     a =\/= b | lengthFL a /= lengthFL b = NotEq
              | otherwise = cmpSameLength a b
@@ -229,7 +231,7 @@ instance (Eq2 p, Commute p) => Eq2 (FL p) where
     xs =/\= ys = reverseFL xs =/\= reverseFL ys
 
 instance (Eq2 p, Commute p) => Eq2 (RL p) where
-    unsafeCompare = bug "Buggy use of unsafeCompare on RL"
+    unsafeCompare = error "Buggy use of unsafeCompare on RL"
     a =/\= b | lengthRL a /= lengthRL b = NotEq
              | otherwise = cmpSameLength a b
              where cmpSameLength :: RL p wX wY -> RL p wW wY -> EqCheck wX wW
@@ -238,21 +240,19 @@ instance (Eq2 p, Commute p) => Eq2 (RL p) where
                    cmpSameLength _ _ = NotEq
     xs =\/= ys = reverseRL xs =\/= reverseRL ys
 
--- |Partition a list into the patches that merge with the given patch and those that don't (including dependencies)
-partitionConflictingFL :: (Commute p1, Invert p1) => CommuteFn p1 p2 -> FL p1 wX wY -> p2 wX wZ -> (FL p1 :> FL p1) wX wY
-partitionConflictingFL _ NilFL _ = NilFL :> NilFL
-partitionConflictingFL commuter (x :>: xs) y =
-   case commuter (invert x :> y) of
-     Nothing -> case commuteWhatWeCanFL (x :> xs) of
-                 xs_ok :> x' :> xs_deps ->
-                   case partitionConflictingFL commuter xs_ok y of
-                     xs_clean :> xs_conflicts -> xs_clean :> (xs_conflicts +>+ (x' :>: xs_deps))
-     Just (y' :> _) ->
-       case partitionConflictingFL commuter xs y' of
-          xs_clean :> xs_conflicts -> (x :>: xs_clean) :> xs_conflicts
-
-inverseCommuter :: (Invert p, Invert q) => CommuteFn p q -> CommuteFn q p
-inverseCommuter commuter (p :> q) =
-   do invp' :> invq' <- commuter (invert q :> invert p)
-      return (invert invq' :> invert invp')
-
+-- | Partition a list into the patches that merge cleanly with the given
+-- patch and those that don't (including dependencies)
+partitionConflictingFL :: (Commute p, CleanMerge p)
+                       => FL p wX wY -> FL p wX wZ -> (FL p :> FL p) wX wY
+partitionConflictingFL NilFL _ = NilFL :> NilFL
+partitionConflictingFL (x :>: xs) ys =
+  case cleanMergeFL (x :\/: ys) of
+    Nothing ->
+      case commuteWhatWeCanFL (x :> xs) of
+        xs_ok :> x' :> xs_deps ->
+          case partitionConflictingFL xs_ok ys of
+            xs_clean :> xs_conflicts ->
+              xs_clean :> (xs_conflicts +>+ (x' :>: xs_deps))
+    Just (ys' :/\: _) ->
+      case partitionConflictingFL xs ys' of
+        xs_clean :> xs_conflicts -> (x :>: xs_clean) :> xs_conflicts
