@@ -1,5 +1,3 @@
-{-# LANGUAGE OverloadedStrings, StandaloneDeriving #-}
-
 -- Copyright (C) 2011 Petr Rockai
 --
 -- Permission is hereby granted, free of charge, to any person
@@ -35,11 +33,10 @@ module Darcs.Patch.Prim.FileUUID.Core
     , FileContent
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Darcs.Patch.Witnesses.Eq ( Eq2(..) )
-import Darcs.Patch.Witnesses.Show ( Show1(..), Show2(..), ShowDict(ShowDictClass) )
+import Darcs.Patch.Witnesses.Show ( Show1, Show2 )
 import Darcs.Patch.Witnesses.Unsafe
 import Darcs.Patch.FileHunk( IsHunk(..) )
 import Darcs.Patch.Invert ( Invert(..) )
@@ -53,11 +50,11 @@ import Darcs.Patch.Prim.FileUUID.ObjectMap
 data Hunk wX wY = H !Int !FileContent !FileContent
   deriving (Eq, Show)
 
-instance Show1 (Hunk wX) where
-  showDict1 = ShowDictClass
+type role Hunk nominal nominal
 
-instance Show2 Hunk where
-  showDict2 = ShowDictClass
+instance Show1 (Hunk wX)
+
+instance Show2 Hunk
 
 invertHunk :: Hunk wX wY -> Hunk wY wX
 invertHunk (H off old new) = H off new old
@@ -71,6 +68,8 @@ instance Eq2 Hunk where
 data HunkMove wX wY = HM !UUID !Int !UUID !Int !FileContent
   deriving (Eq, Show)
 
+type role HunkMove nominal nominal
+
 invertHunkMove :: HunkMove wX wY -> HunkMove wY wX
 invertHunkMove (HM sid soff tid toff content) = HM tid toff sid soff content
 
@@ -91,11 +90,9 @@ data Prim wX wY where
 deriving instance Eq (Prim wX wY)
 deriving instance Show (Prim wX wY)
 
-instance Show1 (Prim wX) where
-  showDict1 = ShowDictClass
+instance Show1 (Prim wX)
 
-instance Show2 Prim where
-  showDict2 = ShowDictClass
+instance Show2 Prim
 
 -- TODO: PrimClassify doesn't make sense for FileUUID prims
 instance PrimClassify Prim where
@@ -122,7 +119,6 @@ instance PrimConstruct Prim where
   tokreplace _ _ _ _ = error "PrimConstruct tokreplace"
   binary _ _ _ = error "PrimConstruct binary"
   primFromHunk _ = error "PrimConstruct primFromHunk"
-  anIdentity = Identity
 
 instance IsHunk Prim where
   isHunk _ = Nothing
