@@ -1,4 +1,5 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
+{-# LANGUAGE MultiWayIf #-}
 module Darcs.Patch.Prim.V1.Apply () where
 
 import Darcs.Prelude
@@ -105,7 +106,23 @@ instance RepairToFL Prim where
                            FP f (Binary x new) :>: NilFL
                           )
              else return Nothing
-    applyAndTryToFixFL p = do apply p; return Nothing
+    applyAndTryToFixFL p@(Move old new) =
+        do old_is_file <- mDoesFileExist old
+           old_is_dir <- mDoesDirectoryExist old
+           new_is_file <- mDoesFileExist new
+           new_is_dir <- mDoesDirectoryExist new
+           if | not (old_is_file || old_is_dir) ->
+                  return $
+                     Just ("WARNING: Dropping move patch with non-existing source "++ap2fp old,
+                           unsafeCoercePStart NilFL
+                          )
+              | new_is_file || new_is_dir ->
+                  return $
+                     Just ("WARNING: Dropping move patch with existing target "++ap2fp old,
+                           unsafeCoercePStart NilFL
+                          )
+              | otherwise -> apply p >> return Nothing
+    applyAndTryToFixFL p = apply p >> return Nothing
 
 instance PrimApply Prim where
     applyPrimFL NilFL = return ()
