@@ -15,49 +15,57 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-module Darcs.Patch.Read ( ReadPatch(..),
-                          readPatch, readPatchPartial,
-                          bracketedFL, peekfor,
-                          readFileName )
-             where
+module Darcs.Patch.Read
+    ( ReadPatch(..)
+    , readPatch
+    , readPatchPartial
+    , bracketedFL
+    , peekfor
+    , readFileName
+    ) where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Darcs.Util.ByteString ( dropSpace, unpackPSFromUTF8, decodeLocale )
-import qualified Data.ByteString       as B  (ByteString, null)
-
-import Darcs.Patch.Bracketed ( Bracketed(..), unBracketedFL )
-import Darcs.Util.Path ( FileName, fp2fn, decodeWhite )
-import Darcs.Patch.Format ( PatchListFormat(..), ListFormat(..), FileNameFormat(..) )
-import Darcs.Patch.ReadMonads (ParserM,
-                               parseStrictly,
-                               choice, lexChar, lexString,
-                               checkConsumes )
-
-import Darcs.Patch.Witnesses.Ordered ( FL(..), RL, reverseFL )
-import Darcs.Patch.Witnesses.Sealed ( Sealed(..), mapSeal )
-
 import Control.Applicative ( (<|>) )
 import Control.Monad ( mzero )
-import qualified Data.ByteString.Char8 as BC ( ByteString, pack )
+import qualified Data.ByteString as B ( ByteString, null )
+import qualified Data.ByteString.Char8 as BC ( ByteString, pack, stripPrefix )
+
+import Darcs.Patch.Bracketed ( Bracketed(..), unBracketedFL )
+import Darcs.Patch.Format
+    ( FileNameFormat(..)
+    , ListFormat(..)
+    , PatchListFormat(..)
+    )
+import Darcs.Util.Parser
+    ( Parser
+    , checkConsumes
+    , choice
+    , lexChar
+    , lexString
+    , lexWord
+    , parse
+    )
+import Darcs.Patch.Witnesses.Ordered ( FL(..), RL, reverseFL )
+import Darcs.Patch.Witnesses.Sealed ( Sealed(..), mapSeal )
+
+import Darcs.Util.ByteString ( decodeLocale, dropSpace, unpackPSFromUTF8 )
+import Darcs.Util.Path ( AnchoredPath, decodeWhite, floatPath )
 
 -- | This class is used to decode patches from their binary representation.
 class ReadPatch p where
-    readPatch'
-        :: ParserM m => m (Sealed (p wX))
+    readPatch' :: Parser (Sealed (p wX))
 
-readPatchPartial :: ReadPatch p => B.ByteString -> Maybe (Sealed (p wX), B.ByteString)
-readPatchPartial ps
-    = case parseStrictly readPatch' ps of
-         Just (p, ps') -> Just (p, ps')
-         _ -> Nothing
+readPatchPartial :: ReadPatch p => B.ByteString -> Either String (Sealed (p wX), B.ByteString)
+readPatchPartial = parse readPatch'
 
-readPatch :: ReadPatch p => B.ByteString -> Maybe (Sealed (p wX))
-readPatch ps
-    = case readPatchPartial ps of
-         Just (p, ps') | B.null (dropSpace ps') -> Just p
-         _ -> Nothing
+readPatch :: ReadPatch p => B.ByteString -> Either String (Sealed (p wX))
+readPatch ps =
+  case parse readPatch' ps of
+    Left e -> Left e
+    Right (p, leftover)
+      | B.null (dropSpace leftover) -> Right p
+      | otherwise -> Left $ unlines ["leftover:",show leftover]
 
 instance ReadPatch p => ReadPatch (Bracketed p) where
     readPatch' = mapSeal Braced <$> bracketedFL readPatch' '{' '}'
@@ -76,7 +84,7 @@ instance (ReadPatch p, PatchListFormat p) => ReadPatch (FL p) where
             = mapSeal unBracketedFL <$> readPatch'
         | otherwise
             = read_patches
-     where read_patches :: ParserM m => m (Sealed (FL p wX))
+     where read_patches :: Parser (Sealed (FL p wX))
            read_patches = do --tracePeek "starting FL read"
                              -- checkConsumes is needed to make sure that something is read,
                              -- to avoid stack overflow when parsing FL (FL p)
@@ -93,29 +101,37 @@ instance (ReadPatch p, PatchListFormat p) => ReadPatch (RL p) where
     readPatch' = mapSeal reverseFL <$> readPatch'
 
 {-# INLINE bracketedFL #-}
-bracketedFL :: forall p m wX . (ParserM m) =>
-               (forall wY . m (Sealed (p wY))) -> Char -> Char -> m (Sealed (FL p wX))
+bracketedFL :: forall p wX .
+               (forall wY . Parser (Sealed (p wY))) -> Char -> Char -> Parser (Sealed (FL p wX))
 bracketedFL parser pre post =
     peekforc pre bfl mzero
-        where bfl :: forall wZ . m (Sealed (FL p wZ))
+        where bfl :: forall wZ . Parser (Sealed (FL p wZ))
               bfl = peekforc post (return $ Sealed NilFL)
                                   (do Sealed p <- parser
                                       Sealed ps <- bfl
                                       return $ Sealed (p:>:ps))
 
 {-# INLINE peekforc #-}
-peekforc :: ParserM m => Char -> m a -> m a -> m a
+peekforc :: Char -> Parser a -> Parser a -> Parser a
 peekforc c ifstr ifnot = choice [ lexChar c >> ifstr
                                 , ifnot ]
 
-peekfor :: ParserM m => BC.ByteString -> m a -> m a -> m a
+peekfor :: BC.ByteString -> Parser a -> Parser a -> Parser a
 peekfor ps ifstr ifnot = choice [ do lexString ps
                                      ifstr
                                 , ifnot ]
 {-# INLINE peekfor #-}
 
 -- See also Darcs.Patch.Show.formatFileName.
-readFileName :: FileNameFormat -> B.ByteString -> FileName
-readFileName OldFormat = fp2fn . decodeWhite . decodeLocale . BC.pack . unpackPSFromUTF8
-readFileName NewFormat = fp2fn . decodeWhite . decodeLocale
-readFileName UserFormat = error "readFileName called with UserFormat"
+readFileName :: FileNameFormat -> Parser AnchoredPath
+readFileName fmt = do
+  raw <- lexWord
+  case BC.stripPrefix (BC.pack "./") raw of
+    Nothing -> fail $ "invalid file path"
+    Just raw' -> return $ convert fmt raw'
+  where
+    convert FileNameFormatV1 =
+      floatPath . decodeWhite . decodeLocale . BC.pack . unpackPSFromUTF8
+    convert FileNameFormatV2 =
+      floatPath . decodeWhite . decodeLocale
+    convert FileNameFormatDisplay = error "readFileName called with FileNameFormatDisplay"
