@@ -15,7 +15,6 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 module Darcs.Patch.V1.Commute
@@ -26,49 +25,59 @@ module Darcs.Patch.V1.Commute
     )
        where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Control.Monad ( MonadPlus, mplus, msum, mzero, guard )
 import Control.Applicative ( Alternative(..) )
+import Data.Maybe ( fromMaybe )
 
 import Darcs.Patch.Commute ( selfCommuter )
 import Darcs.Patch.CommuteFn ( commuterIdFL, commuterFLId )
-import Darcs.Util.Path ( FileName )
-import Darcs.Util.Printer ( errorDoc )
+import Darcs.Util.Path ( AnchoredPath )
 import Darcs.Patch.Invert ( invertRL )
-import Darcs.Patch.Merge ( Merge(..), naturalMerge )
+import Darcs.Patch.Merge ( CleanMerge(..), Merge(..) )
 import Darcs.Patch.Commute ( Commute(..) )
 import Darcs.Patch.Invert ( Invert(..) )
 import Darcs.Patch.Inspect ( PatchInspect(..) )
 import Darcs.Patch.V1.Core ( RepoPatchV1(..),
                              isMerger,
                              mergerUndo )
-import Darcs.Patch.Conflict
-    ( Conflict(..), listConflictedFiles
-    , IsConflictedPrim(..), ConflictState(..), CommuteNoConflicts(..)
-    , mangleUnravelled
+import Darcs.Patch.CommuteNoConflicts
+    ( CommuteNoConflicts(..)
+    , mergeNoConflicts
     )
+import Darcs.Patch.Conflict
+  ( Conflict(..), combineConflicts, mangleOrFail
+  )
+import Darcs.Patch.Unwind ( Unwind(..), Unwound(..), mkUnwound )
 import Darcs.Patch.Effect ( Effect(..) )
 import Darcs.Patch.FileHunk ( IsHunk(..) )
-import Darcs.Patch.Prim ( FromPrim(..), PrimPatch,
-                          is_filepatch, sortCoalesceFL,
-                        )
-import Darcs.Patch.Permutations ( headPermutationsRL, simpleHeadPermutationsFL )
-import Darcs.Util.Printer ( text, vcat, ($$) )
+import Darcs.Patch.Prim ( PrimPatch, is_filepatch )
+import Darcs.Patch.Permutations
+    ( headPermutationsRL
+    , simpleHeadPermutationsFL
+    , removeFL
+    )
+import Darcs.Util.Printer ( renderString, text, vcat, ($$) )
 import Darcs.Patch.V1.Show ( showPatch_ )
-import Data.List ( nub, nubBy )
+import Data.List ( nub )
 import Data.List.Ordered ( nubSort )
+import Darcs.Patch.Summary
+    ( Summary(..)
+    , ConflictState(..)
+    , IsConflictedPrim(..)
+    )
 import Darcs.Patch.Witnesses.Sealed
-    ( Sealed(..) , mapSeal, unseal, FlippedSeal(..), mapFlipped
-    , unsafeUnseal, unsafeUnsealFlipped )
+    ( Sealed(..) , mapSeal, unseal
+    , FlippedSeal(..), mapFlipped, unsealFlipped
+    )
 import Darcs.Patch.Witnesses.Eq ( EqCheck(..), Eq2(..) )
 import Darcs.Patch.Witnesses.Unsafe
     ( unsafeCoerceP, unsafeCoercePStart
     , unsafeCoercePEnd )
 import Darcs.Patch.Witnesses.Ordered
     ( mapFL_FL, mapFL,
-    FL(..), RL(..),
+    FL(..), RL(..), (+>+),
     (:/\:)(..), (:\/:)(..), (:>)(..),
     lengthFL, mapRL,
     reverseFL, reverseRL, concatFL
@@ -94,10 +103,6 @@ instance  Monad Perhaps where
     Failed   >>= _      =  Failed
     Unknown  >>= _      =  Unknown
     return              =  Succeeded
-#if MIN_VERSION_base(4,13,0)
-instance MonadFail Perhaps where
-#endif
-    fail _              =  Unknown
 
 instance Alternative Perhaps where
     empty = Unknown
@@ -164,7 +169,7 @@ then commutex (B^-1, A^-1) == Just (A'^-1, B'^-1)
 -}
 
 unsafeMerger :: PrimPatch prim => String -> RepoPatchV1 prim wX wY -> RepoPatchV1 prim wX wZ -> RepoPatchV1 prim wA wB
-unsafeMerger x p1 p2 = unsafeCoercePStart $ unsafeUnseal $ merger x p1 p2
+unsafeMerger x p1 p2 = unseal unsafeCoerceP $ merger x p1 p2
 
 -- | Attempt to commute two patches, the first of which is a Merger patch.
 mergerCommute :: PrimPatch prim
@@ -179,18 +184,18 @@ mergerCommute (Merger _ _ b' c'' :> Merger _ _ (Merger _ _ c b) (Merger _ _ c' a
                   )
 mergerCommute _ = Unknown
 
+instance PrimPatch prim => CleanMerge (RepoPatchV1 prim) where
+    cleanMerge = mergeNoConflicts
+
 instance PrimPatch prim => Merge (RepoPatchV1 prim) where
-    merge (y :\/: z) =
-        case actualMerge (y:\/:z) of
-        -- actualMerge returns one "arm" of a merge result (@y'@, which applies
-        -- "after" @z@), but we need to return both arms. We therefore commute
-        -- @z@ and @y'@, to obtain a @z'@, which applies "after" @y'' == y@.
-        Sealed y' -> case commute (z :> y') of
-                         Nothing -> errorDoc $ text "merge_patches bug"
-                                    $$ showPatch_ y
-                                   $$ showPatch_ z
-                                   $$ showPatch_ y'
-                         Just (_ :> z') -> unsafeCoercePStart z' :/\: y'
+    merge (p1 :\/: p2) =
+        case mergeNoConflicts (p1 :\/: p2) of
+            Just r -> r
+            Nothing ->
+                case merger "0.0" p1 p2 of
+                    Sealed p2' ->
+                        case merger "0.0" p2 p1 of
+                            Sealed p1' -> unsafeCoercePEnd p2' :/\: unsafeCoercePEnd p1'
 
 instance PrimPatch prim => Commute (RepoPatchV1 prim) where
     commute x = toMaybe $ msum
@@ -210,13 +215,7 @@ instance PrimPatch prim => PatchInspect (RepoPatchV1 prim) where
     hunkMatches f c@(Regrem{}) = hunkMatches f $ invert c
     hunkMatches f (PP p) = hunkMatches f p
 
-commuteNoMerger :: PrimPatch prim => MaybeCommute prim
-commuteNoMerger x =
-    toMaybe $ msum [ speedyCommute x
-                   , everythingElseCommute x
-                   ]
-
-isFilepatchMerger :: PrimPatch prim => RepoPatchV1 prim wX wY -> Maybe FileName
+isFilepatchMerger :: PrimPatch prim => RepoPatchV1 prim wX wY -> Maybe AnchoredPath
 isFilepatchMerger (PP p) = is_filepatch p
 isFilepatchMerger (Merger _ _ p1 p2) = do
      f1 <- isFilepatchMerger p1
@@ -267,14 +266,6 @@ otherCommuteRecursiveMerger (p_old@(Merger _ _ p1' p2') :> pA') = toPerhaps $
 otherCommuteRecursiveMerger _ = Unknown
 
 type CommuteFunction prim = forall wX wY . (RepoPatchV1 prim :> RepoPatchV1 prim) wX wY -> Perhaps ((RepoPatchV1 prim :> RepoPatchV1 prim) wX wY)
-type MaybeCommute prim = forall wX wY . (RepoPatchV1 prim :> RepoPatchV1 prim) wX wY -> Maybe ((RepoPatchV1 prim :> RepoPatchV1 prim) wX wY)
-
-commuteFLId :: MaybeCommute prim -> (RepoPatchV1 prim :> FL (RepoPatchV1 prim)) wX wY -> Maybe ((FL (RepoPatchV1 prim) :> RepoPatchV1 prim) wX wY)
-commuteFLId _        (p :> NilFL) = return (NilFL :> p)
-commuteFLId commuter (p :> (q :>: qs)) = do
-   q' :> p' <- commuter (p :> q)
-   qs' :> p'' <- commuteFLId commuter (p' :> qs)
-   return ((q' :>: qs') :> p'')
 
 {-
 A note about mergers and type witnesses
@@ -299,17 +290,6 @@ these constructors.
 -}
 
 
--- |actualMerge attempts to perform a merge; if successful, it returns the
--- "right" branch of the merge
---
--- TODO: why does this code throw away the other branch, only for merge to
--- rebuild it?
-actualMerge :: PrimPatch prim
-    => (RepoPatchV1 prim :\/: RepoPatchV1 prim) wX wY -> Sealed (RepoPatchV1 prim wY)
-actualMerge (p1 :\/: p2) = case naturalMerge (p1:\/:p2) of
-                             Just (_ :/\: p1') -> Sealed p1'
-                             Nothing -> merger "0.0" p2 p1
-
 -- Recreates a patch history in reverse.
 unwind :: RepoPatchV1 prim wX wY -> Sealed (RL (RepoPatchV1 prim) wX)
 unwind (Merger _ unwindings _ _) = Sealed unwindings
@@ -318,13 +298,14 @@ unwind p = Sealed (NilRL :<: p)
 -- Recreates a patch history in reverse. The patch being unwound is always at
 -- the start of the list of patches.
 trueUnwind :: PrimPatch prim
-    => RepoPatchV1 prim wX wY -> Sealed (RL (RepoPatchV1 prim) wX)
-trueUnwind p@(Merger _ _ p1 p2) =
-    case (unwind p1, unwind p2) of
+    => RepoPatchV1 prim wC wX -> RepoPatchV1 prim wC wD -> Sealed ((RL (RepoPatchV1 prim) :> RepoPatchV1 prim) wX)
+trueUnwind p1 p2 =
+  let fake_p = Merger NilFL NilRL p1 p2
+  in
+  case (unwind p1, unwind p2) of
     (Sealed (p1s:<:_),Sealed (p2s:<:_)) ->
-         Sealed (unsafeUnsealFlipped (reconcileUnwindings p p1s (unsafeCoercePEnd p2s)) :<: unsafeCoerceP p1 :<: p)
-    _ -> impossible
-trueUnwind _ = impossible
+         Sealed (unsealFlipped unsafeCoerceP (reconcileUnwindings fake_p p1s (unsafeCoercePEnd p2s)) :<: unsafeCoerceP p1 :> fake_p)
+    _ -> error "impossible case"
 
 reconcileUnwindings :: PrimPatch prim
     => RepoPatchV1 prim wA wB -> RL (RepoPatchV1 prim) wX wZ -> RL (RepoPatchV1 prim) wY wZ -> FlippedSeal (RL (RepoPatchV1 prim)) wZ
@@ -345,10 +326,11 @@ reconcileUnwindings p (p1s:<:p1) p2s@(tp2s:<:p2) =
               Just p1s' -> mapFlipped (:<:p2) $
                            reconcileUnwindings p p1s' tp2s
               Nothing ->
-                errorDoc $ text "in function reconcileUnwindings"
-                        $$ text "Original patch:"
-                        $$ showPatch_ p
-    _ -> bug "in reconcileUnwindings"
+                error $ renderString
+                  $ text "in function reconcileUnwindings"
+                  $$ text "Original patch:"
+                  $$ showPatch_ p
+    _ -> error "in reconcileUnwindings"
 
 -- This code seems wrong, shouldn't the commute be invert p1 :> p2 ? And why isn't p1' re-inverted?
 -- it seems to have been this way forever:
@@ -363,40 +345,51 @@ putBefore p1 (p2:>:p2s) =
 putBefore _ NilFL = Just (unsafeCoerceP NilFL)
 
 instance PrimPatch prim => CommuteNoConflicts (RepoPatchV1 prim) where
-  commuteNoConflicts (x :> y) = do y' :> x' <- commuteNoMerger (x :> y)
-                                   return (y' :> x')
+  commuteNoConflicts x =
+    toMaybe $ msum [ speedyCommute x
+                   , everythingElseCommute x
+                   ]
 
 instance PrimPatch prim => Conflict (RepoPatchV1 prim) where
-  resolveConflicts patch = rcs NilFL (NilRL :<: patch)
-    where rcs :: FL (RepoPatchV1 prim) wY wW -> RL (RepoPatchV1 prim) wX wY -> [[Sealed (FL prim wW)]]
-          rcs _ NilRL = []
-          rcs passedby (ps:<:p@(Merger{})) =
-              case commuteFLId commuteNoMerger (p :> passedby) of
-              Just (_ :> p'@(Merger _ _ p1 p2)) ->
-                  map Sealed (nubBy unsafeCompare $
-                        effect (unsafeCoercePStart $ unsafeUnseal (glump09 p1 p2)) :
-                          map (unsafeCoercePStart . unsafeUnseal) (unravel p'))
-                  : rcs (p :>: passedby) ps
-              Nothing -> rcs (p :>: passedby) ps
-              _ -> impossible
-          rcs passedby (ps:<:p) = seq passedby $
-                                  rcs (p :>: passedby) ps
+  resolveConflicts _ = map mangleOrFail . combineConflicts resolveOne
+    where
+      resolveOne p | isMerger p = [publicUnravel p]
+      resolveOne _ = []
 
-  conflictedEffect x =
-    case listConflictedFiles x of
-      [] -> mapFL (IsC Okay) $ effect x
-      _ -> mapFL (IsC Conflicted) $ effect x
+instance PrimPatch prim => Unwind (RepoPatchV1 prim) where
+  fullUnwind (PP prim) = mkUnwound NilFL (prim :>: NilFL) NilFL
+  fullUnwind (Merger a _ c d) =
+    case fullUnwind d of
+      Unwound before prim _after ->
+        mkUnwound
+          (invert (effect c) +>+ before)
+          prim
+          (invert prim +>+ invert before +>+ effect c +>+ effect a)
+  fullUnwind (Regrem a b c d) = invert (fullUnwind (Merger a b c d))
 
+instance PrimPatch prim => Summary (RepoPatchV1 prim) where
+  conflictedEffect x
+    | isMerger x = mapFL (IsC Conflicted) $ effect x
+    | otherwise = mapFL (IsC Okay) $ effect x
 
 -- This type seems wrong - the most natural type for the result would seem to be
 -- [Sealed (FL prim wX)], given the type of unwind.
 -- However downstream code in darcs convert assumes the wY type, and I was unable
 -- to figure out whether this could/should reasonably be changed -- Ganesh 13/4/10
+--
+-- bf says: the type here is correct, those of unwind and unravel are wrong,
+-- because conflict resolution applies to the end of the repo.
 publicUnravel :: PrimPatch prim => RepoPatchV1 prim wX wY -> [Sealed (FL prim wY)]
 publicUnravel = map (mapSeal unsafeCoercePStart) . unravel
 
+dropAllInverses :: (Commute p, Invert p, Eq2 p) => FL p wX wY -> FL p wX wY
+dropAllInverses NilFL = NilFL
+dropAllInverses (p :>: ps) =
+  let ps' = dropAllInverses ps in
+  fromMaybe (p :>: ps') $ removeFL (invert p) ps'
+
 unravel :: PrimPatch prim => RepoPatchV1 prim wX wY -> [Sealed (FL prim wX)]
-unravel p = nub $ map (mapSeal (sortCoalesceFL . concatFL . mapFL_FL effect)) $
+unravel p = nub $ map (mapSeal (dropAllInverses . concatFL . mapFL_FL effect)) $
             getSupers $ map (mapSeal reverseRL) $ unseal (newUr p) $ unwind p
 
 getSupers :: PrimPatch prim
@@ -424,27 +417,26 @@ Sealed x `isSuperpatchOf` Sealed y = x `iso` y
 -- the resulting Merger is therefore a representation of @p2@.
 merger :: PrimPatch prim
     => String -> RepoPatchV1 prim wX wY -> RepoPatchV1 prim wX wZ -> Sealed (RepoPatchV1 prim wY)
-merger "0.0" p1 p2 = Sealed $ Merger undoit unwindings p1 p2
-    where fake_p = Merger NilFL NilRL p1 p2
-          unwindings = unsafeUnseal (trueUnwind fake_p)
-          p = Merger NilFL unwindings p1 p2
-          undoit =
+merger "0.0" p1 p2 = final_p
+    where
+          sealed_unwindings = trueUnwind p1 p2
+          final_p =
+            case (sealed_undoit, sealed_unwindings) of
+              (Sealed undoit, Sealed unwindings)
+                -> Sealed $ Merger undoit ((\(a :> b) -> (a :<: b)) unwindings) p1 p2
+          sealed_undoit =
               case (isMerger p1, isMerger p2) of
-              (True ,True ) -> case unwind p of
-                                 Sealed (t:<:_) -> unsafeCoerceP $ invertRL t
-                                 _ -> impossible
-              (False,False) -> unsafeCoerceP $ invert p1 :>: NilFL
-              (True ,False) -> unsafeCoerceP NilFL
-              (False,True ) -> unsafeCoerceP $ invert p1 :>: mergerUndo p2
+              (True ,True ) -> case sealed_unwindings of
+                                 Sealed (t :> _) -> Sealed $ unsafeCoercePStart $ invertRL t
+              (False,False) -> Sealed $ invert p1 :>: NilFL
+              (True ,False) -> Sealed NilFL
+              (False,True ) -> Sealed $ invert p1 :>: mergerUndo p2
 merger g _ _ =
     error $ "Cannot handle mergers other than version 0.0\n"++g
     ++ "\nPlease use darcs optimize --modernize with an older darcs."
 
-glump09 :: PrimPatch prim => RepoPatchV1 prim wX wY -> RepoPatchV1 prim wX wZ -> Sealed (FL (RepoPatchV1 prim) wY)
-glump09 p1 p2 = mapSeal (mapFL_FL fromPrim) $ mangleUnravelled $ unseal unravel $ merger "0.0" p1 p2
-
 instance PrimPatch prim => Effect (RepoPatchV1 prim) where
-    effect p@(Merger{}) = sortCoalesceFL $ effect $ mergerUndo p
+    effect p@(Merger{}) = dropAllInverses $ effect $ mergerUndo p
     effect p@(Regrem{}) = invert $ effect $ invert p
     effect (PP p) = p :>: NilFL
 
@@ -457,7 +449,7 @@ newUr :: PrimPatch prim
 newUr p (ps :<: Merger _ _ p1 p2) =
    case filter (\(_:<:pp) -> pp `unsafeCompare` p1) $ headPermutationsRL ps of
    ((ps':<:_):_) -> newUr p (ps':<:unsafeCoercePStart p1) ++ newUr p (ps':<:unsafeCoercePStart p2)
-   _ -> errorDoc $ text "in function newUr"
+   _ -> error $ renderString $ text "in function newUr"
                  $$ text "Original patch:"
                  $$ showPatch_ p
                  $$ text "Unwound:"
