@@ -1,10 +1,8 @@
-{-# LANGUAGE GeneralizedNewtypeDeriving, StandaloneDeriving #-}
 module Darcs.Patch.V1.Core
     ( RepoPatchV1(..),
       isMerger, mergerUndo
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Darcs.Patch.Format
@@ -12,17 +10,17 @@ import Darcs.Patch.Format
     , ListFormat(ListFormatV1)
     )
 import Darcs.Patch.Debug ( PatchDebug(..) )
-import Darcs.Patch.Prim
-    ( FromPrim(..), PrimPatchBase(..), PrimPatch
+import Darcs.Patch.FromPrim
+    ( FromPrim(..)
+    , PrimPatchBase(..)
+    , ToPrim(..)
     )
+import Darcs.Patch.Ident ( PatchId )
+import Darcs.Patch.Prim ( PrimPatch )
 import Darcs.Patch.Repair ( Check )
 
 import Darcs.Patch.Witnesses.Ordered ( FL(..), RL )
-import Darcs.Patch.Witnesses.Show
-    ( Show1(..), Show2(..)
-    , ShowDict(ShowDictClass)
-    , appPrec, showsPrec2
-    )
+import Darcs.Patch.Witnesses.Show ( Show1, Show2, appPrec, showsPrec2 )
 
 -- This haddock could be put on the individual bits of Merger instead
 -- once haddock supports doc comments on GADT constructors
@@ -41,13 +39,13 @@ data RepoPatchV1 prim wX wY where
     PP :: prim wX wY -> RepoPatchV1 prim wX wY
     Merger :: FL (RepoPatchV1 prim) wX wY
            -> RL (RepoPatchV1 prim) wX wB
-           -> RepoPatchV1 prim wC wB
+           -> RepoPatchV1 prim wC wX
            -> RepoPatchV1 prim wC wD
            -> RepoPatchV1 prim wX wY
     Regrem :: FL (RepoPatchV1 prim) wX wY
            -> RL (RepoPatchV1 prim) wX wB
-           -> RepoPatchV1 prim wC wB
-           -> RepoPatchV1 prim wC wA
+           -> RepoPatchV1 prim wC wX
+           -> RepoPatchV1 prim wC wD
            -> RepoPatchV1 prim wY wX
 
 instance Show2 prim => Show (RepoPatchV1 prim wX wY)  where
@@ -66,17 +64,21 @@ instance Show2 prim => Show (RepoPatchV1 prim wX wY)  where
             showString " " . showsPrec2 (appPrec + 1) conflicting .
             showString " " . showsPrec2 (appPrec + 1) original
 
-instance Show2 prim => Show1 (RepoPatchV1 prim wX) where
-    showDict1 = ShowDictClass
+instance Show2 prim => Show1 (RepoPatchV1 prim wX)
 
-instance Show2 prim => Show2 (RepoPatchV1 prim) where
-    showDict2 = ShowDictClass
+instance Show2 prim => Show2 (RepoPatchV1 prim)
 
 instance PrimPatch prim => PrimPatchBase (RepoPatchV1 prim) where
     type PrimOf (RepoPatchV1 prim) = prim
 
+type instance PatchId (RepoPatchV1 prim) = ()
+
 instance FromPrim (RepoPatchV1 prim) where
-    fromPrim = PP
+    fromAnonymousPrim = PP
+
+instance ToPrim (RepoPatchV1 prim) where
+    toPrim (PP p) = Just p
+    toPrim _ = Nothing
 
 isMerger :: RepoPatchV1 prim wA wB -> Bool
 isMerger (Merger{}) = True
@@ -85,7 +87,7 @@ isMerger _ = False
 
 mergerUndo :: RepoPatchV1 prim wX wY -> FL (RepoPatchV1 prim) wX wY
 mergerUndo (Merger undo _ _ _) = undo
-mergerUndo _ = impossible
+mergerUndo _ = error "impossible case"
 
 instance PatchListFormat (RepoPatchV1 prim) where
    -- In principle we could use ListFormatDefault when prim /= V1 Prim patches,
