@@ -1,9 +1,11 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Darcs.Patch.V1.Viewing () where
 
+import Darcs.Prelude
+
 import Darcs.Patch.Prim ( PrimPatch )
 import Darcs.Patch.Show ( ShowPatch(..), ShowContextPatch(..), showPatch )
-import Darcs.Patch.Summary ( plainSummary )
+import Darcs.Patch.Summary ( plainSummary, plainSummaryFL )
 
 import Darcs.Patch.V1.Apply ()
 import Darcs.Patch.V1.Core ( RepoPatchV1(..) )
@@ -15,5 +17,5 @@ instance PrimPatch prim => ShowContextPatch (RepoPatchV1 prim) where
 
 instance PrimPatch prim => ShowPatch (RepoPatchV1 prim) where
     summary = plainSummary
-    summaryFL = plainSummary
+    summaryFL = plainSummaryFL
     thing _ = "change"
