@@ -25,35 +25,49 @@ module Darcs.Repository.ApplyPatches
     , DefaultIO, runDefault
     ) where
 
-import Control.Exception ( catch, SomeException, IOException )
+import Control.Exception ( IOException, SomeException, catch )
+import Control.Monad ( unless )
+import qualified Data.ByteString as B ( empty, null, readFile )
 import Data.Char ( toLower )
 import Data.List ( isSuffixOf )
-import System.IO ( stderr )
-import System.IO.Error ( isDoesNotExistError, isPermissionError )
-import Control.Monad ( unless, mplus )
-import System.Directory ( createDirectory,
-                          removeDirectory, removeFile,
-                          renameFile, renameDirectory,
-                          doesDirectoryExist, doesFileExist
-                        )
+import System.Directory
+    ( createDirectory
+    , doesDirectoryExist
+    , doesFileExist
+    , removeDirectory
+    , removeFile
+    , renamePath
+    )
+import System.IO ( hPutStrLn, stderr )
+import System.IO.Error ( catchIOError, isDoesNotExistError, isPermissionError )
 
 import Darcs.Prelude
 
-import Darcs.Patch.ApplyMonad( ApplyMonad(..), ApplyMonadTree(..) )
-import Darcs.Patch.ApplyPatches ( applyPatches )
+import Darcs.Patch.Apply ( Apply(..) )
+import Darcs.Patch.ApplyMonad ( ApplyMonad(..), ApplyMonadTree(..) )
+import Darcs.Patch.Info ( displayPatchInfo )
 import Darcs.Patch.MonadProgress ( MonadProgress(..), ProgressAction(..) )
-import Darcs.Repository.Prefs( changePrefval )
-import Darcs.Util.Lock ( writeAtomicFilePS )
-
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, hopefully, info )
+import Darcs.Patch.Witnesses.Ordered ( FL(..), mapFL )
+import Darcs.Repository.Prefs ( changePrefval )
 import Darcs.Util.Exception ( prettyException )
-import Darcs.Util.Progress ( beginTedious, endTedious, tediousSize, finishedOneIO )
-import Darcs.Util.Printer ( hPutDocLn, renderString )
 import Darcs.Util.External ( backupByCopying, backupByRenaming )
+import Darcs.Util.Lock ( writeAtomicFilePS )
 import Darcs.Util.Path ( AnchoredPath, anchorPath )
-import qualified Data.ByteString as B (empty, null, readFile)
-
-import Darcs.Util.Tree( Tree )
+import Darcs.Util.Printer ( hPutDocLn, renderString )
+import Darcs.Util.Printer ( text, ($$) )
+import Darcs.Util.Progress ( beginTedious, endTedious, finishedOneIO, tediousSize )
+import Darcs.Util.Tree ( Tree )
 
+applyPatches :: (MonadProgress m, ApplyMonad (ApplyState p) m, Apply p)
+             => FL (PatchInfoAnd rt p) wX wY -> m ()
+applyPatches ps = runProgressActions "Applying patch" (mapFL doApply ps)
+  where
+    doApply hp = ProgressAction { paAction = apply (hopefully hp)
+                                , paMessage = displayPatchInfo (info hp)
+                                , paOnError = text "Unapplicable patch:" $$
+                                              displayPatchInfo (info hp)
+                                }
 
 ap2fp :: AnchoredPath -> FilePath
 ap2fp = anchorPath ""
@@ -95,13 +109,7 @@ instance ApplyMonadTree DefaultIO where
                             fail $ "Cannot remove non-empty file "++fp
                        removeFile fp
     mRemoveDirectory = DefaultIO . removeDirectory . ap2fp
-    mRename a b = DefaultIO $
-                  catch
-                  (renameDirectory x y `mplus` renameFile x y)
-                  -- We need to catch does not exist errors, since older
-                  -- versions of darcs allowed users to rename nonexistent
-                  -- files.  :(
-                  (\e -> unless (isDoesNotExistError e) $ ioError e)
+    mRename a b = DefaultIO $ renamePath x y
       where x = ap2fp a
             y = ap2fp b
 
@@ -114,7 +122,7 @@ newtype TolerantIO a = TIO { runTIO :: IO a }
     deriving (Functor, Applicative, Monad)
 
 instance TolerantMonad TolerantIO where
-    warning io = TIO $ io `catch` \e -> putStrLn $ "Warning: " ++ prettyException e
+    warning io = TIO $ io `catch` \e -> hPutStrLn stderr $ "Warning: " ++ prettyException e
     runIO (TIO io) = io
     runTM = TIO
 
@@ -140,7 +148,11 @@ runSilently = runSIO . runTolerantWrapper
 -- | The default mode of applying patches: fail if the directory is not
 -- as we expect
 runDefault :: DefaultIO a -> IO a
-runDefault = runDefaultIO
+runDefault action =
+  catchIOError (runDefaultIO action) $ \e ->
+    fail $ "Cannot apply some patch:\n"++show e++
+      "\nYou may want to run 'darcs check' to find out if there are broken"++
+      "\npatches in your repo, and perhaps 'darcs repair' to fix them."
 
 instance TolerantMonad m => ApplyMonad Tree (TolerantWrapper m) where
     type ApplyMonadBase (TolerantWrapper m) = IO
