@@ -1,7 +1,9 @@
 module Darcs.Repository.Cache
     ( cacheHash
     , okayHash
-    , Cache(..)
+    , Cache
+    , mkCache
+    , cacheEntries
     , CacheType(..)
     , CacheLoc(..)
     , WritableOrNot(..)
@@ -22,25 +24,29 @@ module Darcs.Repository.Cache
     , isThisRepo
     , hashedFilePath
     , allHashedDirs
-    , compareByLocality
     , reportBadSources
+    , closestWritableDirectory
+    , dropNonRepos
     ) where
 
+import Control.Concurrent.MVar ( MVar, newMVar, modifyMVar_, readMVar )
 import Control.Monad ( liftM, when, unless, filterM, forM_, mplus )
 import qualified Data.ByteString as B (length, ByteString )
-import Data.List ( nub, intercalate )
-import Data.Maybe ( catMaybes, fromMaybe )
+import Data.List ( nub, intercalate, sortBy )
+import Data.Maybe ( catMaybes, fromMaybe, listToMaybe )
 import System.FilePath.Posix ( (</>), joinPath, dropFileName )
 import System.Directory ( createDirectoryIfMissing, removeFile, doesFileExist,
                           doesDirectoryExist, getDirectoryContents,
                           getPermissions )
 import qualified System.Directory as SD ( writable )
 import System.IO ( hPutStrLn, stderr )
+import System.IO.Unsafe (unsafePerformIO)
 import System.Posix.Files ( createLink, linkCount, getSymbolicLinkStatus )
 
+import Darcs.Prelude
+
 import Darcs.Util.ByteString ( gzWriteFilePS )
-import Darcs.Util.Global ( darcsdir, addBadSource, isBadSource, addReachableSource,
-                      isReachableSource, getBadSourcesList, defaultRemoteDarcsCmd )
+import Darcs.Util.Global ( darcsdir, defaultRemoteDarcsCmd )
 import Darcs.Util.External ( gzFetchFilePS, fetchFilePS
                            , speculateFileOrUrl, copyFileOrUrl
                            , Cachable( Cachable ) )
@@ -53,8 +59,8 @@ import Darcs.Util.File ( withCurrentDirectory )
 import Darcs.Util.Hash ( sha256sum )
 import Darcs.Util.English ( englishNum, Noun(..), Pronoun(..) )
 import Darcs.Util.Exception ( catchall )
-import Darcs.Util.Progress ( progressList, debugMessage, debugFail )
-import qualified Darcs.Util.Download as Download ( ConnectionError(..) )
+import Darcs.Util.Progress ( progressList, debugMessage )
+import qualified Darcs.Util.Download as Download ( ConnectionError )
 
 data HashedDir = HashedPristineDir
                | HashedPatchesDir
@@ -88,6 +94,12 @@ data CacheLoc = Cache
 -- | Cache is an abstract type for hiding the underlying cache locations
 newtype Cache = Ca [CacheLoc]
 
+mkCache :: [CacheLoc] -> Cache
+mkCache = Ca . sortBy compareByLocality . nub
+
+cacheEntries :: Cache -> [CacheLoc]
+cacheEntries (Ca entries) = entries
+
 instance Eq CacheLoc where
     (Cache aTy _ aSrc) == (Cache bTy _ bSrc) = aTy == bTy && aSrc == bSrc
 
@@ -111,7 +123,7 @@ unionCaches (Ca a) (Ca b) = Ca (nub (a ++ b))
 --     network entries can lead to darcs hang when it tries to get to
 --     unaccessible host.
 --
---   * If remote repositoty is local, copy all network cache entries. For local
+--   * If remote repository is local, copy all network cache entries. For local
 --     cache entries if the cache directory exists and is writable it is added
 --     as writable cache, if it exists but is not writable it is added as
 --     read-only cache.
@@ -195,6 +207,21 @@ writable :: CacheLoc -> Bool
 writable (Cache _ NotWritable _) = False
 writable (Cache _ Writable _) = True
 
+-- | This keeps only 'Repo' 'NotWritable' entries.
+dropNonRepos :: Cache -> Cache
+dropNonRepos (Ca cache) = Ca $ filter notRepo cache where
+  notRepo xs = case xs of
+    Cache Directory _ _ -> False
+    -- we don't want to write thisrepo: entries to the disk
+    Cache Repo Writable _ -> False
+    _ -> True
+
+closestWritableDirectory :: Cache -> Maybe String
+closestWritableDirectory (Ca cs) =
+  listToMaybe . catMaybes .flip map cs $ \case
+    Cache Directory Writable x -> Just x
+    _ -> Nothing
+
 isThisRepo :: CacheLoc -> Bool
 isThisRepo (Cache Repo Writable _) = True
 isThisRepo _ = False
@@ -219,9 +246,9 @@ hashedFilePath (Cache Repo _ r) s f =
 -- If directory, assume it is non-bucketed cache (old cache location).
 hashedFilePathReadOnly :: CacheLoc -> HashedDir -> String -> String
 hashedFilePathReadOnly (Cache Directory _ d) s f =
-    d ++ "/" ++ hashedDir s ++ "/" ++ f
+    d </> hashedDir s </> f
 hashedFilePathReadOnly (Cache Repo _ r) s f =
-    r ++ "/" ++ darcsdir ++ "/" ++ hashedDir s ++ "/" ++ f
+    r </> darcsdir </> hashedDir s </> f
 
 -- | @peekInCache cache subdir hash@ tells whether @cache@ and contains an
 -- object with hash @hash@ in a writable position.  Florent: why do we want it
@@ -273,7 +300,7 @@ data OrOnlySpeculate = ActuallyCopy
 copyFileUsingCache :: OrOnlySpeculate -> Cache -> HashedDir -> String -> IO ()
 copyFileUsingCache oos (Ca cache) subdir f = do
     debugMessage $
-        "I'm doing copyFileUsingCache on " ++ hashedDir subdir ++ "/" ++ f
+        "I'm doing copyFileUsingCache on " ++ hashedDir subdir </> f
     Just stickItHere <- cacheLoc cache
     createDirectoryIfMissing True
         (reverse $ dropWhile (/= '/') $ reverse stickItHere)
@@ -374,7 +401,7 @@ checkHashedInventoryReachability cache = withTemp $ \tempout -> do
     `catchNonSignal` const (return False)
 
 -- | Get contents of some hashed file taking advantage of the cache system.
--- We hace a list of locations (@cache@) ordered from "closest/fastest"
+-- We have a list of locations (@cache@) ordered from "closest/fastest"
 -- (typically, the destination repo) to "farthest/slowest" (typically,
 -- the source repo).
 -- First, if possible it copies the file from remote location to local.
@@ -386,7 +413,7 @@ fetchFileUsingCachePrivate fromWhere (Ca cache) subdir f = do
     when (fromWhere == Anywhere) $
         copyFileUsingCache ActuallyCopy (Ca cache) subdir f
     filterBadSources cache >>= ffuc
-    `catchall` debugFail ("Couldn't fetch `" ++ f ++ "'\nin subdir "
+    `catchall` fail ("Couldn't fetch " ++ f ++ "\nin subdir "
                           ++ hashedDir subdir ++ " from sources:\n\n"
                           ++ show (Ca cache))
   where
@@ -446,11 +473,11 @@ fetchFileUsingCachePrivate fromWhere (Ca cache) subdir f = do
                     return (fname, x)
         | otherwise = ffuc cs
 
-    ffuc [] = debugFail $ "No sources from which to fetch file `" ++ f
-                          ++ "'\n"++ show (Ca cache)
+    ffuc [] = fail $ "No sources from which to fetch file " ++ f
+                          ++ "\n"++ show (Ca cache)
 
     tryLinking ff c@(Cache Directory Writable d) = do
-        createDirectoryIfMissing False (d ++ "/" ++ hashedDir subdir)
+        createDirectoryIfMissing False (d </> hashedDir subdir)
         createLink ff (hashedFilePath c subdir f)
         `catchall`
         return ()
@@ -458,7 +485,7 @@ fetchFileUsingCachePrivate fromWhere (Ca cache) subdir f = do
 
 createCache :: CacheLoc -> HashedDir -> IO ()
 createCache (Cache Directory _ d) subdir =
-    createDirectoryIfMissing True (d ++ "/" ++ hashedDir subdir)
+    createDirectoryIfMissing True (d </> hashedDir subdir)
 createCache _ _ = return ()
 
 -- | @write compression filename content@ writes @content@ to the file
@@ -480,7 +507,7 @@ writeFileUsingCache (Ca cache) compr subdir ps = do
     `catchall`
     wfuc cache
     `catchall`
-    debugFail ("Couldn't write `" ++ hash ++ "'\nin subdir "
+    fail ("Couldn't write " ++ hash ++ "\nin subdir "
                ++ hashedDir subdir ++ " to sources:\n\n"++ show (Ca cache))
   where
     hash = cacheHash ps
@@ -491,8 +518,7 @@ writeFileUsingCache (Ca cache) compr subdir ps = do
             -- FIXME: create links in caches
             write compr (hashedFilePath c subdir hash) ps
             return hash
-    wfuc [] = debugFail $ "No location to write file `" ++ hashedDir subdir
-                          ++ "/" ++ hash ++ "'"
+    wfuc [] = fail $ "No location to write file " ++ (hashedDir subdir </> hash)
 
 cleanCaches :: Cache -> HashedDir -> IO ()
 cleanCaches c d = cleanCachesWithHint' c d Nothing
@@ -504,10 +530,10 @@ cleanCachesWithHint' :: Cache -> HashedDir -> Maybe [String] -> IO ()
 cleanCachesWithHint' (Ca cs) subdir hint = mapM_ cleanCache cs
   where
     cleanCache (Cache Directory Writable d) =
-        withCurrentDirectory (d ++ "/" ++ hashedDir subdir) (do
+        withCurrentDirectory (d </> hashedDir subdir) (do
             fs' <- getDirectoryContents "."
             let fs = filter okayHash $ fromMaybe fs' hint
-                cleanMsg = "Cleaning cache " ++ d ++ "/" ++ hashedDir subdir
+                cleanMsg = "Cleaning cache " ++ d </> hashedDir subdir
             mapM_ clean $ progressList cleanMsg fs)
         `catchall`
         return ()
@@ -524,12 +550,47 @@ reportBadSources = do
     sources <- getBadSourcesList
     let size = length sources
     unless (null sources) $ hPutStrLn stderr $
-        concat [ "\nHINT: I could not reach the following "
-               , englishNum size (Noun "repository") ":"
+        concat [ "\nBy the way, I could not reach the following "
+               , englishNum size (Noun "location") ":"
                , "\n"
-               , intercalate "\n" (map ("        " ++) sources)
-               , "\n      If you're not using "
-               , englishNum size It ", you should probably delete"
-               , "\n      the corresponding "
+               , intercalate "\n" (map ("  " ++) sources)
+               , "\nUnless you plan to restore access to "
+               , englishNum size It ", you should delete "
+               , "the corresponding "
                , englishNum size (Noun "entry") " from _darcs/prefs/sources."
                ]
+
+-- * Global Variables
+
+badSourcesList :: MVar [String]
+badSourcesList = unsafePerformIO $ newMVar []
+{-# NOINLINE badSourcesList #-}
+
+addBadSource :: String -> IO ()
+addBadSource cache = modifyMVarPure badSourcesList (cache:)
+
+getBadSourcesList :: IO [String]
+getBadSourcesList = readMVar badSourcesList
+
+isBadSource :: IO (String -> Bool)
+isBadSource = do
+    badSources <- getBadSourcesList
+    return (`elem` badSources)
+
+reachableSourcesList :: MVar [String]
+reachableSourcesList = unsafePerformIO $ newMVar []
+{-# NOINLINE reachableSourcesList #-}
+
+addReachableSource :: String -> IO ()
+addReachableSource src = modifyMVarPure reachableSourcesList (src:)
+
+getReachableSources :: IO [String]
+getReachableSources = readMVar reachableSourcesList
+
+isReachableSource :: IO (String -> Bool)
+isReachableSource =  do
+    reachableSources <- getReachableSources
+    return (`elem` reachableSources)
+
+modifyMVarPure :: MVar a -> (a -> a) -> IO ()
+modifyMVarPure mvar f = modifyMVar_ mvar (return . f)
