@@ -1,21 +1,19 @@
 module Darcs.Repository.Clone
     ( cloneRepository
     , replacePristine
-    , writePatchSet
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Control.Exception ( catch, SomeException )
-import Control.Monad ( when )
+import Control.Monad ( unless, void, when )
 import qualified Data.ByteString.Char8 as BC
 import Data.List( intercalate )
 import Data.Maybe( catMaybes )
 import System.FilePath( (</>) )
 import System.Directory
     ( removeFile
-    , getDirectoryContents
+    , listDirectory
     )
 import System.IO ( stderr )
 
@@ -25,17 +23,21 @@ import Darcs.Repository.Create
     , writePristine
     )
 import Darcs.Repository.State ( invalidateIndex )
-import Darcs.Repository.Pending ( tentativelyAddToPending )
-import Darcs.Repository.Identify
-    ( IdentifyRepo(..)
-    , identifyRepositoryFor
-    , maybeIdentifyRepository )
-import Darcs.Repository.Hashed
-    ( readRepo
-    , tentativelyRemovePatches
-    , finalizeRepositoryChanges
+import Darcs.Repository.Identify ( identifyRepositoryFor, ReadingOrWriting(..) )
+import Darcs.Repository.Pristine
+    ( ApplyDir(..)
+    , applyToTentativePristineCwd
     , createPristineDirectoryTree
+    )
+import Darcs.Repository.Hashed
+    ( copyHashedInventory
+    , finalizeRepositoryChanges
+    , finalizeTentativeChanges
+    , readRepo
     , revertRepositoryChanges
+    , revertTentativeChanges
+    , tentativelyRemovePatches
+    , writeTentativeInventory
     )
 import Darcs.Repository.Working
     ( setScriptsExecutable
@@ -46,7 +48,7 @@ import Darcs.Repository.InternalTypes
     , repoFormat
     , repoCache
     , modifyCache
-    , repoPatchType )
+    )
 import Darcs.Repository.Job ( withUMaskFlag )
 import Darcs.Repository.Cache
     ( unionRemoteCaches
@@ -54,20 +56,17 @@ import Darcs.Repository.Cache
     , fetchFileUsingCache
     , speculateFileUsingCache
     , HashedDir(..)
-    , Cache(..)
-    , CacheLoc(..)
     , repo2cache
+    , dropNonRepos
     )
-import qualified Darcs.Repository.Cache as DarcsCache
 
-import qualified Darcs.Repository.Hashed as HashedRepo
 import Darcs.Repository.ApplyPatches ( runDefault )
-import Darcs.Repository.Hashed
-    ( applyToTentativePristineCwd
-    , peekPristineHash
+import Darcs.Repository.Inventory
+    ( peekPristineHash
+    , getValidHash
     )
 import Darcs.Repository.Format
-    ( RepoProperty ( HashedInventory, Darcs2 )
+    ( RepoProperty ( HashedInventory, Darcs2, Darcs3 )
     , RepoFormat
     , formatHas
     , readProblem
@@ -88,9 +87,15 @@ import Darcs.Repository.Packs
     , fetchAndUnpackPatches
     , packsDir
     )
+import Darcs.Repository.Resolution
+    ( StandardResolution(..)
+    , patchsetConflictResolutions
+    , announceConflicts
+    )
+import Darcs.Repository.Working ( applyToWorking )
 import Darcs.Util.Lock ( appendTextFile, withNewDirectory )
 import Darcs.Repository.Flags
-    ( UpdateWorking(..)
+    ( UpdatePending(..)
     , UseCache(..)
     , RemoteDarcs (..)
     , remoteDarcs
@@ -102,35 +107,37 @@ import Darcs.Repository.Flags
     , SetScriptsExecutable (..)
     , RemoteRepos (..)
     , SetDefault (..)
+    , InheritDefault (..)
     , WithWorkingDir (..)
     , ForgetParent (..)
     , WithPatchIndex (..)
     , PatchFormat (..)
+    , AllowConflicts(..)
+    , ExternalMerge(..)
     )
 
-import Darcs.Patch ( RepoPatch, IsRepoType, apply, invert, effect )
-import Darcs.Patch.Depends ( findCommonWithThem, countUsThem )
-import Darcs.Patch.Set ( Origin
-                       , PatchSet
-                       , patchSet2RL
+import Darcs.Patch ( RepoPatch, IsRepoType, description )
+import Darcs.Patch.Depends ( findUncommon )
+import Darcs.Patch.Set ( patchSet2RL
                        , patchSet2FL
                        , progressPatchSet
                        )
-import Darcs.Patch.Match ( MatchFlag(..), havePatchsetMatch )
+import Darcs.Patch.Match ( MatchFlag(..), patchSetMatch )
 import Darcs.Patch.Progress ( progressRLShowTags, progressFL )
-import Darcs.Patch.Apply( ApplyState )
+import Darcs.Patch.Apply ( Apply(..) )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
 import Darcs.Patch.Witnesses.Ordered
-    ( (:>)(..)
-    , lengthFL
-    , mapFL_FL
+    ( FL(..)
     , RL(..)
+    , (:\/:)(..)
+    , lengthFL
     , bunchFL
     , mapFL
     , mapRL
     , lengthRL
+    , nullFL
     )
-import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, extractHash, hopefully )
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, extractHash )
 
 import Darcs.Util.Tree( Tree, emptyTree )
 
@@ -139,8 +146,9 @@ import Darcs.Util.Exception ( catchall )
 import Darcs.Util.English ( englishNum, Noun(..) )
 import Darcs.Util.Global ( darcsdir )
 import Darcs.Util.URL ( isValidLocalPath )
-import Darcs.Util.SignalHandler ( catchInterrupt )
-import Darcs.Util.Printer ( Doc, text, hPutDocLn, putDocLn, errorDoc )
+import Darcs.Util.SignalHandler ( catchInterrupt, withSignalsBlocked )
+import Darcs.Util.Printer ( Doc, ($$), hPutDocLn, hsep, putDocLn, text )
+import Darcs.Util.Printer.Color ( unsafeRenderStringColored )
 import Darcs.Util.Progress
     ( debugMessage
     , tediousSize
@@ -158,7 +166,9 @@ cloneRepository ::
     -> CloneKind
     -> UMask -> RemoteDarcs
     -> SetScriptsExecutable
-    -> RemoteRepos -> SetDefault
+    -> RemoteRepos
+    -> SetDefault
+    -> InheritDefault
     -> [MatchFlag]
     -> RepoFormat
     -> WithWorkingDir
@@ -166,19 +176,22 @@ cloneRepository ::
     -> Bool   -- use packs
     -> ForgetParent
     -> IO ()
-cloneRepository repodir mysimplename v useCache cloneKind um rdarcs sse remoteRepos
-                setDefault matchFlags rfsource withWorkingDir usePatchIndex usePacks
-                forget =
+cloneRepository repourl mysimplename v useCache cloneKind um rdarcs sse remoteRepos
+                setDefault inheritDefault matchFlags rfsource withWorkingDir
+                usePatchIndex usePacks forget =
   withUMaskFlag um $ withNewDirectory mysimplename $ do
-      let patchfmt = if formatHas Darcs2 rfsource then PatchFormat2 else PatchFormat1
-      EmptyRepository toRepo' <-
+      let patchfmt
+            | formatHas Darcs3 rfsource = PatchFormat3
+            | formatHas Darcs2 rfsource = PatchFormat2
+            | otherwise                 = PatchFormat1
+      EmptyRepository _toRepo <-
         createRepository patchfmt withWorkingDir
           (if cloneKind == LazyClone then NoPatchIndex else usePatchIndex) useCache
       debugMessage "Finished initializing new repository."
-      addRepoSource repodir NoDryRun remoteRepos setDefault
+      addRepoSource repourl NoDryRun remoteRepos setDefault inheritDefault False
 
       debugMessage "Identifying and copying repository..."
-      fromRepo <- identifyRepositoryFor toRepo' useCache repodir
+      fromRepo <- identifyRepositoryFor Reading _toRepo useCache repourl
       let fromLoc = repoLocation fromRepo
       let rffrom = repoFormat fromRepo
       case readProblem rffrom of
@@ -189,58 +202,69 @@ cloneRepository repodir mysimplename v useCache cloneKind um rdarcs sse remoteRe
         (joinUrl [fromLoc, darcsdir, "prefs", "prefs"])
         (darcsdir </> "prefs/prefs") (MaxAge 600) `catchall` return ()
       debugMessage "Copying sources..."
-      cache <- unionRemoteCaches (repoCache toRepo') (repoCache fromRepo) fromLoc
+      cache <- unionRemoteCaches (repoCache _toRepo) (repoCache fromRepo) fromLoc
       appendTextFile (darcsdir </> "prefs/sources")
                      (show $ repo2cache fromLoc `unionCaches` dropNonRepos cache)
       debugMessage "Done copying and filtering sources."
       -- put remote source last
-      let toRepo = modifyCache toRepo' (const $ cache `unionCaches` repo2cache fromLoc)
+      _toRepo <- return $
+        modifyCache (const $ cache `unionCaches` repo2cache fromLoc) _toRepo
       if formatHas HashedInventory rffrom then do
-       -- copying basic repository (hashed_inventory and pristine)
+       debugMessage "Copying basic repository (hashed_inventory and pristine)"
        if usePacks && (not . isValidLocalPath) fromLoc
-         then copyBasicRepoPacked    fromRepo toRepo v rdarcs withWorkingDir
-         else copyBasicRepoNotPacked fromRepo toRepo v rdarcs withWorkingDir
+         then copyBasicRepoPacked    fromRepo _toRepo v rdarcs withWorkingDir
+         else copyBasicRepoNotPacked fromRepo _toRepo v rdarcs withWorkingDir
        when (cloneKind /= LazyClone) $ do
          when (cloneKind /= CompleteClone) $
            putInfo v $ text "Copying patches, to get lazy repository hit ctrl-C..."
-       -- copying complete repository (inventories and patches)
+         debugMessage "Copying complete repository (inventories and patches)"
          if usePacks && (not . isValidLocalPath) fromLoc
-           then copyCompleteRepoPacked    fromRepo toRepo v cloneKind
-           else copyCompleteRepoNotPacked fromRepo toRepo v cloneKind
+           then copyCompleteRepoPacked    fromRepo _toRepo v cloneKind
+           else copyCompleteRepoNotPacked fromRepo _toRepo v cloneKind
       else
        -- old-fashioned repositories are cloned diferently since
        -- we need to copy all patches first and then build pristine
-       copyRepoOldFashioned fromRepo toRepo v withWorkingDir
+       copyRepoOldFashioned fromRepo _toRepo v withWorkingDir
       when (sse == YesSetScriptsExecutable) setScriptsExecutable
-      when (havePatchsetMatch (repoPatchType toRepo) matchFlags) $ do
+      case patchSetMatch matchFlags of
+       Nothing -> return ()
+       Just psm -> do
         putInfo v $ text "Going to specified version..."
-        -- the following is necessary to be able to read repo's patches
-        revertRepositoryChanges toRepo YesUpdateWorking
-        patches <- readRepo toRepo
-        Sealed context <- getOnePatchset toRepo matchFlags
-        when (snd (countUsThem patches context) > 0) $
-             errorDoc $ text "Missing patches from context!" -- FIXME : - (
-        _ :> us' <- return $ findCommonWithThem patches context
-        let ps = mapFL_FL hopefully us'
-        putInfo v $ text $ "Unapplying " ++ show (lengthFL ps) ++ " " ++
-                    englishNum (lengthFL ps) (Noun "patch") ""
-        invalidateIndex toRepo
-        _ <- tentativelyRemovePatches toRepo GzipCompression YesUpdateWorking us'
-        tentativelyAddToPending toRepo YesUpdateWorking $ invert $ effect us'
-        finalizeRepositoryChanges toRepo YesUpdateWorking GzipCompression
-        runDefault (apply (invert $ effect ps)) `catch` \(e :: SomeException) ->
-            fail ("Couldn't undo patch in working dir.\n" ++ show e)
-        when (sse == YesSetScriptsExecutable) $ setScriptsExecutablePatches (invert $ effect ps)
+        -- the following is necessary to be able to read _toRepo's patches
+        _toRepo <- revertRepositoryChanges _toRepo NoUpdatePending
+        patches <- readRepo _toRepo
+        Sealed context <- getOnePatchset _toRepo psm
+        to_remove :\/: only_in_context <- return $ findUncommon patches context
+        case only_in_context of
+          NilFL -> do
+            let num_to_remove = lengthFL to_remove
+            putInfo v $ hsep $ map text
+              [ "Unapplying"
+              , show num_to_remove
+              , englishNum num_to_remove (Noun "patch") ""
+              ]
+            invalidateIndex _toRepo
+            _toRepo <-
+              tentativelyRemovePatches _toRepo GzipCompression NoUpdatePending to_remove
+            _toRepo <-
+              finalizeRepositoryChanges _toRepo NoUpdatePending GzipCompression
+            runDefault (unapply to_remove) `catch` \(e :: SomeException) ->
+                fail ("Couldn't undo patch in working tree.\n" ++ show e)
+            when (sse == YesSetScriptsExecutable) $ setScriptsExecutablePatches to_remove
+          _ ->
+            -- This can only happen if the user supplied a context file that
+            -- doesn't specify a subset of the remote repo.
+            fail $ unsafeRenderStringColored
+              $ text "Missing patches from context:"
+              $$ description only_in_context
       when (forget == YesForgetParent) deleteSources
-
--- | This keeps only NonWritable Repo entries.
-dropNonRepos :: Cache -> Cache
-dropNonRepos (Ca cache) = Ca $ filter notRepo cache where
-  notRepo xs = case xs of
-    Cache DarcsCache.Directory _ _ -> False
-    -- we don't want to write thisrepo: entries to the disk
-    Cache DarcsCache.Repo DarcsCache.Writable _ -> False
-    _ -> True
+      -- check for unresolved conflicts
+      patches <- readRepo _toRepo
+      let conflicts = patchsetConflictResolutions patches
+      _ <- announceConflicts "clone" YesAllowConflictsAndMark NoExternalMerge conflicts
+      Sealed mangled_res <- return $ mangled conflicts
+      unless (nullFL mangled_res) $
+        withSignalsBlocked $ void $ applyToWorking _toRepo v mangled_res
 
 putInfo :: Verbosity -> Doc -> IO ()
 putInfo Quiet _ = return ()
@@ -259,8 +283,8 @@ copyBasicRepoNotPacked  :: forall rt p wR wU wT.
                         -> IO ()
 copyBasicRepoNotPacked fromRepo toRepo verb rdarcs withWorkingDir = do
   putVerbose verb $ text "Copying hashed inventory from remote repo..."
-  HashedRepo.copyHashedInventory toRepo rdarcs (repoLocation fromRepo)
-  putVerbose verb $ text "Writing pristine and working directory contents..."
+  copyHashedInventory toRepo rdarcs (repoLocation fromRepo)
+  putVerbose verb $ text "Writing pristine and working tree contents..."
   createPristineDirectoryTree toRepo "." withWorkingDir
 
 copyCompleteRepoNotPacked :: forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
@@ -291,15 +315,19 @@ copyBasicRepoPacked fromRepo toRepo verb rdarcs withWorkingDir =
      mPackHash <- (Just <$> gzFetchFilePS hashURL Uncachable) `catchall` (return Nothing)
      let hiURL = joinUrl [fromLoc, darcsdir, "hashed_inventory"]
      i <- gzFetchFilePS hiURL Uncachable
-     let currentHash = BC.pack $ peekPristineHash i
+     let currentHash = BC.pack $ getValidHash $ peekPristineHash i
      let copyNormally = copyBasicRepoNotPacked fromRepo toRepo verb rdarcs withWorkingDir
      case mPackHash of
       Just packHash | packHash == currentHash
-              -> ( copyBasicRepoPacked2 fromRepo toRepo verb withWorkingDir
-                    `catch` \(e :: SomeException) ->
+              -> ( do copyBasicRepoPacked2 fromRepo toRepo verb withWorkingDir
+                      -- need to obtain a fresh copy of hashed_inventory as reference
+                      putVerbose verb $ text "Copying hashed inventory from remote repo..."
+                      copyHashedInventory toRepo rdarcs (repoLocation fromRepo)
+                   `catch` \(e :: SomeException) ->
                                do putStrLn ("Exception while getting basic pack:\n" ++ show e)
                                   copyNormally)
-      _       -> do putVerbose verb $ text "Remote repo has no basic pack or outdated basic pack, copying normally."
+      _       -> do putVerbose verb $
+                      text "Remote repo has no basic pack or outdated basic pack, copying normally."
                     copyNormally
 
 copyBasicRepoPacked2 ::
@@ -352,8 +380,7 @@ copyCompleteRepoPacked2 fromRepo toRepo verb cloneKind = do
     when pi $ createPIWithInterrupt toRepo us -- TODO or do another readRepo?
 
 cleanDir :: FilePath -> IO ()
-cleanDir d = mapM_ (\x -> removeFile $ d </> x) .
-  filter (\x -> head x /= '.') =<< getDirectoryContents d
+cleanDir d = mapM_ (\x -> removeFile $ d </> x) =<< listDirectory d
 
 copyRepoOldFashioned :: forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
                         => Repository rt p wR wU wT  -- remote repo
@@ -361,28 +388,28 @@ copyRepoOldFashioned :: forall rt p wR wU wT. (IsRepoType rt, RepoPatch p, Apply
                         -> Verbosity
                         -> WithWorkingDir
                         -> IO ()
-copyRepoOldFashioned fromrepository toRepo verb withWorkingDir = do
-  HashedRepo.revertTentativeChanges
+copyRepoOldFashioned fromrepository _toRepo verb withWorkingDir = do
+  revertTentativeChanges
   patches <- readRepo fromrepository
   let k = "Copying patch"
   beginTedious k
   tediousSize k (lengthRL $ patchSet2RL patches)
   let patches' = progressPatchSet k patches
-  HashedRepo.writeTentativeInventory (repoCache toRepo) GzipCompression patches'
+  writeTentativeInventory (repoCache _toRepo) GzipCompression patches'
   endTedious k
-  HashedRepo.finalizeTentativeChanges toRepo GzipCompression
+  finalizeTentativeChanges _toRepo GzipCompression
   -- apply all patches into current hashed repository
-  HashedRepo.revertTentativeChanges
-  local_patches <- readRepo toRepo
-  replacePristine toRepo emptyTree
+  _toRepo <- revertRepositoryChanges _toRepo NoUpdatePending
+  local_patches <- readRepo _toRepo
+  replacePristine _toRepo emptyTree
   let patchesToApply = progressFL "Applying patch" $ patchSet2FL local_patches
-  sequence_ $ mapFL applyToTentativePristineCwd $ bunchFL 100 patchesToApply
-  finalizeRepositoryChanges toRepo YesUpdateWorking GzipCompression
-  putVerbose verb $ text "Writing pristine and working directory contents..."
-  createPristineDirectoryTree toRepo "." withWorkingDir
+  sequence_ $ mapFL (applyToTentativePristineCwd ApplyNormal) $ bunchFL 100 patchesToApply
+  _toRepo <- finalizeRepositoryChanges _toRepo NoUpdatePending GzipCompression
+  putVerbose verb $ text "Writing pristine and working tree contents..."
+  createPristineDirectoryTree _toRepo "." withWorkingDir
 
 -- | This function fetches all patches that the given repository has
---   with fetchFileUsingCache, unless --lazy is passed.
+--   with fetchFileUsingCache.
 fetchPatchesIfNecessary :: forall rt p wR wU wT. (IsRepoType rt, RepoPatch p)
                         => Repository rt p wR wU wT
                         -> IO ()
@@ -403,49 +430,6 @@ fetchPatchesIfNecessary toRepo =
           mapM_ (speculateFileUsingCache c HashedPatchesDir) ss
         c = repoCache toRepo
 
-{-
--- | patchSetToRepository takes a patch set, and writes a new repository
---   in the current directory that contains all the patches in the patch
---   set.  This function is used when 'darcs get'ing a repository with
---   the --to-match flag.
--- bf: no it is not used anywhere
-patchSetToRepository :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-                     => Repository rt p wR1 wU1 wR1
-                     -> PatchSet rt p Origin wX
-                     -> UseCache -> RemoteDarcs
-                     -> IO ()
-patchSetToRepository fromRepo patchset useCache rDarcs = do
-    when (formatHas HashedInventory (repoFormat fromRepo)) $
-       -- set up sources and all that
-       do writeFile (darcsdir </> "tentative_pristine") "" -- this is hokey
-          repox <- writePatchSet patchset useCache
-          let fromLoc = repoLocation fromRepo
-          HashedRepo.copyHashedInventory repox rDarcs fromLoc
-          void $ copySources repox fromLoc
-    repo <- writePatchSet patchset useCache
-    readRepo repo >>= (runDefault . applyPatches . patchSet2FL)
-    debugMessage "Writing the pristine"
-    withRepoLocation repo $ readWorking >>= replacePristine repo
--}
-
--- | writePatchSet is like patchSetToRepository, except that it doesn't
--- touch the working directory or pristine cache.
-writePatchSet :: (IsRepoType rt, RepoPatch p)
-              => PatchSet rt p Origin wX
-              -> UseCache
-              -> IO (Repository rt p wR wU wT)
-writePatchSet patchset useCache = do
-    maybeRepo <- maybeIdentifyRepository useCache "."
-    let repo =
-          case maybeRepo of
-            GoodRepository r -> r
-            BadRepository e -> bug ("Current directory is a bad repository in writePatchSet: " ++ e)
-            NonRepository e -> bug ("Current directory not a repository in writePatchSet: " ++ e)
-    debugMessage "Writing inventory"
-    HashedRepo.writeTentativeInventory (repoCache repo) GzipCompression patchset
-    HashedRepo.finalizeTentativeChanges repo GzipCompression
-    return repo
-
 -- | Replace the existing pristine with a new one (loaded up in a Tree object).
 replacePristine :: Repository rt p wR wU wT -> Tree IO -> IO ()
 replacePristine = writePristine . repoLocation
