@@ -76,6 +76,7 @@ import Darcs.Util.Lock
     ( writeBinFile
     , writeDocBinFile
     )
+import Darcs.Util.Path ( ioAbsoluteOrRemote, toPath )
 import Darcs.Util.Tree( Tree, emptyTree )
 import Darcs.Util.Tree.Hashed( writeDarcsHashed, darcsAddMissingHashes )
 
@@ -112,30 +113,34 @@ createRepository :: PatchFormat -> WithWorkingDir -> WithPatchIndex -> UseCache
 createRepository patchfmt withWorkingDir withPatchIndex useCache = do
   rfmt <- createRepositoryFiles patchfmt withWorkingDir
   cache <- getCaches useCache here
+  rdir <- toPath <$> ioAbsoluteOrRemote here
   repo@(EmptyRepository r) <- case patchfmt of
-    PatchFormat1 -> return $ EmptyRepository $ mkRepoV1 rfmt cache
-    PatchFormat2 -> return $ EmptyRepository $ mkRepoV2 rfmt cache
-    PatchFormat3 -> return $ EmptyRepository $ mkRepoV3 rfmt cache
+    PatchFormat1 -> return $ EmptyRepository $ mkRepoV1 rdir rfmt cache
+    PatchFormat2 -> return $ EmptyRepository $ mkRepoV2 rdir rfmt cache
+    PatchFormat3 -> return $ EmptyRepository $ mkRepoV3 rdir rfmt cache
   maybeCreatePatchIndex withPatchIndex r
   return repo
 
 mkRepoV1
-  :: RepoFormat
+  :: FilePath
+  -> RepoFormat
   -> Cache
   -> Repository ('RepoType 'NoRebase) (RepoPatchV1 V1.Prim) Origin Origin Origin
-mkRepoV1 repofmt cache = mkRepo here repofmt HashedPristine cache
+mkRepoV1 rdir repofmt cache = mkRepo rdir repofmt HashedPristine cache
 
 mkRepoV2
-  :: RepoFormat
+  :: FilePath
+  -> RepoFormat
   -> Cache
   -> Repository ('RepoType 'NoRebase) (RepoPatchV2 V2.Prim) Origin Origin Origin
-mkRepoV2 repofmt cache = mkRepo here repofmt HashedPristine cache
+mkRepoV2 rdir repofmt cache = mkRepo rdir repofmt HashedPristine cache
 
 mkRepoV3
-  :: RepoFormat
+  :: FilePath
+  -> RepoFormat
   -> Cache
   -> Repository ('RepoType 'NoRebase) (RepoPatchV3 V2.Prim) Origin Origin Origin
-mkRepoV3 repofmt cache = mkRepo here repofmt HashedPristine cache
+mkRepoV3 rdir repofmt cache = mkRepo rdir repofmt HashedPristine cache
 
 createRepositoryV1
   :: WithWorkingDir -> WithPatchIndex -> UseCache
@@ -143,7 +148,8 @@ createRepositoryV1
 createRepositoryV1 withWorkingDir withPatchIndex useCache = do
   rfmt <- createRepositoryFiles PatchFormat1 withWorkingDir
   cache <- getCaches useCache here
-  let repo = mkRepoV1 rfmt cache
+  rdir <- toPath <$> ioAbsoluteOrRemote here
+  let repo = mkRepoV1 rdir rfmt cache
   maybeCreatePatchIndex withPatchIndex repo
   return repo
 
@@ -153,7 +159,8 @@ createRepositoryV2
 createRepositoryV2 withWorkingDir withPatchIndex useCache = do
   rfmt <- createRepositoryFiles PatchFormat2 withWorkingDir
   cache <- getCaches useCache here
-  let repo = mkRepoV2 rfmt cache
+  rdir <- toPath <$> ioAbsoluteOrRemote here
+  let repo = mkRepoV2 rdir rfmt cache
   maybeCreatePatchIndex withPatchIndex repo
   return repo
 
