@@ -33,7 +33,6 @@ module Darcs.Repository.Diff
       treeDiff
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import qualified Data.ByteString.Lazy.Char8 as BLC
@@ -80,7 +79,7 @@ getDiff :: AnchoredPath
 getDiff p Nothing (Just t) = (p, Added t)
 getDiff p (Just from) (Just to) = (p, Changed from to)
 getDiff p (Just t) Nothing = (p, Removed t)
-getDiff _ Nothing Nothing = impossible -- zipTrees should never return this
+getDiff _ Nothing Nothing = error "impossible case" -- zipTrees should never return this
 
 
 treeDiff :: forall m w prim . (Monad m, Gap w, PrimPatch prim)
@@ -111,34 +110,35 @@ treeDiff da ft t1 t2 = do
     diff :: AnchoredPath -> Diff m -> m (w (FL prim))
     diff _ (Changed (SubTree _) (SubTree _)) = return (emptyGap NilFL)
     diff p (Removed (SubTree _)) =
-        return $ freeGap (rmdir (anchorPath "" p) :>: NilFL)
+        -- Note: With files we first make the file empty before removing it.
+        -- But for subtrees this has already been done in previous recursive calls.
+        return $ freeGap (rmdir p :>: NilFL)
     diff p (Added (SubTree _)) =
-        return $ freeGap (adddir (anchorPath "" p) :>: NilFL)
+        return $ freeGap (adddir p :>: NilFL)
     diff p (Added b'@(File _)) =
         do diff' <- diff p (Changed (File emptyBlob) b')
-           return $ joinGap (:>:) (freeGap (addfile (anchorPath "" p))) diff'
+           return $ joinGap (:>:) (freeGap (addfile p)) diff'
     diff p (Removed a'@(File _)) =
         do diff' <- diff p (Changed a' (File emptyBlob))
-           return $ joinGap (+>+) diff' (freeGap (rmfile (anchorPath "" p) :>: NilFL))
+           return $ joinGap (+>+) diff' (freeGap (rmfile p :>: NilFL))
     diff p (Changed (File a') (File b')) =
         do a <- readBlob a'
            b <- readBlob b'
-           let path = anchorPath "" p
-           case ft path of
+           case ft (anchorPath "" p) of
              TextFile | no_bin a && no_bin b ->
-                          return $ text_diff path a b
+                          return $ text_diff p a b
              _ -> return $ if a /= b
-                              then freeGap (binary path (strict a) (strict b) :>: NilFL)
+                              then freeGap (binary p (strict a) (strict b) :>: NilFL)
                               else emptyGap NilFL
     diff p (Changed a'@(File _) subtree@(SubTree _)) =
-        do rmFileP <- diff p (Changed a' (File emptyBlob))
+        do rmFileP <- diff p (Removed a')
            addDirP <- diff p (Added subtree)
            return $ joinGap (+>+) rmFileP addDirP
     diff p (Changed subtree@(SubTree _) b'@(File _)) =
         do rmDirP <- diff p (Removed subtree)
-           addFileP <- diff p (Changed (File emptyBlob) b')
+           addFileP <- diff p (Added b')
            return $ joinGap (+>+) rmDirP addFileP
-    diff p _ = error $ "Missing case at path " ++ show p
+    diff p _ = error $ "Missing case at path " ++ show (anchorPath "" p)
 
     text_diff p a b
         | BL.null a && BL.null b = emptyGap NilFL
