@@ -626,7 +626,11 @@ finalizeRepositoryChanges r updatePending compr
             let r' = unsafeCoerceR r
             debugMessage "Done finalizing changes..."
             ps <- readRepo r'
-            doesPatchIndexExist (repoLocation r') >>= (`when` createOrUpdatePatchIndexDisk r' ps)
+            pi_exists <- doesPatchIndexExist (repoLocation r')
+            when pi_exists $
+              createOrUpdatePatchIndexDisk r' ps
+              `catchIOError` \e ->
+                hPutStrLn stderr $ "Cannot create or update patch index: "++ show e
             updateIndex r'
             return r'
     | otherwise = fail Old.oldRepoFailMsg
