@@ -33,6 +33,7 @@ import System.Directory ( doesDirectoryExist
                         , listDirectory
                         )
 import System.FilePath.Posix ( (</>) )
+import System.IO ( hPutStrLn, stderr )
 import System.IO.Error ( catchIOError )
 import Data.Maybe ( fromMaybe )
 
@@ -164,7 +165,7 @@ amInHashedRepository wd
 
 -- | hunt upwards for the darcs repository
 -- This keeps changing up one parent directory, testing at each
--- step if the current directory is a repository or not.  $
+-- step if the current directory is a repository or not.
 -- The result is:
 --   Nothing, if no repository found
 --   Just (Left errorMessage), if bad repository found
@@ -172,19 +173,24 @@ amInHashedRepository wd
 -- WARNING this changes the current directory for good if matchFn succeeds
 seekRepo :: IO (Maybe (Either String ()))
 seekRepo = getCurrentDirectory >>= helper where
-   helper startpwd = do
+  helper startpwd = do
     status <- maybeIdentifyRepository YesUseCache "."
     case status of
       GoodRepository _ -> return . Just $ Right ()
-      BadRepository e  -> return . Just $ Left e
+      BadRepository e -> return . Just $ Left e
       NonRepository _ ->
-            do cd <- toFilePath `fmap` getCurrentDirectory
-               setCurrentDirectory ".."
-               cd' <- toFilePath `fmap` getCurrentDirectory
-               if cd' /= cd
-                  then helper startpwd
-                  else do setCurrentDirectory startpwd
-                          return Nothing
+        catchIOError
+          (do cd <- toFilePath `fmap` getCurrentDirectory
+              setCurrentDirectory ".."
+              cd' <- toFilePath `fmap` getCurrentDirectory
+              if cd' /= cd
+                then helper startpwd
+                else do
+                  setCurrentDirectory startpwd
+                  return Nothing)
+          (\e -> do
+             hPutStrLn stderr ("Warning: " ++ show e)
+             return Nothing)
 
 -- The performGC in this function is a workaround for a library/GHC bug,
 -- http://hackage.haskell.org/trac/ghc/ticket/2924 -- (doesn't seem to be a
