@@ -8,6 +8,7 @@ module Darcs.Repository.Inventory
     , PristineHash
     , inventoryPatchNames
     , parseInventory
+    , parseHeadInventory -- not used
     , showInventory
     , showInventoryPatches
     , showInventoryEntry
@@ -22,7 +23,6 @@ module Darcs.Repository.Inventory
     , prop_skipPokePristineHash
     ) where
 
-import Prelude ()
 import Darcs.Prelude hiding ( take )
 
 import Control.Applicative ( optional, many )
@@ -32,8 +32,8 @@ import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as BC
 
 import Darcs.Patch.Info ( PatchInfo, showPatchInfo, readPatchInfo )
-import Darcs.Patch.ReadMonads
-    ( ParserM, parseStrictly, string, skipSpace, take, takeTillChar )
+import Darcs.Util.Parser
+    ( Parser, parse, string, skipSpace, take, takeTillChar )
 import Darcs.Patch.Show ( ShowPatchFor(..) )
 import Darcs.Repository.Cache ( okayHash )
 import Darcs.Util.Hash ( sha256sum )
@@ -80,12 +80,11 @@ instance ValidHash PristineHash where
 
 -- * Inventories
 
--- Note: this type and the commented out parser combinators for it
--- aren't actually used (except for testing). They are left here to
--- serve as documentation for the API we would like to use but won't
--- because of efficiency: we want to be able to access the pristine
--- hash with forcing a complete parse of the head inventory. Thus we
--- retain the lower-level peek/poke/skip API for the pristine hash.
+-- This type and the parser combinators for it aren't actually used. They are
+-- here to serve as documentation for the API we would like to use but won't
+-- because of efficiency: we want to be able to access the pristine hash
+-- without forcing a complete parse of the head inventory. Thus we retain the
+-- lower-level peek/poke/skip API for the pristine hash.
 type HeadInventory = (PristineHash, Inventory)
 
 data Inventory = Inventory
@@ -104,47 +103,43 @@ emptyInventory = Inventory Nothing []
 
 -- * Parsing
 
-{-
-parseHeadInventory :: B.ByteString -> Maybe HeadInventory
+parseHeadInventory :: B.ByteString -> Either String HeadInventory
 parseHeadInventory = fmap fst . parse pHeadInv
--}
 
-parseInventory :: B.ByteString -> Maybe Inventory
-parseInventory = fmap fst . parseStrictly pInv
+parseInventory :: B.ByteString -> Either String Inventory
+parseInventory = fmap fst . parse pInv
 
-{-
-pHeadInv :: ParserM m => m HeadInventory
-pHeadInv = (,) <$> pInvPristine <*> pInv
+pHeadInv :: Parser HeadInventory
+pHeadInv = (,) <$> pPristineHash <*> pInv
 
-pInvPristine :: ParserM m => m ValidHash
-pInvPristine = do
+pPristineHash :: Parser PristineHash
+pPristineHash = do
   string pristineName
   skipSpace
   pHash
--}
 
-pInv :: ParserM m => m Inventory
+pInv :: Parser Inventory
 pInv = Inventory <$> pInvParent <*> pInvPatches
 
-pInvParent :: ParserM m => m (Maybe InventoryHash)
+pInvParent :: Parser (Maybe InventoryHash)
 pInvParent = optional $ do
   string parentName
   skipSpace
   pHash
 
-pHash :: (ParserM m, ValidHash h) => m h
+pHash :: ValidHash h => Parser h
 pHash = do
   hash <- BC.unpack <$> pLine
   guard (okayHash hash)
   return (mkValidHash hash)
 
-pLine :: ParserM m => m B.ByteString
+pLine :: Parser B.ByteString
 pLine = takeTillChar '\n' <* take 1
 
-pInvPatches :: ParserM m => m [InventoryEntry]
+pInvPatches :: Parser [InventoryEntry]
 pInvPatches = many pInvEntry
 
-pInvEntry :: ParserM m => m InventoryEntry
+pInvEntry :: Parser InventoryEntry
 pInvEntry = do
   info <- readPatchInfo
   skipSpace
@@ -177,8 +172,8 @@ showParent Nothing = mempty
 
 -- | Replace the pristine hash at the start of a raw, unparsed 'HeadInventory'
 -- or add it if none is present.
-pokePristineHash :: String -> B.ByteString -> Doc
-pokePristineHash h inv =
+pokePristineHash :: PristineHash -> B.ByteString -> Doc
+pokePristineHash (PristineHash h) inv =
   invisiblePS pristineName <> text h $$ invisiblePS (skipPristineHash inv)
 
 takeHash :: B.ByteString -> Maybe (String, B.ByteString)
@@ -188,14 +183,14 @@ takeHash input = do
   guard $ okayHash hash
   return (hash, rest)
 
-peekPristineHash :: B.ByteString -> String
+peekPristineHash :: B.ByteString -> PristineHash
 peekPristineHash inv =
   case tryDropPristineName inv of
     Just rest ->
       case takeHash rest of
-        Just (h, _) -> h
+        Just (h, _) -> mkValidHash h
         Nothing -> error $ "Bad hash in inventory!"
-    Nothing -> sha256sum B.empty
+    Nothing -> mkValidHash $ sha256sum B.empty
 
 -- |skipPristineHash drops the 'pristine: HASH' prefix line, if present.
 skipPristineHash :: B.ByteString -> B.ByteString
@@ -228,12 +223,12 @@ newline = BC.pack "\n"
 
 prop_inventoryParseShow :: Inventory -> Bool
 prop_inventoryParseShow inv =
-  Just inv == parseInventory (renderPS (showInventory inv))
+  Right inv == parseInventory (renderPS (showInventory inv))
 
 prop_peekPokePristineHash :: (PristineHash, B.ByteString) -> Bool
-prop_peekPokePristineHash (PristineHash hash, raw) =
+prop_peekPokePristineHash (hash, raw) =
   hash == peekPristineHash (renderPS (pokePristineHash hash raw))
 
 prop_skipPokePristineHash :: (PristineHash, B.ByteString) -> Bool
-prop_skipPokePristineHash (PristineHash hash, raw) =
+prop_skipPokePristineHash (hash, raw) =
   raw == skipPristineHash (renderPS (pokePristineHash hash raw))
