@@ -30,17 +30,15 @@ module Darcs.Repository.Job
     , withUMaskFlag
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Darcs.Util.Global ( darcsdir )
-
 import Darcs.Patch.Apply ( ApplyState )
 import Darcs.Patch.V1 ( RepoPatchV1 )
 import Darcs.Patch.V2 ( RepoPatchV2 )
+import Darcs.Patch.V3 ( RepoPatchV3 )
 import qualified Darcs.Patch.V1.Prim as V1 ( Prim(..) )
 import qualified Darcs.Patch.V2.Prim as V2 ( Prim(..) )
-import Darcs.Patch.Prim ( PrimOf )
+import Darcs.Patch ( PrimOf )
 import Darcs.Patch.Prim.V1 ( Prim )
 import Darcs.Patch.RepoPatch ( RepoPatch )
 import Darcs.Patch.RepoType
@@ -50,11 +48,13 @@ import Darcs.Patch.RepoType
   )
 
 import Darcs.Repository.Flags
-    ( UseCache(..), UpdateWorking(..), DryRun(..), UMask (..)
+    ( UseCache(..), UpdatePending(..), DryRun(..), UMask (..)
     )
 import Darcs.Repository.Format
     ( RepoProperty( Darcs2
+                  , Darcs3
                   , RebaseInProgress
+                  , RebaseInProgress_2_16
                   , HashedInventory
                   )
     , formatHas
@@ -66,20 +66,22 @@ import Darcs.Repository.InternalTypes
     ( Repository
     , repoFormat
     , repoLocation
+    , unsafeCoerceRepoType
+    , unsafeCoercePatchType
     )
+import Darcs.Repository.Paths ( lockPath )
 import Darcs.Repository.Rebase
-    ( RebaseJobFlags
-    , startRebaseJob
+    ( startRebaseJob
     , rebaseJob
+    , maybeDisplaySuspendedStatus
+    , checkOldStyleRebaseStatus
     )
-import qualified Darcs.Repository.Rebase as Rebase ( maybeDisplaySuspendedStatus )
 import Darcs.Util.Lock ( withLock, withLockCanFail )
 
 import Darcs.Util.Progress ( debugMessage )
 
 import Control.Monad ( when )
 import Control.Exception ( bracket_, finally )
-import Data.Coerce ( coerce )
 import Data.List ( intercalate )
 
 import Foreign.C.String ( CString, withCString )
@@ -88,12 +90,9 @@ import Foreign.C.Types ( CInt(..) )
 
 import Darcs.Util.Tree ( Tree )
 
-getUMask :: UMask -> Maybe String
-getUMask (YesUMask s) = Just s
-getUMask NoUMask = Nothing
-
 withUMaskFlag :: UMask -> IO a -> IO a
-withUMaskFlag = maybe id withUMask . getUMask
+withUMaskFlag NoUMask = id
+withUMaskFlag (YesUMask umask) = withUMask umask
 
 foreign import ccall unsafe "umask.h set_umask" set_umask
     :: CString -> IO CInt
@@ -136,9 +135,10 @@ data RepoJob a
     | PrimV1Job (forall rt p wR wU . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree, IsPrimV1 (PrimOf p))
                => Repository rt p wR wU wR -> IO a)
     -- A job that works on normal darcs repositories, but will want access to the rebase patch if it exists.
-    | RebaseAwareJob RebaseJobFlags (forall rt p wR wU . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => Repository rt p wR wU wR -> IO a)
-    | RebaseJob RebaseJobFlags (forall p wR wU . (RepoPatch p, ApplyState p ~ Tree) => Repository ('RepoType 'IsRebase) p wR wU wR -> IO a)
-    | StartRebaseJob RebaseJobFlags (forall p wR wU . (RepoPatch p, ApplyState p ~ Tree) => Repository ('RepoType 'IsRebase) p wR wU wR -> IO a)
+    | RebaseAwareJob (forall rt p wR wU . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree) => Repository rt p wR wU wR -> IO a)
+    | RebaseJob (forall p wR wU . (RepoPatch p, ApplyState p ~ Tree) => Repository ('RepoType 'IsRebase) p wR wU wR -> IO a)
+    | OldRebaseJob (forall p wR wU . (RepoPatch p, ApplyState p ~ Tree) => Repository ('RepoType 'IsRebase) p wR wU wR -> IO a)
+    | StartRebaseJob (forall p wR wU . (RepoPatch p, ApplyState p ~ Tree) => Repository ('RepoType 'IsRebase) p wR wU wR -> IO a)
 
 onRepoJob :: RepoJob a
           -> (forall rt p wR wU . (RepoPatch p, ApplyState p ~ Tree) => (Repository rt p wR wU wR -> IO a) -> Repository rt p wR wU wR -> IO a)
@@ -147,9 +147,10 @@ onRepoJob (RepoJob job) f = RepoJob (f job)
 onRepoJob (V1Job job) f = V1Job (f job)
 onRepoJob (V2Job job) f = V2Job (f job)
 onRepoJob (PrimV1Job job) f = PrimV1Job (f job)
-onRepoJob (RebaseAwareJob flags job) f = RebaseAwareJob flags (f job)
-onRepoJob (RebaseJob flags job) f      = RebaseJob flags (f job)
-onRepoJob (StartRebaseJob flags job) f = StartRebaseJob flags (f job)
+onRepoJob (RebaseAwareJob job) f = RebaseAwareJob (f job)
+onRepoJob (RebaseJob job) f = RebaseJob (f job)
+onRepoJob (OldRebaseJob job) f = OldRebaseJob (f job)
+onRepoJob (StartRebaseJob job) f = StartRebaseJob (f job)
 
 -- | apply a given RepoJob to a repository in the current working directory
 withRepository :: UseCache -> RepoJob a -> IO a
@@ -160,6 +161,7 @@ withRepository useCache = withRepositoryLocation useCache "."
 data RepoPatchType p where
   RepoV1 :: RepoPatchType (RepoPatchV1 V1.Prim)
   RepoV2 :: RepoPatchType (RepoPatchV2 V2.Prim)
+  RepoV3 :: RepoPatchType (RepoPatchV3 V2.Prim)
 
 -- | This type allows us to check multiple patch types against the
 -- constraints required by most repository jobs
@@ -169,6 +171,7 @@ data IsTree p where
 checkTree :: RepoPatchType p -> IsTree p
 checkTree RepoV1 = IsTree
 checkTree RepoV2 = IsTree
+checkTree RepoV3 = IsTree
 
 class ApplyState p ~ Tree => IsPrimV1 p where
   toPrimV1 :: p wX wY -> Prim wX wY
@@ -185,6 +188,7 @@ data UsesPrimV1 p where
 checkPrimV1 :: RepoPatchType p -> UsesPrimV1 p
 checkPrimV1 RepoV1 = UsesPrimV1
 checkPrimV1 RepoV2 = UsesPrimV1
+checkPrimV1 RepoV3 = UsesPrimV1
 
 -- | apply a given RepoJob to a repository in a given url
 withRepositoryLocation :: UseCache -> String -> RepoJob a -> IO a
@@ -204,15 +208,19 @@ withRepositoryLocation useCache url repojob = do
           :: IsRebaseType rebaseType
           => SRebaseType rebaseType -> Repository rtDummy pDummy wR wU wR -> RepoJob a -> IO a
         runJob1 isRebase =
-          if formatHas Darcs2 rf
-          then runJob RepoV2 (SRepoType isRebase)
-          else runJob RepoV1 (SRepoType isRebase)
+          if formatHas Darcs3 rf
+          then runJob RepoV3 (SRepoType isRebase)
+          else
+            if formatHas Darcs2 rf
+            then runJob RepoV2 (SRepoType isRebase)
+            else runJob RepoV1 (SRepoType isRebase)
 
         runJob2 :: Repository rtDummy pDummy wR wU wR -> RepoJob a -> IO a
         runJob2 =
-          if startRebase || formatHas RebaseInProgress rf
-          then runJob1 SIsRebase
-          else runJob1 SNoRebase
+          if startRebase ||
+             formatHas RebaseInProgress rf || formatHas RebaseInProgress_2_16 rf
+            then runJob1 SIsRebase
+            else runJob1 SNoRebase
 
     runJob2 repo repojob
 
@@ -230,11 +238,12 @@ runJob patchType (SRepoType isRebase) repo repojob = do
   -- The actual type the repository should have is only known when
   -- when this function is called, so we need to "cast" it to its proper type
   let
-    therepo = coerce repo :: Repository rt p wR wU wR
+    therepo = unsafeCoercePatchType (unsafeCoerceRepoType repo) :: Repository rt p wR wU wR
 
     patchTypeString :: String
     patchTypeString =
       case patchType of
+        RepoV3 -> "darcs-3"
         RepoV2 -> "darcs-2"
         RepoV1 -> "darcs-1"
 
@@ -256,70 +265,92 @@ runJob patchType (SRepoType isRebase) repo repojob = do
   case repojob of
     RepoJob job ->
       case checkTree patchType of
-        IsTree ->
+        IsTree -> do
+          checkOldStyleRebaseStatus isRebase therepo
           job therepo
             `finally`
-              Rebase.maybeDisplaySuspendedStatus isRebase therepo
+              maybeDisplaySuspendedStatus isRebase therepo
 
     PrimV1Job job ->
       case checkPrimV1 patchType of
         UsesPrimV1 -> do
+          checkOldStyleRebaseStatus isRebase therepo
           job therepo
             `finally`
-              Rebase.maybeDisplaySuspendedStatus isRebase therepo
+              maybeDisplaySuspendedStatus isRebase therepo
 
     V2Job job ->
       case (patchType, isRebase) of
         (RepoV2, SNoRebase) -> job therepo
+        (RepoV2, SIsRebase) ->
+          fail "This command is not supported while a rebase is in progress."
         (RepoV1, _        ) ->
           fail $    "This repository contains darcs v1 patches,"
                  ++ " but the command requires darcs v2 patches."
-        (RepoV2, SIsRebase) ->
-          fail "This command is not supported while a rebase is in progress."
+        (RepoV3, _        ) ->
+          fail $    "This repository contains darcs v3 patches,"
+                 ++ " but the command requires darcs v2 patches."
 
     V1Job job ->
       case (patchType, isRebase) of
         (RepoV1, SNoRebase) -> job therepo
+        (RepoV1, SIsRebase) ->
+          fail "This command is not supported while a rebase is in progress."
         (RepoV2, _        ) ->
           fail $    "This repository contains darcs v2 patches,"
                  ++ " but the command requires darcs v1 patches."
-        (RepoV1, SIsRebase) ->
-          fail "This command is not supported while a rebase is in progress."
+        (RepoV3, _        ) ->
+          fail $    "This repository contains darcs v3 patches,"
+                 ++ " but the command requires darcs v1 patches."
 
-    RebaseAwareJob flags job ->
+    RebaseAwareJob job ->
       case (checkTree patchType, isRebase) of
         (IsTree, SNoRebase) -> job therepo
-        (IsTree, SIsRebase) -> rebaseJob job therepo flags
+        (IsTree, SIsRebase) -> do
+          checkOldStyleRebaseStatus isRebase therepo
+          rebaseJob job therepo
 
-    RebaseJob flags job ->
+    RebaseJob job ->
       case (checkTree patchType, isRebase) of
         (_     , SNoRebase) -> fail "No rebase in progress. Try 'darcs rebase suspend' first."
-        (IsTree, SIsRebase) -> rebaseJob job therepo flags
+        (IsTree, SIsRebase) -> do
+          checkOldStyleRebaseStatus isRebase therepo
+          rebaseJob job therepo
 
-    StartRebaseJob flags job ->
-       case (checkTree patchType, isRebase) of
-         (_     , SNoRebase) -> impossible
-         (IsTree, SIsRebase) -> startRebaseJob job therepo flags
+    OldRebaseJob job ->
+      case (checkTree patchType, isRebase) of
+        (_     , SNoRebase) -> fail "No rebase in progress."
+        (IsTree, SIsRebase) -> do
+          -- no checkOldStyleRebaseStatus, this is for 'rebase upgrade'
+          job therepo
+            `finally`
+              maybeDisplaySuspendedStatus isRebase therepo
 
--- | apply a given RepoJob to a repository in the current working directory,
---   taking a lock
-withRepoLock :: DryRun -> UseCache -> UpdateWorking -> UMask -> RepoJob a -> IO a
-withRepoLock dry useCache uw um repojob =
-    withRepository useCache $ onRepoJob repojob $ \job repository ->
-    do maybe (return ()) fail $ writeProblem (repoFormat repository)
-       let name = "./"++darcsdir++"/lock"
-       withUMaskFlag um $
-         if dry == YesDryRun
-           then job repository
-           else withLock name (revertRepositoryChanges repository uw >> job repository)
+    StartRebaseJob job ->
+      case (checkTree patchType, isRebase) of
+        (_     , SNoRebase) -> error "impossible case"
+        (IsTree, SIsRebase) -> do
+          -- no checkOldStyleRebaseStatus, startRebaseJob does that
+          startRebaseJob job therepo
+
+-- | Apply a given RepoJob to a repository in the current working directory.
+-- However, before doing the job, take the repo lock and initializes a repo
+-- transaction, unless this is a dry-run.
+withRepoLock :: DryRun -> UseCache -> UpdatePending -> UMask -> RepoJob a -> IO a
+withRepoLock YesDryRun useCache _ _ repojob =
+  withRepository useCache $ onRepoJob repojob $ \job repository -> job repository
+withRepoLock NoDryRun useCache upe um repojob =
+  withLock lockPath $
+    withRepository useCache $ onRepoJob repojob $ \job repository -> do
+      maybe (return ()) fail $ writeProblem (repoFormat repository)
+      withUMaskFlag um $ revertRepositoryChanges repository upe >>= job
 
 -- | run a lock-taking job in an old-fashion repository.
 --   only used by `darcs optimize upgrade`.
 withOldRepoLock :: RepoJob a -> IO a
 withOldRepoLock repojob =
     withRepository NoUseCache $ onRepoJob repojob $ \job repository ->
-    do let name = "./"++darcsdir++"/lock"
-       withLock name $ job repository
+    withLock lockPath $ job repository
 
 -- | Apply a given RepoJob to a repository in the current working directory,
 -- taking a lock. If lock not takeable, do nothing. If old-fashioned
@@ -327,17 +358,20 @@ withOldRepoLock repojob =
 -- because there is no call to revertRepositoryChanges. This entry point is
 -- currently only used for attemptCreatePatchIndex.
 withRepoLockCanFail :: UseCache -> RepoJob () -> IO ()
-withRepoLockCanFail useCache repojob =
-    withRepository useCache $ onRepoJob repojob $ \job repository ->
-    let rf = repoFormat repository in
-    if formatHas HashedInventory rf then do
-       maybe (return ()) fail $ writeProblem rf
-       let name = "./"++darcsdir++"/lock"
-       eitherDone <- withLockCanFail name (job repository)
-       case eitherDone of
-         Left  _ -> debugMessage "Lock could not be obtained, not doing the job."
-         Right _ -> return ()
-      else debugMessage "Not doing the job because this is an old-fashioned repository."
+withRepoLockCanFail useCache repojob = do
+  eitherDone <-
+    withLockCanFail lockPath $
+      withRepository useCache $ onRepoJob repojob $ \job repository -> do
+        let rf = repoFormat repository
+        if formatHas HashedInventory rf then do
+          maybe (return ()) fail $ writeProblem rf
+          job repository
+        else
+          debugMessage
+            "Not doing the job because this is an old-fashioned repository."
+  case eitherDone of
+    Left  _ -> debugMessage "Lock could not be obtained, not doing the job."
+    Right _ -> return ()
 
 -- | If the 'RepoType' of the given repo indicates that we have 'NoRebase',
 -- then 'Just' the repo with the refined type, else 'Nothing'.
