@@ -17,68 +17,57 @@
 
 module Darcs.Repository.Match
     (
-      getNonrangeMatch
+      getRecordedUpToMatch
     , getOnePatchset
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Control.Exception ( throw )
-
 import Darcs.Patch.Match
-    ( getNonrangeMatchS
-    , nonrangeMatcherIsTag
+    ( rollbackToPatchSetMatch
+    , PatchSetMatch(..)
     , getMatchingTag
     , matchAPatchset
-    , nonrangeMatcher
-    , applyNInv
-    , hasIndexRange
-    , MatchFlag(..)
     )
 
-import Darcs.Patch.Bundle ( scanContextFile )
+import Darcs.Patch.Bundle ( readContextFile )
 import Darcs.Patch.ApplyMonad ( ApplyMonad(..) )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch ( RepoPatch, IsRepoType )
-import Darcs.Patch.Set ( PatchSet(..), SealedPatchSet, Origin )
-import Darcs.Patch.Witnesses.Sealed ( seal )
+import Darcs.Patch.Set ( Origin, PatchSet(..), SealedPatchSet, patchSetDrop )
 
 import Darcs.Repository.Flags
     ( WithWorkingDir (WithWorkingDir) )
 import Darcs.Repository.ApplyPatches ( DefaultIO, runDefault )
 import Darcs.Repository.InternalTypes ( Repository )
-import Darcs.Repository.Hashed
-    ( readRepo, createPristineDirectoryTree )
+import Darcs.Repository.Hashed ( readRepo )
+import Darcs.Repository.Pristine ( createPristineDirectoryTree )
 
 import Darcs.Util.Tree ( Tree )
 
 import Darcs.Util.Path ( toFilePath )
 
-getNonrangeMatch :: (ApplyMonad (ApplyState p) DefaultIO, IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-                 => Repository rt p wR wU wT
-                 -> [MatchFlag]
-                 -> IO ()
-getNonrangeMatch r = withRecordedMatch r . getMatch where
-  getMatch fs = case hasIndexRange fs of
-    Just (n, m) | n == m -> applyNInv (n-1)
-                | otherwise -> throw $ userError "Index range is not allowed for this command."
-    _ -> getNonrangeMatchS fs
+-- | Create a new pristine and working tree in the current working directory,
+-- corresponding to the state of the 'PatchSet' returned by 'getOnePatchSet'
+-- for the same 'PatchSetMatch'.
+getRecordedUpToMatch :: (ApplyMonad (ApplyState p) DefaultIO, IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
+                     => Repository rt p wR wU wT
+                     -> PatchSetMatch
+                     -> IO ()
+getRecordedUpToMatch r = withRecordedMatch r . rollbackToPatchSetMatch
 
 getOnePatchset :: (IsRepoType rt, RepoPatch p)
-               => Repository rt p wR wU wT
-               -> [MatchFlag]
+               => Repository rt p wR wU wR
+               -> PatchSetMatch
                -> IO (SealedPatchSet rt p Origin)
-getOnePatchset repository fs =
-    case nonrangeMatcher fs of
-        Just m -> do ps <- readRepo repository
-                     if nonrangeMatcherIsTag fs
-                        then return $ getMatchingTag m ps
-                        else return $ matchAPatchset m ps
-        Nothing -> seal `fmap` (scanContextFile . toFilePath . context_f $ fs)
-    where context_f [] = bug "Couldn't match_nonrange_patchset"
-          context_f (Context f:_) = f
-          context_f (_:xs) = context_f xs
+getOnePatchset repository pm =
+  case pm of
+    IndexMatch n -> patchSetDrop (n-1) <$> readRepo repository
+    PatchMatch m -> matchAPatchset m <$> readRepo repository
+    TagMatch m -> getMatchingTag m <$> readRepo repository
+    ContextMatch path -> do
+      ref <- readRepo repository
+      readContextFile ref (toFilePath path)
 
 withRecordedMatch :: (IsRepoType rt, RepoPatch p)
                   => Repository rt p wR wU wT
