@@ -18,9 +18,9 @@
 module Darcs.Repository.Old ( readOldRepo,
                               oldRepoFailMsg ) where
 
-import Prelude ()
 import Darcs.Prelude
 
+import Control.Applicative ( many )
 import Darcs.Util.Progress ( debugMessage, beginTedious, endTedious, finishedOneIO )
 import Darcs.Util.Path ( ioAbsoluteOrRemote, toPath )
 import System.IO ( hPutStrLn, stderr )
@@ -30,12 +30,11 @@ import Darcs.Patch.PatchInfoAnd ( Hopefully, PatchInfoAnd,
                          patchInfoAndPatch,
                          actually, unavailable )
 
-import qualified Data.ByteString as B ( ByteString, null )
+import qualified Data.ByteString as B ( ByteString )
 import qualified Data.ByteString.Char8 as BC ( break, pack, unpack )
 
-import Darcs.Patch ( RepoPatch, IsRepoType, WrappedNamed,
-                     readPatch )
-import Darcs.Patch.ReadMonads as RM ( parseStrictly )
+import Darcs.Patch ( RepoPatch, Named, readPatch )
+import qualified Darcs.Util.Parser as P ( parse )
 import Darcs.Patch.Witnesses.Ordered ( RL(..) )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed), seal, unseal, mapSeal )
 import Darcs.Patch.Info ( PatchInfo(..), makePatchname, readPatchInfo, displayPatchInfo )
@@ -51,7 +50,7 @@ import Darcs.Util.IsoDate ( readUTCDateOldFashioned, showIsoDateTime )
 
 import Control.Exception ( catch, IOException )
 
-readOldRepo :: (IsRepoType rt, RepoPatch p) => String -> IO (SealedPatchSet rt p Origin)
+readOldRepo :: RepoPatch p => String -> IO (SealedPatchSet rt p Origin)
 readOldRepo repo_dir = do
   realdir <- toPath `fmap` ioAbsoluteOrRemote repo_dir
   let task = "Reading inventory of repository "++repo_dir
@@ -60,18 +59,13 @@ readOldRepo repo_dir = do
                         (\e -> do hPutStrLn stderr ("Invalid repository:  " ++ realdir)
                                   ioError e)
 
-readRepoPrivate :: (IsRepoType rt, RepoPatch p)
+readRepoPrivate :: RepoPatch p
                 => String -> FilePath -> FilePath -> IO (SealedPatchSet rt p Origin)
 readRepoPrivate task repo_dir inventory_name = do
     inventory <- gzFetchFilePS (repo_dir </> darcsdir </> inventory_name) Uncachable
     finishedOneIO task inventory_name
     let parse inf = parse2 inf $ repo_dir </> darcsdir </> "patches" </> makeFilename inf
-        (mt, is) = case BC.break ('\n' ==) inventory of
-                   (swt,pistr) | swt == BC.pack "Starting with tag:" ->
-                                     case readPatchInfos pistr of
-                                     (t:ids) -> (Just t,reverse ids)
-                                     [] -> bug "bad inventory in readRepoPrivate"
-                   _ -> (Nothing, reverse $ readPatchInfos inventory)
+    (mt, is) <- readInventory inventory
     Sealed ts <- unseal seal `fmap` unsafeInterleaveIO (read_ts parse mt)
     Sealed ps <- unseal seal `fmap` unsafeInterleaveIO (read_patches parse is)
     return $ seal (PatchSet ts ps)
@@ -86,12 +80,7 @@ readRepoPrivate task repo_dir inventory_name = do
                       do x <- gzFetchFilePS (repo_dir </> darcsdir </> "inventories" </> makeFilename tag0) Uncachable
                          finishedOneIO task (renderString (displayPatchInfo tag0))
                          return x
-                 let (mt, is) = case BC.break ('\n' ==) i of
-                                (swt,pistr) | swt == BC.pack "Starting with tag:" ->
-                                                case readPatchInfos pistr of
-                                                (t:ids) -> (Just t,reverse ids)
-                                                [] -> bug "bad inventory in readRepoPrivate"
-                                _ -> (Nothing, reverse $ readPatchInfos i)
+                 (mt, is) <- readInventory i
                  Sealed ts <- fmap (unseal seal) $ unsafeInterleaveIO $ read_ts parse mt
                  Sealed ps <- unseal seal `fmap` unsafeInterleaveIO (read_patches parse is)
                  Sealed tag00 <-  parse tag0 `catch`
@@ -99,16 +88,17 @@ readRepoPrivate task repo_dir inventory_name = do
                                         return $ seal $
                                         patchInfoAndPatch tag0 $ unavailable $ show e
                  return $ seal $ ts :<: Tagged tag00 Nothing ps
-          parse2 :: (IsRepoType rt, RepoPatch p)
+          parse2 :: RepoPatch p
                  => PatchInfo -> FilePath
                  -> IO (Sealed (PatchInfoAnd rt p wX))
           parse2 i fn = do ps <- unsafeInterleaveIO $ gzFetchFilePS fn Cachable
                            return $ patchInfoAndPatch i
                              `mapSeal` hopefullyNoParseError (toPath fn) (readPatch ps)
-          hopefullyNoParseError :: String -> Maybe (Sealed (WrappedNamed rt a1dr wX))
-                                -> Sealed (Hopefully (WrappedNamed rt a1dr) wX)
-          hopefullyNoParseError _ (Just (Sealed x)) = seal $ actually x
-          hopefullyNoParseError s Nothing = seal $ unavailable $ "Couldn't parse file "++s
+          hopefullyNoParseError :: String -> Either String (Sealed (Named a1dr wX))
+                                -> Sealed (Hopefully (Named a1dr) wX)
+          hopefullyNoParseError _ (Right (Sealed x)) = seal $ actually x
+          hopefullyNoParseError s (Left e) =
+              seal $ unavailable $ unlines ["Couldn't parse file " ++ s, e]
           read_patches :: RepoPatch p =>
                           (forall wB . PatchInfo -> IO (Sealed (PatchInfoAnd rt p wB)))
                        -> [PatchInfo] -> IO (Sealed (RL (PatchInfoAnd rt p) wX))
@@ -146,8 +136,20 @@ makeFilename pi = showIsoDateTime d++"-"++sha1_a++"-"++ (show $ makePatchname pi
     where d = readUTCDateOldFashioned $ BC.unpack $ _piDate pi
           sha1_a = take 5 $ show $ sha1PS $ _piAuthor pi
 
-readPatchInfos :: B.ByteString -> [PatchInfo]
-readPatchInfos inv | B.null inv = []
-readPatchInfos inv = case parseStrictly readPatchInfo inv of
-                     Just (pinfo,r) -> pinfo : readPatchInfos r
-                     _ -> []
+readPatchInfos :: B.ByteString -> IO [PatchInfo]
+readPatchInfos inv =
+    case P.parse (many readPatchInfo) inv of
+        Right (r, _) -> return r
+        Left e -> fail $ unlines ["cannot parse inventory:", e]
+
+readInventory :: B.ByteString -> IO (Maybe PatchInfo, [PatchInfo])
+readInventory inv =
+    case BC.break ('\n' ==) inv of
+        (swt,pistr) | swt == BC.pack "Starting with tag:" -> do
+            infos <- readPatchInfos pistr
+            case infos of
+                (t:ids) -> return (Just t, reverse ids)
+                [] -> fail $ unlines ["empty parent inventory:", BC.unpack pistr]
+        _ -> do
+            infos <- readPatchInfos inv
+            return (Nothing, reverse infos)
