@@ -35,13 +35,13 @@ import System.IO.Unsafe ( unsafeInterleaveIO )
 
 import qualified Data.ByteString.Lazy.Char8 as BLC
 import Data.List ( isPrefixOf, sort )
-import Data.Maybe( catMaybes, listToMaybe )
 
 import System.Directory ( createDirectoryIfMissing
                         , renameFile
                         , removeFile
                         , doesFileExist
                         , getModificationTime
+                        , listDirectory
                         )
 import System.FilePath ( (</>)
                        , (<.>)
@@ -52,32 +52,35 @@ import System.FilePath ( (</>)
                        )
 import System.Posix.Files ( createLink )
 
+import Darcs.Prelude
+
 import Darcs.Util.ByteString ( gzReadFilePS )
 import Darcs.Util.Lock ( withTemp )
 import Darcs.Util.External ( Cachable(..), fetchFileLazyPS )
 import Darcs.Util.Global ( darcsdir )
-import Darcs.Util.Progress ( debugMessage )
+import Darcs.Util.Progress ( debugMessage, progressList )
 
 import Darcs.Patch ( IsRepoType, RepoPatch )
 import Darcs.Patch.PatchInfoAnd ( extractHash )
+import Darcs.Patch.Progress ( progressFL )
 import Darcs.Patch.Witnesses.Ordered ( mapFL )
 import Darcs.Patch.Set ( patchSet2FL )
 
+import Darcs.Repository.Traverse ( listInventories )
 import Darcs.Repository.InternalTypes ( Repository )
-import qualified Darcs.Repository.Hashed as HashedRepo
-import Darcs.Repository.Hashed ( filterDirContents, readRepo, readHashedPristineRoot )
+import Darcs.Repository.Hashed ( readRepo )
+import Darcs.Repository.Inventory ( getValidHash )
 import Darcs.Repository.Format
     ( identifyRepoFormat, formatHas, RepoProperty ( HashedInventory ) )
 import Darcs.Repository.Cache ( fetchFileUsingCache
                               , HashedDir(..)
-                              , Cache(..)
-                              , CacheLoc(..)
-                              , WritableOrNot(..)
+                              , Cache
+                              , closestWritableDirectory
                               , hashedDir
                               , bucketFolder
-                              , CacheType(Directory)
                               )
 import Darcs.Repository.Old ( oldRepoFailMsg )
+import Darcs.Repository.Pristine ( readHashedPristineRoot )
 
 packsDir, basicPack, patchesPack :: String
 packsDir     = "packs"
@@ -119,7 +122,7 @@ unpackTar c dir (Tar.Next e es) = case Tar.entryContent e of
           else do
             if p == darcsdir </> "hashed_inventory"
               then writeFile' Nothing p bs
-              else writeFile' (cacheDir c) p $ GZ.compress bs
+              else writeFile' (closestWritableDirectory c) p $ GZ.compress bs
             debugMessage $ "TAR thread: GET " ++ p
             unpackTar c dir es
   _ -> fail "Unexpected non-file tar entry"
@@ -149,11 +152,6 @@ fetchFilesUsingCache cache dir = mapM_ go where
      then debugMessage $ "FILE thread: exists " ++ path
      else void $ fetchFileUsingCache cache dir path
 
-cacheDir :: Cache -> Maybe String
-cacheDir (Ca cs) = listToMaybe . catMaybes .flip map cs $ \x -> case x of
-  Cache Directory Writable x' -> Just x'
-  _ -> Nothing
-
 -- | Create packs from the current recorded version of the repository.
 createPacks :: (IsRepoType rt, RepoPatch p)
             => Repository rt p wR wU wT -> IO ()
@@ -169,27 +167,26 @@ createPacks repo = flip finally (mapM_ removeFileIfExists
   createDirectoryIfMissing False (darcsdir </> packsDir)
   -- pristine hash
   Just hash <- readHashedPristineRoot repo
-  writeFile ( darcsdir </> packsDir </> "pristine" ) hash
+  writeFile ( darcsdir </> packsDir </> "pristine" ) $ getValidHash hash
   -- pack patchesTar
-  ps <- mapFL hashedPatchFileName . patchSet2FL <$> readRepo repo
-  is <- map ((darcsdir </> "inventories") </>) <$> HashedRepo.listInventories
+  ps <- mapFL hashedPatchFileName . progressFL "Packing patches" . patchSet2FL <$> readRepo repo
+  is <- map ((darcsdir </> "inventories") </>) <$> listInventories
   writeFile (darcsdir </> "meta-filelist-inventories") . unlines $
     map takeFileName is
   -- Note: tinkering with zlib's compression parameters does not make
   -- any noticeable difference in generated archive size;
   -- switching to bzip2 would provide ~25% gain OTOH.
   BLC.writeFile (patchesTar <.> "part") . GZ.compress . Tar.write =<<
-    mapM fileEntry' ((darcsdir </> "meta-filelist-inventories") : ps ++
-    reverse is)
+    mapM fileEntry' ((darcsdir </> "meta-filelist-inventories") : ps ++ reverse is)
   renameFile (patchesTar <.> "part") patchesTar
   -- pack basicTar
-  pr <- sortByMTime =<< dirContents "pristine.hashed"
+  pr <- sortByMTime =<< dirContents (darcsdir </> "pristine.hashed")
   writeFile (darcsdir </> "meta-filelist-pristine") . unlines $
     map takeFileName pr
   BLC.writeFile (basicTar <.> "part") . GZ.compress . Tar.write =<< mapM fileEntry' (
     [ darcsdir </> "meta-filelist-pristine"
     , darcsdir </> "hashed_inventory"
-    ] ++ reverse pr)
+    ] ++ progressList "Packing pristine" (reverse pr))
   renameFile (basicTar <.> "part") basicTar
  where
   basicTar = darcsdir </> packsDir </> basicPack
@@ -198,8 +195,7 @@ createPacks repo = flip finally (mapM_ removeFileIfExists
     content <- BLC.fromChunks . return <$> gzReadFilePS x
     tp <- either fail return $ toTarPath False x
     return $ fileEntry tp content
-  dirContents d = map ((darcsdir </> d) </>) <$>
-    filterDirContents d (const True)
+  dirContents dir = map (dir </>) <$> listDirectory dir
   hashedPatchFileName x = case extractHash x of
     Left _ -> fail "unexpected unhashed patch"
     Right h -> darcsdir </> "patches" </> h
