@@ -180,7 +180,6 @@ applyPatchMods pmods pindex =
          insertTouch fid pid
          stopFidSpan fn pid
          stopFpSpan fid pid
-       go (_, PInvalid _) = return () -- just ignore invalid changes
        go (pid, PDuplicateTouch fn) = do
          fidm <- gets fidspans
          case M.lookup fn fidm of
@@ -323,14 +322,11 @@ patches2patchMods patches fns = snd $ mapAccumL go fns patches
                                      PRemove f -> [f]; _ -> []}
             touched_all = listTouchedFiles p
             touched_effect = concatMap touched pmods_effect
-            touched_invalid = [ f | (PInvalid f) <- pmods_effect]
             -- listTouchedFiles returns all files that touched by these
             --  patches, even if they have no effect, e.g. by duplicate patches
             pmods_dup = map PDuplicateTouch . S.elems
                             $ S.difference (S.fromList touched_all)
-                                           (S.fromList touched_invalid
-                                            `S.union`
-                                            S.fromList touched_effect)
+                                           (S.fromList touched_effect)
 
 -- | return set of current filenames in patch index
 fpSpans2fileNames :: FilePathSpans -> Set AnchoredPath
@@ -388,7 +384,7 @@ updatePatchIndexDisk repo patches = do
     let newpatches = drop len_common $ mapFL seal2 flpatches
         newpmods = patches2patchMods newpatches filenames
     inv_hash <- getInventoryHash repodir
-    storePatchIndex repodir cdir inv_hash (applyPatchMods newpmods pindex')
+    storePatchIndex cdir inv_hash (applyPatchMods newpmods pindex')
   where
     -- return uncommon suffixes and length of common prefix of as and bs
     uncommon = uncommon' 0
@@ -403,7 +399,7 @@ createPatchIndexFrom :: Repository rt p wR wU wT
                      -> [(PatchId, [PatchMod AnchoredPath])] -> IO ()
 createPatchIndexFrom repo pmods = do
     inv_hash <- getInventoryHash repodir
-    storePatchIndex repodir cdir inv_hash (applyPatchMods pmods emptyPatchIndex)
+    storePatchIndex cdir inv_hash (applyPatchMods pmods emptyPatchIndex)
   where repodir = repoLocation repo
         cdir = repodir </> indexDir
         emptyPatchIndex = PatchIndex [] M.empty M.empty M.empty
@@ -518,10 +514,10 @@ isPatchIndexInSync repo = do
     else return False
 
 -- | Stores patch-index on disk.
-storePatchIndex :: FilePath -> FilePath -> String -> PatchIndex -> IO ()
-storePatchIndex repodir cdir inv_hash (PatchIndex pids fidspans fpspans infom) = do
+storePatchIndex :: FilePath -> String -> PatchIndex -> IO ()
+storePatchIndex cdir inv_hash (PatchIndex pids fidspans fpspans infom) = do
   createDirectory cdir `catch` \(_ :: IOError) -> return ()
-  tmpdir <- withPermDir repodir $ \dir -> do
+  tmpdir <- withPermDir cdir $ \dir -> do
               debugMessage "About to create patch index..."
               let tmpdir = toFilePath dir
               storeRepoState (tmpdir </> repoStateFile) inv_hash
