@@ -30,7 +30,6 @@ module Darcs.Repository.Prefs
     , changePrefval
     , defPrefval
     , writeDefaultPrefs
-    , boringRegexps
     , isBoring
     , FileType(..)
     , filetypeFunction
@@ -66,7 +65,6 @@ import System.IO.Error ( isDoesNotExistError, catchIOError )
 import System.IO ( stdout, stderr )
 import System.Info ( os )
 import System.Posix.Files ( getFileStatus, fileOwner )
-import Text.Regex ( Regex, mkRegex, matchRegex )
 
 import Darcs.Repository.Cache ( Cache, mkCache, CacheType(..), CacheLoc(..),
                                 WritableOrNot(..) )
@@ -84,6 +82,7 @@ import Darcs.Util.Global ( darcsdir, debugMessage )
 import Darcs.Util.Path ( AbsolutePath, ioAbsolute, toFilePath,
                          getCurrentDirectory )
 import Darcs.Util.Printer( hPutDocLn, text )
+import Darcs.Util.Regex ( Regex, mkRegex, matchRegex )
 import Darcs.Util.URL ( isValidLocalPath )
 import Darcs.Util.File ( osxCacheDir, xdgCacheDir, removeFileMayNotExist )
 
