@@ -76,7 +76,7 @@ import Darcs.Util.Tree( Tree, emptyTree, list, restrict, expand, itemHash, zipTr
 import Darcs.Util.Tree.Monad( TreeIO )
 import Darcs.Util.Tree.Hashed( darcsUpdateHashes, hashedTreeIO )
 import Darcs.Util.Tree.Plain( readPlainTree )
-import Darcs.Util.Index( updateIndex )
+import Darcs.Util.Index( treeFromIndex )
 
 import qualified Data.ByteString.Char8 as BC
 
@@ -243,7 +243,7 @@ checkIndex
   -> Bool
   -> IO Bool
 checkIndex repo quiet = do
-  index <- updateIndex =<< readIndex repo
+  index <- treeFromIndex =<< readIndex repo
   pristine <- expand =<< readRecordedAndPending repo
   working <- expand =<< restrict pristine <$> readPlainTree "."
   working_hashed <- darcsUpdateHashes working
