@@ -39,21 +39,14 @@ module Darcs.Repository.State
 
 import Darcs.Prelude
 
-import Control.Monad ( when, foldM, forM )
+import Control.Monad ( when, foldM, forM, void )
 import Control.Monad.State ( StateT, runStateT, get, put, liftIO )
 import Control.Exception ( catch, IOException )
-import Data.Maybe ( isJust )
 import Data.Ord ( comparing )
 import Data.List ( sortBy, union, delete )
-import Text.Regex( matchRegex )
 
-import System.Directory( removeFile, doesFileExist, doesDirectoryExist, renameFile )
-import System.FilePath
-    ( (</>)
-#if mingw32_HOST_OS
-    , (<.>)
-#endif
-    )
+import System.Directory( doesFileExist, doesDirectoryExist, renameFile )
+import System.FilePath ( (<.>), (</>) )
 import System.IO ( hPutStrLn, stderr )
 import System.IO.Error ( catchIOError )
 
@@ -87,7 +80,7 @@ import Darcs.Repository.Flags ( UseIndex(..), ScanKnown(..), DiffAlgorithm(..)
 import Darcs.Repository.InternalTypes ( Repository, repoFormat, repoLocation )
 import Darcs.Repository.Format(formatHas, RepoProperty(NoWorkingDir))
 import qualified Darcs.Repository.Pending as Pending
-import Darcs.Repository.Prefs ( filetypeFunction, boringRegexps )
+import Darcs.Repository.Prefs ( filetypeFunction, isBoring )
 import Darcs.Repository.Diff ( treeDiff )
 import Darcs.Repository.Inventory ( peekPristineHash, getValidHash )
 import Darcs.Repository.Paths
@@ -100,9 +93,11 @@ import Darcs.Repository.Paths
     , indexInvalidPath
     )
 
+import Darcs.Util.File ( removeFileMayNotExist )
+import Darcs.Util.Global ( debugMessage )
 import Darcs.Util.Path
     ( AnchoredPath
-    , anchorPath
+    , realPath
     , filterPaths
     , inDarcsdir
     , parents
@@ -115,7 +110,13 @@ import Darcs.Util.Tree( Tree, restrict, FilterTree, expand, emptyTree, overlay,
 import qualified Darcs.Util.Tree.Plain as PlainTree ( readPlainTree )
 import Darcs.Util.Tree.Hashed
     ( darcsTreeHash, readDarcsHashed, decodeDarcsHash, decodeDarcsSize )
-import qualified Darcs.Util.Index as I
+import Darcs.Util.Index
+    ( Index
+    , indexFormatValid
+    , openIndex
+    , treeFromIndex
+    , updateIndexFrom
+    )
 import qualified Darcs.Util.Tree as Tree
 import Darcs.Util.Index ( listFileIDs, getFileID )
 
@@ -171,14 +172,14 @@ maybeRestrictSubpaths pending repo =
 -- to darcs. The result is usually applied to the full working state.
 restrictBoring :: Tree m -> IO (TreeFilter m)
 restrictBoring guide = do
-  boring <- boringRegexps
-  let boring' p | inDarcsdir p = False
-      boring' p = not $ any (\rx -> isJust $ matchRegex rx p') boring
-          where p' = anchorPath "" p
+  boring <- isBoring
+  let exclude p = inDarcsdir p || boring (realPath p)
       restrictTree :: FilterTree t m => t m -> t m
-      restrictTree = Tree.filter $ \p _ -> case find guide p of
-                                             Nothing -> boring' p
-                                             _ -> True
+      restrictTree =
+        Tree.filter $ \p _ ->
+          case find guide p of
+            Nothing -> not (exclude p)
+            _ -> True
   return (TreeFilter restrictTree)
 
 -- | Construct a Tree filter that removes any darcs metadata files the
@@ -245,14 +246,18 @@ readPendingAndWorking _ _ _ r _ | formatHas NoWorkingDir (repoFormat r) = do
   IsEq <- return $ workDirLessRepoWitness r
   return (NilFL :> NilFL)
 readPendingAndWorking (useidx, scan, diffalg) lfm lfr repo mbpaths = do
+  debugMessage "readPendingAndWorking: start"
   (pending_tree, working_tree, (pending :> moves)) <-
     readPendingAndMovesAndUnrecorded repo useidx scan lfm mbpaths
+  debugMessage "readPendingAndWorking: after readPendingAndMovesAndUnrecorded"
   (pending_tree_with_replaces, Sealed replaces) <-
     getReplaces lfr diffalg repo pending_tree working_tree
+  debugMessage "readPendingAndWorking: after getReplaces"
   ft <- filetypeFunction
   wrapped_diff <- treeDiff diffalg ft pending_tree_with_replaces working_tree
   case unFreeLeft wrapped_diff of
     Sealed diff -> do
+      debugMessage "readPendingAndWorking: done"
       return $ unsafeCoercePEnd $ pending :> (moves +>+ replaces +>+ diff)
 
 readPendingAndMovesAndUnrecorded
@@ -267,17 +272,28 @@ readPendingAndMovesAndUnrecorded
         , (FL (PrimOf p) :> FL (PrimOf p)) wR wU -- pending :> moves
         )
 readPendingAndMovesAndUnrecorded repo useidx scan lfm mbpaths = do
+  debugMessage "readPendingAndMovesAndUnrecorded: start"
   (pending_tree, Sealed pending) <- readPending repo
   moves <- getMoves lfm repo mbpaths
-  let pending' = pending +>+ moves
-  relevant <- maybeRestrictSubpaths pending' repo mbpaths
-  pending_tree' <-
+  -- we want to include any user specified paths before and after pending
+  -- and detected moves
+  relevant <- maybeRestrictSubpaths (pending +>+ moves) repo mbpaths
+  pending_tree_with_moves <-
     applyTreeFilter relevant <$> applyToTree moves pending_tree
+  debugMessage "readPendingAndMovesAndUnrecorded: before readIndexOrPlainTree"
+  -- the moves are detected i.e. they are already applied in the working tree;
+  -- also note that we have to use the amended pending tree to restrict the
+  -- working tree in case we don't use the index (here and below)
+  index <- readIndexOrPlainTree repo useidx relevant pending_tree_with_moves
+  debugMessage "readPendingAndMovesAndUnrecorded: before filteredWorking"
+  -- TODO this conditional looks wrong; so if we do have detected moves,
+  -- then we cannot use the index to read the working state? Why not?
   let useidx' = if nullFL moves then useidx else IgnoreIndex
-  index <-
-    applyToTree moves =<< readIndexOrPlainTree repo useidx relevant pending_tree
-  working_tree <- filteredWorking repo useidx' scan relevant index pending_tree'
-  return (pending_tree', working_tree, unsafeCoercePEnd (pending :> moves))
+  working_tree <-
+    filteredWorking repo useidx' scan relevant index pending_tree_with_moves
+  debugMessage "readPendingAndMovesAndUnrecorded: done"
+  return
+    (pending_tree_with_moves, working_tree, unsafeCoercePEnd (pending :> moves))
 
 -- | @filteredWorking useidx scan relevant index pending_tree@ reads the
 -- working tree and filters it according to options and @relevant@ file paths.
@@ -389,7 +405,7 @@ readIndexOrPlainTree :: (ApplyState p ~ Tree, RepoPatch p)
 #if TEST_INDEX
 readIndexOrPlainTree repo useidx treeFilter pending_tree = do
   indexTree <-
-    I.updateIndex =<< applyTreeFilter treeFilter <$> readIndex repo
+    treeFromIndex =<< applyTreeFilter treeFilter <$> readIndex repo
   plainTree <- do
     guide <- expand pending_tree
     expand =<< applyTreeFilter treeFilter . restrict guide <$> readPlainTree repo
@@ -400,7 +416,7 @@ readIndexOrPlainTree repo useidx treeFilter pending_tree = do
       IgnoreIndex -> plainTree
 #else
 readIndexOrPlainTree repo UseIndex treeFilter pending_tree =
-  (I.updateIndex =<< applyTreeFilter treeFilter <$> readIndex repo)
+  (treeFromIndex =<< applyTreeFilter treeFilter <$> readIndex repo)
     `catchIOError` \e -> do
       hPutStrLn stderr ("Warning, cannot access the index:\n" ++ show e)
       readIndexOrPlainTree repo IgnoreIndex treeFilter pending_tree
@@ -455,48 +471,56 @@ readPending repo = do
        return (pristine, seal NilFL)
 
 -- | Mark the existing index as invalid. This has to be called whenever the
--- listing of pristine changes and will cause darcs to update the index next
--- time it tries to read it. (NB. This is about files added and removed from
+-- listing of pristine+pending changes and will cause darcs to update the index.
+-- This will happen either when we call updateIndex in finalizeRepositoryChanges
+-- or else when we try to read the index the next time.
+-- (NB. This is about files added and removed from
 -- pristine: changes to file content in either pristine or working are handled
 -- transparently by the index reading code.)
 invalidateIndex :: t -> IO ()
 invalidateIndex _ = B.writeFile indexInvalidPath B.empty
 
 readIndex :: (RepoPatch p, ApplyState p ~ Tree)
-          => Repository rt p wR wU wR -> IO I.Index
+          => Repository rt p wR wU wR -> IO Index
 readIndex repo = do
-  (invalid, exists, formatValid) <- checkIndex
-  if not exists || invalid || not formatValid
-     then do pris <- readRecordedAndPending repo
-             idx <- I.updateIndexFrom indexPath darcsTreeHash pris
-             when invalid $ removeFile indexInvalidPath
-             return idx
-     else I.readIndex indexPath darcsTreeHash
+  okay <- checkIndex
+  if not okay
+    then internalUpdateIndex repo
+    else openIndex indexPath darcsTreeHash
 
+-- | Update the index so that it matches pristine+pending. If the index does
+-- not exist or is invalid, create a new one. Returns the updated index.
+internalUpdateIndex :: (RepoPatch p, ApplyState p ~ Tree)
+            => Repository rt p wR wU wR -> IO Index
+internalUpdateIndex repo = do
+  pris <- readRecordedAndPending repo
+  idx <- updateIndexFrom indexPath darcsTreeHash pris
+  removeFileMayNotExist indexInvalidPath
+  return idx
+
+-- | Update the index so that it matches pristine+pending. If the index does
+-- not exist or is invalid, create a new one.
 updateIndex :: (RepoPatch p, ApplyState p ~ Tree)
             => Repository rt p wR wU wR -> IO ()
 updateIndex repo = do
-  (invalid, _, _) <- checkIndex
-  pris <- readRecordedAndPending repo
-  _ <- I.updateIndexFrom indexPath darcsTreeHash pris
-  when invalid $ removeFile indexInvalidPath
+  -- call checkIndex to throw away the index if it is invalid;
+  -- this can happen if we are called with --ignore-times
+  -- TODO make this impossible i.e. honor UseIndex here
+  void checkIndex
+  void $ internalUpdateIndex repo
 
-checkIndex :: IO (Bool, Bool, Bool)
+-- | Check if we have a valid index. This means that the index file exists, is
+-- readable, and can be mmapped, /and/ that indexInvalidPath does /not/ exist.
+-- We do not yet remove indexInvalidPath in case updating the index fails.
+checkIndex :: IO Bool
 checkIndex = do
   invalid <- doesFileExist $ indexInvalidPath
-  exists <- doesFileExist indexPath
-  formatValid <- if exists
-                     then I.indexFormatValid indexPath
-                     else return True
-  when (exists && not formatValid) $ do
--- TODO this conditional logic (rename or delete) is mirrored in
--- Darcs.Util.Index.updateIndexFrom and should be refactored
-#if mingw32_HOST_OS
-    renameFile indexPath (indexPath <.> "old")
-#else
-    removeFile indexPath
-#endif
-  return (invalid, exists, formatValid)
+  formatValid <- indexFormatValid indexPath
+  exist <- doesFileExist indexPath
+  -- this fails with a permission (access denied) error on windows
+  -- if we use removeFileMayNotExist instead of renameFile
+  when (exist && not formatValid) $ renameFile indexPath (indexPath <.> "old")
+  return (not invalid && formatValid)
 
 -- |Remove any patches (+dependencies) from a sequence that
 -- conflict with the recorded or unrecorded changes in a repo
