@@ -21,40 +21,44 @@ module Darcs.UI.Commands.Clone
     , clone
     , makeRepoName
     , cloneToSSH
+    , otherHelpInheritDefault
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import System.Directory ( doesDirectoryExist, doesFileExist
                         , setCurrentDirectory )
 import System.Exit ( ExitCode(..) )
-import Control.Exception ( catch, SomeException )
 import Control.Monad ( when, unless )
-import Data.Maybe ( listToMaybe )
 
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts
                       , nodefaults
                       , commandStub
                       , commandAlias
                       , putInfo
+                      , putFinished
                       )
 import Darcs.UI.Completion ( noArgs )
-import Darcs.UI.Flags( DarcsFlag( NewRepo
-                                  , UpToPattern
-                                  , UpToPatch
-                                  , UpToHash
-                                  , OnePattern
-                                  , OnePatch
-                                  , OneHash
-                                  )
-                     , matchAny, useCache, umask, remoteRepos
-                     , setDefault, quiet, usePacks
-                     , remoteDarcs, cloneKind, verbosity, setScriptsExecutable
-                     , withWorkingDir, patchIndexNo )
-import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, (?) )
+import Darcs.UI.Flags
+    ( DarcsFlag
+    , cloneKind
+    , patchIndexNo
+    , quiet
+    , remoteDarcs
+    , remoteRepos
+    , setDefault
+    , setScriptsExecutable
+    , umask
+    , useCache
+    , usePacks
+    , verbosity
+    , withNewRepo
+    , withWorkingDir
+    )
+import Darcs.UI.Options ( (^), odesc, ocheck, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.UI.Commands.Util ( getUniqueRepositoryName )
+import Darcs.Patch.Match ( MatchFlag(..) )
 import Darcs.Repository ( cloneRepository )
 import Darcs.Repository.Format ( identifyRepoFormat
                                , RepoProperty ( HashedInventory
@@ -63,59 +67,60 @@ import Darcs.Repository.Format ( identifyRepoFormat
                                , formatHas
                                )
 import Darcs.Util.Lock ( withTempDir )
-import Darcs.Util.Ssh ( getSSH, SSHCmd(SCP) )
+import Darcs.Util.Ssh ( getSSH, SSHCmd(SCP,SSH) )
 import Darcs.Repository.Flags
     ( CloneKind(CompleteClone), SetDefault(NoSetDefault), ForgetParent(..) )
-import Darcs.Patch.Bundle ( scanContextFile )
-import Darcs.Patch.Dummy ( DummyPatch )
-import Darcs.Patch.Set ( PatchSet, Origin )
 import Darcs.Repository.Prefs ( showMotd )
 import Darcs.Util.Progress ( debugMessage )
-import Darcs.Util.Printer ( text, ($$) )
-import Darcs.Util.Path ( toFilePath, toPath, ioAbsoluteOrRemote, AbsolutePath )
+import Darcs.Util.Printer ( Doc, formatWords, formatText, text, vsep, ($$), ($+$) )
+import Darcs.Util.Path ( toPath, ioAbsoluteOrRemote, AbsolutePath )
 import Darcs.Util.Workaround ( getCurrentDirectory )
-import Darcs.Util.URL ( isSshUrl )
+import Darcs.Util.URL ( SshFilePath(..), isSshUrl, splitSshUrl )
 import Darcs.Util.Exec ( exec, Redirect(..), )
 
 cloneDescription :: String
 cloneDescription = "Make a copy of an existing repository."
 
-cloneHelp :: String
-cloneHelp =
- unlines
-  [ "Clone creates a copy of a repository.  The optional second"
-  , "argument specifies a destination directory for the new copy;"
-  , "if omitted, it is inferred from the source location."
-  , ""
-  , "By default Darcs will copy every patch from the original repository."
-  , "If you expect the original repository to remain accessible, you can"
-  , "use `--lazy` to avoid copying patches until they are needed ('copy on"
-  , "demand').  This is particularly useful when copying a remote"
-  , "repository with a long history that you don't care about."
-  , ""
-  , "When cloning locally, Darcs automatically uses hard linking where"
-  , "possible.  As well as saving time and space, this enables to move or"
-  , "delete the original repository without affecting the copy."
-  , "Hard linking requires that the copy be on the same filesystem as the"
-  , "original repository, and that the filesystem support hard linking."
-  , "This includes NTFS, HFS+ and all general-purpose Unix filesystems"
-  , "(such as ext, UFS and ZFS). FAT does not support hard links."
-  , ""
-  , "When cloning from a remote location, Darcs will look for and attempt"
-  , "to use packs created by `darcs optimize http` in the remote repository."
-  , "Packs are single big files that can be downloaded faster than many"
-  , "little files."
-  , ""
-  , "Darcs clone will not copy unrecorded changes to the source repository's"
-  , "working tree."
-  , ""
-  , "You can copy a repository to a ssh url, in which case the new repository"
-  , "will always be complete."
-  , ""
-  ] ++ cloneHelpTag
-    ++ cloneHelpSSE
+cloneHelp :: Doc
+cloneHelp = vsep $
+  map formatWords
+  [ [ "Clone creates a copy of a repository.  The optional second"
+    , "argument specifies a destination directory for the new copy;"
+    , "if omitted, it is inferred from the source location."
+    ]
+  , [ "By default Darcs will copy every patch from the original repository."
+    , "If you expect the original repository to remain accessible, you can"
+    , "use `--lazy` to avoid copying patches until they are needed ('copy on"
+    , "demand').  This is particularly useful when copying a remote"
+    , "repository with a long history that you don't care about."
+    ]
+  , [ "When cloning locally, Darcs automatically uses hard linking where"
+    , "possible.  As well as saving time and space, this enables to move or"
+    , "delete the original repository without affecting the copy."
+    , "Hard linking requires that the copy be on the same filesystem as the"
+    , "original repository, and that the filesystem support hard linking."
+    , "This includes NTFS, HFS+ and all general-purpose Unix filesystems"
+    , "(such as ext, UFS and ZFS). FAT does not support hard links."
+    ]
+  , [ "When cloning from a remote location, Darcs will look for and attempt"
+    , "to use packs created by `darcs optimize http` in the remote repository."
+    , "Packs are single big files that can be downloaded faster than many"
+    , "little files."
+    ]
+  , [ "Darcs clone will not copy unrecorded changes to the source repository's"
+    , "working tree."
+    ]
+  , [ "You can copy a repository to a ssh url, in which case the new repository"
+    , "will always be complete."
+    ]
+  ]
+  ++
+  [ cloneHelpTag
+  , cloneHelpSSE
+  , cloneHelpInheritDefault
+  ]
 
-clone :: DarcsCommand [DarcsFlag]
+clone :: DarcsCommand
 clone = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "clone"
@@ -124,45 +129,44 @@ clone = DarcsCommand
     , commandExtraArgs = -1
     , commandExtraArgHelp = ["<REPOSITORY>", "[<DIRECTORY>]"]
     , commandCommand = cloneCmd
-    , commandPrereq = validContextFile
+    , commandPrereq = \_ -> return $ Right ()
     , commandCompleteArgs = noArgs
     , commandArgdefaults = nodefaults
     , commandAdvancedOptions = odesc cloneAdvancedOpts
     , commandBasicOptions = odesc cloneBasicOpts
     , commandDefaults = defaultFlags cloneOpts
     , commandCheckOptions = ocheck cloneOpts
-    , commandParseOptions = onormalise cloneOpts
     }
   where
     cloneBasicOpts
-      = O.reponame
+      = O.newRepo
       ^ O.cloneKind
       ^ O.matchOneContext
       ^ O.setDefault
+      ^ O.inheritDefault
       ^ O.setScriptsExecutable
       ^ O.withWorkingDir
-    cloneAdvancedOpts = O.usePacks ^ O.patchIndexNo ^ O.network
+    cloneAdvancedOpts = O.usePacks ^ O.patchIndexNo ^ O.umask ^ O.network
     cloneOpts = cloneBasicOpts `withStdOpts` cloneAdvancedOpts
 
-get :: DarcsCommand [DarcsFlag]
+get :: DarcsCommand
 get = commandAlias "get" Nothing clone
 
 putDescription :: String
 putDescription = "Deprecated command, replaced by clone."
 
-putHelp :: String
-putHelp = unlines
+putHelp :: Doc
+putHelp = formatText 80
  [ "This command is deprecated."
- , ""
- , "To clone the current repository to a ssh destination,"
- , "use the syntax `darcs clone . user@server:path` ."
+ , "To clone the current repository to a ssh destination, " ++
+   "use the syntax `darcs clone . user@server:path` ."
  ]
 
-put :: DarcsCommand [DarcsFlag]
+put :: DarcsCommand
 put = commandStub "put" putHelp putDescription clone
 
 cloneCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-cloneCmd fps opts [inrepodir, outname] = cloneCmd fps (NewRepo outname:opts) [inrepodir]
+cloneCmd fps opts [inrepodir, outname] = cloneCmd fps (withNewRepo outname opts) [inrepodir]
 cloneCmd _ opts [inrepodir] = do
   debugMessage "Starting work on clone..."
   typed_repodir <- ioAbsoluteOrRemote inrepodir
@@ -171,12 +175,11 @@ cloneCmd _ opts [inrepodir] = do
   rfsource <- identifyRepoFormat repodir
   debugMessage $ "Found the format of "++repodir++"..."
 
-  -- there's no fundamental reason for banning gets of repositories with
-  -- rebase in progress, but it seems a bit dubious to actually copy the
-  -- rebase state, and removing it is a bit of work since the current
-  -- implementation just copies the inventory file
+  -- This merely forbids clone from an old-style rebase in progress, which is
+  -- exactly what we want. Transferring patches from repos with new-style
+  -- rebase in progress is unproblematic and fully supported.
   when (formatHas RebaseInProgress rfsource) $
-      fail "Can't clone a repository with a rebase in progress"
+      fail "Cannot clone a repository with an old-style rebase in progress"
 
   unless (formatHas HashedInventory rfsource) $ putInfo opts $
        text "***********************************************************************"
@@ -192,6 +195,7 @@ cloneCmd _ opts [inrepodir] = do
   case cloneToSSH opts of
     Just repo -> do
       withTempDir "clone" $ \_ -> do
+         guardRemoteDirDoesNotExist repo
          putInfo opts $ text "Creating local clone..."
          currentDir <- getCurrentDirectory
          mysimplename <- makeRepoName True [] repodir -- give correct name to local clone
@@ -199,7 +203,8 @@ cloneCmd _ opts [inrepodir] = do
                          CompleteClone (umask ? opts) (remoteDarcs opts)
                          (setScriptsExecutable ? opts)
                          (remoteRepos ? opts) (NoSetDefault True)
-                         (matchAny ? map convertUpToToOne opts)
+                         O.NoInheritDefault -- never inherit defaultrepo when cloning to ssh
+                         (map convertUpToToOne (O.matchOneContext ? opts))
                          rfsource
                          (withWorkingDir ? opts)
                          (patchIndexNo ? opts)
@@ -217,24 +222,35 @@ cloneCmd _ opts [inrepodir] = do
                   (cloneKind ? opts) (umask ? opts) (remoteDarcs opts)
                   (setScriptsExecutable ? opts)
                   (remoteRepos ? opts) (setDefault True opts)
-                  (matchAny ? map convertUpToToOne opts)
+                  (O.inheritDefault ? opts)
+                  (map convertUpToToOne (O.matchOneContext ? opts))
                   rfsource
                   (withWorkingDir ? opts)
                   (patchIndexNo ? opts)
                   (usePacks ? opts)
                   NoForgetParent
-      putInfo opts $ text "Finished cloning."
+      putFinished opts "cloning"
 
 cloneCmd _ _ _ = fail "You must provide 'clone' with either one or two arguments."
 
 cloneToSSH :: [DarcsFlag] -> Maybe String
-cloneToSSH fs = case O.reponame ? fs of
+cloneToSSH fs = case O.newRepo ? fs of
   Nothing -> Nothing
   Just r -> if isSshUrl r then Just r else Nothing
 
+-- | Make sure we do not overwrite an existing remote directory.
+guardRemoteDirDoesNotExist :: String -> IO ()
+guardRemoteDirDoesNotExist rpath = do
+  (ssh, ssh_args) <- getSSH SSH
+  let sshfp = splitSshUrl rpath
+  let ssh_cmd = "mkdir '" ++ sshRepo sshfp ++ "'"
+  r <- exec ssh (ssh_args ++ [sshUhost sshfp, ssh_cmd]) (AsIs,AsIs,AsIs)
+  when (r /= ExitSuccess) $
+    fail $ "Cannot create remote directory '"  ++ sshRepo sshfp ++ "'."
+
 makeRepoName :: Bool -> [DarcsFlag] -> FilePath -> IO String
 makeRepoName talkative fs d =
-  case O.reponame ? fs of
+  case O.newRepo ? fs of
     Just n -> do
       exists <- doesDirectoryExist n
       file_exists <- doesFileExist n
@@ -252,10 +268,10 @@ makeRepoName talkative fs d =
       where mkName = dropWhile (== '.') . reverse .
               takeWhile (not . (`elem` "/:")) . dropWhile (== '/') . reverse
 
-cloneHelpTag :: String
-cloneHelpTag =
- unlines
-  [ "It is often desirable to make a copy of a repository that excludes"
+cloneHelpTag :: Doc
+cloneHelpTag = formatWords
+  [ ""
+  , "It is often desirable to make a copy of a repository that excludes"
   , "some patches.  For example, if releases are tagged then `darcs clone"
   , "--tag .` would make a copy of the repository as at the latest release."
   , ""
@@ -272,48 +288,50 @@ cloneHelpTag =
   , "patch.  Because these options treat the set of patches as an ordered"
   , "sequence, you may get different results after reordering with `darcs"
   , "optimize reorder`."
-  , ""
   ]
 
-cloneHelpSSE :: String
-cloneHelpSSE =
-    unlines
-    [ "The `--set-scripts-executable` option causes scripts to be made"
-    , "executable in the working tree. A script is any file that starts"
-    , "with a shebang (\"#!\")."
-    ]
+cloneHelpSSE :: Doc
+cloneHelpSSE = formatWords
+  [ "The `--set-scripts-executable` option causes scripts to be made"
+  , "executable in the working tree. A script is any file that starts"
+  , "with a shebang (\"#!\")."
+  ]
 
-validContextFile :: [DarcsFlag] -> IO (Either String ())
-validContextFile opts =
-   case getContext opts of
-     Nothing -> return $ Right ()
-     Just ctxAbsolutePath -> do
-         let ctxFilePath = toFilePath ctxAbsolutePath
-         exists <- doesFileExist ctxFilePath
-         if exists
-             then do
-                (ps :: PatchSet rt DummyPatch Origin wX) <-
-                    scanContextFile ctxFilePath
-                (ps `seq` return $ Right ()) `catch` \(_ :: SomeException) ->
-                    return . Left $ "File " ++ ctxFilePath
-                                    ++ " is not a valid context file"
-             else return . Left $
-                 "Context file " ++ ctxFilePath ++ " does not exist"
+cloneHelpInheritDefault :: Doc
+cloneHelpInheritDefault = commonHelpInheritDefault $+$ formatWords
+  [ "For the clone command it means the following:"
+  , "If the source repository already has a defaultrepo set (either because"
+  , "you cloned it or because you explicitly used the --set-default option),"
+  , "and both source and target are locally valid paths on the same host,"
+  , "then the target repo will get the same defaultrepo as the source repo."
+  , "Otherwise the target repo gets the source repo itself as defaultrepo,"
+  , "i.e. we fall back to the defalt behavior (--no-inherit-default)."
+  ]
 
--- TODO getContext choses arbitrarily the first --context flag
--- should instead report an error when more than one is given
+otherHelpInheritDefault :: Doc
+otherHelpInheritDefault = commonHelpInheritDefault $+$ formatWords
+  [ "For the commands push, pull, and send it means the following:"
+  , "Changes the meaning of the --set-default option so that it sets the"
+  , "(local) defaultrepo to the defaultrepo of the remote repo, instead of"
+  , "the remote repo itself. This happens only if the remote repo does have"
+  , "a defaultrepo set and both local and remote repositories are locally"
+  , "valid paths on the same host, otherwise fall back to the default behavior"
+  , "(--no-inherit-default)."
+  ]
 
--- | 'getContext' takes a list of flags and returns the context
--- specified by @Context c@ in that list of flags, if any.
--- This flag is present if darcs was invoked with @--context=FILE@
-getContext :: [DarcsFlag] -> Maybe AbsolutePath
-getContext fs = listToMaybe [ f | O.Context f <- O.context ? fs ]
+commonHelpInheritDefault :: Doc
+commonHelpInheritDefault = formatWords
+  [ "The --inherit-default option is meant to support a work flow where"
+  , "you have different branches of the same upstream repository and want"
+  , "all your branches to have the same upstream repo as the defaultrepo."
+  , "It is most useful when enabled globally by adding 'ALL --inherit-default'"
+  , "to your ~/darcs/defaults file."
+  ]
 
--- The 'clone' command takes --to-patch and --to-match as arguments,
--- but internally wants to handle them as if they were --patch and --match
--- TODO: remove this when we get rid of directly looking at [DarcsFlag]
--- for this command.
-convertUpToToOne :: DarcsFlag -> DarcsFlag
+-- | The 'clone' command takes --to-patch and --to-match as arguments,
+-- but internally wants to handle them as if they were --patch and --match.
+-- This function does the conversion.
+convertUpToToOne :: MatchFlag -> MatchFlag
 convertUpToToOne (UpToPattern p) = OnePattern p
 convertUpToToOne (UpToPatch p) = OnePatch p
 convertUpToToOne (UpToHash p) = OneHash p
