@@ -25,7 +25,6 @@ module Darcs.UI.Commands.GZCRCs
     , doCRCWarnings
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Control.Monad ( when, unless, forM_ )
@@ -37,7 +36,7 @@ import Data.IORef ( newIORef, readIORef, writeIORef )
 import Data.Monoid ( Any(..), Sum(..) )
 import System.Directory ( doesFileExist, doesDirectoryExist )
 import System.Exit ( ExitCode(..), exitWith )
-import System.IO ( hPutStr, hPutStrLn, stderr )
+import System.IO ( stderr )
 import Darcs.Util.File ( getRecursiveContentsFullPath )
 import Darcs.Util.ByteString ( isGZFile, gzDecompress )
 import Darcs.Util.Global ( getCRCWarnings, resetCRCWarnings )
@@ -46,22 +45,26 @@ import Darcs.Repository ( Repository, withRepository, RepoJob(..), repoCache )
 -- get at the caches and inspect them directly)
 -- Could move the relevant code into Darcs.Repository modules
 -- but it doesn't really seem worth it.
-import Darcs.Repository.Cache ( Cache(..), writable, isThisRepo,
-                                hashedFilePath, allHashedDirs )
+import Darcs.Repository.Cache
+    ( allHashedDirs
+    , cacheEntries
+    , hashedFilePath
+    , isThisRepo
+    , writable
+    )
 import Darcs.Util.Lock ( gzWriteAtomicFilePSs )
 import Darcs.UI.Commands
     ( DarcsCommand(..), withStdOpts, nodefaults, amInRepository
     , putInfo, putVerbose
     )
 import Darcs.UI.Completion ( noArgs )
-import Darcs.UI.Options ( (^), oid, odesc, ocheck, onormalise, defaultFlags, (?) )
+import Darcs.UI.Options ( (^), oid, odesc, ocheck, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Util.Path ( AbsolutePath )
 import Darcs.UI.Flags ( DarcsFlag, useCache )
-import Darcs.Util.Text ( formatText )
-import Darcs.Util.Printer ( text )
+import Darcs.Util.Printer ( Doc, ($$), formatText, hPutDocLn, pathlist, text )
 
-gzcrcsHelp :: String
+gzcrcsHelp :: Doc
 gzcrcsHelp = formatText 80
     [ "Versions of darcs >=1.0.4 and <2.2.0 had a bug that caused compressed "
       ++ "files with bad CRCs (but valid data) to be written out. CRCs were "
@@ -94,22 +97,20 @@ doCRCWarnings verbose = do
     files <- getCRCWarnings
     resetCRCWarnings
     unless (null files) $ do
-        hPutStr stderr . formatText 80 $
-            [""
-            , "Warning: CRC errors found. These are probably harmless but "
+        hPutDocLn stderr . formatText 80 $
+            [ "Warning: CRC errors found. These are probably harmless but "
               ++ "should be repaired. See 'darcs gzcrcs --help' for more "
               ++ "information."
-            , ""
             ]
         when verbose $
-            hPutStrLn stderr . unlines $
-                "The following corrupt files were found:" : files
+            hPutDocLn stderr $
+                text "The following corrupt files were found:" $$ pathlist files
 
 gzcrcsDescription :: String
 gzcrcsDescription = "Check or repair the CRCs of compressed files in the "
                     ++ "repository."
 
-gzcrcs :: DarcsCommand [DarcsFlag]
+gzcrcs :: DarcsCommand
 gzcrcs = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "gzcrcs"
@@ -125,7 +126,6 @@ gzcrcs = DarcsCommand
     , commandBasicOptions = odesc gzcrcsBasicOpts
     , commandDefaults = defaultFlags gzcrcsOpts
     , commandCheckOptions = ocheck gzcrcsOpts
-    , commandParseOptions = onormalise gzcrcsOpts
     }
   where
     gzcrcsBasicOpts = O.gzcrcsActions ^ O.justThisRepo ^ O.repoDir
@@ -143,7 +143,7 @@ gzcrcs' action opts repo = do
     -- pre-filter the list of locs to check and then decide whether to print
     -- the message up front.
     warnRelatedRepos <- newIORef $ not isJustThisRepo
-    let Ca locs = repoCache repo
+    let locs = cacheEntries $ repoCache repo
     (_, Any checkFailed) <- runWriterT $ forM_ locs $ \loc ->
         unless (isJustThisRepo && not (isThisRepo loc)) $ do
             let isWritable = writable loc
