@@ -15,81 +15,99 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-module Darcs.UI.Commands.Help (
- helpCmd,
- commandControlList, environmentHelp,          -- these are for preproc.hs
- printVersion,
- listAvailableCommands ) where
+{-# LANGUAGE OverloadedStrings #-}
+module Darcs.UI.Commands.Help
+    ( helpCmd
+    , commandControlList
+    , printVersion
+    , listAvailableCommands
+    ) where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Darcs.UI.Flags
-    ( DarcsFlag
-    , environmentHelpEmail
-    , environmentHelpSendmail
-    )
-import Darcs.UI.Options.Markdown ( optionsMarkdown )
+import Control.Arrow ( (***) )
+import Data.Char ( isAlphaNum, toLower, toUpper )
+import Data.Either ( partitionEithers )
+import Data.List ( groupBy, intercalate, lookup, nub )
+import System.Exit ( exitSuccess )
+import Version ( version )
+
+import Darcs.Patch.Match ( helpOnMatchers )
+import Darcs.Repository.Prefs ( environmentHelpHome, prefsFilesHelp )
+
 import Darcs.UI.Commands
     ( CommandArgs(..)
     , CommandControl(..)
-    , normalCommand
     , DarcsCommand(..)
-    , WrappedCommand(..)
-    , wrappedCommandName
+    , commandName
     , disambiguateCommands
     , extractCommands
     , getSubcommands
     , nodefaults
+    , normalCommand
     )
 import Darcs.UI.External ( viewDoc )
-import Darcs.UI.Usage
-    ( getCommandHelp
-    , usage
-    , subusage
-    )
-import Darcs.Util.Lock ( environmentHelpTmpdir, environmentHelpKeepTmpdir
-                       , environmentHelpLocks )
-import Darcs.Patch.Match ( helpOnMatchers )
-import Darcs.Repository.Prefs ( environmentHelpHome, prefsFilesHelp )
-import Darcs.Util.Ssh ( environmentHelpSsh, environmentHelpScp, environmentHelpSshPort )
-import Darcs.Util.File ( withCurrentDirectory )
-import Darcs.Util.Path ( AbsolutePath )
-import Control.Arrow ( (***) )
-import Data.Char ( isAlphaNum, toLower, toUpper )
-import Data.Either ( partitionEithers )
-import Data.List ( groupBy, isPrefixOf, intercalate, nub, lookup )
-import Darcs.Util.English ( andClauses )
-import Darcs.Util.Printer (text, vcat, vsep, ($$), empty)
-import Darcs.Util.Printer.Color ( environmentHelpColor, environmentHelpEscape, environmentHelpEscapeWhite )
-import System.Exit ( exitSuccess )
-import Version ( version )
-import Darcs.Util.Download ( environmentHelpProxy, environmentHelpProxyPassword )
-import Darcs.Util.Workaround ( getCurrentDirectory )
-import Darcs.UI.Options ( defaultFlags, ocheck, onormalise, oid )
+import Darcs.UI.Flags ( DarcsFlag, environmentHelpEmail, environmentHelpSendmail )
+import Darcs.UI.Options ( defaultFlags, ocheck, oid )
+import Darcs.UI.Options.Markdown ( optionsMarkdown )
 import qualified Darcs.UI.TheCommands as TheCommands
+import Darcs.UI.Usage ( getCommandHelp, getSuperCommandHelp, subusage, usage )
+
+import Darcs.Util.Download ( environmentHelpProxy, environmentHelpProxyPassword )
+import Darcs.Util.English ( andClauses )
+import Darcs.Util.File ( withCurrentDirectory )
+import Darcs.Util.Lock
+    ( environmentHelpKeepTmpdir
+    , environmentHelpLocks
+    , environmentHelpTmpdir
+    )
+import Darcs.Util.Path ( AbsolutePath )
+import Darcs.Util.Printer
+    ( Doc
+    , empty
+    , formatWords
+    , quoted
+    , renderString
+    , text
+    , vcat
+    , vsep
+    , ($$)
+    , ($+$)
+    , (<+>)
+    )
+import Darcs.Util.Printer.Color
+    ( environmentHelpColor
+    , environmentHelpEscape
+    , environmentHelpEscapeWhite
+    )
+import Darcs.Util.Ssh
+    ( environmentHelpScp
+    , environmentHelpSsh
+    , environmentHelpSshPort
+    )
+import Darcs.Util.Workaround ( getCurrentDirectory )
+
 
 helpDescription :: String
 helpDescription = "Display help about darcs and darcs commands."
 
-helpHelp :: String
-helpHelp =
- "Without arguments, `darcs help` prints a categorized list of darcs\n" ++
- "commands and a short description of each one.  With an extra argument,\n" ++
- "`darcs help foo` prints detailed help about the darcs command foo.\n"
+helpHelp :: Doc
+helpHelp = formatWords
+  [ "Without arguments, `darcs help` prints a categorized list of darcs"
+  , "commands and a short description of each one.  With an extra argument,"
+  , "`darcs help foo` prints detailed help about the darcs command foo."
+  ]
 
 -- | Starting from a list of 'CommandControl's, unwrap one level
 -- to get a list of command names together with their subcommands.
 unwrapTree :: [CommandControl] -> [(String, [CommandControl])]
-unwrapTree cs = [ (wrappedCommandName c, subcmds c) | CommandData c <- cs ]
-  where
-    subcmds (WrappedCommand sc) = getSubcommands sc
+unwrapTree cs = [ (commandName c, getSubcommands c) | CommandData c <- cs ]
 
 -- | Given a list of (normal) arguments to the help command, produce a list
 -- of possible completions for the next (normal) argument.
 completeArgs :: [String] -> [String]
 completeArgs [] = map fst (unwrapTree commandControlList) ++ extraArgs where
-  extraArgs = [ "manpage", "markdown", "patterns", "environment" ]
+  extraArgs = [ "patterns", "preferences", "environment", "manpage", "markdown" ]
 completeArgs (arg:args) = exploreTree arg args commandControlList where
   exploreTree cmd cmds cs =
     case lookup cmd (unwrapTree cs) of
@@ -98,7 +116,7 @@ completeArgs (arg:args) = exploreTree arg args commandControlList where
         [] -> map fst (unwrapTree cs')
         sub:cmds' -> exploreTree sub cmds' cs'
 
-help :: DarcsCommand [DarcsFlag]
+help :: DarcsCommand
 help = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "help"
@@ -114,25 +132,29 @@ help = DarcsCommand
     , commandBasicOptions = []
     , commandDefaults = defaultFlags oid
     , commandCheckOptions = ocheck oid
-    , commandParseOptions = onormalise oid
     }
 
 helpCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-helpCmd _ _ ["manpage"] = putStr $ unlines manpageLines
-helpCmd _ _ ["markdown"] = putStr $ unlines markdownLines
-helpCmd _ _ ["patterns"] = viewDoc $ text $ unlines helpOnMatchers
+helpCmd _ _ ["manpage"] = viewDoc manpage
+helpCmd _ _ ["markdown"] = viewDoc $ vcat $ map text markdownLines
+helpCmd _ _ ["patterns"] = viewDoc $ vcat $ map text helpOnMatchers
+helpCmd _ _ ["preferences"] =
+    viewDoc $ header $+$ vcat (map render prefsFilesHelp)
+  where
+    header = "Preference Files" $$
+             "================"
+    render (f, h) =
+      let item = "_darcs/prefs/" ++ f in
+        text item $$ text (replicate (length item) '-') $$ text h
 helpCmd _ _ ("environment":vs_) =
-    viewDoc $ header $$
-              vsep (map render known) $$
-              footer
+    viewDoc $ vsep (header : map render known) $+$ footer
   where
     header | null known = empty
-           | otherwise = text "Environment Variables" $$
-                         text "====================="
+           | otherwise = "Environment Variables" $$
+                         "====================="
 
     footer | null unknown = empty
-           | otherwise = text "" $$
-                         text ("Unknown environment variables: "
+           | otherwise = text ("Unknown environment variables: "
                                ++ intercalate ", " unknown)
 
     render (ks, ds) = text (andClauses ks ++ ":") $$
@@ -156,12 +178,12 @@ helpCmd _ _ (cmd:args) =
          Left err -> fail err
          Right (cmds,as) ->
              let msg = case cmds of
-                         CommandOnly c       -> getCommandHelp Nothing  c
+                         CommandOnly c       -> getCommandHelp Nothing c
                          SuperCommandOnly c  ->
                           if null as then
-                            getCommandHelp Nothing  c
+                            getSuperCommandHelp c
                           else
-                            text $ "Invalid subcommand!\n\n" ++ subusage c
+                            "Invalid subcommand!" $+$ subusage c
                          SuperCommandSub c s -> getCommandHelp (Just c) s
              in viewDoc $ msg
 
@@ -169,9 +191,9 @@ listAvailableCommands :: IO ()
 listAvailableCommands =
     do here <- getCurrentDirectory
        is_valid <- mapM
-                   (\(WrappedCommand c)-> withCurrentDirectory here $ commandPrereq c [])
+                   (\c -> withCurrentDirectory here $ commandPrereq c [])
                    (extractCommands commandControlList)
-       putStr $ unlines $ map (wrappedCommandName . fst) $
+       putStr $ unlines $ map (commandName . fst) $
                 filter (isRight.snd) $
                 zip (extractCommands commandControlList) is_valid
        putStrLn "--help"
@@ -214,7 +236,7 @@ environmentHelp = [
  environmentHelpProxyPassword,
  environmentHelpTimeout]
 
--- | This module is responsible for emitting a darcs "man-page", a
+-- | This function is responsible for emitting a darcs "man-page", a
 -- reference document used widely on Unix-like systems.  Manpages are
 -- primarily used as a quick reference, or "memory jogger", so the
 -- output should be terser than the user manual.
@@ -222,11 +244,9 @@ environmentHelp = [
 -- Before modifying the output, please be sure to read the man(7) and
 -- man-pages(7) manpages, as these respectively describe the relevant
 -- syntax and conventions.
-
--- | The lines of the manpage to be printed.
-manpageLines :: [String]
-manpageLines = [
- ".TH DARCS 1 \"" ++ version ++ "\"",
+manpage :: Doc
+manpage = vcat [
+ ".TH DARCS 1" <+> quoted version,
  ".SH NAME",
  "darcs \\- an advanced revision control system",
  ".SH SYNOPSIS",
@@ -234,44 +254,22 @@ manpageLines = [
  "",
  "Where the", ".I commands", "and their respective", ".I arguments", "are",
  "",
- unlines synopsis,
+ synopsis,
  ".SH DESCRIPTION",
- -- FIXME: this is copy-and-pasted from darcs.cabal, so
- -- it'll get out of date as people forget to maintain
- -- both in sync.
- "Darcs is a free, open source revision control",
- "system. It is:",
- ".TP 3", "\\(bu",
- "Distributed: Every user has access to the full",
- "command set, removing boundaries between server and",
- "client or committer and non\\(hycommitters.",
- ".TP", "\\(bu",
- "Interactive: Darcs is easy to learn and efficient to",
- "use because it asks you questions in response to",
- "simple commands, giving you choices in your work",
- "flow. You can choose to record one change in a file,",
- "while ignoring another. As you update from upstream,",
- "you can review each patch name, even the full `diff'",
- "for interesting patches.",
- ".TP", "\\(bu",
- "Smart: Originally developed by physicist David",
- "Roundy, darcs is based on a unique algebra of",
- "patches.",
- "This smartness lets you respond to changing demands",
- "in ways that would otherwise not be possible. Learn",
- "more about spontaneous branches with darcs.",
+ description,
  ".SH OPTIONS",
  "Different options are accepted by different Darcs commands.",
  "Each command's most important options are listed in the",
  ".B COMMANDS",
  "section.  For a full list of all options accepted by",
  "a particular command, run `darcs", ".I command", "\\-\\-help'.",
- ".SS " ++ escape (unlines helpOnMatchers), -- FIXME: this is a kludge.
+ ".SS " <> vcat (map text helpOnMatchers),
  ".SH COMMANDS",
- unlines commands,
- unlines environment,
+ commands,
+ ".SH ENVIRONMENT",
+ environment,
  ".SH FILES",
- unlines prefFiles,
+ prefFiles,
  ".SH BUGS",
  "At http://bugs.darcs.net/ you can find a list of known",
  "bugs in Darcs.  Unknown bugs can be reported at that",
@@ -294,43 +292,52 @@ manpageLines = [
       -- | A synopsis line for each command.  Uses 'foldl' because it is
       -- necessary to avoid blank lines from Hidden_commands, as groff
       -- translates them into annoying vertical padding (unlike TeX).
-      synopsis :: [String]
-      synopsis = foldl iter [] commandControlList
-          where iter :: [String] -> CommandControl -> [String]
+      synopsis :: Doc
+      synopsis = foldl iter mempty commandControlList
+          where iter :: Doc -> CommandControl -> Doc
                 iter acc (GroupName _) = acc
                 iter acc (HiddenCommand _) = acc
-                iter acc (CommandData (WrappedCommand c@SuperCommand {})) =
-                    acc ++ concatMap
+                iter acc (CommandData (c@SuperCommand {})) =
+                    acc $$ vcat (map
                             (render (commandName c ++ " "))
-                            (extractCommands (commandSubCommands c))
-                iter acc (CommandData c) = acc ++ render "" c
-                render :: String -> WrappedCommand -> [String]
-                render prefix (WrappedCommand c) =
-                    [".B darcs " ++ prefix ++ commandName c] ++
-                    map mangle_args (commandExtraArgHelp c) ++
+                            (extractCommands (commandSubCommands c)))
+                iter acc (CommandData c) = acc $$ render "" c
+                render :: String -> DarcsCommand -> Doc
+                render prefix c =
+                    ".B darcs " <> text prefix <> text (commandName c) $$
+                    vcat (map (text.mangle_args) (commandExtraArgHelp c)) $$
                     -- In the output, we want each command to be on its own
                     -- line, but we don't want blank lines between them.
-                    -- AFAICT this can only be achieved with the .br
-                    -- directive, which is probably a GNUism.
-                    [".br"]
+                    ".br"
 
       -- | As 'synopsis', but make each group a subsection (.SS), and
       -- include the help text for each command.
-      commands :: [String]
-      commands = foldl iter [] commandControlList
-          where iter :: [String] -> CommandControl -> [String]
-                iter acc (GroupName x) = acc ++ [".SS \"" ++ x ++ "\""]
-                iter acc (HiddenCommand _) = acc
-                iter acc (CommandData (WrappedCommand c@SuperCommand {})) =
-                    acc ++ concatMap
-                            (render (commandName c ++ " "))
-                            (extractCommands (commandSubCommands c))
-                iter acc (CommandData c) = acc ++ render "" c
-                render :: String -> WrappedCommand -> [String]
-                render prefix (WrappedCommand c) =
-                    [".B darcs " ++ prefix ++ commandName c] ++
-                    map mangle_args (commandExtraArgHelp c) ++
-                    [".RS 4", escape $ commandHelp c, ".RE"]
+      commands :: Doc
+      commands = vsep $ map iter commandControlList
+          where iter :: CommandControl -> Doc
+                iter (GroupName x) = ".SS" <+> quoted x
+                iter (HiddenCommand _) = mempty
+                iter (CommandData (c@SuperCommand {})) =
+                  vcat
+                  [ ".B darcs " <> text (commandName c)
+                  , text (mangle_args "subcommand")
+                  , ".RS 4"
+                  , commandHelp c
+                  , ".RE"
+                  ]
+                  $+$
+                  vsep (map (render (commandName c ++ " "))
+                    (extractCommands (commandSubCommands c)))
+                iter (CommandData c) = render "" c
+                render :: String -> DarcsCommand -> Doc
+                render prefix c =
+                  vcat
+                  [ ".B darcs " <> text prefix <> text (commandName c)
+                  , vcat (map (text.mangle_args) (commandExtraArgHelp c))
+                  , ".RS 4"
+                  , commandHelp c
+                  , ".RE"
+                  ]
 
       -- | Now I'm showing off: mangle the extra arguments of Darcs commands
       -- so as to use the ideal format for manpages, italic words and roman
@@ -338,32 +345,37 @@ manpageLines = [
       mangle_args :: String -> String
       mangle_args s =
           ".RI " ++ unwords (map show (groupBy cmp $ map toLower $ gank s))
-              where cmp x y = not $ xor (isAlphaNum x) (isAlphaNum y)
-                    xor x y = (x && not y) || (y && not x)
+              where cmp x y = isAlphaNum x == isAlphaNum y
                     gank (' ':'o':'r':' ':xs) = '|' : gank xs
                     gank (x:xs) = x : gank xs
                     gank [] = []
 
-      environment :: [String]
-      environment = ".SH ENVIRONMENT" : concat
-                    [(".SS \"" ++ andClauses ks ++ "\"") : map escape ds
+      environment :: Doc
+      environment = vcat $ concat
+                    [(".SS" <+> quoted (andClauses ks)) : map text ds
                      | (ks, ds) <- environmentHelp]
 
-      escape :: String -> String
-      escape = minus . bs       -- Order is important
-        where
-          minus      = replace "-"     "\\-"
-          bs         = replace "\\"    "\\\\"
+      prefFiles :: Doc
+      prefFiles = vcat $ map go prefsFilesHelp
+        where go (f,h) = ".SS" <+> quoted("_darcs/prefs/" <> f) $$ text h
 
-          replace :: Eq a => [a] -> [a] -> [a] -> [a]
-          replace _ _ [] = []
-          replace find repl s =
-              if find `isPrefixOf` s
-                  then repl ++ replace find repl (drop (length find) s)
-                  else head s : replace find repl (tail s)
-
-      prefFiles = concatMap go prefsFilesHelp
-        where go (f,h) = [".SS \"_darcs/prefs/" ++ f ++ "\"", escape h]
+      description = vcat
+        [ "Unlike conventional revision control systems, Darcs is based on tracking"
+        , "changes, rather than versions: it can and does automatically re-order"
+        , "independent changes when needed. This means that in Darcs the state of"
+        , "a repository should be regarded as a"
+        , ".I set of patches"
+        , "rather than a"
+        , ".I sequence of versions."
+        , ""
+        , "Another distinguishing feature of darcs is that most commands are"
+        , "interactive by default. For instance, `darcs record' (the equivalent of"
+        , "what is usually called `commit') presents you with"
+        , "each unrecorded change and asks you whether it should be included in"
+        , "the patch to be recorded. Similarly, `darcs push' and `darcs pull'"
+        , "present you with each patch, allowing you to select which patches to"
+        , "push or pull."
+        ]
 
 markdownLines :: [String]
 markdownLines =
@@ -389,18 +401,18 @@ markdownLines =
       iter :: [String] -> CommandControl -> [String]
       iter acc (GroupName x) = acc ++ ["## " ++ x, ""]
       iter acc (HiddenCommand _) = acc
-      iter acc (CommandData (WrappedCommand c@SuperCommand {})) =
+      iter acc (CommandData (c@SuperCommand {})) =
           acc ++ concatMap
                   (render (commandName c ++ " "))
                   (extractCommands (commandSubCommands c))
       iter acc (CommandData c) = acc ++ render "" c
-      render :: String -> WrappedCommand -> [String]
-      render prefix (WrappedCommand c) =
+      render :: String -> DarcsCommand -> [String]
+      render prefix c =
           [ "### " ++ prefix ++ commandName c
           , "", "darcs " ++ prefix ++ commandName c ++ " [OPTION]... " ++
           unwords (commandExtraArgHelp c)
           , "", commandDescription c
-          , "", commandHelp c
+          , "", renderString (commandHelp c)
           , "Options:", optionsMarkdown $ commandBasicOptions c
           , if null opts2 then ""
              else unlines ["Advanced Options:", optionsMarkdown opts2]
