@@ -15,25 +15,27 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE PatternGuards #-}
-
 module Darcs.UI.Commands.Log
-    ( changes, log
-    , changelog, getLogInfo
+    ( changes
+    , log
+    , changelog
+    , logInfoFL
+    , simpleLogInfo -- for darcsden
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Data.List ( intersect, sort, nub, find )
-import Data.Maybe ( fromMaybe, fromJust, isJust )
+import Data.List ( intersect, find )
+import Data.List.Ordered ( nubSort )
+import Data.Maybe ( fromMaybe, isJust )
 import Control.Arrow ( second )
 import Control.Exception ( catch, IOException )
 import Control.Monad.State.Strict
 
 import Darcs.UI.PrintPatch ( showFriendly )
-import Darcs.Patch.PatchInfoAnd ( fmapFLPIAP, hopefullyM, info )
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAndG, fmapFLPIAP, hopefullyM, info )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, commandAlias, findRepository )
+import Darcs.UI.Commands.Util ( matchRange )
 import Darcs.UI.Completion ( knownFileArgs )
 import Darcs.UI.External ( viewDocWith )
 import Darcs.UI.Flags
@@ -41,68 +43,103 @@ import Darcs.UI.Flags
     , changesReverse, onlyToFiles
     , useCache, maxCount, hasXmlOutput
     , verbosity, withContext, isInteractive, verbose
-    , fixSubPaths, getRepourl )
-import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, parseFlags, (?) )
+    , getRepourl, pathSetFromArgs )
+import Darcs.UI.Options ( (^), odesc, ocheck, defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
-import Darcs.Util.Path ( SubPath(), toFilePath,
-                    fp2fn, fn2fp, normPath, AbsolutePath, simpleSubPath )
-import Darcs.Repository ( PatchSet, PatchInfoAnd,
+import Darcs.Util.Path
+    ( SubPath
+    , AbsolutePath
+    , simpleSubPath
+    , AnchoredPath
+    , floatSubPath
+    , displayPath
+    )
+import Darcs.Repository ( PatchInfoAnd,
                           withRepositoryLocation, RepoJob(..),
                           readRepo, unrecordedChanges,
                           withRepoLockCanFail )
 import Darcs.Repository.Flags ( UseIndex(..), ScanKnown(..), DiffAlgorithm(MyersDiff) )
 import Darcs.Util.Lock ( withTempDir )
-import Darcs.Patch.Set ( PatchSet(..), patchSet2RL )
-import Darcs.Patch.Conflict ( Conflict, CommuteNoConflicts )
+import Darcs.Patch.Set ( PatchSet, patchSet2RL, Origin )
 import Darcs.Patch.Format ( PatchListFormat )
-import Darcs.Patch.FileHunk ( IsHunk )
 import Darcs.Patch.Info ( toXml, toXmlShort, showPatchInfo, displayPatchInfo, escapeXML, PatchInfo )
-import Darcs.Patch.Depends ( findCommonWithThem )
-import Darcs.Patch.Bundle( contextPatches )
-import Darcs.Patch.Prim ( PrimPatchBase )
+import Darcs.Patch.Ident ( PatchId )
+import Darcs.Patch.Invertible ( mkInvertible )
+import Darcs.Patch.Depends ( contextPatches )
 import Darcs.Patch.Show ( ShowPatch, ShowPatchFor(..) )
 import Darcs.Patch.TouchesFiles ( lookTouch )
-import Darcs.Patch.Type ( PatchType(PatchType) )
-import Darcs.Patch.Apply ( Apply, ApplyState )
-import Darcs.Patch ( IsRepoType, invert, xmlSummary, description,
-                     effectOnFilePaths, listTouchedFiles, showPatch )
-import Darcs.Patch.Named.Wrapped ( (:~:)(..), getdeps )
+import Darcs.Patch.Apply ( ApplyState )
+import Darcs.Patch ( PrimPatchBase(..), invert, xmlSummary, description,
+                     effectOnPaths, listTouchedFiles, showPatch )
+import Darcs.Patch.Named ( HasDeps, getdeps )
+import Darcs.Patch.Prim.Class ( PrimDetails )
+import Darcs.Patch.Summary ( Summary )
 import Darcs.Patch.Witnesses.Eq ( EqCheck(..) )
 import Darcs.Patch.Witnesses.Ordered
     ( FL(NilFL), RL(..), filterOutFLFL, filterRL,
-    reverseFL, (:>)(..), mapRL )
-import Darcs.Patch.Witnesses.Sealed ( Sealed2(..), unseal2, Sealed(..), seal2 )
+    reverseFL, (:>)(..), mapFL, mapRL )
+import Darcs.Patch.Witnesses.Sealed ( Sealed2(..), unseal2, Sealed(..) )
 import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 import Darcs.Patch.Match
     ( MatchFlag
-    , firstMatch
-    , secondMatch
+    , Matchable
+    , MatchableRP
     , matchAPatch
     , haveNonrangeMatch
-    , matchFirstPatchset
-    , matchSecondPatchset
     )
-import Darcs.Patch.Matchable ( Matchable )
-import Darcs.Util.Printer ( Doc, simplePrinters, (<+>), prefix, text, vcat,
-                 vsep, ($$), errorDoc, insertBeforeLastline, empty )
+import Darcs.Util.Printer
+    ( Doc
+    , ($$)
+    , (<+>)
+    , formatWords
+    , hsep
+    , insertBeforeLastline
+    , prefix
+    , simplePrinters
+    , text
+    , vcat
+    , vsep
+    )
 import Darcs.Util.Printer.Color ( fancyPrinters )
 import Darcs.Util.Progress ( setProgressMode, debugMessage )
-import Darcs.Util.URL ( isValidLocalPath )
 import Darcs.UI.SelectChanges ( viewChanges )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
 import Darcs.Repository.PatchIndex ( PatchFilter, maybeFilterPatches, attemptCreatePatchIndex )
 import Darcs.Util.Tree( Tree )
 
-logHelp :: String
-logHelp = unlines
- [ "The `darcs log` command lists patches of the current repository or,"
- , "with `--repo`, a remote repository.  Without options or arguments,"
- , "ALL patches will be listed."
- , ""]
- ++ logHelp'
- ++ logHelp''
+logHelp :: Doc
+logHelp = vsep $ map formatWords
+  [ [ "The `darcs log` command lists patches of the current repository or,"
+    , "with `--repo`, a remote repository.  Without options or arguments,"
+    , "ALL patches will be listed."
+    ]
+  , [ "When given files or directories paths as arguments, only patches which"
+    , "affect those paths are listed.  This includes patches that happened to"
+    , "files before they were moved or renamed."
+    ]
+  , [ "When given `--from-tag` or `--from-patch`, only patches since that tag"
+    , "or patch are listed.  Similarly, the `--to-tag` and `--to-patch`"
+    , "options restrict the list to older patches."
+    ]
+  , [ "The `--last` and `--max-count` options both limit the number of patches"
+    , "listed.  The former applies BEFORE other filters, whereas the latter"
+    , "applies AFTER other filters.  For example `darcs log foo.c"
+    , "--max-count 3` will print the last three patches that affect foo.c,"
+    , "whereas `darcs log --last 3 foo.c` will, of the last three"
+    , "patches, print only those that affect foo.c."
+    ]
+  , [ "Four output formats exist.  The default is `--human-readable`. The slightly"
+    , "different `--machine-readable` format enables to see patch dependencies in"
+    , "non-interactive mode. You can also select `--context`, which is an internal"
+    , "format that can be re-read by Darcs (e.g. `darcs clone --context`)."
+    ]
+  , [ "Finally, there is `--xml-output`, which emits valid XML... unless a the"
+    , "patch metadata (author, name or description) contains a non-ASCII"
+    , "character and was recorded in a non-UTF8 locale."
+    ]
+  ]
 
-log :: DarcsCommand [DarcsFlag]
+log :: DarcsCommand
 log = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "log"
@@ -118,7 +155,6 @@ log = DarcsCommand
     , commandBasicOptions = odesc logBasicOpts
     , commandDefaults = defaultFlags logOpts
     , commandCheckOptions = ocheck logOpts
-    , commandParseOptions = onormalise logOpts
     }
   where
     logBasicOpts
@@ -126,7 +162,7 @@ log = DarcsCommand
       ^ O.maxCount
       ^ O.onlyToFiles
       ^ O.changesFormat
-      ^ O.summary
+      ^ O.withSummary
       ^ O.changesReverse
       ^ O.possiblyRemoteRepo
       ^ O.repoDir
@@ -142,27 +178,26 @@ logCmd fps opts args
   | hasRemoteRepo opts = do
       (fs, es) <- remoteSubPaths args []
       if null es then
-        withTempDir "darcs.log" (\_ -> showLog opts $ maybeNotNull $ nub $ sort fs)
+        withTempDir "darcs.log"
+          (\_ -> showLog opts $ maybeNotNull $ nubSort $ map floatSubPath fs)
       else
         fail $ "For a remote repo I can only handle relative paths.\n"
             ++ "Invalid arguments: "++unwords es
   | null args = showLog opts Nothing
   | otherwise = do
-      fs <- fixSubPaths fps args
-      case fs of
-        [] -> putStrLn "No valid arguments were given, nothing to do."
-        _ -> do unless (isInteractive False opts)
-                 $ when (O.patchIndexNo ? opts == O.YesPatchIndex)
-                  $ withRepoLockCanFail (useCache ? opts)
-                    $ RepoJob (\repo -> readRepo repo >>= attemptCreatePatchIndex repo)
-                showLog opts $ Just $ nub $ sort fs
+      unless (isInteractive False opts)
+        $ when (O.patchIndexNo ? opts == O.YesPatchIndex)
+          $ withRepoLockCanFail (useCache ? opts)
+            $ RepoJob (\repo -> readRepo repo >>= attemptCreatePatchIndex repo)
+      paths <- pathSetFromArgs fps args
+      showLog opts paths
 
 maybeNotNull :: [a] -> Maybe [a]
 maybeNotNull [] = Nothing
 maybeNotNull xs = Just xs
 
 hasRemoteRepo :: [DarcsFlag] -> Bool
-hasRemoteRepo = maybe False (not . isValidLocalPath) . parseFlags O.possiblyRemoteRepo
+hasRemoteRepo = isJust . getRepourl
 
 remoteSubPaths :: [String] -> [String] -> IO ([SubPath],[String])
 remoteSubPaths [] es = return ([], es)
@@ -172,7 +207,7 @@ remoteSubPaths (arg:args) es = case simpleSubPath arg of
     (sps, es') <- remoteSubPaths args es
     return (sp:sps, es')
 
-showLog :: [DarcsFlag] -> Maybe [SubPath] -> IO ()
+showLog :: [DarcsFlag] -> Maybe [AnchoredPath] -> IO ()
 showLog opts files =
   let repodir = fromMaybe "." (getRepourl opts) in
   withRepositoryLocation (useCache ? opts) repodir $ RepoJob $ \repository -> do
@@ -186,9 +221,7 @@ showLog opts files =
   debugMessage "About to read the repository..."
   patches <- readRepo repository
   debugMessage "Done reading the repository."
-  let normfp = fn2fp . normPath . fp2fn
-      undoUnrecordedOnFPs = effectOnFilePaths (invert unrec)
-      recFiles = map normfp . undoUnrecordedOnFPs . map toFilePath <$> files
+  let recFiles = effectOnPaths (invert unrec) <$> files
       filtered_changes p =
           maybe_reverse <$>
           getLogInfo
@@ -199,80 +232,81 @@ showLog opts files =
               (maybeFilterPatches repository patches)
               p
   if isInteractive False opts
-    then do (fp_and_fs, _, _) <- filtered_changes patches
-            let fp = map fst fp_and_fs
-            viewChanges (logPatchSelOpts opts) fp
-    else do let header = if isJust files && hasXmlOutput opts
-                          then text $ "Changes to "++unwords (fromJust recFiles)++":\n"
-                          else empty
+    then do li <- filtered_changes patches
+            viewChanges (logPatchSelOpts opts) (map fst (liPatches li))
+    else do let header =
+                  case recFiles of
+                    Just fs | not (hasXmlOutput opts) ->
+                      let pathlist = map (text . displayPath) fs
+                      in hsep (text "Changes to" : pathlist) <> text ":" $$ text ""
+                    _ -> mempty
             debugMessage "About to print the patches..."
             let printers = if hasXmlOutput opts then simplePrinters else fancyPrinters
             ps <- readRepo repository -- read repo again to prevent holding onto
                                        -- values forced by filtered_changes
-            logOutput <- changelog opts ps `fmap` filtered_changes patches
-            viewDocWith printers $ header $$ logOutput
-  where maybe_reverse (xs,b,c) = if changesReverse ? opts
-                                 then (reverse xs, b, c)
-                                 else (xs, b, c)
-
-logHelp' :: String
-logHelp' = unlines
- [ "When given files or directories paths as arguments, only patches which"
- , "affect those paths are listed.  This includes patches that happened to"
- , "files before they were moved or renamed."
- , ""
- , "When given `--from-tag` or `--from-patch`, only patches since that tag"
- , "or patch are listed.  Similarly, the `--to-tag` and `--to-patch`"
- , "options restrict the list to older patches."
- , ""
- , "The `--last` and `--max-count` options both limit the number of patches"
- , "listed.  The former applies BEFORE other filters, whereas the latter"
- , "applies AFTER other filters.  For example `darcs log foo.c"
- , "--max-count 3` will print the last three patches that affect foo.c,"
- , "whereas `darcs log --last 3 foo.c` will, of the last three"
- , "patches, print only those that affect foo.c."
- , ""
- ]
-
-getLogInfo :: forall rt p wX wY
-                . (IsRepoType rt, Matchable p, ApplyState p ~ Tree)
-               => Maybe Int -> [MatchFlag] -> Bool
-               -> Maybe [FilePath]
-               -> PatchFilter rt p
-               -> PatchSet rt p wX wY
-               -> IO ( [(Sealed2 (PatchInfoAnd rt p), [FilePath])]
-                     , [(FilePath, FilePath)]
-                     , Maybe Doc )
-getLogInfo maxCountFlag matchFlags onlyToFilesFlag plain_fs patchFilter ps =
-    case (sp1s, sp2s) of
-      (Sealed p1s, Sealed p2s) ->
-          case findCommonWithThem p2s p1s of
-            _ :> us ->
-              let ps' = filterRL pf (reverseFL us) in
-                case plain_fs of
-                  Nothing -> return $ foldr (\x xs -> (x, []) -:- xs) ([], [], Nothing) $
-                    maybe id take maxCountFlag ps'
-                  Just fs -> let fs' = map (\x -> "./" ++ x) fs in do
-                    filterOutUnrelatedChanges <$> do
-                       ps'' <- patchFilter fs' ps'
-                       return $ filterPatchesByNames maxCountFlag fs' ps''
+            logOutput <- changelog opts (patchSet2RL ps) `fmap` filtered_changes patches
+            viewDocWith printers (header $$ logOutput)
   where
-        sp1s = if firstMatch matchFlags
-               then matchFirstPatchset matchFlags ps
-               else Sealed $ PatchSet NilRL NilRL
-        sp2s = if secondMatch matchFlags
-               then matchSecondPatchset matchFlags ps
-               else Sealed ps
-        pf = if haveNonrangeMatch (PatchType :: PatchType rt p) matchFlags
-             then matchAPatch matchFlags
-             else \_ -> True
+    maybe_reverse li@(LogInfo xs b c) =
+      if changesReverse ? opts then LogInfo (reverse xs) b c else li
 
-        filterOutUnrelatedChanges (pfs, renames, doc)
-          | onlyToFilesFlag = (map onlyRelated pfs, renames, doc)
-          | otherwise       = (pfs, renames, doc)
+data LogInfo p = LogInfo
+  { liPatches :: [(Sealed2 p, [AnchoredPath])]
+  , liRenames :: [(AnchoredPath, AnchoredPath)]
+  , liErrorMsg :: Maybe Doc
+  }
+
+mkLogInfo :: [Sealed2 p] -> LogInfo p
+mkLogInfo ps = LogInfo (map (,[]) ps) [] Nothing
+
+logInfoFL :: FL p wX wY -> LogInfo p
+logInfoFL = mkLogInfo . mapFL Sealed2
+
+matchNonrange :: (Matchable p, PatchId p ~ PatchInfo)
+              => [MatchFlag] -> RL p wA wB -> [Sealed2 p]
+matchNonrange matchFlags
+  | haveNonrangeMatch matchFlags = filterRL (matchAPatch matchFlags)
+  | otherwise = mapRL Sealed2
+
+simpleLogInfo :: ( MatchableRP p
+                 , ApplyState p ~ Tree
+                 )
+              => AnchoredPath
+              -> PatchFilter rt p
+              -> PatchSet rt p Origin wY
+              -> IO [Sealed2 (PatchInfoAnd rt p)]
+simpleLogInfo path pf ps =
+  map fst . liPatches <$> getLogInfo Nothing [] False (Just [path]) pf ps
+
+getLogInfo :: forall rt p wY.
+              ( MatchableRP p
+              , ApplyState p ~ Tree
+              )
+           => Maybe Int -> [MatchFlag] -> Bool
+           -> Maybe [AnchoredPath]
+           -> PatchFilter rt p
+           -> PatchSet rt p Origin wY
+           -> IO (LogInfo (PatchInfoAnd rt p))
+getLogInfo maxCountFlag matchFlags onlyToFilesFlag paths patchFilter ps =
+  case matchRange matchFlags ps of
+    Sealed2 range ->
+      let ps' = matchNonrange matchFlags (reverseFL range) in
+      case paths of
+        Nothing -> return $ mkLogInfo $ maybe id take maxCountFlag ps'
+        Just fs -> do
+          filterOutUnrelatedChanges <$> do
+            ps'' <- patchFilter fs ps'
+            return $ filterPatchesByNames maxCountFlag fs ps''
+  where
+        -- What we do here is somewhat unclean: we modify the contents of
+        -- our patches and throw out everything not related to our files.
+        -- This is okay because we only use the result for display.
+        filterOutUnrelatedChanges li
+          | onlyToFilesFlag = li { liPatches = map onlyRelated (liPatches li) }
+          | otherwise       = li
 
         onlyRelated (Sealed2 p, fs) =
-          (Sealed2 $ fmapFLPIAP (filterOutFLFL (unrelated fs)) (\_ -> ReflPatch) p, fs)
+          (Sealed2 $ fmapFLPIAP (filterOutFLFL (unrelated fs)) p, fs)
 
         unrelated fs p
           -- If the change does not affect the patches we are looking at,
@@ -288,80 +322,79 @@ getLogInfo maxCountFlag matchFlags onlyToFilesFlag plain_fs patchFilter ps =
 -- "Just n" -- returns at most n patches touching the file (starting from the
 -- beginning of the patch list).
 filterPatchesByNames
-    :: forall rt p
-     . (Matchable p, ApplyState p ~ Tree)
-    => Maybe Int -- ^ maxcount
-    -> [FilePath] -- ^ filenames
-    -> [Sealed2 (PatchInfoAnd rt p)] -- ^ patchlist
-    -> ([(Sealed2 (PatchInfoAnd rt p),[FilePath])], [(FilePath, FilePath)], Maybe Doc)
-filterPatchesByNames maxcount fns patches = removeNonRenames $
-    evalState (filterPatchesByNames' fns patches) (maxcount, initRenames) where
-        removeNonRenames (ps, renames, doc) = (ps, removeIds renames, doc)
+    :: forall rt p.
+       ( MatchableRP p
+       , ApplyState p ~ Tree
+       )
+    => Maybe Int                      -- ^ maxcount
+    -> [AnchoredPath]                 -- ^ paths
+    -> [Sealed2 (PatchInfoAnd rt p)]  -- ^ patches
+    -> LogInfo (PatchInfoAnd rt p)
+filterPatchesByNames maxcount paths patches = removeNonRenames $
+    evalState (filterPatchesByNamesM paths patches) (maxcount, initRenames) where
+        removeNonRenames li = li { liRenames = removeIds (liRenames li) }
         removeIds = filter $ uncurry (/=)
-        initRenames = map (\x -> (x, x)) fns
-        returnFinal = (\renames -> ([], renames, Nothing)) <$> gets snd
-        filterPatchesByNames' [] _ = returnFinal
-        filterPatchesByNames' _ [] = returnFinal
-        filterPatchesByNames' fs (s2hp@(Sealed2 hp) : ps) = do
+        initRenames = map (\x -> (x, x)) paths
+        returnFinal = (\renames -> LogInfo [] renames Nothing) <$> gets snd
+        filterPatchesByNamesM [] _ = returnFinal
+        filterPatchesByNamesM _ [] = returnFinal
+        filterPatchesByNamesM fs (s2hp@(Sealed2 hp) : ps) = do
             (count, renames) <- get
-            let stopNow = case count of
-                                Nothing -> False
-                                Just c -> c <= 0
-            if stopNow
-                then returnFinal
-                else case hopefullyM hp of
+            case count of
+                Just c | c <= 0 -> returnFinal
+                _ ->
+                  case hopefullyM hp of
                     Nothing -> do
                         let err = text "Can't find patches prior to:"
                                   $$ displayPatchInfo (info hp)
-                        return ([], renames, Just err)
+                        return (LogInfo [] renames (Just err))
                     Just p ->
-                        case lookTouch (Just renames) fs (invert p) of
+                        case lookTouch (Just renames) fs (invert (mkInvertible p)) of
                             (True, affected, [], renames') ->
-                                return ([(s2hp, affected)], renames', Nothing)
+                                return (LogInfo [(s2hp, affected)] renames' Nothing)
                             (True, affected, fs', renames') -> do
                                 let sub1Mb c = subtract 1 <$> c
                                 modify $ \(c, _) -> (sub1Mb c, renames')
-                                rest <- filterPatchesByNames' fs' ps
-                                return $ (s2hp, affected) -:- rest
+                                rest <- filterPatchesByNamesM fs' ps
+                                return $ rest {
+                                    liPatches = (s2hp, affected) : liPatches rest
+                                  }
                             (False, _, fs', renames') -> do
                                 modify $ second (const renames')
-                                filterPatchesByNames' fs' ps
-
--- | Note, lazy pattern matching is required to make functions like
--- filterPatchesByNames lazy in case you are only not interested in
--- the first element. E.g.:
---
---   let (fs, _, _) = filterPatchesByNames ...
-(-:-) :: a -> ([a],b,c) -> ([a],b,c)
-x -:- ~(xs,y,z) = (x:xs,y,z)
+                                filterPatchesByNamesM fs' ps
 
 changelog :: forall rt p wStart wX
-           . ( Apply p, ApplyState p ~ Tree, ShowPatch p, IsHunk p
-             , PrimPatchBase p, PatchListFormat p
-             , Conflict p, CommuteNoConflicts p
+           . ( ShowPatch p, PatchListFormat p
+             , Summary p, HasDeps p, PrimDetails (PrimOf p)
              )
-          => [DarcsFlag] -> PatchSet rt p wStart wX
-          -> ([(Sealed2 (PatchInfoAnd rt p), [FilePath])], [(FilePath, FilePath)], Maybe Doc)
+          => [DarcsFlag] -> RL (PatchInfoAndG rt p) wStart wX
+          -> LogInfo (PatchInfoAndG rt p)
           -> Doc
-changelog opts patchset (pis_and_fs, createdAsFs, mbErr)
-    | O.changesFormat ? opts == Just O.CountPatches = text $ show $ length pis_and_fs
-    | hasXmlOutput opts =
-         text "<changelog>"
-      $$ vcat created_as_xml
-      $$ vcat actual_xml_changes
-      $$ text "</changelog>"
-    | O.yes (O.summary ? opts) || verbose opts =
-        mbAppendErr $ vsep (map (number_patch change_with_summary) pis)
-    | otherwise = mbAppendErr $ vsep (map (number_patch description') pis)
-    where mbAppendErr = maybe id (\err -> ($$ err)) mbErr
-          change_with_summary :: Sealed2 (PatchInfoAnd rt p) -> Doc
+changelog opts patches li
+    | O.changesFormat ? opts == Just O.CountPatches =
+        text $ show $ length $ liPatches li
+    | hasXmlOutput opts = xml_changelog
+    | O.yes (O.withSummary ? opts) || verbose opts =
+        vsep (map (number_patch change_with_summary) ps) $$ mbErr
+    | otherwise = vsep (map (number_patch description') ps) $$ mbErr
+    where ps_and_fs = liPatches li
+          mbErr = fromMaybe mempty (liErrorMsg li)
+          change_with_summary :: Sealed2 (PatchInfoAndG rt p) -> Doc
           change_with_summary (Sealed2 hp)
             | Just p <- hopefullyM hp =
               if O.changesFormat ? opts == Just O.MachineReadable
                 then showPatch ForStorage p
-                else showFriendly (verbosity ? opts) (O.summary ? opts) p
+                else showFriendly (verbosity ? opts) (O.withSummary ? opts) p
             | otherwise = description hp $$ indent (text "[this patch is unavailable]")
 
+          xml_changelog = vcat
+            [ text "<changelog>"
+            , vcat xml_created_as
+            , vcat xml_changes
+            , text "</changelog>"
+            ]
+
+          xml_with_summary :: Sealed2 (PatchInfoAndG rt p) -> Doc
           xml_with_summary (Sealed2 hp) | Just p <- hopefullyM hp =
                     let
                       deps = getdeps p
@@ -375,71 +408,61 @@ changelog opts patchset (pis_and_fs, createdAsFs, mbErr)
                       insertBeforeLastline (toXml $ info hp) summary
           xml_with_summary (Sealed2 hp) = toXml (info hp)
           indent = prefix "    "
-          actual_xml_changes =
-            case O.summary ? opts of
-              O.YesSummary -> map xml_with_summary pis
-              O.NoSummary -> map (toXml . unseal2 info) pis
-
-          created_as_xml = map create createdAsFs where
+          xml_changes =
+            case O.withSummary ? opts of
+              O.YesSummary -> map xml_with_summary ps
+              O.NoSummary -> map (toXml . unseal2 info) ps
+          xml_created_as = map create (liRenames li) where
+            create :: (AnchoredPath, AnchoredPath) -> Doc
             create rename@(_, as) = createdAsXml (first_change_of as) rename
             -- We need to reorder the patches when they haven't been reversed
             -- already, so that we find the *first* patch that modifies a given
             -- file, not the last (by default, the list is oldest->newest).
             reorderer = if not (changesReverse ? opts) then reverse else id
-            oldest_first_pis_and_fs = reorderer pis_and_fs
+            oldest_first_ps_and_fs = reorderer ps_and_fs
             couldnt_find fn = error $ "Couldn't find first patch affecting " ++
-                                      fn ++ " in pis_and_fs"
-            mb_first_change_of fn = find ((fn `elem`) . snd) oldest_first_pis_and_fs
+                                      (displayPath fn) ++ " in ps_and_fs"
+            mb_first_change_of fn = find ((fn `elem`) . snd) oldest_first_ps_and_fs
             find_first_change_of fn = fromMaybe (couldnt_find fn)
               (mb_first_change_of fn)
+            first_change_of :: AnchoredPath -> PatchInfo
             first_change_of = unseal2 info . fst . find_first_change_of
+
           number_patch f x = if O.changesFormat ? opts == Just O.NumberPatches
                              then case get_number x of
                                   Just n -> text (show n++":") <+> f x
                                   Nothing -> f x
                              else f x
-          get_number :: Sealed2 (PatchInfoAnd re p) -> Maybe Int
-          get_number (Sealed2 y) = gn 1 (patchSet2RL patchset)
+
+          get_number :: Sealed2 (PatchInfoAndG re p) -> Maybe Int
+          get_number (Sealed2 y) = gn 1 patches
               where iy = info y
-                    gn :: Int -> RL (PatchInfoAnd rt p) wStart wY -> Maybe Int
+                    gn :: Int -> RL (PatchInfoAndG rt p) wStart wY -> Maybe Int
                     gn n (bs:<:b) | seq n (info b) == iy = Just n
                                   | otherwise = gn (n+1) bs
                     gn _ NilRL = Nothing
-          pis = map fst pis_and_fs
+          ps = map fst ps_and_fs
           description' = unseal2 description
 
-logHelp'' :: String
-logHelp'' = unlines
- [ "Four output formats exist.  The default is `--human-readable`. The slightly"
- , "different `--machine-readable` format enables to see patch dependencies in"
- , "non-interactive mode. You can also select `--context`, which is an internal"
- , "format that can be re-read by Darcs (e.g. `darcs clone --context`)."
- , ""
- , "Finally, there is `--xml-output`, which emits valid XML... unless a the"
- , "patch metadata (author, name or description) contains a non-ASCII"
- , "character and was recorded in a non-UTF8 locale."
- ]
-
 logContext :: [DarcsFlag] -> IO ()
 logContext opts = do
   let repodir = fromMaybe "." $ getRepourl opts
   withRepositoryLocation (useCache ? opts) repodir $ RepoJob $ \repository -> do
-      (_ :> ps') <- contextPatches `fmap` readRepo repository
-      let pis = mapRL seal2 ps'
+      (_ :> ps) <- contextPatches `fmap` readRepo repository
       let header = text "\nContext:\n"
-      let logOutput = maybe (vsep $ map (unseal2 (showPatchInfo ForStorage . info)) pis) errorDoc Nothing
-      viewDocWith simplePrinters $ header $$ logOutput
+      viewDocWith simplePrinters $ vsep
+          (header : mapRL (showPatchInfo ForStorage . info) ps)
 
 -- | changes is an alias for log
-changes :: DarcsCommand [DarcsFlag]
+changes :: DarcsCommand
 changes = commandAlias "changes" Nothing log
 
-createdAsXml :: PatchInfo -> (String, String) -> Doc
+createdAsXml :: PatchInfo -> (AnchoredPath, AnchoredPath) -> Doc
 createdAsXml pinfo (current, createdAs) =
     text "<created_as current_name='"
-       <> escapeXML current
+       <> escapeXML (displayPath current)
        <> text "' original_name='"
-       <> escapeXML createdAs
+       <> escapeXML (displayPath createdAs)
        <> text "'>"
     $$    toXml pinfo
     $$    text "</created_as>"
@@ -450,6 +473,6 @@ logPatchSelOpts flags = S.PatchSelectionOptions
     , S.matchFlags = parseFlags O.matchSeveralOrRange flags
     , S.interactive = isInteractive False flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
-    , S.summary = O.summary ? flags
+    , S.withSummary = O.withSummary ? flags
     , S.withContext = withContext ? flags
     }
