@@ -19,34 +19,36 @@
 
 module Darcs.UI.Commands.MarkConflicts ( markconflicts ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import System.Exit ( exitSuccess )
 import Data.List.Ordered ( nubSort, isect )
-import Data.Maybe ( fromJust )
-import Control.Monad ( when, unless )
-import Control.Exception ( catch, IOException )
+import Control.Monad ( when, unless, void )
 
 import Darcs.Util.Prompt ( promptYorn )
 import Darcs.Util.SignalHandler ( withSignalsBlocked )
-import Darcs.Util.Path ( AbsolutePath, SubPath, toFilePath, simpleSubPath )
+import Darcs.Util.Path ( AbsolutePath, AnchoredPath, anchorPath )
 import Darcs.Util.Printer
-    ( Doc, putDocLnWith, text, redText, debugDocLn, vsep, (<+>), ($$) )
+    ( Doc, pathlist, putDocLnWith, text, redText, debugDocLn, vsep, (<+>), ($$) )
 import Darcs.Util.Printer.Color ( fancyPrinters )
-import Darcs.Util.Text ( pathlist )
 
 import Darcs.UI.Commands
-    ( DarcsCommand(..), withStdOpts, nodefaults, amInHashedRepository, putInfo )
+    ( DarcsCommand(..)
+    , withStdOpts
+    , nodefaults
+    , amInHashedRepository
+    , putInfo
+    , putFinished
+    )
 import Darcs.UI.Commands.Util ( filterExistingPaths )
 import Darcs.UI.Completion ( knownFileArgs )
 import Darcs.UI.Flags
     ( DarcsFlag, diffingOpts, verbosity, dryRun, umask
-    , useCache, fixSubPaths )
-import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, (?) )
+    , useCache, pathSetFromArgs )
+import Darcs.UI.Options ( (^), odesc, ocheck, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 
-import Darcs.Repository.Flags ( UpdateWorking (..) )
+import Darcs.Repository.Flags ( UpdatePending (..) )
 import Darcs.Repository
     ( withRepoLock
     , RepoJob(..)
@@ -55,12 +57,16 @@ import Darcs.Repository
     , readRepo
     , unrecordedChanges )
 
-import Darcs.Patch ( invert, listTouchedFiles, effectOnFilePaths )
+import Darcs.Patch ( invert, listTouchedFiles, effectOnPaths )
 import Darcs.Patch.Show
 import Darcs.Patch.TouchesFiles ( chooseTouching )
 import Darcs.Patch.Witnesses.Ordered ( FL(..), mapFL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed) )
-import Darcs.Repository.Resolution ( patchsetConflictResolutions )
+import Darcs.Repository.Resolution
+    ( StandardResolution(..)
+    , patchsetConflictResolutions
+    , warnUnmangled
+    )
 
 -- * The mark-conflicts command
 
@@ -68,8 +74,8 @@ markconflictsDescription :: String
 markconflictsDescription =
  "Mark unresolved conflicts in working tree, for manual resolution."
 
-markconflictsHelp :: String
-markconflictsHelp = unlines
+markconflictsHelp :: Doc
+markconflictsHelp = text $ unlines
  ["Darcs requires human guidance to unify changes to the same part of a"
  ,"source file.  When a conflict first occurs, darcs will add the"
  ,"initial state and both choices to the working tree, delimited by the"
@@ -94,7 +100,7 @@ markconflictsHelp = unlines
  ,"You will be prompted for confirmation before this takes place."
  ]
 
-markconflicts :: DarcsCommand [DarcsFlag]
+markconflicts :: DarcsCommand
 markconflicts = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "mark-conflicts"
@@ -110,7 +116,6 @@ markconflicts = DarcsCommand
     , commandBasicOptions = odesc markconflictsBasicOpts
     , commandDefaults = defaultFlags markconflictsOpts
     , commandCheckOptions = ocheck markconflictsOpts
-    , commandParseOptions = onormalise markconflictsOpts
     }
   where
     markconflictsBasicOpts
@@ -123,9 +128,10 @@ markconflicts = DarcsCommand
 
 markconflictsCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 markconflictsCmd fps opts args = do
-  paths <- if null args then return Everything else sps2ps <$> fixSubPaths fps args -- Applicative IO
+  paths <- maybeToOnly <$> pathSetFromArgs fps args
   debugDocLn $ "::: paths =" <+>  (text . show) paths
-  withRepoLock (dryRun ? opts) (useCache ? opts) YesUpdateWorking (umask ? opts) $ RepoJob $ \repository -> do
+  withRepoLock (dryRun ? opts) (useCache ? opts) YesUpdatePending (umask ? opts) $
+    RepoJob $ \_repository -> do
 
 {-
     What we do here:
@@ -149,28 +155,32 @@ markconflictsCmd fps opts args = do
     let (useidx, scan, _) = diffingOpts opts
         verb = verbosity ? opts
     classified_paths <-
-      traverse (filterExistingPaths repository verb useidx scan O.NoLookForMoves) paths
+      traverse (filterExistingPaths _repository verb useidx scan O.NoLookForMoves) paths
 
     unrecorded <- unrecordedChanges (diffingOpts opts)
       O.NoLookForMoves O.NoLookForReplaces
-      repository (fromOnly Everything)
+      _repository (fromOnly Everything)
 
-    let forward_renames = liftToPathSet (effectOnFilePaths unrecorded)
-        backward_renames = liftToPathSet (effectOnFilePaths (invert unrecorded))
+    let forward_renames = effectOnPaths unrecorded
+        backward_renames = effectOnPaths (invert unrecorded)
         existing_paths = fmap snd classified_paths
-        pre_pending_paths = backward_renames existing_paths
+        pre_pending_paths = fmap backward_renames existing_paths
     debugDocLn $ "::: pre_pending_paths =" <+> (text . show) pre_pending_paths
 
-    r <- readRepo repository
+    r <- readRepo _repository
     Sealed res <- case patchsetConflictResolutions r of
-      Sealed raw_res -> do
-        let raw_res_paths = fps2ps (listTouchedFiles raw_res)
+      conflicts -> do
+        -- FIXME this should warn only about unmangled conflicts
+        -- involving the file paths we care about
+        warnUnmangled conflicts
+        Sealed mangled_res <- return $ mangled conflicts
+        let raw_res_paths = pathSet $ listTouchedFiles mangled_res
         debugDocLn $ "::: raw_res_paths =" <+>  (text . show) raw_res_paths
-        return $ chooseTouching (ps2fps pre_pending_paths) raw_res
-    let res_paths = fps2ps (listTouchedFiles res)
+        return $ chooseTouching (fromOnly pre_pending_paths) mangled_res
+    let res_paths = pathSet $ listTouchedFiles res
     debugDocLn $ "::: res_paths =" <+>  (text . show) res_paths
 
-    let affected_paths = isectPathSet res_paths pre_pending_paths
+    let affected_paths = res_paths `isectPathSet` pre_pending_paths
     debugDocLn $ "::: affected_paths =" <+>  (text . show) affected_paths
 
     when (affected_paths == Only []) $ do
@@ -179,9 +189,9 @@ markconflictsCmd fps opts args = do
 
     to_revert <- unrecordedChanges (diffingOpts opts)
       O.NoLookForMoves O.NoLookForReplaces
-      repository (fromOnly affected_paths)
+      _repository (fromOnly affected_paths)
 
-    let post_pending_affected_paths = forward_renames affected_paths
+    let post_pending_affected_paths = forward_renames <$> affected_paths
     putInfo opts $ "Marking conflicts in:" <+> showPathSet post_pending_affected_paths <> "."
 
     debugDocLn $ "::: to_revert =" $$ vsep (mapFL displayPatch to_revert)
@@ -190,8 +200,8 @@ markconflictsCmd fps opts args = do
         putInfo opts $ "Conflicts will not be marked: this is a dry run."
         exitSuccess
 
-    repository' <- case to_revert of
-      NilFL -> return repository
+    _repository <- case to_revert of
+      NilFL -> return _repository
       _ -> do
         -- TODO:
         -- (1) create backups for all files where we revert changes
@@ -209,19 +219,16 @@ markconflictsCmd fps opts args = do
         case commute (norevert:>p) of
           Just (p':>_) -> writeUnrevert repository p' recorded NilFL
           Nothing -> writeUnrevert repository (norevert+>+p) recorded NilFL
-        debugMessage "About to apply to the working directory."
+        debugMessage "About to apply to the working tree."
 -}
 
         let to_add = invert to_revert
-        addToPending repository YesUpdateWorking to_add
-        applyToWorking repository (verbosity ? opts) to_add `catch` \(e :: IOException) ->
-           bug ("Can't undo pending changes!" ++ show e)
+        addToPending _repository (O.useIndex ? opts) to_add
+        applyToWorking _repository (verbosity ? opts) to_add
     withSignalsBlocked $
-      do addToPending repository' YesUpdateWorking res
-         _ <- applyToWorking repository' (verbosity ? opts) res `catch` \(e :: IOException) ->
-             bug ("Problem marking conflicts in mark-conflicts!" ++ show e)
-         return ()
-    putInfo opts "Finished marking conflicts."
+      do addToPending _repository (O.useIndex ? opts) res
+         void $ applyToWorking _repository (verbosity ? opts) res
+    putFinished opts "marking conflicts"
 
 -- * Generic 'PathSet' support
 
@@ -269,21 +276,20 @@ fromOnly :: Only a -> Maybe a
 fromOnly Everything = Nothing
 fromOnly (Only x) = Just x
 
-{- | A set of repository paths. 'Everything' means every path in the repo, it
-usually originates from an empty list of path arguments. The list of
-'SubPath's is always kept in sorted order with no duplicates and normalised
-(as in 'FilePath.normalise'). This has the nice effect of getting rid of the
-idiotic "./" that Darcs insists on prepending to repo paths (which can make
-things like comparing paths returned from different parts of the code base a
-nightmare).
+maybeToOnly :: Maybe a -> Only a
+maybeToOnly Nothing = Everything
+maybeToOnly (Just x) = Only x
 
-It uses 'SubPath' for easier compatibility and lists because the number of
-elements is expected to be small.
+{- | A set of repository paths. 'Everything' means every path in the repo,
+it usually originates from an empty list of path arguments. The list of
+'AnchoredPath's is always kept in sorted order with no duplicates.
+
+It uses lists because the number of elements is expected to be small.
 -}
-type PathSet = Only [SubPath]
+type PathSet a = Only [a]
 
 -- | Intersection of two 'PathSet's
-isectPathSet :: PathSet -> PathSet -> PathSet
+isectPathSet :: Ord a => PathSet a -> PathSet a -> PathSet a
 isectPathSet Everything ys = ys
 isectPathSet xs Everything = xs
 isectPathSet (Only xs) (Only ys) = Only (isect xs ys)
@@ -296,39 +302,11 @@ union xs Everything = Everything
 union (Only xs) (Only ys) = Only (union xs ys)
 -}
 
--- | Convert a list of 'SubPath's to a 'PathSet'.
-sps2ps :: [SubPath] -> PathSet
-sps2ps = Only . nubSort
-
--- | Convert a list of repo paths to a 'PathSet'.
--- Partial function! Use only with repo paths.
-fps2ps :: [FilePath] -> PathSet
-fps2ps = sps2ps . map fp2sp
-
--- | Convert a 'PathSet' to something that e.g. 'chooseTouching'
--- takes as parameter.
-ps2fps :: PathSet -> Maybe [FilePath]
-ps2fps = fmap (map sp2fp) . fromOnly
+pathSet :: Ord a => [a] -> PathSet a
+pathSet = Only . nubSort
 
 -- | Convert a 'PathSet' to a 'Doc'. Uses the English module
 -- to generate a nicely readable list of file names.
-showPathSet :: Only [SubPath] -> Doc
+showPathSet :: PathSet AnchoredPath -> Doc
 showPathSet Everything = text "all paths"
-showPathSet (Only xs) = pathlist (map sp2fp xs)
-
--- | Lift a function transforming a list of 'FilePath' to one that
--- transforms a 'PathSet'.
-liftToPathSet :: ([FilePath] -> [FilePath]) -> PathSet -> PathSet
-liftToPathSet f = fmap (nubSort . map fp2sp . f . map sp2fp)
-
--- | Convert a 'FilePath' to a 'SubPath'.
---
--- Note: Should call this only with paths we get from the repository.
--- This guarantees that they are relative (to the repo dir).
-fp2sp :: FilePath -> SubPath
-fp2sp = fromJust . simpleSubPath
-
--- | Convert a 'SubPath' to a 'FilePath'. Same as 'toFilePath' and
--- only here for symmetry.
-sp2fp :: SubPath -> FilePath
-sp2fp = toFilePath
+showPathSet (Only xs) = pathlist (map (anchorPath "") xs)
