@@ -202,8 +202,12 @@ moveFile opts old new = withRepoAndState opts $ \(repo, work, cur, recorded) ->
 
 moveFilesToDir :: [DarcsFlag] -> [AnchoredPath] -> AnchoredPath -> IO ()
 moveFilesToDir opts froms to =
-  withRepoAndState opts $ \(repo, work, cur, _) ->
-    moveToDir repo opts cur work froms to
+  withRepoAndState opts $ \(repo, work, cur, _) -> do
+    froms_exist <- and <$> forM froms (treeHas cur)
+    if froms_exist then
+      moveToDir repo opts cur work froms to
+    else
+      fail "Some of the paths you want to move aren't know to darcs. Use `darcs add` to add them first."
 
 withRepoAndState :: [DarcsFlag]
                  -> (forall rt p wR wU .
