@@ -19,19 +19,19 @@
 
 module Darcs.UI.Commands.Optimize ( optimize ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Control.Monad ( when, unless, forM_ )
 import Data.List ( nub )
 import Data.Maybe ( fromJust, isJust )
 import System.Directory
-    ( getDirectoryContents
+    ( listDirectory
     , doesDirectoryExist
     , renameFile
     , createDirectoryIfMissing
     , removeFile
     , getHomeDirectory
+    , removeDirectoryRecursive
     )
 import qualified Data.ByteString.Char8 as BC
 import Darcs.UI.Commands ( DarcsCommand(..), nodefaults
@@ -51,13 +51,32 @@ import Darcs.Repository
     )
 import Darcs.Repository.Job ( withOldRepoLock )
 import Darcs.Repository.Identify ( findAllReposInDir )
-import Darcs.Repository.Hashed ( inventoriesDir, patchesDir, pristineDir,
-                                    hashedInventory,
-                                    listInventoriesRepoDir,
-                                    listPatchesLocalBucketed, diffHashLists, peekPristineHash )
+import Darcs.Repository.Traverse
+    ( diffHashLists
+    , listInventoriesRepoDir
+    , listPatchesLocalBucketed
+    , specialPatches
+    )
+import Darcs.Repository.Inventory ( peekPristineHash )
+import Darcs.Repository.Paths
+    ( formatPath
+    , hashedInventoryPath
+    , inventoriesDir
+    , inventoriesDirPath
+    , oldCheckpointDirPath
+    , oldCurrentDirPath
+    , oldInventoryPath
+    , oldPristineDirPath
+    , oldTentativeInventoryPath
+    , patchesDir
+    , patchesDirPath
+    , pristineDir
+    , pristineDirPath
+    , tentativePristinePath
+    )
 import Darcs.Repository.Packs ( createPacks )
-import Darcs.Repository.Pending ( pendingName )
 import Darcs.Repository.HashedIO ( getHashedFiles )
+import Darcs.Repository.Inventory ( getValidHash )
 import Darcs.Patch.Witnesses.Ordered
      ( mapFL
      , bunchFL
@@ -71,12 +90,11 @@ import Darcs.Patch.Set
     )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Util.ByteString ( gzReadFilePS )
-import Darcs.Util.Printer ( text )
+import Darcs.Util.Printer ( Doc, formatWords, text, wrapText, ($+$) )
 import Darcs.Util.Lock
     ( maybeRelink
     , gzWriteAtomicFilePS
     , writeAtomicFilePS
-    , rmRecursive
     , removeFileMayNotExist
     , writeBinFile
     )
@@ -85,7 +103,7 @@ import Darcs.Util.File
     , getRecursiveContents
     , doesDirectoryReallyExist
     )
-import Darcs.UI.External ( catchall )
+import Darcs.Util.Exception ( catchall )
 import Darcs.Util.Progress
     ( beginTedious
     , endTedious
@@ -101,12 +119,12 @@ import System.FilePath.Posix
     )
 import Text.Printf ( printf )
 import Darcs.UI.Flags
-    (  DarcsFlag, verbosity, useCache, umask )
-import Darcs.UI.Options ( DarcsOption, (^), oid, odesc, ocheck, onormalise
+    (  DarcsFlag, useCache, umask )
+import Darcs.UI.Options ( DarcsOption, (^), odesc, ocheck
                         , defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Repository.Flags
-    ( UpdateWorking (..), DryRun ( NoDryRun ), UseCache (..), UMask (..)
+    ( UpdatePending (..), DryRun ( NoDryRun ), UseCache (..), UMask (..)
     , WithWorkingDir(WithWorkingDir), PatchFormat(PatchFormat1) )
 import Darcs.Patch.Progress ( progressFL )
 import Darcs.Repository.Cache ( hashedDir, bucketFolder,
@@ -119,7 +137,14 @@ import Darcs.Repository.Format
     , RepoProperty ( HashedInventory )
     )
 import Darcs.Repository.PatchIndex
-import qualified Darcs.Repository.Hashed as HashedRepo
+import Darcs.Repository.Hashed
+    ( writeTentativeInventory
+    , finalizeTentativeChanges
+    )
+import Darcs.Repository.Pristine
+    ( ApplyDir(ApplyNormal)
+    , applyToTentativePristineCwd
+    )
 import Darcs.Repository.State ( readRecorded )
 
 import Darcs.Util.Tree
@@ -129,7 +154,7 @@ import Darcs.Util.Tree
     , expand
     , emptyTree
     )
-import Darcs.Util.Path( anchorPath, toFilePath, AbsolutePath )
+import Darcs.Util.Path( realPath, toFilePath, AbsolutePath )
 import Darcs.Util.Tree.Plain( readPlainTree )
 import Darcs.Util.Tree.Hashed
     ( writeDarcsHashed
@@ -139,12 +164,14 @@ import Darcs.Util.Tree.Hashed
 optimizeDescription :: String
 optimizeDescription = "Optimize the repository."
 
-optimizeHelp :: String
-optimizeHelp =
- "The `darcs optimize` command modifies the current repository in an\n" ++
- "attempt to reduce its resource requirements."
+optimizeHelp :: Doc
+optimizeHelp = formatWords
+  [ "The `darcs optimize` command modifies internal data structures of"
+  , "the current repository in an attempt to reduce its resource requirements."
+  ]
+  $+$ "For further details see the descriptions of the subcommands."
 
-optimize :: DarcsCommand [DarcsFlag]
+optimize :: DarcsCommand
 optimize = SuperCommand {
       commandProgramName = "darcs"
     , commandName = "optimize"
@@ -165,13 +192,13 @@ optimize = SuperCommand {
                            ]
     }
 
-commonBasicOpts :: DarcsOption a (Maybe String -> UMask -> a)
-commonBasicOpts = O.repoDir ^ O.umask
+commonBasicOpts :: DarcsOption a (Maybe String -> a)
+commonBasicOpts = O.repoDir
 
-commonAdvancedOpts :: DarcsOption a a
-commonAdvancedOpts = oid
+commonAdvancedOpts :: DarcsOption a (UMask -> a)
+commonAdvancedOpts = O.umask
 
-common :: DarcsCommand [DarcsFlag]
+common :: DarcsCommand
 common = DarcsCommand
     { commandProgramName = "darcs"
     , commandExtraArgs = 0
@@ -187,37 +214,37 @@ common = DarcsCommand
     , commandBasicOptions = odesc commonBasicOpts
     , commandDefaults = defaultFlags commonOpts
     , commandCheckOptions = ocheck commonOpts
-    , commandParseOptions = onormalise commonOpts
     }
   where
     commonOpts = commonBasicOpts `withStdOpts` commonAdvancedOpts
 
 
-optimizeClean :: DarcsCommand [DarcsFlag]
+optimizeClean :: DarcsCommand
 optimizeClean = common
     { commandName = "clean"
-    , commandHelp = "This command deletes obsolete files within the repository."
     , commandDescription = "garbage collect pristine, inventories and patches"
+    , commandHelp = optimizeHelpClean
     , commandCommand = optimizeCleanCmd
     }
 
 optimizeCleanCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeCleanCmd _ opts _ =
-    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    withRepoLock NoDryRun (useCache ? opts) YesUpdatePending (umask ? opts) $
     RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       putInfo opts "Done cleaning repository!"
 
-optimizeUpgrade :: DarcsCommand [DarcsFlag]
+optimizeUpgrade :: DarcsCommand
 optimizeUpgrade = common
     { commandName = "upgrade"
-    , commandHelp = "Convert old-fashioned repositories to the current default hashed format."
+    , commandHelp = wrapText 80
+        "Convert old-fashioned repositories to the current default hashed format."
     , commandDescription = "upgrade repository to latest compatible format"
     , commandPrereq = amInRepository
     , commandCommand = optimizeUpgradeCmd
     }
 
-optimizeHttp :: DarcsCommand [DarcsFlag]
+optimizeHttp :: DarcsCommand
 optimizeHttp = common
     { commandName = "http"
     , commandHelp = optimizeHelpHttp
@@ -227,30 +254,31 @@ optimizeHttp = common
 
 optimizeHttpCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeHttpCmd _ opts _ =
-    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    withRepoLock NoDryRun (useCache ? opts) YesUpdatePending (umask ? opts) $
     RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       createPacks repository
       putInfo opts "Done creating packs!"
 
-optimizePristine :: DarcsCommand [DarcsFlag]
+optimizePristine :: DarcsCommand
 optimizePristine = common
     { commandName = "pristine"
-    , commandHelp = "This command updates the format of `_darcs/pristine.hashed/`, which was different\n"
-                 ++ "before darcs 2.3.1."
+    , commandHelp = wrapText 80 $
+        "This command updates the format of `"++pristineDirPath++
+        "`, which was different\nbefore darcs 2.3.1."
     , commandDescription = "optimize hashed pristine layout"
     , commandCommand = optimizePristineCmd
     }
 
 optimizePristineCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizePristineCmd _ opts _ =
-    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    withRepoLock NoDryRun (useCache ? opts) YesUpdatePending (umask ? opts) $
     RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       doOptimizePristine opts repository
       putInfo opts "Done optimizing pristine!"
 
-optimizeCompress :: DarcsCommand [DarcsFlag]
+optimizeCompress :: DarcsCommand
 optimizeCompress = common
     { commandName = "compress"
     , commandHelp = optimizeHelpCompression
@@ -258,7 +286,7 @@ optimizeCompress = common
     , commandCommand = optimizeCompressCmd
     }
 
-optimizeUncompress :: DarcsCommand [DarcsFlag]
+optimizeUncompress :: DarcsCommand
 optimizeUncompress = common
     { commandName = "uncompress"
     , commandHelp = optimizeHelpCompression
@@ -268,7 +296,7 @@ optimizeUncompress = common
 
 optimizeCompressCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeCompressCmd _ opts _ =
-    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    withRepoLock NoDryRun (useCache ? opts) YesUpdatePending (umask ? opts) $
     RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       optimizeCompression O.GzipCompression opts
@@ -276,7 +304,7 @@ optimizeCompressCmd _ opts _ =
 
 optimizeUncompressCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeUncompressCmd _ opts _ =
-    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    withRepoLock NoDryRun (useCache ? opts) YesUpdatePending (umask ? opts) $
     RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       optimizeCompression O.NoCompression opts
@@ -285,44 +313,45 @@ optimizeUncompressCmd _ opts _ =
 optimizeCompression :: O.Compression -> [DarcsFlag] -> IO ()
 optimizeCompression compression opts = do
     putInfo opts "Optimizing (un)compression of patches..."
-    do_compress (darcsdir ++ "/patches")
+    do_compress patchesDirPath
     putInfo opts "Optimizing (un)compression of inventories..."
-    do_compress (darcsdir ++ "/inventories")
+    do_compress inventoriesDirPath
     where
       do_compress f = do
         isd <- doesDirectoryExist f
         if isd
           then withCurrentDirectory f $ do
-                 fs <- filter notdot `fmap` getDirectoryContents "."
+                 fs <- filter (`notElem` specialPatches) <$> listDirectory "."
                  mapM_ do_compress fs
           else gzReadFilePS f >>=
                case compression of
                  O.GzipCompression -> gzWriteAtomicFilePS f
                  O.NoCompression -> writeAtomicFilePS f
-      notdot ('.':_) = False
-      notdot _ = True
 
-optimizeEnablePatchIndex :: DarcsCommand [DarcsFlag]
+optimizeEnablePatchIndex :: DarcsCommand
 optimizeEnablePatchIndex = common
     { commandName = "enable-patch-index"
-    , commandHelp = "Build the patch index, an internal data structure that accelerates\n"
-                 ++ "commands that need to know what patches touch a given file. Such as\n"
-                 ++ "annotate and log."
+    , commandHelp = formatWords
+        [ "Build the patch index, an internal data structure that accelerates"
+        , "commands that need to know what patches touch a given file. Such as"
+        , "annotate and log."
+        ]
     , commandDescription = "Enable patch index"
     , commandCommand = optimizeEnablePatchIndexCmd
     }
 
-optimizeDisablePatchIndex :: DarcsCommand [DarcsFlag]
+optimizeDisablePatchIndex :: DarcsCommand
 optimizeDisablePatchIndex = common
     { commandName = "disable-patch-index"
-    , commandHelp = "Delete and stop maintaining the patch index from the repository."
+    , commandHelp = wrapText 80
+        "Delete and stop maintaining the patch index from the repository."
     , commandDescription = "Disable patch index"
     , commandCommand = optimizeDisablePatchIndexCmd
     }
 
 optimizeEnablePatchIndexCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeEnablePatchIndexCmd _ opts _ =
-    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    withRepoLock NoDryRun (useCache ? opts) YesUpdatePending (umask ? opts) $
     RepoJob $ \repository -> do
       ps <- readRepo repository
       createOrUpdatePatchIndexDisk repository ps
@@ -330,30 +359,32 @@ optimizeEnablePatchIndexCmd _ opts _ =
 
 optimizeDisablePatchIndexCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeDisablePatchIndexCmd _ opts _ =
-    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    withRepoLock NoDryRun (useCache ? opts) YesUpdatePending (umask ? opts) $
     RepoJob $ \repo -> do
       deletePatchIndex (repoLocation repo)
       putInfo opts "Done disabling patch index!"
 
-optimizeReorder :: DarcsCommand [DarcsFlag]
+optimizeReorder :: DarcsCommand
 optimizeReorder = common
     { commandName = "reorder"
-    , commandHelp = "This command moves recent patches (those not included in\n" ++
-                    "the latest tag) to the \"front\", reducing the amount that a typical\n" ++
-                    "remote command needs to download.  It should also reduce the CPU time\n" ++
-                    "needed for some operations."
+    , commandHelp = formatWords
+        [ "This command moves recent patches (those not included in"
+        , "the latest tag) to the \"front\", reducing the amount that a typical"
+        , "remote command needs to download.  It should also reduce the CPU time"
+        , "needed for some operations."
+        ]
     , commandDescription = "reorder the patches in the repository"
     , commandCommand = optimizeReorderCmd
     }
 
 optimizeReorderCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeReorderCmd _ opts _ =
-    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    withRepoLock NoDryRun (useCache ? opts) YesUpdatePending (umask ? opts) $
     RepoJob $ \repository -> do
-      reorderInventory repository (O.compress ? opts) YesUpdateWorking (verbosity ? opts)
+      reorderInventory repository (O.compress ? opts)
       putInfo opts "Done reordering!"
 
-optimizeRelink :: DarcsCommand [DarcsFlag]
+optimizeRelink :: DarcsCommand
 optimizeRelink = common
     { commandName = "relink"
     , commandHelp = optimizeHelpRelink 
@@ -363,7 +394,6 @@ optimizeRelink = common
     , commandBasicOptions = odesc optimizeRelinkBasicOpts
     , commandDefaults = defaultFlags optimizeRelinkOpts
     , commandCheckOptions = ocheck optimizeRelinkOpts
-    , commandParseOptions = onormalise optimizeRelinkOpts
     }
   where
     optimizeRelinkBasicOpts = commonBasicOpts ^ O.siblings
@@ -371,46 +401,59 @@ optimizeRelink = common
 
 optimizeRelinkCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 optimizeRelinkCmd _ opts _ =
-    withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $
+    withRepoLock NoDryRun (useCache ? opts) YesUpdatePending (umask ? opts) $
     RepoJob $ \repository -> do
       cleanRepository repository -- garbage collect pristine.hashed, inventories and patches directories
       doRelink opts
       putInfo opts "Done relinking!"
 
-optimizeHelpHttp :: String
-optimizeHelpHttp = unlines
-  [ "Using this option creates 'repository packs' that could dramatically"
+optimizeHelpHttp :: Doc
+optimizeHelpHttp = formatWords
+  [ "Using this option creates 'repository packs' that can dramatically"
   , "speed up performance when a user does a `darcs clone` of the repository"
   , "over HTTP. To make use of packs, the clients must have a darcs of at"
   , "least version 2.10."
   ]
 
-optimizeHelpCompression :: String
-optimizeHelpCompression =
- "By default patches are compressed with zlib (RFC 1951) to reduce\n" ++
- "storage (and download) size.  In exceptional circumstances, it may be\n" ++
- "preferable to avoid compression.  In this case the `--dont-compress`\n" ++
- "option can be used (e.g. with `darcs record`) to avoid compression.\n" ++
- "\n" ++
- "The `darcs optimize uncompress` and `darcs optimize compress`\n" ++
- "commands can be used to ensure existing patches in the current\n" ++
- "repository are respectively uncompressed or compressed."
+optimizeHelpClean :: Doc
+optimizeHelpClean = formatWords
+  [ "Darcs normally does not delete hashed files that are no longer"
+  , "referenced by the current repository state. This command can be"
+  , "use to get rid of these files to save some disk space."
+  ]
 
-optimizeHelpRelink :: String
-optimizeHelpRelink =
- "The `darcs optimize relink` command hard-links patches that the\n" ++
- "current repository has in common with its peers.  Peers are those\n" ++
- "repositories listed in `_darcs/prefs/sources`, or defined with the\n" ++
- "`--sibling` option (which can be used multiple times).\n" ++
- "\n" ++
- "Darcs uses hard-links automatically, so this command is rarely needed.\n" ++
- "It is most useful if you used `cp -r` instead of `darcs clone` to copy a\n" ++
- "repository, or if you pulled the same patch from a remote repository\n" ++
- "into multiple local repositories."
+optimizeHelpCompression :: Doc
+optimizeHelpCompression =
+  formatWords
+  [ "By default patches are compressed with zlib (RFC 1951) to reduce"
+  , "storage (and download) size.  In exceptional circumstances, it may be"
+  , "preferable to avoid compression.  In this case the `--dont-compress`"
+  , "option can be used (e.g. with `darcs record`) to avoid compression."
+  ]
+  $+$ formatWords
+  [ "The `darcs optimize uncompress` and `darcs optimize compress`"
+  , "commands can be used to ensure existing patches in the current"
+  , "repository are respectively uncompressed or compressed."
+  ]
+
+optimizeHelpRelink :: Doc
+optimizeHelpRelink = 
+  formatWords
+  [ "The `darcs optimize relink` command hard-links patches that the"
+  , "current repository has in common with its peers.  Peers are those"
+  , "repositories listed in `_darcs/prefs/sources`, or defined with the"
+  , "`--sibling` option (which can be used multiple times)."
+  ]
+  $+$ formatWords
+  [ "Darcs uses hard-links automatically, so this command is rarely needed."
+  , "It is most useful if you used `cp -r` instead of `darcs clone` to copy a"
+  , "repository, or if you pulled the same patch from a remote repository"
+  , "into multiple local repositories."
+  ]
 
 doOptimizePristine :: [DarcsFlag] -> Repository rt p wR wU wT -> IO ()
 doOptimizePristine opts repo = do
-    inv <- BC.readFile (darcsdir </> "hashed_inventory")
+    inv <- BC.readFile hashedInventoryPath
     let linesInv = BC.split '\n' inv
     case linesInv of
       [] -> return ()
@@ -428,9 +471,9 @@ doRelink opts =
        if null siblings
           then putInfo opts "No siblings -- no relinking done."
           else do debugMessage "Relinking patches..."
-                  patch_tree <- expand =<< readPlainTree (darcsdir </> "patches")
-                  let patches = [ anchorPath "" p | (p, File _) <- list patch_tree ]
-                  maybeRelinkFiles siblings patches $ darcsdir </> "patches"
+                  patch_tree <- expand =<< readPlainTree patchesDirPath
+                  let patches = [ realPath p | (p, File _) <- list patch_tree ]
+                  maybeRelinkFiles siblings patches patchesDirPath
                   debugMessage "Done relinking."
 
 maybeRelinkFiles :: [String] -> [String] -> String -> IO ()
@@ -464,41 +507,41 @@ actuallyUpgradeFormat repository = do
   beginTedious k
   tediousSize k (lengthRL $ patchSet2RL patches)
   let patches' = progressPatchSet k patches
+  -- darcs optimize subcommands do not support
+  -- the --no-cache option, so use default
   cache <- getCaches YesUseCache "."
-  -- TODO Why use the default and not what the user provided?
-  -- Is it because the author couldn't be bothered to add the option?
-  -- Or is there a more profound reason?
-  -- Such things justify a few lines of comment!
   let compressDefault = O.compress ? []
-  HashedRepo.writeTentativeInventory cache compressDefault patches'
+  writeTentativeInventory cache compressDefault patches'
   endTedious k
   -- convert pristine by applying patches
-  -- the faster alternative would be to copy pristine, but the apply method is more reliable
+  -- the faster alternative would be to copy pristine, but the apply method
+  -- is more reliable
+  -- TODO we should do both and then comapre them
   let patchesToApply = progressFL "Applying patch" $ patchSet2FL patches'
   createDirectoryIfMissing False $ darcsdir </> hashedDir HashedPristineDir
   -- We ignore the returned root hash, we don't use it.
   _ <- writeDarcsHashed emptyTree $ darcsdir </> hashedDir HashedPristineDir
-  writeBinFile (darcsdir++"/tentative_pristine") ""
-  sequence_ $ mapFL HashedRepo.applyToTentativePristineCwd $ bunchFL 100 patchesToApply
+  writeBinFile tentativePristinePath ""
+  sequence_ $
+    mapFL (applyToTentativePristineCwd ApplyNormal) $
+    bunchFL 100 patchesToApply
   -- now make it official
-  HashedRepo.finalizeTentativeChanges repository compressDefault
-  writeRepoFormat (createRepoFormat PatchFormat1 WithWorkingDir) (darcsdir </> "format")
+  finalizeTentativeChanges repository compressDefault
+  writeRepoFormat (createRepoFormat PatchFormat1 WithWorkingDir) formatPath
   -- clean out old-fashioned junk
   debugMessage "Cleaning out old-fashioned repository files..."
-  removeFileMayNotExist $ darcsdir </> "inventory"
-  removeFileMayNotExist $ darcsdir </> "tentative_inventory"
-  rmRecursive (darcsdir </> "pristine") `catchall` rmRecursive (darcsdir </> "current")
-  rmGzsIn (darcsdir </> "patches")
-  rmGzsIn (darcsdir </> "inventories")
-  let checkpointDir = darcsdir </> "checkpoints"
-  hasCheckPoints <- doesDirectoryExist checkpointDir
-  when hasCheckPoints $ rmRecursive checkpointDir
-  removeFileMayNotExist (pendingName ++ ".tentative")
-  removeFileMayNotExist pendingName
+  removeFileMayNotExist oldInventoryPath
+  removeFileMayNotExist oldTentativeInventoryPath
+  removeDirectoryRecursive oldPristineDirPath
+    `catchall` removeDirectoryRecursive oldCurrentDirPath
+  rmGzsIn patchesDirPath
+  rmGzsIn inventoriesDirPath
+  hasCheckPoints <- doesDirectoryExist oldCheckpointDirPath
+  when hasCheckPoints $ removeDirectoryRecursive oldCheckpointDirPath
  where
   rmGzsIn dir =
     withCurrentDirectory dir $ do
-      gzs <- filter ((== ".gz") . takeExtension) `fmap` getDirectoryContents "."
+      gzs <- filter ((== ".gz") . takeExtension) `fmap` listDirectory "."
       mapM_ removeFile gzs
 
 optimizeBucketed :: [DarcsFlag] -> IO ()
@@ -526,7 +569,7 @@ optimizeBucketed opts = do
                 debugMessage $ "Making " ++ src ++ " bucketed in " ++ dest
                 forM_ subDirSet $ \subDir ->
                   createDirectoryIfMissing True (dest </> subDir)
-                fileNames <- getDirectoryContents src
+                fileNames <- listDirectory src
                 forM_ fileNames $ \file -> do
                   exists <- doesDirectoryReallyExist (src </> file)
                   if not $ exists
@@ -546,7 +589,7 @@ optimizeBucketed opts = do
     toStrHex = printf "%02x"
 
 
-optimizeGlobalCache :: DarcsCommand [DarcsFlag]
+optimizeGlobalCache :: DarcsCommand
 optimizeGlobalCache = common
     { commandName = "cache"
     , commandExtraArgs            = -1
@@ -557,16 +600,17 @@ optimizeGlobalCache = common
     , commandPrereq = \_ -> return $ Right ()
     }
 
-optimizeHelpGlobalCache :: String
-optimizeHelpGlobalCache = unlines
+optimizeHelpGlobalCache :: Doc
+optimizeHelpGlobalCache = formatWords
   [ "This command deletes obsolete files within the global cache."
   , "It takes one or more directories as arguments, and recursively"
   , "searches all repositories within these directories. Then it deletes"
   , "all files in the global cache not belonging to these repositories."
   , "When no directory is given, it searches repositories in the user's"
   , "home directory."
-  , ""
-  , "It also automatically migrates the global cache to the (default)"
+  ]
+  $+$ formatWords
+  [ "It also automatically migrates the global cache to the (default)"
   , "bucketed format."
   ]
 
@@ -614,8 +658,6 @@ cleanGlobalCache dirs opts = do
       (diffHashLists s1 s2)
 
   getPristine :: String -> IO [String]
-  getPristine darcsDir = do
-    i <- gzReadFilePS (darcsDir </> darcsdir </> hashedInventory)
-    getHashedFiles (darcsDir </> darcsdir </> pristineDir) [peekPristineHash i]
-
-
+  getPristine repoDir = do
+    i <- gzReadFilePS (repoDir </> hashedInventoryPath)
+    getHashedFiles (repoDir </> pristineDirPath) [getValidHash $ peekPristineHash i]
