@@ -15,14 +15,14 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
+{-# LANGUAGE OverloadedStrings #-}
 module Darcs.UI.Commands.Pull ( -- * Commands.
                                 pull, fetch,
                                 pullCmd, StandardPatchApplier,
                                 -- * Utility functions.
-                                fetchPatches, revertable
+                                fetchPatches
                               ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import System.Exit ( exitSuccess )
@@ -39,6 +39,7 @@ import Darcs.UI.Commands
     , defaultRepo
     , amInHashedRepository
     )
+import Darcs.UI.Commands.Clone ( otherHelpInheritDefault )
 import Darcs.UI.Flags
     ( DarcsFlag
     , fixUrl, getOutput
@@ -47,53 +48,62 @@ import Darcs.UI.Flags
     , withContext, hasXmlOutput
     , isInteractive, quiet
     )
-import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, parseFlags, (?) )
+import Darcs.UI.Options ( (^), odesc, ocheck, defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
-import Darcs.Repository.Flags ( UpdateWorking (..) )
+import Darcs.Repository.Flags ( UpdatePending (..) )
 import Darcs.Repository
     ( Repository
     , identifyRepositoryFor
+    , ReadingOrWriting(..)
     , withRepoLock
     , RepoJob(..)
     , readRepo
     , modifyCache
-    , modifyCache
-    , Cache(..)
+    , mkCache
+    , cacheEntries
     , CacheLoc(..)
     , WritableOrNot(..)
+    , CacheType(..)
     , filterOutConflicts
     )
 
-import qualified Darcs.Repository.Cache as DarcsCache
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info, hopefully, patchDesc )
 import Darcs.Patch ( IsRepoType, RepoPatch, description )
-import Darcs.Patch.Bundle( makeBundleN, patchFilename )
+import qualified Darcs.Patch.Bundle as Bundle ( makeBundle )
 import Darcs.Patch.Apply( ApplyState )
-import Darcs.Patch.Set ( Origin, PatchSet(..), SealedPatchSet )
+import Darcs.Patch.Set ( PatchSet, Origin, emptyPatchSet, SealedPatchSet )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..), seal )
-import Darcs.Patch.Witnesses.Unsafe ( unsafeCoercePEnd )
 import Darcs.Patch.Witnesses.Ordered
-    ( (:>)(..), (:\/:)(..), FL(..), RL(..)
-    , mapFL, nullFL, reverseFL, mapFL_FL )
+    ( (:>)(..), (:\/:)(..), FL(..), Fork(..)
+    , mapFL, nullFL, mapFL_FL )
 import Darcs.Patch.Permutations ( partitionFL )
 import Darcs.Repository.Prefs ( addToPreflist, addRepoSource, getPreflist, showMotd )
-import Darcs.Patch.Depends ( findUncommon, findCommonWithThem,
+import Darcs.Patch.Depends ( findUncommon, findCommonAndUncommon,
                              patchSetIntersection, patchSetUnion )
 import Darcs.UI.ApplyPatches ( PatchApplier(..), StandardPatchApplier(..) )
 import Darcs.UI.Completion ( prefArgs )
-import Darcs.UI.Commands.Util ( checkUnrelatedRepos )
+import Darcs.UI.Commands.Util ( checkUnrelatedRepos, getUniqueDPatchName )
 import Darcs.UI.SelectChanges
     ( WhichChanges(..)
     , runSelection
-    , selectionContext
+    , selectionConfig
     )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
-import Darcs.Util.Exception ( clarifyErrors )
-import Darcs.Util.Printer ( vcat, ($$), text, putDoc )
+import Darcs.Util.Printer
+    ( Doc
+    , ($$)
+    , ($+$)
+    , (<+>)
+    , formatWords
+    , hsep
+    , putDoc
+    , quoted
+    , text
+    , vcat
+    )
 import Darcs.Util.Lock ( writeDocBinFile )
 import Darcs.Util.Path ( useAbsoluteOrStd, stdOut, AbsolutePath )
 import Darcs.Util.Workaround ( getCurrentDirectory )
-import Darcs.Util.Text ( quote )
 import Darcs.Util.Tree( Tree )
 
 pullDescription :: String
@@ -104,38 +114,45 @@ fetchDescription :: String
 fetchDescription =
  "Fetch patches from another repository, but don't apply them."
 
-pullHelp :: String
-pullHelp = unlines
- [ "Pull is used to bring patches made in another repository into the current"
- , "repository (that is, either the one in the current directory, or the one"
- , "specified with the `--repodir` option). Pull accepts arguments, which are"
- , "URLs from which to pull, and when called without an argument, pull will"
- , "use the repository specified at `_darcs/prefs/defaultrepo`."
- , ""
- , "The default (`--union`) behavior is to pull any patches that are in any of"
- , "the specified repositories.  If you specify the `--intersection` flag, darcs"
- , "will only pull those patches which are present in all source repositories."
- , "If you specify the `--complement` flag, darcs will only pull elements in the"
- , "first repository that do not exist in any of the remaining repositories."
- , ""
- , "If `--reorder` is supplied, the set of patches that exist only in the current"
- , "repository is brought at the top of the current history. This will work even"
- , "if there are no new patches to pull."
- , ""
- , "See `darcs help apply` for detailed description of many options."
- ]
+pullHelp :: Doc
+pullHelp =
+  formatWords
+  [ "Pull is used to bring patches made in another repository into the current"
+  , "repository (that is, either the one in the current directory, or the one"
+  , "specified with the `--repodir` option). Pull accepts arguments, which are"
+  , "URLs from which to pull, and when called without an argument, pull will"
+  , "use the repository specified at `_darcs/prefs/defaultrepo`."
+  ]
+  $+$ formatWords
+  [ "The default (`--union`) behavior is to pull any patches that are in any of"
+  , "the specified repositories.  If you specify the `--intersection` flag, darcs"
+  , "will only pull those patches which are present in all source repositories."
+  , "If you specify the `--complement` flag, darcs will only pull elements in the"
+  , "first repository that do not exist in any of the remaining repositories."
+  ]
+  $+$ formatWords
+  [ "If `--reorder` is supplied, the set of patches that exist only in the current"
+  , "repository is brought at the top of the current history. This will work even"
+  , "if there are no new patches to pull."
+  ]
+  $+$ otherHelpInheritDefault
+  $+$ formatWords
+  [ "See `darcs help apply` for detailed description of many options."
+  ]
 
-fetchHelp :: String
-fetchHelp = unlines
- [ "Fetch is similar to `pull` except that it does not apply any patches"
- , "to the current repository. Instead, it generates a patch bundle that"
- , "you can apply later with `apply`."
- , ""
- , "Fetch's behaviour is essentially similar to pull's, so please consult"
- , "the help of `pull` to know more."
- ]
+fetchHelp :: Doc
+fetchHelp =
+  formatWords
+  [ "Fetch is similar to `pull` except that it does not apply any patches"
+  , "to the current repository. Instead, it generates a patch bundle that"
+  , "you can apply later with `apply`."
+  ]
+  $+$ formatWords
+  [ "Fetch's behaviour is essentially similar to pull's, so please consult"
+  , "the help of `pull` to know more."
+  ]
 
-fetch :: DarcsCommand [DarcsFlag]
+fetch :: DarcsCommand
 fetch = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "fetch"
@@ -151,16 +168,16 @@ fetch = DarcsCommand
     , commandBasicOptions = odesc basicOpts
     , commandDefaults = defaultFlags allOpts
     , commandCheckOptions = ocheck allOpts
-    , commandParseOptions = onormalise allOpts
     }
   where
     basicOpts
       = O.matchSeveral
       ^ O.interactive -- True
       ^ O.dryRun
-      ^ O.summary
+      ^ O.withSummary
       ^ O.selectDeps
       ^ O.setDefault
+      ^ O.inheritDefault
       ^ O.repoDir
       ^ O.output
       ^ O.allowUnrelatedRepos
@@ -171,7 +188,7 @@ fetch = DarcsCommand
       ^ O.network
     allOpts = basicOpts `withStdOpts` advancedOpts
 
-pull :: DarcsCommand [DarcsFlag]
+pull :: DarcsCommand
 pull = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "pull"
@@ -187,7 +204,6 @@ pull = DarcsCommand
     , commandBasicOptions = odesc basicOpts
     , commandDefaults = defaultFlags allOpts
     , commandCheckOptions = ocheck allOpts
-    , commandParseOptions = onormalise allOpts
     }
   where
     basicOpts
@@ -198,9 +214,10 @@ pull = DarcsCommand
       ^ O.externalMerge
       ^ O.runTest
       ^ O.dryRunXml
-      ^ O.summary
+      ^ O.withSummary
       ^ O.selectDeps
       ^ O.setDefault
+      ^ O.inheritDefault
       ^ O.repoDir
       ^ O.allowUnrelatedRepos
       ^ O.diffAlgorithm
@@ -211,7 +228,6 @@ pull = DarcsCommand
       ^ O.remoteRepos
       ^ O.setScriptsExecutable
       ^ O.umask
-      ^ O.restrictPaths
       ^ O.changesReverse
       ^ O.pauseForGui
       ^ O.network
@@ -223,29 +239,28 @@ pullCmd
 pullCmd patchApplier (_,o) opts repos =
   do
     pullingFrom <- mapM (fixUrl o) repos
-    withRepoLock (dryRun ? opts) (useCache ? opts) YesUpdateWorking (umask ? opts) $
-     repoJob patchApplier opts $ \patchProxy initRepo -> do
-
-      let repository = modifyCache initRepo $ addReposToCache pullingFrom
-      (_, Sealed (us' :\/: to_be_pulled))
-          <- fetchPatches o opts repos "pull" repository
-      let from_whom = error "Internal error: pull shouldn't need a 'from' address"
-      applyPatches patchApplier patchProxy "pull" opts from_whom repository us' to_be_pulled
+    withRepoLock (dryRun ? opts) (useCache ? opts) YesUpdatePending (umask ? opts) $
+     repoJob patchApplier $ \patchProxy initRepo -> do
+      let repository = modifyCache (addReposToCache pullingFrom) initRepo
+      Sealed fork <- fetchPatches o opts repos "pull" repository
+      applyPatches patchApplier patchProxy "pull" opts repository fork
     where
-      addReposToCache repos' (Ca cache) = Ca $ [ toReadOnlyCache r | r <- repos' ] ++  cache
-      toReadOnlyCache = Cache DarcsCache.Repo NotWritable
+      addReposToCache repos' cache =
+        mkCache $ [ toReadOnlyCache r | r <- repos' ] ++ cacheEntries cache
+      toReadOnlyCache = Cache Repo NotWritable
 
 
 fetchCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 fetchCmd (_,o) opts repos =
-    withRepoLock (dryRun ? opts) (useCache ? opts) YesUpdateWorking (umask ? opts) $ RepoJob $
+    withRepoLock (dryRun ? opts) (useCache ? opts) YesUpdatePending (umask ? opts) $ RepoJob $
         fetchPatches o opts repos "fetch" >=> makeBundle opts
 
 fetchPatches :: forall rt p wR wU . (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
              => AbsolutePath -> [DarcsFlag] -> [String] -> String
              -> Repository rt p wR wU wR
-             -> IO (SealedPatchSet rt p Origin,
-                    Sealed ((FL (PatchInfoAnd rt p)  :\/: FL (PatchInfoAnd rt p)) wR))
+             -> IO (Sealed (Fork (PatchSet rt p)
+                                 (FL (PatchInfoAnd rt p))
+                                 (FL (PatchInfoAnd rt p)) Origin wR))
 fetchPatches o opts unfixedrepodirs@(_:_) jobname repository = do
   here <- getCurrentDirectory
   repodirs <- (nub . filter (/= here)) `fmap` mapM (fixUrl o) unfixedrepodirs
@@ -257,16 +272,16 @@ fetchPatches o opts unfixedrepodirs@(_:_) jobname repository = do
       let pulling = case dryRun ? opts of
                       O.YesDryRun -> "Would pull"
                       O.NoDryRun -> "Pulling"
-      in  putInfo opts $ text $ pulling++" from "++concatMap quote repodirs++"..."
+      in  putInfo opts $ text pulling <+> "from" <+> hsep (map quoted repodirs) <> "..."
   (Sealed them, Sealed compl) <- readRepos repository opts repodirs
-  addRepoSource (head repodirs) (dryRun ? opts) (remoteRepos ? opts) (setDefault False opts)
+  addRepoSource (head repodirs) (dryRun ? opts) (remoteRepos ? opts)
+      (setDefault False opts) (O.inheritDefault ? opts) (isInteractive True opts)
   mapM_ (addToPreflist "repos") repodirs
   unless (quiet opts || hasXmlOutput opts) $ mapM_ showMotd repodirs
   us <- readRepo repository
   checkUnrelatedRepos (parseFlags O.allowUnrelatedRepos opts) us them
 
-  common :> _ <- return $ findCommonWithThem us them
-  us' :\/: them' <- return $ findUncommon us them
+  Fork common us' them' <- return $ findCommonAndUncommon us them
   _   :\/: compl' <- return $ findUncommon us compl
 
   let avoided = mapFL info compl'
@@ -282,44 +297,36 @@ fetchPatches o opts unfixedrepodirs@(_:_) jobname repository = do
       vcat (mapFL description ps)
   (hadConflicts, Sealed psFiltered)
     <- if O.conflictsYes ? opts == Nothing
-        then filterOutConflicts (reverseFL us') repository ps
+        then filterOutConflicts repository us' ps
         else return (False, Sealed ps)
   when hadConflicts $ putInfo opts $ text "Skipping some patches which would cause conflicts."
   when (nullFL psFiltered) $ do putInfo opts $ text "No remote patches to pull in!"
                                 setEnvDarcsPatches psFiltered
                                 when (reorder ? opts /= O.Reorder) exitSuccess
   let direction = if changesReverse ? opts then FirstReversed else First
-      context = selectionContext direction jobname (pullPatchSelOpts opts) Nothing Nothing
-  (to_be_pulled :> _) <- runSelection psFiltered context
-  return (seal common, seal $ us' :\/: to_be_pulled)
+      selection_config = selectionConfig direction jobname (pullPatchSelOpts opts) Nothing Nothing
+  (to_be_pulled :> _) <- runSelection psFiltered selection_config
+  return (Sealed (Fork common us' to_be_pulled))
 
 fetchPatches _ _ [] jobname _ = fail $
   "No default repository to " ++ jobname ++ " from, please specify one"
 
 makeBundle :: forall rt p wR . (RepoPatch p, ApplyState p ~ Tree)
            => [DarcsFlag]
-           -> (SealedPatchSet rt p Origin,
-               Sealed ((FL (PatchInfoAnd rt p) :\/: FL (PatchInfoAnd rt p)) wR))
+           -> (Sealed (Fork (PatchSet rt p)
+                      (FL (PatchInfoAnd rt p))
+                      (FL (PatchInfoAnd rt p)) Origin wR))
            -> IO ()
-makeBundle opts (Sealed common, Sealed (_ :\/: to_be_fetched)) =
+makeBundle opts (Sealed (Fork common _ to_be_fetched)) =
     do
-      bundle <- makeBundleN Nothing (unsafeCoercePEnd common) $
+      bundle <- Bundle.makeBundle Nothing common $
                  mapFL_FL hopefully to_be_fetched
-      let fname = case to_be_fetched of
-                    (x:>:_)-> patchFilename $ patchDesc x
-                    _ -> impossible
-          o = fromMaybe stdOut (getOutput opts fname)
+      fname <- case to_be_fetched of
+                    (x:>:_)-> getUniqueDPatchName $ patchDesc x
+                    _ -> error "impossible case"
+      let o = fromMaybe stdOut (getOutput opts fname)
       useAbsoluteOrStd writeDocBinFile putDoc o bundle
 
-revertable :: IO a -> IO a
-revertable x =
-    x `clarifyErrors` unlines
-          ["Error applying patch to the working directory.","",
-           "This may have left your working directory an inconsistent",
-           "but recoverable state. If you had no un-recorded changes",
-           "by using 'darcs revert' you should be able to make your",
-           "working directory consistent again."]
-
 {- Read in the specified pull-from repositories.  Perform
 Intersection, Union, or Complement read.  In patch-theory terms
 (stated in set algebra, where + is union and & is intersection
@@ -341,15 +348,15 @@ the second patchset(s) to be complemented against Rc.
 readRepos :: (IsRepoType rt, RepoPatch p)
           => Repository rt p wR wU wT -> [DarcsFlag] -> [String]
           -> IO (SealedPatchSet rt p Origin,SealedPatchSet rt p Origin)
-readRepos _ _ [] = impossible
+readRepos _ _ [] = error "impossible case"
 readRepos to_repo opts us =
-    do rs <- mapM (\u -> do r <- identifyRepositoryFor to_repo (useCache ? opts) u
+    do rs <- mapM (\u -> do r <- identifyRepositoryFor Reading to_repo (useCache ? opts) u
                             ps <- readRepo r
                             return $ seal ps) us
        return $ case parseFlags O.repoCombinator opts of
-                  O.Intersection -> (patchSetIntersection rs, seal (PatchSet NilRL NilRL))
+                  O.Intersection -> (patchSetIntersection rs, seal emptyPatchSet)
                   O.Complement -> (head rs, patchSetUnion $ tail rs)
-                  O.Union -> (patchSetUnion rs, seal (PatchSet NilRL NilRL))
+                  O.Union -> (patchSetUnion rs, seal emptyPatchSet)
 
 pullPatchSelOpts :: [DarcsFlag] -> S.PatchSelectionOptions
 pullPatchSelOpts flags = S.PatchSelectionOptions
@@ -357,6 +364,6 @@ pullPatchSelOpts flags = S.PatchSelectionOptions
     , S.matchFlags = parseFlags O.matchSeveral flags
     , S.interactive = isInteractive True flags
     , S.selectDeps = selectDeps ? flags
-    , S.summary = O.summary ? flags
+    , S.withSummary = O.withSummary ? flags
     , S.withContext = withContext ? flags
     }
