@@ -15,11 +15,9 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE TypeOperators #-}
-
+{-# LANGUAGE OverloadedStrings #-}
 module Darcs.UI.Commands.Push ( push ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import System.Exit ( exitWith, ExitCode( ExitSuccess, ExitFailure ), exitSuccess )
@@ -29,11 +27,13 @@ import Darcs.UI.Commands
     ( DarcsCommand(..), withStdOpts
     , putVerbose
     , putInfo
+    , putFinished
     , abortRun
     , setEnvDarcsPatches
     , defaultRepo
     , amInHashedRepository
     )
+import Darcs.UI.Commands.Clone ( otherHelpInheritDefault )
 import Darcs.UI.Commands.Util ( printDryRunMessageAndExit, checkUnrelatedRepos )
 import Darcs.UI.Completion ( prefArgs )
 import Darcs.UI.Flags
@@ -42,14 +42,20 @@ import Darcs.UI.Flags
     , xmlOutput, selectDeps, applyAs, remoteDarcs
     , changesReverse, dryRun, useCache, remoteRepos, setDefault, fixUrl )
 import Darcs.UI.Options
-    ( (^), odesc, ocheck, onormalise
+    ( (^), odesc, ocheck
     , defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Repository.Flags ( DryRun (..) )
 import qualified Darcs.Repository.Flags as R ( remoteDarcs )
 import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, hopefully )
-import Darcs.Repository ( Repository, withRepository, RepoJob(..), identifyRepositoryFor,
-                          readRepo )
+import Darcs.Repository
+    ( RepoJob(..)
+    , Repository
+    , identifyRepositoryFor
+    , ReadingOrWriting(..)
+    , readRepo
+    , withRepository
+    )
 import Darcs.Patch ( IsRepoType, RepoPatch, description )
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Witnesses.Ordered
@@ -64,19 +70,29 @@ import Darcs.Util.URL ( isHttpUrl, isValidLocalPath
 import Darcs.Util.Path ( AbsolutePath )
 import Darcs.UI.SelectChanges
     ( WhichChanges(..)
-    , selectionContext
+    , selectionConfig
     , runSelection
     )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
 import Darcs.Patch.Depends ( findCommonWithThem, countUsThem )
-import Darcs.Patch.Bundle ( makeBundleN )
+import Darcs.Patch.Bundle ( makeBundle )
 import Darcs.Patch.Show( ShowPatch )
 import Darcs.Patch.Set ( PatchSet, Origin )
-import Darcs.Util.Printer ( Doc, vcat, empty, text, ($$) )
+import Darcs.Util.Printer.Color ( ePutDocLn )
+import Darcs.Util.Printer
+    ( Doc
+    , ($$)
+    , ($+$)
+    , (<+>)
+    , empty
+    , formatWords
+    , quoted
+    , text
+    , vcat
+    )
 import Darcs.UI.Email ( makeEmail )
 import Darcs.Util.English (englishNum, Noun(..))
 import Darcs.Util.Workaround ( getCurrentDirectory )
-import Darcs.Util.Text ( quote )
 import Darcs.Util.Tree( Tree )
 
 
@@ -84,25 +100,30 @@ pushDescription :: String
 pushDescription =
  "Copy and apply patches from this repository to another one."
 
-pushHelp :: String
-pushHelp = unlines
- [ "Push is the opposite of pull.  Push allows you to copy patches from the"
- , "current repository into another repository."
- , ""
- , "If you give the `--apply-as` flag, darcs will use `sudo` to apply the"
- , "patches as a different user.  This can be useful if you want to set up a"
- , "system where several users can modify the same repository, but you don't"
- , "want to allow them full write access.  This isn't secure against skilled"
- , "malicious attackers, but at least can protect your repository from clumsy,"
- , "inept or lazy users."
- , ""
- , "`darcs push` will compress the patch data before sending it to a remote"
- , "location via ssh. This works as long as the remote darcs is not older"
- , "than version 2.5. If you get errors that indicate a corrupt patch bundle,"
- , "you should try again with the `--no-compress` option."
- ]
+pushHelp :: Doc
+pushHelp =
+  formatWords
+    [ "Push is the opposite of pull.  Push allows you to copy patches from the"
+    , "current repository into another repository."
+    ]
+  $+$ formatWords
+    [ "If you give the `--apply-as` flag, darcs will use `sudo` to apply the"
+    , "patches as a different user.  This can be useful if you want to set up a"
+    , "system where several users can modify the same repository, but you don't"
+    , "want to allow them full write access.  This isn't secure against skilled"
+    , "malicious attackers, but at least can protect your repository from clumsy,"
+    , "inept or lazy users."
+    ]
+  $+$ formatWords
+    [ "`darcs push` will compress the patch data before sending it to a remote"
+    , "location via ssh. This works as long as the remote darcs is not older"
+    , "than version 2.5. If you get errors that indicate a corrupt patch bundle,"
+    , "you should try again with the `--no-compress` option."
+    ]
+  $+$
+  otherHelpInheritDefault
 
-push :: DarcsCommand [DarcsFlag]
+push :: DarcsCommand
 push = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "push"
@@ -118,7 +139,6 @@ push = DarcsCommand
     , commandBasicOptions = odesc pushBasicOpts
     , commandDefaults = defaultFlags pushOpts
     , commandCheckOptions = ocheck pushOpts
-    , commandParseOptions = onormalise pushOpts
     }
   where
     pushBasicOpts
@@ -127,9 +147,10 @@ push = DarcsCommand
       ^ O.interactive
       ^ O.sign
       ^ O.dryRunXml
-      ^ O.summary
+      ^ O.withSummary
       ^ O.repoDir
       ^ O.setDefault
+      ^ O.inheritDefault
       ^ O.allowUnrelatedRepos
     pushAdvancedOpts
       = O.applyAs
@@ -156,9 +177,9 @@ pushCmd (_, o) opts [unfixedrepodir] = do
   rval <- remoteApply opts repodir body
   case rval of
     ExitFailure ec -> do
-      putStrLn "Apply failed!"
+      ePutDocLn (text "Apply failed!")
       exitWith (ExitFailure ec)
-    ExitSuccess -> putInfo opts $ text "Push successful."
+    ExitSuccess -> putFinished opts "pushing"
 pushCmd _ _ [] = die "No default repository to push to, please specify one."
 pushCmd _ _ _ = die "Cannot push to more than one repo."
 
@@ -168,15 +189,16 @@ prepareBundle opts repodir repository = do
   old_default <- getPreflist "defaultrepo"
   when (old_default == [repodir]) $
        let pushing = if dryRun ? opts == YesDryRun then "Would push" else "Pushing"
-       in  putInfo opts $ text $ pushing++" to "++quote repodir++"..."
-  them <- identifyRepositoryFor repository (useCache ? opts) repodir >>= readRepo
-  addRepoSource repodir (dryRun ? opts) (remoteRepos ? opts) (setDefault False opts)
+       in  putInfo opts $ text pushing <+> "to" <+> quoted repodir <> "..."
+  them <- identifyRepositoryFor Writing repository (useCache ? opts) repodir >>= readRepo
+  addRepoSource repodir (dryRun ? opts) (remoteRepos ? opts)
+      (setDefault False opts) (O.inheritDefault ? opts) (isInteractive True opts)
   us <- readRepo repository
   common :> us' <- return $ findCommonWithThem us them
   prePushChatter opts us (reverseFL us') them
   let direction = if changesReverse ? opts then FirstReversed else First
-      context = selectionContext direction "push" (pushPatchSelOpts opts) Nothing Nothing
-  runSelection us' context
+      selection_config = selectionConfig direction "push" (pushPatchSelOpts opts) Nothing Nothing
+  runSelection us' selection_config
                    >>= bundlePatches opts common
 
 prePushChatter :: forall rt p a wX wY wT . (RepoPatch p, ShowPatch a) =>
@@ -203,7 +225,7 @@ bundlePatches opts common (to_be_pushed :> _) =
       setEnvDarcsPatches to_be_pushed
       printDryRunMessageAndExit "push"
         (verbosity ? opts)
-        (O.summary ? opts)
+        (O.withSummary ? opts)
         (dryRun ? opts)
         (xmlOutput ? opts)
         (isInteractive True opts)
@@ -212,7 +234,7 @@ bundlePatches opts common (to_be_pushed :> _) =
           putInfo opts $
             text "You don't want to push any patches, and that's fine with me!"
           exitSuccess
-      makeBundleN Nothing common (mapFL_FL hopefully to_be_pushed)
+      makeBundle Nothing common (mapFL_FL hopefully to_be_pushed)
 
 checkOptionsSanity :: [DarcsFlag] -> String -> IO ()
 checkOptionsSanity opts repodir =
@@ -232,7 +254,7 @@ pushPatchSelOpts flags = S.PatchSelectionOptions
     , S.matchFlags = parseFlags O.matchSeveral flags
     , S.interactive = isInteractive True flags
     , S.selectDeps = selectDeps ? flags
-    , S.summary = O.summary ? flags
+    , S.withSummary = O.withSummary ? flags
     , S.withContext = withContext ? flags
     }
 
