@@ -20,25 +20,24 @@ module Darcs.UI.Commands.Replace
     , defaultToks
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import qualified Data.ByteString.Char8 as BC
 import qualified Data.ByteString.Lazy as BL
 import qualified Data.ByteString as B
 import Data.Char ( isSpace )
+import Data.List.Ordered ( nubSort )
 import Data.Maybe ( fromJust, isJust )
-import Control.Exception ( catch, IOException )
-import Control.Monad ( unless, filterM, void )
+import Control.Monad ( unless, filterM, void, when )
 import Darcs.Util.Tree( readBlob, modifyTree, findFile, TreeItem(..), Tree
                       , makeBlobBS )
-import Darcs.Util.Path( SubPath, toFilePath, AbsolutePath )
+import Darcs.Util.Path( AbsolutePath )
 import Darcs.UI.Flags
     ( DarcsFlag
-    , verbosity, useCache, dryRun, umask, diffAlgorithm, fixSubPaths )
-import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, (?) )
+    , verbosity, useCache, dryRun, umask, diffAlgorithm, pathsFromArgs )
+import Darcs.UI.Options ( (^), odesc, ocheck, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
-import Darcs.Repository.Flags ( UpdateWorking(..) )
+import Darcs.Repository.Flags ( UpdatePending(..) )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInHashedRepository )
 import Darcs.UI.Completion ( knownFileArgs )
 import Darcs.Repository.Diff( treeDiff )
@@ -55,15 +54,17 @@ import Darcs.Repository
     )
 import Darcs.Patch.TokenReplace ( defaultToks )
 import Darcs.Repository.Prefs ( FileType(TextFile) )
-import Darcs.Util.Path ( floatSubPath )
+import Darcs.Util.Path ( AnchoredPath, displayPath )
+import Darcs.Util.Printer ( Doc, text )
+import Darcs.Util.SignalHandler ( withSignalsBlocked )
 import Darcs.Patch.Witnesses.Ordered ( FL(..), (+>+), concatFL, toFL, nullFL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..), mapSeal, FreeLeft, Gap(..), unFreeLeft, unseal )
 
 replaceDescription :: String
 replaceDescription = "Substitute one word for another."
 
-replaceHelp :: String
-replaceHelp =
+replaceHelp :: Doc
+replaceHelp = text $
  "In addition to line-based patches, Darcs supports a limited form of\n" ++
  "lexical substitution.  Files are treated as sequences of words, and\n" ++
  "each occurrence of the old word is replaced by the new word.\n" ++
@@ -118,7 +119,7 @@ replaceHelp =
  "`[[:alnum:]]`) are NOT supported by `--token-chars`, and will be silently\n" ++
  "treated as a simple set of characters.\n"
 
-replace :: DarcsCommand [DarcsFlag]
+replace :: DarcsCommand
 replace = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "replace"
@@ -137,7 +138,6 @@ replace = DarcsCommand
     , commandBasicOptions = odesc replaceBasicOpts
     , commandDefaults = defaultFlags replaceOpts
     , commandCheckOptions = ocheck replaceOpts
-    , commandParseOptions = onormalise replaceOpts
     }
   where
     replaceBasicOpts = O.tokens ^ O.forceReplace ^ O.repoDir
@@ -151,35 +151,36 @@ replaceArgs fps flags args =
     else knownFileArgs fps flags args
 
 replaceCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
-replaceCmd fps opts (old : new : relfs@(_ : _)) =
-  withRepoLock  (dryRun ? opts) (useCache ? opts) YesUpdateWorking (umask ? opts) $ RepoJob $
+replaceCmd fps opts (old : new : args@(_ : _)) =
+  withRepoLock  (dryRun ? opts) (useCache ? opts) YesUpdatePending (umask ? opts) $ RepoJob $
     \repository -> do
-        fs <- fixSubPaths fps relfs
+        paths <- nubSort <$> pathsFromArgs fps args
+        when (null paths) $ fail "No valid repository paths were given."
         toks <- chooseToks (O.tokens ? opts) old new
         let checkToken tok = unless (isTok toks tok) $
                                  fail $ "'" ++ tok ++ "' is not a valid token!"
         mapM_ checkToken [ old, new ]
-        working <- readUnrecorded repository Nothing
-        files <- filterM (exists working) fs
+        working <- readUnrecorded repository (O.useIndex ? opts) Nothing
+        files <- filterM (exists working) paths
         Sealed replacePs <- mapSeal concatFL . toFL <$>
             mapM (doReplace toks working) files
-        -- Note: addToPending takes care of commuting the replace patch and
-        -- everything it depends on past the diff between pending and working
-        addToPending repository YesUpdateWorking replacePs
-        void $ applyToWorking repository (verbosity ? opts) replacePs `catch` \(e :: IOException) ->
-            bug $ "Can't do replace on working!\n" ++ show e
+        withSignalsBlocked $ do
+          -- Note: addToPending takes care of commuting the replace patch and
+          -- everything it depends on past the diff between pending and working
+          addToPending repository (O.useIndex ? opts) replacePs
+          void $ applyToWorking repository (verbosity ? opts) replacePs
   where
-    exists tree file = if isJust $ findFile tree (floatSubPath file)
+    exists tree file = if isJust $ findFile tree file
                            then return True
                            else do putStrLn $ skipmsg file
                                    return False
 
-    skipmsg f = "Skipping file '" ++ toFilePath f
+    skipmsg f = "Skipping file '" ++ displayPath f
                 ++ "' which isn't in the repository."
 
     doReplace :: forall prim . (PrimPatch prim,
               ApplyState prim ~ Tree) => String -> Tree IO
-              -> SubPath -> IO (FreeLeft (FL prim))
+              -> AnchoredPath -> IO (FreeLeft (FL prim))
     doReplace toks work f = do
         workReplaced <- maybeApplyToTree replacePatch work
         case workReplaced of
@@ -189,12 +190,13 @@ replaceCmd fps opts (old : new : relfs@(_ : _)) =
             | O.forceReplace ? opts -> getForceReplace f toks work
             | otherwise -> putStrLn existsMsg >> return gapNilFL
       where
-        existsMsg = "Skipping file '" ++ fp ++ "'\nPerhaps the working"
+        -- FIXME Why do we say "perhaps"? Aren't we sure? Are there other
+        -- reasons maybeApplyToTree can fail and what to do about them?
+        existsMsg = "Skipping file '" ++ displayPath f ++ "'\nPerhaps the working"
                     ++ " version of this file already contains '" ++ new
                     ++ "'?\nUse the --force option to override."
         gapNilFL = emptyGap NilFL
-        fp = toFilePath f
-        replacePatch = tokreplace fp toks old new
+        replacePatch = tokreplace f toks old new
 
     ftf _ = TextFile
 
@@ -202,10 +204,9 @@ replaceCmd fps opts (old : new : relfs@(_ : _)) =
     -- hunk patches to normalise all occurences of the target token (changing
     -- them back to the source token) and then the replace patches from
     -- oldToken -> newToken.
-    getForceReplace :: PrimPatch prim => SubPath -> String -> Tree IO
-                    -> IO (FreeLeft (FL prim))
-    getForceReplace f toks tree = do
-        let path = floatSubPath f
+    getForceReplace :: PrimPatch prim
+                    => AnchoredPath -> String -> Tree IO -> IO (FreeLeft (FL prim))
+    getForceReplace path toks tree = do
         content <- readBlob $ fromJust $ findFile tree path
         let newcontent = forceTokReplace toks (BC.pack new) (BC.pack old)
                             (B.concat $ BL.toChunks content)
@@ -218,7 +219,7 @@ replaceCmd fps opts (old : new : relfs@(_ : _)) =
                        ++ "so that darcs replace can token-replace them"
                        ++ " back into '" ++ new ++ "' again."
         return . joinGap (+>+) normaliseNewTokPatch $ freeGap $
-            tokreplace (toFilePath f) toks old new :>: NilFL
+            tokreplace path toks old new :>: NilFL
 replaceCmd _ _ [_, _] = fail "You need to supply a list of files to replace in!"
 replaceCmd _ _ _ = fail "Usage: darcs replace <OLD> <NEW> <FILE>..."
 
