@@ -18,24 +18,41 @@
 {-# LANGUAGE OverloadedStrings #-}
 module Darcs.UI.Commands.Revert ( revert ) where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Control.Exception ( catch, IOException )
-import Data.List ( sort )
+import Control.Monad ( void )
 
 import Darcs.UI.Flags
-    ( DarcsFlag, diffingOpts, verbosity, diffAlgorithm, isInteractive, withContext
-    , dryRun, umask, useCache, fixSubPaths )
-import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, (?) )
+    ( DarcsFlag
+    , diffAlgorithm
+    , diffingOpts
+    , dryRun
+    , isInteractive
+    , pathSetFromArgs
+    , umask
+    , useCache
+    , verbosity
+    , withContext
+    )
+import Darcs.UI.Options ( (^), odesc, ocheck, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
-import Darcs.Repository.Flags ( UpdateWorking(..) )
-import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInHashedRepository, putInfo )
+import Darcs.Repository.Flags ( UpdatePending(..) )
+import Darcs.UI.Commands
+    ( DarcsCommand(..)
+    , amInHashedRepository
+    , nodefaults
+    , putInfo
+    , putFinished
+    , withStdOpts
+    )
 import Darcs.UI.Commands.Util ( announceFiles )
 import Darcs.UI.Commands.Unrevert ( writeUnrevert )
 import Darcs.UI.Completion ( modifiedFileArgs )
+
 import Darcs.Util.Global ( debugMessage )
-import Darcs.Util.Path ( toFilePath, AbsolutePath )
+import Darcs.Util.Path ( AbsolutePath )
+import Darcs.Util.Printer ( Doc, text )
+import Darcs.Util.SignalHandler ( withSignalsBlocked )
 import Darcs.Repository
     ( withRepoLock
     , RepoJob(..)
@@ -44,14 +61,21 @@ import Darcs.Repository
     , readRecorded
     , unrecordedChanges
     )
-import Darcs.Patch ( invert, effectOnFilePaths, commute )
+import Darcs.Patch ( invert, effectOnPaths, commuteFL )
+import Darcs.Patch.Permutations ( genCommuteWhatWeCanRL )
 import Darcs.Patch.Split ( reversePrimSplitter )
-import Darcs.Patch.Witnesses.Ordered ( FL(..), (:>)(..), nullFL, (+>+) )
+import Darcs.Patch.Witnesses.Ordered
+    ( FL(..)
+    , (:>)(..)
+    , nullFL
+    , (+>>+)
+    , reverseFL
+    )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
 import Darcs.UI.SelectChanges
     ( WhichChanges(Last)
-    , selectionContextPrim
-    , runSelection
+    , selectionConfigPrim
+    , runInvertibleSelection
     )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions(..) )
 import Darcs.Patch.TouchesFiles ( chooseTouching )
@@ -60,8 +84,8 @@ import Darcs.Patch.TouchesFiles ( chooseTouching )
 revertDescription :: String
 revertDescription = "Discard unrecorded changes."
 
-revertHelp :: String
-revertHelp =
+revertHelp :: Doc
+revertHelp = text $
  "The `darcs revert` command discards unrecorded changes the working\n" ++
  "tree.  As with `darcs record`, you will be asked which hunks (changes)\n" ++
  "to revert.  The `--all` switch can be used to avoid such prompting. If\n" ++
@@ -80,11 +104,11 @@ patchSelOpts flags = S.PatchSelectionOptions
     , S.matchFlags = []
     , S.interactive = isInteractive True flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
-    , S.summary = O.NoSummary -- option not supported, use default
+    , S.withSummary = O.NoSummary -- option not supported, use default
     , S.withContext = withContext ? flags
     }
 
-revert :: DarcsCommand [DarcsFlag]
+revert :: DarcsCommand
 revert = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "revert"
@@ -100,7 +124,6 @@ revert = DarcsCommand
     , commandBasicOptions = odesc revertBasicOpts
     , commandDefaults = defaultFlags revertOpts
     , commandCheckOptions = ocheck revertOpts
-    , commandParseOptions = onormalise revertOpts
     }
   where
     revertBasicOpts
@@ -113,33 +136,44 @@ revert = DarcsCommand
 
 revertCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 revertCmd fps opts args =
- withRepoLock (dryRun ? opts) (useCache ? opts) YesUpdateWorking (umask ? opts) $ RepoJob $ \repository -> do
-  files <- if null args then return Nothing
-    else Just . sort <$> fixSubPaths fps args
+ withRepoLock (dryRun ? opts) (useCache ? opts) YesUpdatePending (umask ? opts) $
+ RepoJob $ \repository -> do
+  files <- pathSetFromArgs fps args
   announceFiles (verbosity ? opts) files "Reverting changes in"
   changes <- unrecordedChanges (diffingOpts opts {- always ScanKnown here -})
     O.NoLookForMoves O.NoLookForReplaces repository files
-  let pre_changed_files = effectOnFilePaths (invert changes) . map toFilePath <$> files
+  let pre_changed_files = effectOnPaths (invert changes) <$> files
   recorded <- readRecorded repository
   Sealed touching_changes <- return (chooseTouching pre_changed_files changes)
   case touching_changes of
     NilFL -> putInfo opts "There are no changes to revert!"
     _ -> do
-      let context = selectionContextPrim
-                          Last "revert" (patchSelOpts opts)
-                          (Just (reversePrimSplitter (diffAlgorithm ? opts)))
-                          pre_changed_files (Just recorded)
-      (norevert:>p) <- runSelection changes context
-      if nullFL p
-       then putInfo opts $ "If you don't want to revert after all, that's fine with me!"
-       else do
-                 addToPending repository YesUpdateWorking $ invert p
+      let selection_config = selectionConfigPrim
+                                Last "revert" (patchSelOpts opts)
+                                (Just (reversePrimSplitter (diffAlgorithm ? opts)))
+                                pre_changed_files (Just recorded)
+      norevert :> torevert <- runInvertibleSelection changes selection_config
+      if nullFL torevert
+       then putInfo opts $
+              "If you don't want to revert after all, that's fine with me!"
+       else withSignalsBlocked $ do
+                 addToPending repository (O.useIndex ? opts) $ invert torevert
                  debugMessage "About to write the unrevert file."
-                 case commute (norevert:>p) of
-                   Just (p':>_) -> writeUnrevert repository p' recorded NilFL
-                   Nothing -> writeUnrevert repository (norevert+>+p) recorded NilFL
-                 debugMessage "About to apply to the working directory."
-                 _ <- applyToWorking repository (verbosity ? opts) (invert p) `catch` \(e :: IOException) ->
-                     fail ("Unable to apply inverse patch!" ++ show e)
-                 return ()
-      putInfo opts "Finished reverting."
+                 {- The user has split unrecorded into the sequence 'norevert' then 'torevert',
+                    which is natural as the bit we keep in unrecorded should have recorded
+                    as the context.
+
+                    But the unrevert patch also needs to have recorded as the context, not
+                    unrecorded (which can be changed by the user at any time).
+
+                    So we need to commute 'torevert' with 'norevert', and if that fails then
+                    we need to keep some of 'norevert' in the actual unrevert patch so it
+                    still makes sense. The use of genCommuteWhatWeCanRL minimises the amount
+                    of 'norevert' that we need to keep.
+                 -}
+                 case genCommuteWhatWeCanRL commuteFL (reverseFL norevert :> torevert) of
+                   deps :> torevert' :> _ ->
+                      writeUnrevert repository (deps +>>+ torevert') recorded NilFL
+                 debugMessage "About to apply to the working tree."
+                 void $ applyToWorking repository (verbosity ? opts) (invert torevert)
+      putFinished opts "reverting"
