@@ -17,59 +17,56 @@
 
 module Darcs.UI.Commands.Rollback ( rollback ) where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Control.Exception ( catch, IOException )
-import Control.Monad ( when )
-import Data.List ( sort )
+import Control.Monad ( when, void )
 import Darcs.Util.Tree( Tree )
 import System.Exit ( exitSuccess )
 
 import Darcs.Patch.Apply( ApplyState )
 import Darcs.Patch.Match ( firstMatch )
 import Darcs.Patch.PatchInfoAnd ( n2pia )
-import Darcs.Patch ( IsRepoType, RepoPatch, invert, effect, fromPrims, sortCoalesceFL,
+import Darcs.Patch ( IsRepoType, RepoPatch, invert, effect, sortCoalesceFL,
                      canonize, PrimOf )
-import Darcs.Patch.Named.Wrapped ( anonymous )
-import Darcs.Patch.Set ( PatchSet(..), patchSet2FL )
+import Darcs.Patch.Named ( anonymous )
+import Darcs.Patch.Set ( PatchSet, Origin, emptyPatchSet, patchSet2FL )
 import Darcs.Patch.Split ( reversePrimSplitter )
-import Darcs.Patch.Witnesses.Ordered ( FL(..), (:>)(..), RL(..), concatFL,
-                                       nullFL, mapFL_FL )
+import Darcs.Patch.Witnesses.Ordered ( Fork(..), FL(..), (:>)(..), concatFL, nullFL, mapFL_FL )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(..) )
 import Darcs.Repository.Flags ( AllowConflicts (..), UseIndex(..), Reorder(..),
-                                ScanKnown(..), UpdateWorking(..), DryRun(NoDryRun))
+                                ScanKnown(..), UpdatePending(..), DryRun(NoDryRun))
 import Darcs.Repository ( Repository, withRepoLock, RepoJob(..),
                           applyToWorking, readRepo,
                           finalizeRepositoryChanges, tentativelyAddToPending,
                           considerMergeToWorking )
 import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, setEnvDarcsPatches,
                            amInHashedRepository, putInfo )
-import Darcs.UI.Commands.Unrecord ( getLastPatches )
-import Darcs.UI.Commands.Util ( announceFiles )
+import Darcs.UI.Commands.Util ( announceFiles, getLastPatches )
 import Darcs.UI.Completion ( knownFileArgs )
 import Darcs.UI.Flags ( DarcsFlag, verbosity, umask, useCache,
                         compress, externalMerge, wantGuiPause,
-                        diffAlgorithm, fixSubPaths, isInteractive )
+                        diffAlgorithm, isInteractive, pathSetFromArgs )
 import Darcs.UI.Options
-    ( (^), odesc, ocheck, onormalise
+    ( (^), odesc, ocheck
     , defaultFlags, parseFlags, (?)
     )
 import qualified Darcs.UI.Options.All as O
 import Darcs.UI.SelectChanges ( WhichChanges(..),
-                                selectionContext, selectionContextPrim,
-                                runSelection )
+                                selectionConfig, selectionConfigPrim,
+                                runSelection, runInvertibleSelection )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions(..) )
-import Darcs.Util.Path ( toFilePath, AbsolutePath )
-import Darcs.Util.Printer ( text )
+import Darcs.Util.Path ( AbsolutePath )
+import Darcs.Util.Printer ( Doc, text )
 import Darcs.Util.Progress ( debugMessage )
+import Darcs.Util.SignalHandler ( withSignalsBlocked )
+
 
 rollbackDescription :: String
 rollbackDescription =
  "Apply the inverse of recorded changes to the working tree."
 
-rollbackHelp :: String
-rollbackHelp = unlines
+rollbackHelp :: Doc
+rollbackHelp = text $ unlines
     [ "Rollback is used to undo the effects of some changes from patches"
     , "in the repository. The selected changes are undone in your working"
     , "tree, but the repository is left unchanged. First you are offered a"
@@ -88,11 +85,11 @@ patchSelOpts flags = S.PatchSelectionOptions
     , S.matchFlags = parseFlags O.matchSeveralOrLast flags
     , S.interactive = isInteractive True flags
     , S.selectDeps = O.PromptDeps
-    , S.summary = O.NoSummary
+    , S.withSummary = O.NoSummary
     , S.withContext = O.NoContext
     }
 
-rollback :: DarcsCommand [DarcsFlag]
+rollback :: DarcsCommand
 rollback = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "rollback"
@@ -108,7 +105,6 @@ rollback = DarcsCommand
     , commandBasicOptions = odesc rollbackBasicOpts
     , commandDefaults = defaultFlags rollbackOpts
     , commandCheckOptions = ocheck rollbackOpts
-    , commandParseOptions = onormalise rollbackOpts
     }
   where
     rollbackBasicOpts
@@ -125,52 +121,49 @@ exitIfNothingSelected ps what =
 
 rollbackCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 rollbackCmd fps opts args = withRepoLock NoDryRun (useCache ? opts)
-    YesUpdateWorking (umask ? opts) $ RepoJob $ \repository -> do
-        files <- if null args
-                     then return Nothing
-                     else Just . sort <$> fixSubPaths fps args
-        when (files == Just []) $
-            fail "No valid arguments were given."
+    YesUpdatePending (umask ? opts) $ RepoJob $ \repository -> do
+        files <- pathSetFromArgs fps args
         announceFiles (verbosity ? opts) files "Rolling back changes in"
         allpatches <- readRepo repository
         let matchFlags = parseFlags O.matchSeveralOrLast opts
         (_ :> patches) <- return $
             if firstMatch matchFlags
                 then getLastPatches matchFlags allpatches
-                else PatchSet NilRL NilRL :> patchSet2FL allpatches
-        let filesFps = map toFilePath <$> files
-            patchCtx = selectionContext LastReversed "rollback" (patchSelOpts opts) Nothing filesFps
+                else emptyPatchSet :> patchSet2FL allpatches
         (_ :> ps) <-
-            runSelection patches patchCtx
+            runSelection patches $
+                selectionConfig LastReversed "rollback" (patchSelOpts opts) Nothing files
+
         exitIfNothingSelected ps "patches"
         setEnvDarcsPatches ps
-        let hunkContext = selectionContextPrim Last "rollback" (patchSelOpts opts)
-                              (Just (reversePrimSplitter (diffAlgorithm ? opts)))
-                              filesFps Nothing
-            hunks = concatFL . mapFL_FL (canonize $ diffAlgorithm ? opts) . sortCoalesceFL . effect $ ps
-        whatToUndo <- runSelection hunks hunkContext
-        undoItNow opts repository whatToUndo
+        let prim_selection_context =
+              selectionConfigPrim
+                  Last "rollback" (patchSelOpts opts)
+                  (Just (reversePrimSplitter (diffAlgorithm ? opts)))
+                  files Nothing
+            hunks = concatFL . mapFL_FL (canonize $ diffAlgorithm ? opts) . sortCoalesceFL . effect
+        whatToUndo <- runInvertibleSelection (hunks ps) prim_selection_context
+        undoItNow opts repository allpatches whatToUndo
 
 undoItNow :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
-          => [DarcsFlag] -> Repository rt p wR wU wT
-          -> (q :> FL (PrimOf p)) wA wT -> IO ()
-undoItNow opts repo (_ :> prims) = do
+          => [DarcsFlag] -> Repository rt p wR wU wR
+          -> PatchSet rt p Origin wR
+          -> (q :> FL (PrimOf p)) wA wR -> IO ()
+undoItNow opts _repo context (_ :> prims) = do
     exitIfNothingSelected prims "changes"
-    rbp <- n2pia `fmap` anonymous (fromPrims $ invert prims)
-    Sealed pw <- considerMergeToWorking repo "rollback"
-                     YesAllowConflictsAndMark YesUpdateWorking
+    -- Note: use of anonymous is unproblematic here because we
+    -- only store effects by adding them to pending and working)
+    rbp <- n2pia `fmap` anonymous (invert prims)
+    Sealed pw <- considerMergeToWorking _repo "rollback"
+                     YesAllowConflictsAndMark
                      (externalMerge ? opts) (wantGuiPause opts)
                      (compress ? opts) (verbosity ? opts) NoReorder
                      (UseIndex, ScanKnown, diffAlgorithm ? opts)
-                     NilFL (rbp :>: NilFL)
-    tentativelyAddToPending repo YesUpdateWorking pw
-    finalizeRepositoryChanges repo YesUpdateWorking
-        (compress ? opts)
-    _ <- applyToWorking repo (verbosity ? opts) pw
-            `catch`
-            \(e :: IOException) -> fail $
-                "error applying rolled back patch to working directory\n"
-                ++ show e
+                     (Fork context NilFL (rbp :>: NilFL))
+    tentativelyAddToPending _repo pw
+    withSignalsBlocked $ do
+        _repo <- finalizeRepositoryChanges _repo YesUpdatePending (compress ? opts)
+        void $ applyToWorking _repo (verbosity ? opts) pw
     debugMessage "Finished applying unrecorded rollback patch"
-    putInfo opts $ text "Changes rolled back in working directory"
+    putInfo opts $ text "Changes rolled back in working tree"
 
