@@ -15,20 +15,14 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 module Darcs.UI.Commands.Send ( send ) where
 
 import Darcs.Prelude
 
 import System.Directory ( renameFile )
-import System.Exit
-    ( exitSuccess
-#ifndef HAVE_MAPI
-    , ExitCode ( ExitFailure )
-    , exitWith
-#endif
-    )
+import System.Exit ( exitSuccess )
 import System.IO ( hClose )
 import Control.Exception ( catch, IOException, onException )
 import Control.Monad ( when, unless, forM_ )
@@ -105,11 +99,7 @@ import Darcs.UI.External
     , sendEmailDoc
     , generateEmail
     , editFile
-    , getSystemEncoding
-    , isUTF8Locale
-#ifndef HAVE_MAPI
-    , haveSendmail
-#endif
+    , checkDefaultSendmail
     )
 import Darcs.Util.ByteString ( mmapFilePS, isAscii )
 import qualified Data.ByteString.Char8 as BC (unpack)
@@ -179,7 +169,8 @@ send = DarcsCommand
       ^ O.headerFields
       ^ O.author
       ^ O.charset
-      ^ O.sendmail
+      ^ O.mail
+      ^ O.sendmailCmd
       ^ O.output
       ^ O.sign
       ^ O.dryRunXml
@@ -203,6 +194,12 @@ sendCmd fps opts [""] = sendCmd fps opts []
 sendCmd (_,o) opts [unfixedrepodir] =
  withRepository (useCache ? opts) $ RepoJob $
   \(repository :: Repository rt p wR wU wR) -> do
+  when (O.mail ? opts && dryRun ? opts == O.NoDryRun) $ do
+    -- If --mail is used and the user has not provided a --sendmail-command
+    -- and we can detect that the system has no default way to send emails, 
+    -- then we want to fail early i.e. before asking the user any questions.
+    sm_cmd <- getSendmailCmd opts
+    when (isNothing sm_cmd) checkDefaultSendmail
   case O.sendToContext ? opts of
     Just contextfile -> do
         wtds <- decideOnBehavior opts (Nothing :: Maybe (Repository rt p wR wU wR))
@@ -230,16 +227,6 @@ sendToThem :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
            => Repository rt p wR wU wT -> [DarcsFlag] -> [WhatToDo] -> String
            -> PatchSet rt p Origin wX -> IO ()
 sendToThem repo opts wtds their_name them = do
-#ifndef HAVE_MAPI
-  when (fst (O.sendmail ? opts) && dryRun ? opts == O.NoDryRun) $ do
-    -- If --mail is used, check if the user has sendmail or
-    -- provided a --sendmail-cmd
-    sendmail <- haveSendmail
-    sm_cmd <- getSendmailCmd opts
-    when (not sendmail && sm_cmd == "") $ do
-        putInfo opts noWorkingSendmail
-        exitWith $ ExitFailure 1
-#endif
   us <- readRepo repo
   common :> us' <- return $ findCommonWithThem us them
   checkUnrelatedRepos (O.allowUnrelatedRepos ? opts) us them
@@ -276,7 +263,7 @@ sendToThem repo opts wtds their_name them = do
   fname <- make_fname to_be_sent
   let outname = case getOutput opts fname of
                     Just f  -> Just f
-                    Nothing | fst (O.sendmail ? opts) -> Nothing
+                    Nothing | O.mail ? opts -> Nothing
                             | not $ null [ p | Post p <- wtds] -> Nothing
                             | otherwise        -> Just (makeAbsoluteOrStd here fname)
   case outname of
@@ -339,13 +326,7 @@ sendBundle opts to_be_sent bundle fname wtds their_name=
                                 Nothing -> do
                                   warnCharset charsetCouldNotGuess
                                   return mailcharset
-                                Just "utf-8" -> do
-                                  -- Check the locale encoding for consistency
-                                  encoding <- getSystemEncoding
-                                  debugMessage $ currentEncodingIs encoding
-                                  unless (isUTF8Locale encoding) $
-                                    warnCharset charsetUtf8MailDiffLocale
-                                  return mailcharset
+                                Just "utf-8" -> return mailcharset
                                 -- Trust other cases (us-ascii)
                                 Just _ -> return mailcharset
 
@@ -359,8 +340,8 @@ sendBundle opts to_be_sent bundle fname wtds their_name=
 
                sendmail =
                 (do
-                 sm_cmd <- getSendmailCmd opts
                  let to = generateEmailToString thetargets
+                 sm_cmd <- getSendmailCmd opts
                  sendEmailDoc from to thesubject (getCc opts)
                                sm_cmd contentAndBundle body
                  putInfo opts (success to (getCc opts)))
@@ -600,11 +581,6 @@ cannotSendToSelf = "Can't send to current repository! Did you mean send --contex
 creatingPatch :: String -> Doc
 creatingPatch repodir = "Creating patch to" <+> quoted repodir <> "..."
 
-#ifndef HAVE_MAPI
-noWorkingSendmail :: Doc
-noWorkingSendmail = "No working sendmail instance on your machine!"
-#endif
-
 nothingSendable :: Doc
 nothingSendable = "No recorded local changes to send!"
 
@@ -626,12 +602,6 @@ charsetAborted = "Aborted.  You can specify charset with the --charset option."
 charsetCouldNotGuess :: String
 charsetCouldNotGuess = "darcs could not guess the charset of your mail."
 
-currentEncodingIs :: String -> String
-currentEncodingIs e = "Current locale encoding: " ++ e
-
-charsetUtf8MailDiffLocale :: String
-charsetUtf8MailDiffLocale = "your mail is valid UTF-8 but your locale differs."
-
 aborted :: Doc
 aborted = "Aborted."
 
