@@ -28,7 +28,6 @@ import Data.Ord ( comparing )
 import System.IO.Error ( catchIOError )
 import Text.ParserCombinators.Parsec hiding ( lower, count, Line )
 import Text.ParserCombinators.Parsec.Error
-import Text.Regex ( Regex, mkRegexWithOpts, matchRegex )
 
 import Darcs.Prelude
 
@@ -46,6 +45,7 @@ import Darcs.Patch.Witnesses.Ordered ( mapRL )
 import Darcs.Util.Lock ( readTextFile )
 import Darcs.Util.Printer ( Doc, text )
 import Darcs.Util.Path ( AbsolutePath )
+import Darcs.Util.Regex ( Regex, mkRegexWithOpts, matchRegex )
 
 data Spelling = Spelling String String [Regex] -- name, email, regexps
 type ParsedLine = Maybe Spelling -- Nothing for blank lines
