@@ -38,7 +38,7 @@ import Darcs.Repository.State ( readRecorded )
 
 import Darcs.Util.Hash( encodeBase16, Hash( NoHash ) )
 import Darcs.Util.Tree( list, expand, itemHash, Tree, TreeItem( SubTree ) )
-import Darcs.Util.Index( updateIndex, listFileIDs )
+import Darcs.Util.Index( treeFromIndex, listFileIDs )
 import Darcs.Util.Path( anchorPath, AbsolutePath, floatPath )
 import Darcs.Util.Printer ( Doc, text )
 
@@ -98,7 +98,7 @@ dump opts fileids tree = do
 showIndexCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 showIndexCmd _ opts _ = withRepository (useCache ? opts) $ RepoJob $ \repo ->
   do index <- readIndex repo
-     index_tree <- updateIndex index
+     index_tree <- treeFromIndex index
      fileids <- (M.fromList . map (\((a,_),b) -> (anchorPath "" a,b))) <$> listFileIDs index
      dump opts (Just fileids) index_tree
 
