@@ -17,51 +17,57 @@
 
 module Darcs.UI.Commands.Unrevert ( unrevert, writeUnrevert ) where
 
-import Prelude ()
 import Darcs.Prelude
 
-import Control.Exception ( catch, IOException )
 import System.Exit ( exitSuccess )
 import Darcs.Util.Tree( Tree )
 
-import Darcs.UI.Commands ( DarcsCommand(..), withStdOpts, nodefaults, amInHashedRepository )
+import Darcs.UI.Commands
+    ( DarcsCommand(..)
+    , withStdOpts
+    , nodefaults
+    , amInHashedRepository
+    , putFinished
+    )
 import Darcs.UI.Completion ( noArgs )
 import Darcs.UI.Flags
     ( diffingOpts, verbosity, useCache, umask, compress, diffAlgorithm
     , isInteractive, withContext )
 import Darcs.Repository.Flags
     ( UseIndex(..), ScanKnown (..), Reorder(..), AllowConflicts(..), ExternalMerge(..)
-    , WantGuiPause(..), UpdateWorking(..), DryRun(NoDryRun) )
+    , WantGuiPause(..), UpdatePending(..), DryRun(NoDryRun) )
 import Darcs.UI.Flags ( DarcsFlag )
-import Darcs.UI.Options ( (^), odesc, ocheck, onormalise, defaultFlags, (?) )
+import Darcs.UI.Options ( (^), odesc, ocheck, defaultFlags, (?) )
 import qualified Darcs.UI.Options.All as O
 import Darcs.Repository ( SealedPatchSet, Repository, withRepoLock, RepoJob(..),
-                          unrevertUrl, considerMergeToWorking,
+                          considerMergeToWorking,
                           tentativelyAddToPending, finalizeRepositoryChanges,
                           readRepo,
                           readRecorded,
                           applyToWorking, unrecordedChanges )
-import Darcs.Patch ( IsRepoType, RepoPatch, PrimOf, commute, fromPrims )
+import Darcs.Repository.Paths ( unrevertPath )
+import Darcs.Patch ( IsRepoType, RepoPatch, PrimOf, commute )
 import Darcs.Patch.Apply( ApplyState )
-import Darcs.Patch.Named.Wrapped ( namepatch )
-import Darcs.Patch.Rebase ( dropAnyRebase )
-import Darcs.Patch.Set ( Origin )
+import Darcs.Patch.Info ( patchinfo )
+import Darcs.Patch.Named ( infopatch )
+import Darcs.Patch.Set ( PatchSet, Origin )
 import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed) )
-import Darcs.Patch.Witnesses.Ordered ( FL(..), (:>)(..), (+>+) )
+import Darcs.Patch.Witnesses.Ordered ( Fork(..), FL(..), (:>)(..), (+>+) )
 import Darcs.UI.SelectChanges
     ( WhichChanges(First)
-    , runSelection
-    , selectionContextPrim
+    , runInvertibleSelection
+    , selectionConfigPrim
     )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
 import qualified Data.ByteString as B
 import Darcs.Util.Lock ( writeDocBinFile, removeFileMayNotExist )
 import Darcs.Patch.Depends ( mergeThem )
-import Darcs.UI.External ( catchall )
+import Darcs.Util.Exception ( catchall )
 import Darcs.Util.Prompt ( askUser )
-import Darcs.Patch.Bundle ( scanBundle, makeBundleN )
+import Darcs.Patch.Bundle ( parseBundle, interpretBundle, makeBundle )
 import Darcs.Util.IsoDate ( getIsoDateTime )
 import Darcs.Util.SignalHandler ( withSignalsBlocked )
+import Darcs.Util.Printer ( Doc, text )
 import Darcs.Util.Progress ( debugMessage )
 import Darcs.Util.Path ( AbsolutePath )
 
@@ -69,8 +75,8 @@ unrevertDescription :: String
 unrevertDescription =
  "Undo the last revert."
 
-unrevertHelp :: String
-unrevertHelp =
+unrevertHelp :: Doc
+unrevertHelp = text $
  "Unrevert is a rescue command in case you accidentally reverted\n" ++
  "something you wanted to keep (for example, typing `darcs rev -a`\n" ++
  "instead of `darcs rec -a`).\n" ++
@@ -85,11 +91,11 @@ patchSelOpts flags = S.PatchSelectionOptions
     , S.matchFlags = []
     , S.interactive = isInteractive True flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
-    , S.summary = O.NoSummary -- option not supported, use default
+    , S.withSummary = O.NoSummary -- option not supported, use default
     , S.withContext = withContext ? flags
     }
 
-unrevert :: DarcsCommand [DarcsFlag]
+unrevert :: DarcsCommand
 unrevert = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "unrevert"
@@ -105,7 +111,6 @@ unrevert = DarcsCommand
     , commandBasicOptions = odesc unrevertBasicOpts
     , commandDefaults = defaultFlags unrevertOpts
     , commandCheckOptions = ocheck unrevertOpts
-    , commandParseOptions = onormalise unrevertOpts
     }
   where
     unrevertBasicOpts
@@ -119,36 +124,37 @@ unrevert = DarcsCommand
 
 unrevertCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 unrevertCmd _ opts [] =
- withRepoLock NoDryRun (useCache ? opts) YesUpdateWorking (umask ? opts) $ RepoJob $ \repository -> do
-  us <- readRepo repository
-  Sealed them <- unrevertPatchBundle repository
-  recorded <- readRecorded repository
+ withRepoLock NoDryRun (useCache ? opts) YesUpdatePending (umask ? opts) $ RepoJob $ \_repository -> do
+  us <- readRepo _repository
+  Sealed them <- unrevertPatchBundle us
+  recorded <- readRecorded _repository
   unrecorded <- unrecordedChanges (diffingOpts opts {- always ScanKnown here -})
-    O.NoLookForMoves O.NoLookForReplaces repository Nothing
+    O.NoLookForMoves O.NoLookForReplaces _repository Nothing
   Sealed h_them <- return $ mergeThem us them
-  Sealed pw <- considerMergeToWorking repository "unrevert"
-                      YesAllowConflictsAndMark YesUpdateWorking
+  Sealed pw <- considerMergeToWorking _repository "unrevert"
+                      YesAllowConflictsAndMark
                       NoExternalMerge NoWantGuiPause
                       (compress ? opts) (verbosity ? opts) NoReorder
                       ( UseIndex, ScanKnown, diffAlgorithm ? opts )
-                      NilFL h_them
-  let context = selectionContextPrim First "unrevert" (patchSelOpts opts) Nothing Nothing (Just recorded)
-  (p :> skipped) <- runSelection pw context
-  tentativelyAddToPending repository YesUpdateWorking p
+                      (Fork us NilFL h_them)
+  let selection_config =
+        selectionConfigPrim
+            First "unrevert" (patchSelOpts opts)
+            Nothing Nothing (Just recorded)
+  (p :> skipped) <- runInvertibleSelection pw selection_config
+  tentativelyAddToPending _repository p
   withSignalsBlocked $
-      do finalizeRepositoryChanges repository YesUpdateWorking (compress ? opts)
-         _ <- applyToWorking repository (verbosity ? opts) p `catch` \(e :: IOException) ->
-             fail ("Error applying unrevert to working directory...\n"
-                   ++ show e)
+      do _repository <- finalizeRepositoryChanges _repository YesUpdatePending (compress ? opts)
+         _ <- applyToWorking _repository (verbosity ? opts) p
          debugMessage "I'm about to writeUnrevert."
-         writeUnrevert repository skipped recorded (unrecorded+>+p)
-  debugMessage "Finished unreverting."
-unrevertCmd _ _ _ = impossible
+         writeUnrevert _repository skipped recorded (unrecorded+>+p)
+  putFinished opts "unreverting"
+unrevertCmd _ _ _ = error "impossible case"
 
 writeUnrevert :: (IsRepoType rt, RepoPatch p, ApplyState p ~ Tree)
               => Repository rt p wR wU wT -> FL (PrimOf p) wX wY
               -> Tree IO -> FL (PrimOf p) wR wX -> IO ()
-writeUnrevert repository NilFL _ _ = removeFileMayNotExist $ unrevertUrl repository
+writeUnrevert _ NilFL _ _ = removeFileMayNotExist unrevertPath
 writeUnrevert repository ps recorded pend =
   case commute (pend :> ps) of
     Nothing -> do really <- askUser "You will not be able to unrevert this operation! Proceed? "
@@ -156,19 +162,22 @@ writeUnrevert repository ps recorded pend =
                                  _ -> exitSuccess
                   writeUnrevert repository NilFL recorded pend
     Just (p' :> _) -> do
-        rep <- dropAnyRebase <$> readRepo repository
+        rep <- readRepo repository
         date <- getIsoDateTime
-        np <- namepatch date "unrevert" "anon" [] (fromRepoPrims repository p')
-        bundle <- makeBundleN (Just recorded) rep (np :>: NilFL)
-        writeDocBinFile (unrevertUrl repository) bundle
-        where fromRepoPrims :: RepoPatch p => Repository rt p wR wU wT -> FL (PrimOf p) wR wY -> FL p wR wY
-              fromRepoPrims _ = fromPrims
+        info <- patchinfo date "unrevert" "anon" []
+        let np = infopatch info p'
+        bundle <- makeBundle (Just recorded) rep (np :>: NilFL)
+        writeDocBinFile unrevertPath bundle
 
-unrevertPatchBundle :: RepoPatch p => Repository rt p wR wU wT -> IO (SealedPatchSet rt p Origin)
-unrevertPatchBundle repository = do
-  pf <- B.readFile (unrevertUrl repository)
+unrevertPatchBundle :: RepoPatch p
+                    => PatchSet rt p Origin wR
+                    -> IO (SealedPatchSet rt p Origin)
+unrevertPatchBundle us = do
+  pf <- B.readFile unrevertPath
         `catchall` fail "There's nothing to unrevert!"
-  case scanBundle pf of
-      Right ps -> return ps
+  case parseBundle pf of
+      Right (Sealed bundle) -> do
+        case interpretBundle us bundle of
+          Left msg -> fail msg
+          Right ps -> return (Sealed ps)
       Left err -> fail $ "Couldn't parse unrevert patch:\n" ++ err
-
