@@ -15,46 +15,45 @@
 --  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 --  Boston, MA 02110-1301, USA.
 
+{-# LANGUAGE OverloadedStrings #-}
 module Darcs.UI.Commands.WhatsNew
-    (
-      whatsnew
+    ( whatsnew
     , status
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Control.Monad ( void, when )
 import Control.Monad.Reader ( runReaderT )
 import Control.Monad.State ( evalStateT, liftIO )
-import Darcs.Util.Tree ( Tree )
 import System.Exit ( ExitCode (..), exitSuccess, exitWith )
-import Data.List.Ordered ( nubSort )
 
 import Darcs.Patch
     ( PrimOf, PrimPatch, RepoPatch
     , applyToTree, plainSummaryPrims, primIsHunk
     )
-import Darcs.Patch.Apply ( Apply, ApplyState )
+import Darcs.Patch.Apply ( ApplyState )
 import Darcs.Patch.Choices ( mkPatchChoices, labelPatches, unLabel )
 import Darcs.Patch.Commute ( Commute )
 import Darcs.Patch.FileHunk ( IsHunk (..) )
-import Darcs.Patch.Format ( PatchListFormat )
 import Darcs.Patch.Inspect ( PatchInspect (..) )
 import Darcs.Patch.Permutations ( partitionRL )
 import Darcs.Patch.Prim.Class ( PrimDetails (..) )
-import Darcs.Patch.Show ( ShowPatch, ShowContextPatch )
-import Darcs.Patch.Split ( primSplitter )
-import Darcs.Patch.TouchesFiles ( choosePreTouching )
+import Darcs.Patch.Show
+    ( ShowContextPatch
+    , ShowPatch(..)
+    , ShowPatchBasic(..)
+    , displayPatch
+    )
+import Darcs.Patch.TouchesFiles ( chooseTouching )
 import Darcs.Patch.Witnesses.Ordered
-    ( (:>) (..), FL (..), RL (..)
-    , lengthFL, reverseFL, reverseRL
+    ( (:>) (..), FL (..)
+    , reverseFL, reverseRL
     )
 import Darcs.Patch.Witnesses.Sealed
     ( Sealed (..), Sealed2 (..)
     , unFreeLeft
     )
-import Darcs.Patch.Witnesses.WZipper ( FZipper (..) )
 import Darcs.Repository
     ( RepoJob (..), Repository
     , readRecorded
@@ -68,38 +67,39 @@ import Darcs.UI.Commands
     )
 import Darcs.UI.Completion ( modifiedFileArgs )
 import Darcs.UI.Commands.Util ( announceFiles, filterExistingPaths )
+import Darcs.UI.External ( viewDocWith )
 import Darcs.UI.Flags
     ( DarcsFlag, diffAlgorithm
-    , withContext, useCache, fixSubPaths
+    , withContext, useCache, pathSetFromArgs
     , verbosity, isInteractive
     , lookForAdds, lookForMoves, lookForReplaces
     , scanKnown, useIndex, diffingOpts
     )
 import Darcs.UI.Options
-    ( DarcsOption, (^), odesc, ocheck, onormalise, defaultFlags, parseFlags, (?) )
+    ( DarcsOption, (^), odesc, ocheck, defaultFlags, parseFlags, (?) )
 import qualified Darcs.UI.Options.All as O
-import Darcs.UI.PrintPatch
-    ( contextualPrintPatch, printPatch
-    , printPatchPager
-    )
+import Darcs.UI.PrintPatch ( contextualPrintPatch )
 import Darcs.UI.SelectChanges
-    ( InteractiveSelectionContext (..)
-    , InteractiveSelectionM, KeyPress (..)
-    , WhichChanges (..), backAll
+    ( InteractiveSelectionM, KeyPress (..)
+    , WhichChanges (..)
+    , initialSelectionState
+    , backAll
     , backOne, currentFile
     , currentPatch, decide
     , decideWholeFile, helpFor
     , keysFor, prompt
-    , selectionContextPrim, skipMundane
-    , skipOne, printSummary
+    , selectionConfigPrim, skipMundane
+    , skipOne
     )
 import qualified Darcs.UI.SelectChanges as S ( PatchSelectionOptions (..) )
-import Darcs.Util.Path ( AbsolutePath, SubPath, toFilePath )
+import Darcs.Util.Path ( AbsolutePath, AnchoredPath )
 import Darcs.Util.Printer
-    ( putDocLn, renderString
-    , text, vcat
+    ( Doc, formatWords, putDocLn, renderString
+    , text, vcat, ($+$)
     )
+import Darcs.Util.Printer.Color ( fancyPrinters )
 import Darcs.Util.Prompt ( PromptConfig (..), promptChar )
+import Darcs.Util.Tree ( Tree )
 
 commonAdvancedOpts :: DarcsOption a (O.UseIndex -> O.IncludeBoring -> a)
 commonAdvancedOpts = O.useIndex ^ O.includeBoring
@@ -110,14 +110,14 @@ patchSelOpts flags = S.PatchSelectionOptions
     , S.matchFlags = []
     , S.interactive = isInteractive True flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
-    , S.summary = getSummary flags
+    , S.withSummary = getSummary flags
     , S.withContext = withContext ? flags
     }
 
 -- lookForAdds and machineReadable set YesSummary
 -- unless NoSummary was given expressly
 -- (or by default e.g. status)
-getSummary :: [DarcsFlag] -> O.Summary
+getSummary :: [DarcsFlag] -> O.WithSummary
 getSummary flags = case O.maybeSummary Nothing ? flags of
   Just O.NoSummary -> O.NoSummary
   Just O.YesSummary -> O.YesSummary
@@ -126,7 +126,7 @@ getSummary flags = case O.maybeSummary Nothing ? flags of
     | O.machineReadable ? flags -> O.YesSummary
     | otherwise -> O.NoSummary
 
-whatsnew :: DarcsCommand [DarcsFlag]
+whatsnew :: DarcsCommand
 whatsnew = DarcsCommand
     { commandProgramName = "darcs"
     , commandName = "whatsnew"
@@ -142,7 +142,6 @@ whatsnew = DarcsCommand
     , commandBasicOptions = odesc whatsnewBasicOpts
     , commandDefaults = defaultFlags whatsnewOpts
     , commandCheckOptions = ocheck whatsnewOpts
-    , commandParseOptions = onormalise whatsnewOpts
     }
   where
     whatsnewBasicOpts
@@ -158,49 +157,54 @@ whatsnew = DarcsCommand
 whatsnewDescription :: String
 whatsnewDescription = "List unrecorded changes in the working tree."
 
-whatsnewHelp :: String
+whatsnewHelp :: Doc
 whatsnewHelp =
- "The `darcs whatsnew` command lists unrecorded changes to the working\n" ++
- "tree.  If you specify a set of files and directories, only unrecorded\n" ++
- "changes to those files and directories are listed.\n" ++
- "\n" ++
- "With the `--summary` option, the changes are condensed to one line per\n" ++
- "file, with mnemonics to indicate the nature and extent of the change.\n" ++
- "The `--look-for-adds` option causes candidates for `darcs add` to be\n" ++
- "included in the summary output.  Summary mnemonics are as follows:\n" ++
- "\n" ++
- "* `A f` and `A d/` respectively mean an added file or directory.\n" ++
- "* `R f` and `R d/` respectively mean a removed file or directory.\n" ++
- "* `M f -N +M rP` means a modified file, with `N` lines deleted, `M`\n" ++
- "  lines added, and `P` lexical replacements.\n" ++
- "* `f -> g` means a moved file or directory.\n" ++
- "* `a f` and `a d/` respectively mean a new, but unadded, file or\n" ++
- "  directory, when using `--look-for-adds`.\n" ++
- "\n" ++
- "  An exclamation mark (!) as in `R! foo.c`, means the change is known to\n" ++
- "  conflict with a change in another patch.  The phrase `duplicated`\n" ++
- "  means the change is known to be identical to a change in another patch.\n" ++
- "\n" ++
- "The `--machine-readable` option implies `--summary` while making it more\n" ++
- "parsable. Modified files are only shown as `M f`, and moves are shown in\n" ++
- "two lines: `F f` and `T g` (as in 'From f To g').\n" ++
- "\n" ++
- "By default, `darcs whatsnew` uses Darcs' internal format for changes.\n" ++
- "To see some context (unchanged lines) around each change, use the\n" ++
- "`--unified` option.  To view changes in conventional `diff` format, use\n" ++
- "the `darcs diff` command; but note that `darcs whatsnew` is faster.\n" ++
- "\n" ++
- "This command exits unsuccessfully (returns a non-zero exit status) if\n" ++
- "there are no unrecorded changes.\n"
+  formatWords
+  [ "The `darcs whatsnew` command lists unrecorded changes to the working"
+  , "tree.  If you specify a set of files and directories, only unrecorded"
+  , "changes to those files and directories are listed."
+  ]
+  $+$ formatWords
+  [ "With the `--summary` option, the changes are condensed to one line per"
+  , "file, with mnemonics to indicate the nature and extent of the change."
+  , "The `--look-for-adds` option causes candidates for `darcs add` to be"
+  , "included in the summary output.  WithSummary mnemonics are as follows:"
+  ]
+  -- TODO autoformat bullet lists
+  $+$ vcat
+  [ "  * `A f` and `A d/` respectively mean an added file or directory."
+  , "  * `R f` and `R d/` respectively mean a removed file or directory."
+  , "  * `M f -N +M rP` means a modified file, with `N` lines deleted, `M`"
+  , "    lines added, and `P` lexical replacements."
+  , "  * `f -> g` means a moved file or directory."
+  , "  * `a f` and `a d/` respectively mean a new, but unadded, file or"
+  , "    directory, when using `--look-for-adds`."
+  , "  * An exclamation mark (!) as in `R! foo.c`, means the change"
+  , "    conflicts with a change in an earlier patch. The phrase `duplicated`"
+  , "    means the change is identical to a change in an earlier patch."
+  ]
+  $+$ formatWords
+  [ "The `--machine-readable` option implies `--summary` while making it more"
+  , "parsable. Modified files are only shown as `M f`, and moves are shown in"
+  , "two lines: `F f` and `T g` (as in 'From f To g')."
+  ]
+  $+$ formatWords
+  [ "By default, `darcs whatsnew` uses Darcs' internal format for changes."
+  , "To see some context (unchanged lines) around each change, use the"
+  , "`--unified` option.  To view changes in conventional `diff` format, use"
+  , "the `darcs diff` command; but note that `darcs whatsnew` is faster."
+  ]
+  $+$ formatWords
+  [ "This command exits unsuccessfully (returns a non-zero exit status) if"
+  , "there are no unrecorded changes."
+  ]
 
 whatsnewCmd :: (AbsolutePath, AbsolutePath) -> [DarcsFlag] -> [String] -> IO ()
 whatsnewCmd fps opts args =
    withRepository (useCache ? opts) $ RepoJob $ \(repo :: Repository rt p wR wU wR) -> do
     let scan = scanKnown (lookForAdds opts) (O.includeBoring ? opts)
     existing_files <- do
-      files <- if null args then return Nothing
-               else Just . nubSort <$> fixSubPaths fps args
-      when (files == Just []) $ fail "No valid arguments were given."
+      files <- pathSetFromArgs fps args
       files' <- traverse
         (filterExistingPaths
           repo (verbosity ? opts) (useIndex ? opts) scan (lookForMoves opts))
@@ -252,7 +256,7 @@ whatsnewCmd fps opts args =
     if maybeIsInteractive opts
       then
         runInteractive (interactiveHunks pristine) (patchSelOpts opts)
-          (diffAlgorithm ? opts) pristine allInterestingChanges
+          pristine allInterestingChanges
       else
         if haveLookForAddsAndSummary
           then do
@@ -294,50 +298,39 @@ whatsnewCmd fps opts args =
 
     -- Appropriately print changes, according to the passed flags.
     -- Note this cannot make distinction between unadded and added files.
-    printChanges :: ( IsHunk p, ShowPatch p, ShowContextPatch p
-                    , PatchListFormat p, Apply p
-                    , PrimDetails p, ApplyState p ~ Tree)
+    printChanges :: ( PrimPatch p, ApplyState p ~ Tree)
                  => Tree IO -> FL p wX wY
                  -> IO ()
     printChanges pristine changes
         | haveSummary = putDocLn $ plainSummaryPrims machineReadable changes
         | O.yes (withContext ? opts) = contextualPrintPatch pristine changes
-        | otherwise = printPatch changes
+        | otherwise = printPatchPager changes
      where machineReadable = parseFlags O.machineReadable opts
 
     -- return the unrecorded changes that affect an optional list of paths.
-    filteredUnrecordedChanges :: forall rt p wR wU wT. (RepoPatch p, ApplyState p ~ Tree)
+    filteredUnrecordedChanges :: forall rt p wR wU. (RepoPatch p, ApplyState p ~ Tree)
                               => (O.UseIndex, O.ScanKnown, O.DiffAlgorithm)
                               -> O.LookForMoves
                               -> O.LookForReplaces
-                              -> Repository rt p wR wU wT -> Maybe [SubPath]
-                              -> IO (Sealed (FL (PrimOf p) wT))
-    filteredUnrecordedChanges diffing lfm lfr repo files =
-        let filePaths = map toFilePath <$> files in
-        choosePreTouching filePaths <$>
-          unrecordedChanges diffing lfm lfr repo files
+                              -> Repository rt p wR wU wR
+                              -> Maybe [AnchoredPath]
+                              -> IO (Sealed (FL (PrimOf p) wR))
+    filteredUnrecordedChanges diffing lfm lfr repo paths =
+        chooseTouching paths <$> unrecordedChanges diffing lfm lfr repo paths
 
 -- | Runs the 'InteractiveSelectionM' code
-runInteractive :: PrimPatch p
-               => InteractiveSelectionM p wX wY () -- Selection to run
+runInteractive :: InteractiveSelectionM p wX wY () -- Selection to run
                -> S.PatchSelectionOptions
-               -> O.DiffAlgorithm
                -> Tree IO         -- Pristine
                -> FL p wX wY      -- A list of patches
                -> IO ()
-runInteractive i patchsel diffalg pristine ps' = do
+runInteractive i patchsel pristine ps' = do
     let lps' = labelPatches Nothing ps'
         choices' = mkPatchChoices lps'
-        ps = evalStateT i $
-             ISC { total   = lengthFL lps'
-                 , current = 0
-                 , lps     = FZipper NilRL lps'
-                 , choices = choices'
-                 }
-    void $ runReaderT ps $
-           selectionContextPrim First "view" patchsel
-             (Just (primSplitter diffalg))
-             Nothing (Just pristine)
+        ps = evalStateT i (initialSelectionState lps' choices')
+    void $
+      runReaderT ps $
+        selectionConfigPrim First "view" patchsel Nothing Nothing (Just pristine)
 
 -- | The interactive part of @darcs whatsnew@
 interactiveHunks :: (IsHunk p, ShowPatch p, ShowContextPatch p, Commute p,
@@ -348,7 +341,7 @@ interactiveHunks pristine = do
     case c of
         Nothing -> liftIO $ putStrLn "No more changes!"
         Just (Sealed2 lp) -> do
-            liftIO $ printPatch (unLabel lp)
+            liftIO $ printPatchPager (unLabel lp)
             repeatThis lp
   where
     repeatThis lp = do
@@ -361,7 +354,7 @@ interactiveHunks pristine = do
             'v' -> liftIO (contextualPrintPatch pristine (unLabel lp))
                    >> repeatThis lp
             -- View summary of the change
-            'x' -> liftIO (printSummary (unLabel lp))
+            'x' -> liftIO (putDocLn $ summary $ unLabel lp)
                    >> repeatThis lp
             -- View change and move on
             'y' -> liftIO (contextualPrintPatch pristine (unLabel lp))
@@ -409,21 +402,22 @@ interactiveHunks pristine = do
     basic_options = [ options_yn ]
     adv_options = [ optionsView, optionsNav ]
 
+printPatchPager :: ShowPatchBasic p => p wX wY -> IO ()
+printPatchPager = viewDocWith fancyPrinters . displayPatch
 
--- | status is an alias for whatsnew, with implicit Summary and LookForAdds
--- flags. We override the default description, to include the implicit flags.
-status :: DarcsCommand [DarcsFlag]
+-- | An alias for 'whatsnew', with implicit @-l@ (and thus implicit @-s@)
+-- flags. We override the default description, to include these flags.
+status :: DarcsCommand
 status = statusAlias
     { commandDescription = statusDesc
     , commandAdvancedOptions = odesc commonAdvancedOpts
     , commandBasicOptions = odesc statusBasicOpts
     , commandDefaults = defaultFlags statusOpts
     , commandCheckOptions = ocheck statusOpts
-    , commandParseOptions = onormalise statusOpts
     }
   where
     statusAlias = commandAlias "status" Nothing whatsnew
-    statusDesc = "Alias for `darcs " ++ commandName whatsnew ++ " -ls '."
+    statusDesc = "Alias for `darcs " ++ commandName whatsnew ++ " -ls`."
     statusBasicOpts
       = O.maybeSummary (Just O.YesSummary)
       ^ O.withContext
