@@ -4,7 +4,6 @@ module Darcs.UI.External
     ( sendEmail
     , generateEmail
     , sendEmailDoc
-    , resendEmail
     , signString
     , verifyPS
     , execDocPipe
@@ -12,33 +11,43 @@ module Darcs.UI.External
     , pipeDocSSH
     , viewDoc
     , viewDocWith
-    , haveSendmail
-    , sendmailPath
+    , checkDefaultSendmail
     , diffProgram
     , darcsProgram
     , editText
     , editFile
     --  * Locales
     , setDarcsEncodings
-    , getSystemEncoding
-    , isUTF8Locale
     ) where
 
 import Darcs.Prelude
 
-import Data.Maybe ( isJust, isNothing, maybeToList )
-import Control.Monad ( unless, when, filterM, liftM2, void )
+import Data.Maybe ( isJust )
+#ifndef WIN32
+import Data.Maybe ( isNothing, maybeToList )
+#endif
+import Control.Monad ( unless, when, filterM, void )
+#ifndef WIN32
+import Control.Monad ( liftM2 )
+#endif
 import Control.Concurrent.MVar ( MVar )
 import System.Exit ( ExitCode(..) )
 import System.Environment
     ( getEnv
     , getExecutablePath
     )
-import System.IO ( hPutStr, hPutStrLn, hClose,
-                   hIsTerminalDevice, stdout, stderr, Handle )
-import System.Directory ( doesFileExist,
-                          findExecutable )
+import System.Directory ( doesFileExist, findExecutable )
+import System.IO
+    ( Handle
+    , hClose
+    , hIsTerminalDevice
+    , hPutStr
+    , stderr
+    , stdout
+    )
+#ifndef WIN32
 import System.FilePath.Posix ( (</>) )
+#endif
 import System.Process ( createProcess, proc, CreateProcess(..), runInteractiveProcess, waitForProcess, StdStream(..) )
 import System.Process.Internals ( ProcessHandle )
 
@@ -49,19 +58,12 @@ import GHC.IO.Encoding
     , setLocaleEncoding )
 #endif
 
-import Foreign.C.String ( CString, peekCString )
-
 import Control.Concurrent ( forkIO, newEmptyMVar, putMVar, takeMVar )
-import Control.Exception
-    ( try, finally, catch, IOException )
+import Control.Exception ( IOException, finally, try )
 import System.IO.Error ( ioeGetErrorType )
 import GHC.IO.Exception ( IOErrorType(ResourceVanished) )
-import Data.Char ( toLower )
-import Text.Regex
-#if defined (HAVE_MAPI)
+#ifdef WIN32
 import Foreign.C ( withCString )
-#endif
-#ifdef HAVE_MAPI
 import Foreign.Ptr ( nullPtr )
 import Darcs.Util.Lock ( canonFilename, writeDocBinFile )
 #endif
@@ -75,10 +77,8 @@ import Darcs.Util.Path
 import Darcs.Util.Progress ( withoutProgress, debugMessage )
 
 import Darcs.Util.ByteString (linesPS, unlinesPS)
-import qualified Data.ByteString as B (ByteString, empty, null, readFile
-            ,hGetContents, writeFile, hPut, length
-            ,take, concat, drop, isPrefixOf, singleton, append)
-import qualified Data.ByteString.Char8 as BC (unpack, pack)
+import qualified Data.ByteString as B
+import qualified Data.ByteString.Char8 as BC
 
 import Darcs.Util.Lock
     ( withTemp
@@ -87,24 +87,42 @@ import Darcs.Util.Lock
     )
 import Darcs.Util.Ssh ( getSSH, SSHCmd(..) )
 import Darcs.Util.CommandLine ( parseCmd, addUrlencoded )
+#ifndef WIN32
+import Darcs.Util.English ( orClauses )
+#endif
 import Darcs.Util.Exception ( catchall )
 import Darcs.Util.Exec ( execInteractive, exec, Redirect(..), withoutNonBlock )
 import Darcs.Util.URL ( SshFilePath, sshUhost )
-import Darcs.Util.Printer ( Doc, Printers, hPutDocLnWith, hPutDoc, hPutDocLn, hPutDocWith, ($$), renderPS,
-                 simplePrinters,
-                 hPutDocCompr,
-                 text, empty, packedString, vcat, renderString )
+import Darcs.Util.Printer
+    ( Doc
+    , Printers
+    , hPutDoc
+    , hPutDocCompr
+    , hPutDocLn
+    , hPutDocLnWith
+    , hPutDocWith
+    , packedString
+    , renderPS
+    , renderString
+    , simplePrinters
+    , text
+    )
 import qualified Darcs.Util.Ratified as Ratified
 import Darcs.UI.Email ( formatHeader )
 
+#ifndef WIN32
 sendmailPath :: IO String
 sendmailPath = do
+  let searchPath = [ "/usr/sbin", "/sbin", "/usr/lib" ]
   l <- filterM doesFileExist $ liftM2 (</>)
-                [ "/usr/sbin", "/sbin", "/usr/lib" ]
+                searchPath
                 [ "sendmail" ]
   ex <- findExecutable "sendmail"
-  when (isNothing ex && null l) $ fail "Cannot find the \"sendmail\" program."
+  when (isNothing ex && null l) $
+    fail $ "Cannot find the 'sendmail' program in " ++
+      orClauses ("your PATH" : searchPath) ++ "."
   return $ head $ maybeToList ex ++ l
+#endif
 
 diffProgram :: IO String
 diffProgram = do
@@ -144,7 +162,7 @@ pipeDocSSH compress remoteAddr args input = do
     (ssh, ssh_args) <- getSSH SSH
     pipeDocInternal (PipeToSsh compress) ssh (ssh_args ++ ("--":sshUhost remoteAddr:args)) input
 
-sendEmail :: String -> String -> String -> String -> String -> String -> IO ()
+sendEmail :: String -> String -> String -> String -> Maybe String -> String -> IO ()
 sendEmail f t s cc scmd body =
   sendEmailDoc f t s cc scmd Nothing (text body)
 
@@ -168,8 +186,19 @@ generateEmail h f t s cc body = do
             = B.hPut h (B.append (formatHeader field value) newline)
         newline = B.singleton 10
 
-haveSendmail :: IO Bool
-haveSendmail = (sendmailPath >> return True) `catch` (\(_ :: IOException) -> return False)
+checkDefaultSendmail :: IO ()
+#ifndef WIN32
+checkDefaultSendmail = void sendmailPath
+#else
+-- FIXME find a way to detect if sending via MAPI would work
+checkDefaultSendmail = return ()
+{-
+  TODO do we want a warning message like this one here?
+  hPutStr stderr
+    "Warning: Using MAPI for sending mail. This may hang indefinitely " ++
+    "if not properly configured.\n"
+-}
+#endif
 
 -- | Send an email, optionally containing a patch bundle
 --   (more precisely, its description and the bundle itself)
@@ -178,35 +207,16 @@ sendEmailDoc
   -> String           -- ^ to
   -> String           -- ^ subject
   -> String           -- ^ cc
-  -> String           -- ^ send command
+  -> Maybe String     -- ^ send command
   -> Maybe (Doc, Doc) -- ^ (content,bundle)
   -> Doc              -- ^ body
   -> IO ()
 sendEmailDoc _ "" _ "" _ _ _ = return ()
 sendEmailDoc f "" s cc scmd mbundle body =
   sendEmailDoc f cc s "" scmd mbundle body
-sendEmailDoc f t s cc scmd mbundle body = do
-  use_sendmail <- haveSendmail
-  if use_sendmail || scmd /= "" then
-    withOpenTemp $ \(h,fn) -> do
-      generateEmail h f t s cc body
-      hClose h
-      withOpenTemp $ \(hat,at) -> do
-        ftable' <- case mbundle of
-                   Just (content,bundle) -> do
-                       hPutDocLn hat bundle
-                       return [ ('b', renderString content) , ('a', at) ]
-                   Nothing ->
-                       return [ ('b', renderString body) ]
-        hClose hat
-        let ftable = [ ('t',addressOnly t),('c',cc),('f',f),('s',s) ] ++ ftable'
-        r <- execSendmail ftable scmd fn
-        when (r /= ExitSuccess) $ fail ("failed to send mail to: "
-                                       ++ t ++ cc_list cc
-                                       ++ "\nPerhaps sendmail is not configured.")
-#ifdef HAVE_MAPI
-   else do
-     r <- withCString t $ \tp ->
+#ifdef WIN32
+sendEmailDoc f t s cc Nothing _mbundle body = do
+  r <- withCString t $ \tp ->
            withCString f $ \fp ->
             withCString cc $ \ccp ->
              withCString s $ \sp ->
@@ -217,71 +227,53 @@ sendEmailDoc f t s cc scmd mbundle body = do
                cfn <- canonFilename fn
                withCString cfn $ \pcfn ->
                 c_send_email fp tp ccp sp nullPtr pcfn
-     when (r /= 0) $ fail ("failed to send mail to: " ++ t)
-#else
-   else fail "no mail facility (sendmail or mapi) located at configure time!"
+  when (r /= 0) $ do
+    fail $ "Failed to send mail via MAPI to: " ++ recipients t cc
 #endif
+sendEmailDoc f t s cc scmd mbundle body =
+  withOpenTemp $ \(h,fn) -> do
+    generateEmail h f t s cc body
+    hClose h
+    withOpenTemp $ \(hat,at) -> do
+      ftable' <- case mbundle of
+                 Just (content,bundle) -> do
+                     hPutDocLn hat bundle
+                     return [ ('b', renderString content) , ('a', at) ]
+                 Nothing ->
+                     return [ ('b', renderString body) ]
+      hClose hat
+      let ftable = [ ('t',addressOnly t),('c',cc),('f',f),('s',s) ] ++ ftable'
+      r <- execSendmail ftable scmd fn
+      when (r /= ExitSuccess) $
+        fail ("Failed to send mail to: " ++ recipients t cc)
   where addressOnly a =
           case dropWhile (/= '<') a of
           ('<':a2) -> takeWhile (/= '>') a2
           _        -> a
 
-        cc_list [] = []
-        cc_list c = " and cc'ed " ++ c
+recipients :: String -> String -> String
+recipients to "" = to
+recipients to cc = to ++ " and cc'ed " ++ cc
 
-resendEmail :: String -> String -> B.ByteString -> IO ()
-resendEmail "" _ _ = return ()
-resendEmail t scmd body = do
-  use_sendmail <- haveSendmail
-  if use_sendmail || scmd /= ""
-   then
-    withOpenTemp $ \(h,fn) -> do
-     hPutStrLn h $ "To: "++ t
-     hPutStrLn h $ find_from (linesPS body)
-     hPutStrLn h $ find_subject (linesPS body)
-     hPutDocLn h $ fixit $ linesPS body
-     hClose h
-     let ftable = [('t',t)]
-     r <-  execSendmail ftable scmd fn
-     when (r /= ExitSuccess) $ fail ("failed to send mail to: " ++ t)
-   else
-#ifdef HAVE_MAPI
-    fail "Don't know how to resend email with MAPI"
+execSendmail :: [(Char,String)] -> Maybe String -> String -> IO ExitCode
+#ifdef WIN32
+execSendmail _ Nothing _ = error "impossible"
 #else
-    fail "no mail facility (sendmail or mapi) located at configure time (use the sendmail-command option)!"
+execSendmail _ Nothing fn = do
+  scmd <- sendmailPath
+  exec scmd ["-i", "-t"] (File fn, Null, AsIs)
 #endif
-  where br            = BC.pack "\r"
-        darcsurl      = BC.pack "DarcsURL:"
-        content       = BC.pack "Content-"
-        from_start    = BC.pack "From:"
-        subject_start = BC.pack "Subject:"
-        fixit (l:ls)
-         | B.null l = packedString B.empty $$ vcat (map packedString ls)
-         | l == br = packedString B.empty $$ vcat (map packedString ls)
-         | B.take 9 l == darcsurl || B.take 8 l == content
-            = packedString l $$ fixit ls
-         | otherwise = fixit ls
-        fixit [] = empty
-        find_from (l:ls) | B.take 5 l == from_start = BC.unpack l
-                         | otherwise = find_from ls
-        find_from [] = "From: unknown"
-        find_subject (l:ls) | B.take 8 l == subject_start = BC.unpack l
-                            | otherwise = find_subject ls
-        find_subject [] = "Subject: (no subject)"
+execSendmail ftable (Just scmd) fn =
+  case parseCmd (addUrlencoded ftable) scmd of
+    Right (arg0:opts, wantstdin) ->
+      let stdin = if wantstdin then File fn else Null
+      in exec arg0 opts (stdin, Null, AsIs)
+    Right ([], _) ->
+      fail $ "Invalid sendmail-command "++show scmd
+    Left e ->
+      fail $ "Invalid sendmail-command "++show scmd++"\n"++show e
 
-execSendmail :: [(Char,String)] -> String -> String -> IO ExitCode
-execSendmail ftable scmd fn =
-  if scmd == "" then do
-     cmd <- sendmailPath
-     exec cmd ["-i", "-t"] (File fn, Null, AsIs)
-  else case parseCmd (addUrlencoded ftable) scmd of
-         Right (arg0:opts, wantstdin) ->
-           do let stdin = if wantstdin then File fn else Null
-              exec arg0 opts (stdin, Null, AsIs)
-         Left e -> fail $ "failed to send mail, invalid sendmail-command: "++show e
-         _ -> fail "failed to send mail, invalid sendmail-command"
-
-#ifdef HAVE_MAPI
+#ifdef WIN32
 foreign import ccall "win32/send_email.h send_email" c_send_email
              :: CString -> {- sender -}
                 CString -> {- recipient -}
@@ -370,7 +362,7 @@ verifyGPG goodkeys s =
             fix_line x | B.length x < 3 = x
                        | BC.pack "- -" `B.isPrefixOf` x = B.drop 2 x
                        | otherwise = x
-#if defined(WIN32)
+#ifdef WIN32
             fix_sep c | c=='/' = '\\'   | otherwise = c
             fix_path p = map fix_sep p
 #else
@@ -550,47 +542,3 @@ setDarcsEncodings = do
     setForeignEncoding e
     setLocaleEncoding e
 #endif
-
--- The following functions are copied from the encoding package (BSD3
--- licence, by Henning Günther).
-
--- | @getSystemEncoding@ fetches the current encoding from locale
-foreign import ccall "system_encoding.h get_system_encoding"
-     get_system_encoding :: IO CString
-
-
-getSystemEncoding :: IO String
-getSystemEncoding = do
-    enc <- get_system_encoding
-    peekCString enc
-
-
--- | @isUTF8@ checks if an encoding is UTF-8 (or ascii, since it is a
--- subset of UTF-8).
-isUTF8Locale :: String -> Bool
-isUTF8Locale codeName = case normalizeEncoding codeName of
-    -- ASCII
-    "ascii"              -> True
-    "646"                -> True
-    "ansi_x3_4_1968"     -> True
-    "ansi_x3.4_1986"     -> True
-    "cp367"              -> True
-    "csascii"            -> True
-    "ibm367"             -> True
-    "iso646_us"          -> True
-    "iso_646.irv_1991"   -> True
-    "iso_ir_6"           -> True
-    "us"                 -> True
-    "us_ascii"           -> True
-    -- UTF-8
-    "utf_8"              -> True
-    "u8"                 -> True
-    "utf"                -> True
-    "utf8"               -> True
-    "utf8_ucs2"          -> True
-    "utf8_ucs4"          -> True
-    -- Everything else
-    _                    -> False
-  where
-    normalizeEncoding s = map toLower $ subRegex sep s "_"
-    sep = mkRegex "[^0-9A-Za-z]+"
