@@ -454,10 +454,11 @@ environmentHelpSendmail = (["SENDMAIL"], [
 -- This flag is present if darcs was invoked with @--sendmail-command=COMMAND@
 -- Alternatively the user can set @$S@@ENDMAIL@ which will be used as a
 -- fallback if present.
-getSendmailCmd :: Config -> IO String
-getSendmailCmd fs = case parseFlags O.sendmailCmd fs of
-  Just cmd -> return cmd
-  Nothing -> fmap (maybe "" id) $ lookupEnv "SENDMAIL"
+getSendmailCmd :: Config -> IO (Maybe String)
+getSendmailCmd fs =
+  case parseFlags O.sendmailCmd fs of
+    Nothing -> lookupEnv "SENDMAIL"
+    justcmd -> return justcmd
 
 -- | Accessor for output option
 getOutput :: Config -> FilePath -> Maybe AbsolutePathOrStd
