@@ -4,9 +4,9 @@ module Darcs.UI.Options
     , PrimDarcsOption
     , DarcsOptDescr
     , optDescr
+    , Config
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Data.Functor.Compose ( getCompose )
@@ -14,9 +14,11 @@ import System.Console.GetOpt ( OptDescr )
 
 import Darcs.UI.Options.All ( DarcsOption )
 import Darcs.UI.Options.Core
-import Darcs.UI.Options.Util ( DarcsOptDescr, PrimDarcsOption )
+import Darcs.UI.Options.Util ( DarcsOptDescr, Flag, PrimDarcsOption )
 import Darcs.Util.Path ( AbsolutePath )
 
 -- | Instantiate a 'DarcsOptDescr' with an 'AbsolutePath'
 optDescr :: AbsolutePath -> DarcsOptDescr f -> OptDescr f
 optDescr path = fmap ($ path) . getCompose
+
+type Config = [Flag]
