@@ -140,7 +140,7 @@ module Darcs.UI.Options.All
     , HeaderFields (..)
     , headerFields
     , sendToContext
-    , sendmail
+    , mail
     , sendmailCmd
     , charset
     , editDescription
@@ -839,7 +839,7 @@ data TestChanges = NoTestChanges | YesTestChanges LeaveTestDir deriving (Eq)
 
 testChanges :: PrimDarcsOption TestChanges
 testChanges = imap (Iso fw bw) $ runTest ^ leaveTestDir where
-  fw k NoTestChanges = k NoRunTest {- undefined -} YesLeaveTestDir
+  fw k NoTestChanges = k NoRunTest NoLeaveTestDir
   fw k (YesTestChanges ltd) = k YesRunTest ltd
   bw k NoRunTest _ = k NoTestChanges
   bw k YesRunTest ltd = k (YesTestChanges ltd)
@@ -850,7 +850,7 @@ runTest = withDefault NoRunTest
   , RawNoArg [] ["no-test"] F.NoTest NoRunTest "don't run the test script" ]
 
 leaveTestDir :: PrimDarcsOption LeaveTestDir
-leaveTestDir = withDefault YesLeaveTestDir
+leaveTestDir = withDefault NoLeaveTestDir
   [ RawNoArg [] ["leave-test-directory"]
     F.LeaveTestDir YesLeaveTestDir "don't remove the test directory"
   , RawNoArg [] ["remove-test-directory"]
@@ -900,20 +900,6 @@ sendToContext = singleAbsPathArg [] ["context"] F.Context arg "FILENAME"
   where arg (F.Context s) = Just s
         arg _ = Nothing
 
--- TODO: do something about the nonsensical case (False, Just s)
---
--- Some of the tests actually do this (pass --sendmail-command without
--- passing --mail) and it's unclear if it's deliberate or just a historical
--- accident after the issue2204 changes. We should untangle that and
--- perhaps turn this into a single option with an optional argument.
--- The other question to resolve is the interaction with the 'output'
--- options to darcs send.
-sendmailIso :: Iso (Bool -> Maybe String -> a) ((Bool, Maybe String) -> a)
-sendmailIso = Iso uncurry curry
-
-sendmail :: PrimDarcsOption (Bool, Maybe String)
-sendmail = imap sendmailIso $ mail ^ sendmailCmd
-
 mail :: PrimDarcsOption Bool
 mail = singleNoArg [] ["mail"] F.Mail "send patch using sendmail"
 
@@ -1005,7 +991,7 @@ conflicts def = withDefault (Just def)
   , RawNoArg [] ["skip-conflicts"]
       F.SkipConflicts Nothing "filter out any patches that would create conflicts" ]
 
--- Technically not an isomorphism, see 'sendmailIso'.
+-- Technically not an isomorphism.
 externalMerge :: PrimDarcsOption ExternalMerge
 externalMerge = imap (Iso fw bw) $ singleStrArg [] ["external-merge"] F.ExternalMerge arg
     "COMMAND" "use external tool to merge conflicts"
