@@ -1,38 +1,49 @@
-{-# LANGUAGE RecordWildCards #-}
 -- | Constructing 'OptSpec's and 'OptDescr's
 module Darcs.UI.Options.Util
     ( Flag
-    , PrimDarcsOption
+    -- * Instantiating 'OptSpec' and 'PrimOptSpec'
     , DarcsOptDescr
+    , PrimDarcsOption
+    -- * Constructing 'DarcsOptDescr's
     , noArg
     , strArg
     , optStrArg
     , absPathArg
     , absPathOrStdArg
     , optAbsPathArg
+      -- * Raw option specs
     , RawOptSpec(..)
     , withDefault
+    -- * Simple primitive scalar valued options
     , singleNoArg
     , singleStrArg
+    -- * Simple primitive list valued options
     , multiStrArg
     , multiOptStrArg
     , singleAbsPathArg
     , multiAbsPathArg
     , deprecated
-    -- Re-exports
+    -- * Parsing/showing option arguments
+    , parseIntArg
+    , parseIndexRangeArg
+    , showIntArg
+    , showIndexRangeArg
+    -- * Re-exports
     , AbsolutePath
     , AbsolutePathOrStd
     , makeAbsolute
     , makeAbsoluteOrStd
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
-import System.Console.GetOpt ( OptDescr(..), ArgDescr(..) )
+import Control.Exception ( Exception, throw )
 import Data.Functor.Compose
 import Data.List ( intercalate )
 import Data.Maybe ( maybeToList, fromMaybe )
+import Data.Typeable ( Typeable )
+import System.Console.GetOpt ( OptDescr(..), ArgDescr(..) )
+
 import Darcs.UI.Options.Core
 import Darcs.UI.Options.Flags ( DarcsFlag )
 import Darcs.UI.Options.Iso
@@ -53,19 +64,19 @@ type Flag = DarcsFlag
 'System.Console.GetOpt.OptDescr'. Instead we (post-) compose it with @(->)
 'DarcsUtil.Path.AbsolutePath'@. Modulo newtype noise, this is the same as
 
-@ type 'DarcsOptDescr f = 'System.Console.GetOpt.OptDescr' ('AbsolutePath' -> f)@
+@ type 'DarcsOptDescr' f = 'System.Console.GetOpt.OptDescr' ('AbsolutePath' -> f)@
 
 This is so we can pass a directory relative to which an option argument is
 interpreted (if it has the form of a relative path).
 -}
 type DarcsOptDescr = Compose OptDescr ((->) AbsolutePath)
 
--- | This is 'PrimOptSpec' instantiated with 'DarcsOptDescr and 'Flag'.
+-- | This is 'PrimOptSpec' instantiated with 'DarcsOptDescr' and 'Flag'.
 type PrimDarcsOption v = forall a. PrimOptSpec DarcsOptDescr Flag a v
 
--- * Constructing 'OptDescr's
+-- * Constructing 'DarcsOptDescr's
 
--- | Construct an 'DarcsOptDescr with no arguments.
+-- | Construct a 'DarcsOptDescr' with no arguments.
 noArg :: [Char] -> [String] -> f -> String -> DarcsOptDescr f
 noArg s l f h = Compose $ Option s l (NoArg (const f)) h
 
@@ -74,25 +85,25 @@ noArg s l f h = Compose $ Option s l (NoArg (const f)) h
 type SingleArgOptDescr a f =
         [Char] -> [String] -> (a -> f) -> String -> String -> DarcsOptDescr f
 
--- | Construct an 'DarcsOptDescr with a 'String' argument.
+-- | Construct a 'DarcsOptDescr' with a 'String' argument.
 strArg :: SingleArgOptDescr String f
 strArg s l f a h = Compose $ Option s l (ReqArg (\x _ -> f x) a) h
 
--- | Construct an 'DarcsOptDescr with an optional 'String' argument.
+-- | Construct a 'DarcsOptDescr' with an optional 'String' argument.
 optStrArg :: SingleArgOptDescr (Maybe String) f
 optStrArg s l f a h = Compose $ Option s l (OptArg (\x _ -> f x) a) h
 
--- | Construct an 'DarcsOptDescr with an 'AbsolutePath'
+-- | Construct a 'DarcsOptDescr' with an 'AbsolutePath'
 -- argument.
 absPathArg :: SingleArgOptDescr AbsolutePath f
 absPathArg s l f a h = Compose $ Option s l (ReqArg (\x wd -> f $ makeAbsolute wd x) a) h
 
--- | Construct an 'DarcsOptDescr with an 'AbsolutePathOrStd'
+-- | Construct a 'DarcsOptDescr' with an 'AbsolutePathOrStd'
 -- argument.
 absPathOrStdArg :: SingleArgOptDescr AbsolutePathOrStd f
 absPathOrStdArg s l f a h = Compose $ Option s l (ReqArg (\x wd -> f $ makeAbsoluteOrStd wd x) a) h
 
--- | Construct an 'DarcsOptDescr with an optional 'AbsolutePath'
+-- | Construct a 'DarcsOptDescr' with an optional 'AbsolutePath'
 -- argument.
 optAbsPathArg :: [Char] -> [String] -> String -> (AbsolutePath -> f)
               -> String -> String -> DarcsOptDescr f
@@ -271,3 +282,33 @@ deprecated comments ropts = OptSpec {..} where
   noDefaultHelp (RawAbsPathArg s l mkF _ _ _ a h) = absPathArg s l mkF a h
   noDefaultHelp (RawAbsPathOrStdArg s l mkF _ _ _ a h) = absPathOrStdArg s l mkF a h
   noDefaultHelp (RawOptAbsPathArg s l mkF _ _ _ d a h) = optAbsPathArg s l d mkF a h
+
+-- * Parsing option arguments
+
+data ArgumentParseError = ArgumentParseError String String
+  deriving (Eq, Typeable)
+
+instance Exception ArgumentParseError
+
+instance Show ArgumentParseError where
+  show (ArgumentParseError arg expected) =
+    unwords ["cannot parse flag argument",show arg,"as",expected]
+
+parseIntArg :: String -> (Int -> Bool) -> String -> Int
+parseIntArg expected cond s =
+  case reads s of
+    (n,""):_ | cond n -> n
+    _ -> throw (ArgumentParseError s expected)
+
+parseIndexRangeArg :: String -> (Int,Int)
+parseIndexRangeArg s =
+  case reads s of
+    (n,""):_ | n > 0 -> (n,n)
+    (n,'-':s'):_ | n > 0, (m,""):_ <- reads s', m > 0 -> (n,m)
+    _ -> throw (ArgumentParseError s "index range")
+
+showIntArg :: Int -> String
+showIntArg = show
+
+showIndexRangeArg :: (Int,Int) -> String
+showIndexRangeArg (n,m) = show n ++ "-" ++ show m
