@@ -346,7 +346,11 @@ runInvertibleSelection ps psc = runReaderT (selection ps) psc where
   -- after selecting with a splitter, the results may not be canonical
   canonizeAfterSplitter :: (FL p :> FL p) wA wB -> (FL p :> FL p) wA wB
   canonizeAfterSplitter (x :> y) =
-    let canonizeIfNeeded = maybe id canonizeSplit mspl
+    let
+      canonizeIfNeeded =
+        case mspl of
+          Just s -> canonizeSplit s
+          Nothing -> id
     in canonizeIfNeeded x :> canonizeIfNeeded y
 
   -- retrieve the results of patch selection
