@@ -57,12 +57,11 @@ module Darcs.Util.ByteString
 
 import Darcs.Prelude
 
-import Codec.Binary.Base16 ( b16Enc, b16Dec )
-
 import qualified Data.ByteString            as B
 import qualified Data.ByteString.Char8      as BC
 import qualified Data.ByteString.Lazy       as BL
 import Data.ByteString (intercalate)
+import qualified Data.ByteString.Base16     as B16
 
 import System.IO ( withFile, IOMode(ReadMode)
                  , hSeek, SeekMode(SeekFromEnd,AbsoluteSeek)
@@ -72,10 +71,11 @@ import System.IO.Error          ( catchIOError )
 import System.IO.Unsafe         ( unsafePerformIO )
 
 import Data.Bits                ( rotateL )
-import Data.Char                ( ord, toLower, toUpper )
+import Data.Char                ( ord )
 import Data.Word                ( Word8 )
 import Data.Int                 ( Int32, Int64 )
 import Data.List                ( intersperse )
+import Control.Exception ( throw )
 import Control.Monad            ( when )
 import Control.Monad.ST.Lazy    ( ST )
 
@@ -325,17 +325,21 @@ mmapFilePS f =
 -- fromPS2Hex
 
 fromPS2Hex :: B.ByteString -> B.ByteString
-fromPS2Hex = BC.map toLower . b16Enc
+fromPS2Hex = B16.encode
 
 -- -------------------------------------------------------------------------
 -- fromHex2PS
 
 fromHex2PS :: B.ByteString -> B.ByteString
 fromHex2PS s =
-  case b16Dec $ BC.map toUpper s of
-    Right (result, remaining)
-      | B.null remaining -> result
-    _ -> error "fromHex2PS: input is not hex encoded"
+  case B16.decode s of
+#if MIN_VERSION_base16_bytestring(1,0,0)
+    Right result -> result
+    Left msg -> throw $ userError $ "fromHex2PS: input is not hex encoded: "++msg
+#else
+    (result, rest) | B.null rest -> result
+    _ -> throw $ userError $ "fromHex2PS: input is not hex encoded"
+#endif
 
 propHexConversion :: B.ByteString -> Bool
 propHexConversion x = fromHex2PS (fromPS2Hex x) == x
