@@ -18,7 +18,6 @@ module Darcs.Util.Diff.Patience
     ( getChanges
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Data.List ( sort )
@@ -130,8 +129,8 @@ genNestedChanges (br:brs) i0 o0 n0 = nc i0 (lcus ol nl) ol nl
                       (na, _:nb) ->
                          i' `seq` easydiff i oa na ++ nc i' xs ob nb
                              where i' = i + length (concat na) + length x
-                      (_,[]) -> impossible
-                (_,[]) -> impossible
+                      (_,[]) -> error "impossible case"
+                (_,[]) -> error "impossible case"
           easydiff i o n = genNestedChanges brs i oo nn
               where (oo, nn) = (concat o, concat n)
 genNestedChanges [] i o n = mkdiff (all (`elem` borings)) i mylcs o n
@@ -264,8 +263,8 @@ patientLcs0 xs0@(cc1:cc1s) ys0@(cc2:cc2s)
                    (c1,_:c2) -> case break (==b) ds of
                                   ([],_:d2) -> b : joinU bs c2 d2
                                   (d1,_:d2) -> lcs c1 d1 ++ b : joinU bs c2 d2
-                                  _ -> impossible
-                   _ -> impossible
+                                  _ -> error "impossible case"
+                   _ -> error "impossible case"
           findUnique xs = S.fromList $ gru $ sort xs
           gru (x:x':xs) | x == x' = gru (dropWhile (==x) xs)
           gru (x:xs) = x : gru xs
