@@ -9,21 +9,31 @@
 -- Portability : portable
 
 module Darcs.Util.Download
-    ( copyUrl
-    , copyUrlFirst
-    , setDebugHTTP
+    ( setDebugHTTP
     , disableHTTPPipelining
     , maxPipelineLength
-    , waitUrl
     , Cachable(Cachable, Uncachable, MaxAge)
     , environmentHelpProxy
     , environmentHelpProxyPassword
-    , ConnectionError(..)
+    , ConnectionError
+#ifdef HAVE_CURL
+    , copyUrl
+    , copyUrlFirst
+    , waitUrl
+#endif
     ) where
 
-import Prelude ( (^) )
+import Data.IORef ( newIORef, readIORef, writeIORef, IORef )
+import System.IO.Unsafe ( unsafePerformIO )
+
 import Darcs.Prelude
 
+import Darcs.Util.Download.Request
+    ( Cachable(Cachable,MaxAge,Uncachable)
+    , ConnectionError
+    )
+
+#ifdef HAVE_CURL
 import Control.Arrow ( (&&&) )
 import Control.Concurrent ( forkIO )
 import Control.Concurrent.STM.TChan
@@ -34,25 +44,19 @@ import Control.Monad ( unless, when )
 import Control.Monad.State ( evalStateT, get, modify, put, StateT )
 import Control.Monad.STM ( atomically )
 import Control.Monad.Trans ( liftIO )
-import Data.IORef ( newIORef, readIORef, writeIORef, IORef )
 import Data.Map ( Map )
 import qualified Data.Map as Map
 import Data.Tuple ( swap )
-import System.Directory ( copyFile )
-import System.IO.Unsafe ( unsafePerformIO )
-import System.Random ( randomRIO )
+import System.Directory ( copyFile, renameFile )
+import Crypto.Random ( seedNew, seedToInteger )
 
 import Darcs.Util.AtExit ( atexit )
 import Darcs.Util.File ( removeFileMayNotExist )
 import Numeric ( showHex )
 import Darcs.Util.Progress ( debugMessage )
-import Darcs.Util.Download.Request
-import Darcs.Util.Workaround ( renameFile )
 
-#ifdef HAVE_CURL
+import Darcs.Util.Download.Request
 import qualified Darcs.Util.Download.Curl as Curl
-#else
-import qualified Darcs.Util.Download.HTTP as HTTP
 #endif
 
 {-# NOINLINE maxPipelineLengthRef #-}
@@ -69,6 +73,7 @@ maxPipelineLengthRef = unsafePerformIO $ do
 maxPipelineLength :: IO Int
 maxPipelineLength = readIORef maxPipelineLengthRef
 
+#ifdef HAVE_CURL
 {-# NOINLINE urlNotifications #-}
 urlNotifications :: MVar (Map String (MVar (Maybe String)))
 urlNotifications = unsafePerformIO $ newMVar Map.empty
@@ -84,11 +89,9 @@ type UrlM a = StateT UrlState IO a
 
 urlThread :: TChan UrlRequest -> IO ()
 urlThread ch = do
-    junk <- flip showHex "" `fmap` randomRIO rrange
+    junk <- flip showHex "" <$> seedToInteger <$> seedNew
     evalStateT urlThread' (UrlState Map.empty emptyQ 0 junk)
   where
-    rrange = (0, 2 ^ (128 :: Integer) :: Integer)
-
     urlThread' :: UrlM ()
     urlThread' = do
         empty <- liftIO $ atomically $ isEmptyTChan ch
@@ -165,7 +168,7 @@ checkWaitToStart = do
             Nothing -> return ()
             Just (u, rest) -> do
                 case Map.lookup u (inProgress st) of
-                    Nothing -> bug $ "bug in URL.checkWaitToStart " ++ u
+                    Nothing -> error $ "bug in URL.checkWaitToStart " ++ u
                     Just (f, _, c) -> do
                         dbg $ "URL.requestUrl (" ++ u ++ "\n"
                               ++ "-> " ++ f ++ ")"
@@ -220,7 +223,7 @@ waitNextUrl = do
         liftIO $ case Map.lookup u p of
             Nothing ->
                 -- A url finished downloading, but we don't have a record of it
-                bug $ "bug in URL.waitNextUrl: " ++ u
+                error $ "bug in URL.waitNextUrl: " ++ u
             Just (f, fs, _) -> if null e
                 then do -- Succesful download
                     renameFile (createDownloadFileName f st) f
@@ -266,6 +269,12 @@ waitUrl u = do
 dbg :: String -> StateT a IO ()
 dbg = liftIO . debugMessage
 
+requestUrl :: String -> FilePath -> Cachable -> IO String
+requestUrl = Curl.requestUrl
+
+waitNextUrl' :: IO (String, String, Maybe ConnectionError)
+waitNextUrl' = Curl.waitNextUrl
+
 minCachable :: Cachable -> Cachable -> Cachable
 minCachable Uncachable _          = Uncachable
 minCachable _          Uncachable = Uncachable
@@ -273,28 +282,23 @@ minCachable (MaxAge a) (MaxAge b) = MaxAge $ min a b
 minCachable (MaxAge a) _          = MaxAge a
 minCachable _          (MaxAge b) = MaxAge b
 minCachable _          _          = Cachable
+#endif
 
 disableHTTPPipelining :: IO ()
 disableHTTPPipelining = writeIORef maxPipelineLengthRef 1
 
 setDebugHTTP :: IO ()
-requestUrl :: String -> FilePath -> Cachable -> IO String
-waitNextUrl' :: IO (String, String, Maybe ConnectionError)
 pipeliningEnabled :: IO Bool
 
 #ifdef HAVE_CURL
 
 setDebugHTTP = Curl.setDebugHTTP
-requestUrl = Curl.requestUrl
-waitNextUrl' = Curl.waitNextUrl
 pipeliningEnabled = Curl.pipeliningEnabled
 
 #else
 
 setDebugHTTP = return ()
-requestUrl = HTTP.requestUrl
-waitNextUrl' = HTTP.waitNextUrl
-pipeliningEnabled = return False
+pipeliningEnabled = return True
 
 #endif
 
