@@ -1,24 +1,37 @@
-{-# Language MultiParamTypeClasses #-}
 module Darcs.Util.Exception
     ( firstJustIO
     , catchall
+    , catchNonExistence
     , clarifyErrors
     , prettyException
     , prettyError
     , die
+    , handleOnly
+    , handleOnlyIOError
+    , ifIOError
+    , ifDoesNotExistError
     ) where
 
 
-import Prelude ()
 import Darcs.Prelude
 
-import Control.Exception ( SomeException, Exception(fromException), catch )
+import Control.Exception
+    ( Exception(fromException)
+    , SomeException
+    , catch
+    , handle
+    , throwIO
+    )
 import Data.Maybe ( isJust )
 
 import System.Exit ( exitFailure )
 import System.IO ( stderr, hPutStrLn )
-import System.IO.Error ( isUserError, ioeGetErrorString
-                       , isDoesNotExistError, ioeGetFileName )
+import System.IO.Error
+    ( ioeGetErrorString
+    , ioeGetFileName
+    , isDoesNotExistError
+    , isUserError
+    )
 
 import Darcs.Util.SignalHandler ( catchNonSignal )
 
@@ -27,6 +40,12 @@ catchall :: IO a
          -> IO a
 a `catchall` b = a `catchNonSignal` (\_ -> b)
 
+catchNonExistence :: IO a -> a -> IO a
+catchNonExistence job nonexistval =
+    catch job $
+    \e -> if isDoesNotExistError e then return nonexistval
+                                   else ioError e
+
 -- | The firstJustM returns the first Just entry in a list of monadic
 -- operations. This is close to `listToMaybe `fmap` sequence`, but the sequence
 -- operator evaluates all monadic members of the list before passing it along
@@ -70,3 +89,23 @@ prettyError e | isUserError e = ioeGetErrorString e
 -- | Terminate the program with an error message.
 die :: String -> IO a
 die msg = hPutStrLn stderr msg >> exitFailure
+
+-- | Handle only actual IO exceptions i.e. not "user errors" e.g. those raised
+-- by calling 'fail'.
+--
+-- We use 'fail' all over the place to signify erroneous conditions and we
+-- normally don't want to handle such errors.
+handleOnlyIOError :: IO a -> IO a -> IO a
+handleOnlyIOError = handleOnly (not . isUserError)
+
+-- | Like 'handleOnlyIOError' but restricted to returning a given value.
+ifIOError :: a -> IO a -> IO a
+ifIOError use_instead = handleOnlyIOError (return use_instead)
+
+-- | Like 'ifIOError' but restricted to handling non-existence.
+ifDoesNotExistError :: a -> IO a -> IO a
+ifDoesNotExistError use_instead = handleOnly isDoesNotExistError (return use_instead)
+
+-- | Handle only a those exceptions for which the predicate succeeds.
+handleOnly :: Exception e => (e -> Bool) -> IO a -> IO a -> IO a
+handleOnly pred handler = handle (\e -> if pred e then handler else throwIO e)
