@@ -15,7 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, DeriveDataTypeable #-}
+{-# LANGUAGE CPP #-}
 
 -- |
 -- Module      : Darcs.Util.Exec
@@ -37,11 +37,11 @@ module Darcs.Util.Exec
     , ExecException(..)
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 #ifndef WIN32
 import Control.Exception ( bracket )
+import Control.Monad ( forM_ )
 import System.Posix.Env ( setEnv, getEnv, unsetEnv )
 import System.Posix.IO ( queryFdOption, setFdOption, FdOption(..), stdInput )
 #else
@@ -119,7 +119,10 @@ instance Show ExecException where
 
 _devNull :: FilePath
 #ifdef WIN32
-_devNull = "NUL"
+-- since GHC 8.6, Windows special devices need to be referred to using
+-- "device namespace" syntax. See
+-- https://ghc.gitlab.haskell.org/ghc/doc/users_guide/win32-dlls.html#windows-file-paths
+_devNull = "\\\\.\\NUL"
 #else
 _devNull = "/dev/null"
 #endif
@@ -149,7 +152,7 @@ exec cmd args (inp,out,err) = withoutProgress $ do
     -- because runProcess closes the Handles we pass it.
     redirect Stdout             _    = Just `fmap` hDuplicate stdout
 
-execInteractive :: String -> String -> IO ExitCode
+execInteractive :: String -> Maybe String -> IO ExitCode
 #ifndef WIN32
 {-
 This should handle arbitrary commands interpreted by the shell on Unix since
@@ -157,18 +160,19 @@ that's what people expect. But we don't want to allow the shell to interpret
 the argument in any way, so we set an environment variable and call
 cmd "$DARCS_ARGUMENT"
 -}
-execInteractive cmd arg = withoutProgress $ do
+execInteractive cmd mArg = withoutProgress $ do
     let var = "DARCS_ARGUMENT"
     stdin `seq` return ()
     withoutNonBlock $ bracket
       (do oldval <- getEnv var
-          setEnv var arg True
+          forM_ mArg $ \arg ->
+            setEnv var arg True
           return oldval)
       (\oldval ->
          case oldval of
               Nothing -> unsetEnv var
               Just val -> setEnv var val True)
-      (\_ -> withExit127 $ system $ cmd++" \"$"++var++"\"")
+      (\_ -> withExit127 $ system $ cmd++ maybe "" (const (" \"$"++var++"\"")) mArg)
 
 #else
 -- The `system' function passes commands to execute via cmd.exe (or
@@ -185,9 +189,9 @@ execInteractive cmd arg = withoutProgress $ do
 -- SETLOCAL EnableDelayedExpansion makes sure that !variable! expansion is done
 -- correctly on systems where that function is not enabled by default.
 --
-execInteractive cmd arg = withoutProgress $
+execInteractive cmd mArg = withoutProgress $
   withExit127 $ system $ "SETLOCAL EnableDelayedExpansion & " ++
-                          cmd ++ " " ++ arg ++
+                          cmd ++ maybe "" (" " ++) mArg ++
                           " & exit !errorlevel!"
 #endif
 
