@@ -1,6 +1,7 @@
 --  Copyright (C) 2009-2011 Petr Rockai BSD3
 --  Copyright (C) 2001, 2004 Ian Lynagh <igloo@earth.li>
 
+{-# LANGUAGE CPP #-}
 module Darcs.Util.Hash
     ( Hash(..)
     , encodeBase16, decodeBase16, sha256, sha256sum, rawHash
@@ -17,13 +18,11 @@ import Data.ByteArray ( convert )
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as BL
 import qualified Data.ByteString.Char8 as BC
-
-import qualified Codec.Binary.Base16 as B16
+import qualified Data.ByteString.Base16 as B16
 
 import qualified Crypto.Hash as H
 
-import Data.Maybe( isJust, fromJust )
-import Data.Char( toLower, toUpper, intToDigit, ord )
+import Data.Char( intToDigit, ord )
 
 import Data.Binary ( Binary(..), decode, encode )
 import Data.Bits ( xor, shiftL, (.|.) )
@@ -36,25 +35,23 @@ data Hash = SHA256 !B.ByteString
           | NoHash
             deriving (Show, Eq, Ord, Read)
 
-base16 :: B.ByteString -> B.ByteString
-base16 = BC.map toLower . B16.b16Enc
-
-debase16 :: B.ByteString -> Maybe B.ByteString
-debase16 bs = case B16.b16Dec $ BC.map toUpper bs of
-                Right (s, _) -> Just s
-                Left _ -> Nothing
-
 -- | Produce a base16 (ascii-hex) encoded string from a hash. This can be
 -- turned back into a Hash (see "decodeBase16". This is a loss-less process.
 encodeBase16 :: Hash -> B.ByteString
-encodeBase16 (SHA256 bs) = base16 bs
+encodeBase16 (SHA256 bs) = B16.encode bs
 encodeBase16 NoHash = B.empty
 
 -- | Take a base16-encoded string and decode it as a "Hash". If the string is
 -- malformed, yields NoHash.
 decodeBase16 :: B.ByteString -> Hash
-decodeBase16 bs | B.length bs == 64 && isJust (debase16 bs) = SHA256 (fromJust $ debase16 bs)
-                | otherwise = NoHash
+decodeBase16 bs
+  | B.length bs == 64
+#if MIN_VERSION_base16_bytestring(1,0,0)
+  , Right dbs <- B16.decode bs = SHA256 dbs
+#else
+  , (dbs, rest) <- B16.decode bs, B.null rest = SHA256 dbs
+#endif
+  | otherwise = NoHash
 
 -- | Compute a sha256 of a (lazy) ByteString.
 sha256 :: BL.ByteString -> Hash
@@ -62,7 +59,7 @@ sha256 bits = SHA256 (convert (H.hashlazy bits :: H.Digest H.SHA256))
 
 -- | Same as previous but general purpose.
 sha256sum :: B.ByteString -> String
-sha256sum = BC.unpack . base16 . convert . H.hashWith H.SHA256
+sha256sum = BC.unpack . B16.encode . convert . H.hashWith H.SHA256
 
 rawHash :: Hash -> B.ByteString
 rawHash NoHash = error "Cannot obtain raw hash from NoHash."
@@ -138,4 +135,4 @@ sha1Read bs
 
 {-# INLINE sha1Show #-}
 sha1Show :: SHA1 -> B.ByteString
-sha1Show = base16 . BL.toStrict . encode
+sha1Show = B16.encode . BL.toStrict . encode
