@@ -70,7 +70,7 @@
 -- index, so we can efficiently skip reading the whole subtree starting at a
 -- given directory (by just seeking aux bytes forward). The lines are
 -- pre-ordered with respect to directory structure -- the directory comes first
--- and after it come all its items. Cf. 'readIndex''.
+-- and after it come all its items. Cf. 'openIndex''.
 --
 -- For files, the aux field holds a timestamp.
 --
@@ -93,10 +93,10 @@
 -- paths.
 
 module Darcs.Util.Index
-    ( readIndex
+    ( openIndex
     , updateIndexFrom
     , indexFormatValid
-    , updateIndex
+    , treeFromIndex
     , listFileIDs
     , Index
     , filter
@@ -146,12 +146,8 @@ import Foreign.Ptr( Ptr, plusPtr )
 import System.IO ( hPutStrLn, stderr )
 import System.IO.MMap( mmapFileForeignPtr, Mode(..) )
 import System.Directory( doesFileExist, getCurrentDirectory, doesDirectoryExist )
-#if mingw32_HOST_OS
 import System.Directory( renameFile )
 import System.FilePath( (<.>) )
-#else
-import System.Directory( removeFile )
-#endif
 
 #ifdef WIN32
 import System.Win32.File
@@ -187,7 +183,7 @@ import System.Posix.Types ( FileID, EpochTime, FileOffset )
 -- used by git...) It turns out that it's hard to efficiently read a flat index
 -- with our internal data structures -- we need to turn the flat index into a
 -- recursive Tree object, which is rather expensive... As a bonus, we can also
--- efficiently implement subtree queries this way (cf. 'readIndex').
+-- efficiently implement subtree queries this way (cf. 'openIndex').
 data Item = Item { iBase :: !(Ptr ())
                  , iHashAndDescriptor :: !B.ByteString
                  } deriving Show
@@ -575,18 +571,18 @@ readFileFileID _ state item =
 -- * Reading and writing 'Tree's from/to the index
 
 -- | Read an index and build up a 'Tree' object from it, referring to current
--- working directory. The initial Index object returned by readIndex is not
+-- working directory. The initial Index object returned by openIndex is not
 -- directly useful. However, you can use 'Tree.filter' on it. Either way, to
 -- obtain the actual Tree object, call update.
 --
 -- The usual use pattern is this:
 --
--- > do (idx, update) <- readIndex
+-- > do (idx, update) <- openIndex
 -- >    tree <- update =<< filter predicate idx
 --
 -- The resulting tree will be fully expanded.
-readIndex :: FilePath -> (Tree IO -> Hash) -> IO Index
-readIndex indexpath ht = do
+openIndex :: FilePath -> (Tree IO -> Hash) -> IO Index
+openIndex indexpath ht = do
   (mmap_ptr, mmap_size) <- mmapIndex indexpath 0
   base <- getCurrentDirectory
   return $ if mmap_size == 0 then EmptyIndex
@@ -638,32 +634,32 @@ formatIndex mmap_ptr old reference =
 -- does not exist in a plain form in current working directory).
 updateIndexFrom :: FilePath -> (Tree IO -> Hash) -> Tree IO -> IO Index
 updateIndexFrom indexpath hashtree' ref =
-    do old_idx <- updateIndex =<< readIndex indexpath hashtree'
+    do old_tree <- treeFromIndex =<< openIndex indexpath hashtree'
        reference <- expand ref
        let len_root = itemAllocSize anchoredRoot
            len = len_root + sum [ itemAllocSize p | (p, _) <- list reference ]
        exist <- doesFileExist indexpath
--- TODO this conditional logic (rename or delete) is mirrored in
--- Darcs.Repository.State.checkIndex and should be refactored
-#if mingw32_HOST_OS
+       -- Note that the file is still open via the mmaped pointer in
+       -- the open index, and we /are/ going to write the index using
+       -- that pointer. If we could rely on posix semantics,
+       -- we would just delete the file. However, on windows this
+       -- would fail, so instead we rename it.
        when exist $ renameFile indexpath (indexpath <.> "old")
-#else
-       when exist $ removeFile indexpath -- to avoid clobbering oldidx
-#endif
        (mmap_ptr, _) <- mmapIndex indexpath len
-       formatIndex mmap_ptr old_idx reference
-       readIndex indexpath hashtree'
+       formatIndex mmap_ptr old_tree reference
+       openIndex indexpath hashtree'
 
-updateIndex :: Index -> IO (Tree IO)
-updateIndex EmptyIndex = return emptyTree
-updateIndex index =
+-- | Read the index, starting with the root, to create a 'Tree'.
+treeFromIndex :: Index -> IO (Tree IO)
+treeFromIndex EmptyIndex = return emptyTree
+treeFromIndex index =
     do let initial = State { start = size_header
                            , dirlength = 0
                            , path = anchoredRoot }
        res <- readItem index initial
        case treeitem res of
          Just (SubTree tree) -> return $ filter (predicate index) tree
-         _ -> fail "Unexpected failure in updateIndex!"
+         _ -> fail "Unexpected failure in treeFromIndex!"
 
 -- | Check that a given file is an index file with a format we can handle. You
 -- should remove and re-create the index whenever this is not true.
