@@ -23,26 +23,14 @@
 
 {-# LANGUAGE CPP #-}
 module Darcs.Util.Path
-    ( FileName( )
-    , fp2fn
-    , fn2fp
-    , fn2ps
-    , ps2fn
-    , breakOnDir
-    , normPath
-    , ownName
-    , superName
-    , movedirfilename
-    , encodeWhite
+    ( encodeWhite
     , decodeWhite
     , encodeWhiteName
     , decodeWhiteName
-    , isParentOrEqOf
     -- * AbsolutePath
     , AbsolutePath
     , makeAbsolute
     , ioAbsolute
-    , rootDirectory
     -- * AbsolutePathOrStd
     , AbsolutePathOrStd
     , makeAbsoluteOrStd
@@ -57,10 +45,8 @@ module Darcs.Util.Path
     , SubPath
     , makeSubPathOf
     , simpleSubPath
-    , isSubPathOf
     , floatSubPath
     -- * Miscellaneous
-    , sp2fn
     , FilePathOrURL(..)
     , FilePathLike(toFilePath)
     , getCurrentDirectory
@@ -68,27 +54,38 @@ module Darcs.Util.Path
     , getUniquePathName
     , doesPathExist
     -- * Check for malicious paths
-    , isMaliciousPath
     , isMaliciousSubPath
     -- * Tree filtering.
-    , filterFilePaths
     , filterPaths
     -- * AnchoredPaths: relative paths within a Tree. All paths are
     -- anchored at a certain root (this is usually the Tree root). They are
     -- represented by a list of Names (these are just strict bytestrings).
     , Name
-    , unsafeMakeName
+    , name2fp
+    , makeName
+    , rawMakeName
     , eqAnycase
     , AnchoredPath(..)
     , anchoredRoot
     , appendPath
     , anchorPath
     , isPrefix
-    , parent, parents, catPaths, flatten, makeName, appendToName
+    , breakOnDir
+    , movedirfilename
+    , parent
+    , parents
+    , replaceParent
+    , catPaths
+    , flatten
+    , inDarcsdir
+    , displayPath
+    , realPath
+    , isRoot
+    , darcsdirName
     -- * Unsafe AnchoredPath functions.
-    , floatPath, replacePrefixPath ) where
+    , floatPath
+    ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Data.List
@@ -99,16 +96,17 @@ import Data.List
     , inits
     )
 import Data.Char ( isSpace, chr, ord, toLower )
-import Control.Exception ( tryJust, bracket_ )
+import Data.Typeable ( Typeable )
+import Control.Exception ( tryJust, bracket_, throw, Exception )
 import Control.Monad ( when )
 import System.IO.Error ( isDoesNotExistError )
 
 import qualified Darcs.Util.Workaround as Workaround ( getCurrentDirectory )
 import qualified System.Directory ( setCurrentDirectory )
 import System.Directory ( doesDirectoryExist, doesFileExist )
-import qualified System.FilePath.Posix as FilePath ( normalise, isRelative )
+import qualified System.FilePath.Posix as FilePath ( (</>), normalise, isRelative )
 import qualified System.FilePath as NativeFilePath ( takeFileName, takeDirectory )
-import System.FilePath( (</>), splitDirectories, normalise, dropTrailingPathSeparator )
+import System.FilePath( splitDirectories, normalise, dropTrailingPathSeparator )
 import System.Posix.Files ( isDirectory, getSymbolicLinkStatus )
 
 import Darcs.Util.ByteString ( encodeLocale, decodeLocale )
@@ -119,37 +117,23 @@ import Data.Binary
 import Darcs.Util.Global ( darcsdir )
 import Darcs.Util.URL ( isAbsolute, isRelative, isSshNopath )
 
--- | FileName is an abstract type intended to facilitate the input and output of
--- unicode filenames.
-newtype FileName = FN FilePath deriving ( Eq, Ord )
 
-instance Show FileName where
-   showsPrec d (FN fp) = showParen (d > appPrec) $ showString "fp2fn " . showsPrec (appPrec + 1) fp
-      where appPrec = 10
+-- Utilities for use by command implementations
 
-instance Binary FileName where
-  put (FN h) = put h
-  get = FN `fmap` get
+-- | For displaying paths to the user. It should never be used
+-- for on-disk patch storage. This adds the "./" for consistency
+-- with how repo paths are displayed by 'showPatch' and friends,
+-- except for the root path which is displayed as plain ".".
+displayPath :: AnchoredPath -> FilePath
+displayPath p
+  | isRoot p = "."
+  | otherwise = anchorPath "." p
 
-{-# INLINE fp2fn #-}
-fp2fn :: FilePath -> FileName
-fp2fn = FN
-
-{-# INLINE fn2fp #-}
-fn2fp :: FileName -> FilePath
-fn2fp (FN fp) = fp
-
-{-# INLINE fn2ps #-}
-fn2ps :: FileName -> B.ByteString
-fn2ps (FN fp) = encodeLocale $ encodeWhite fp
-
-{-# INLINE ps2fn #-}
-ps2fn :: B.ByteString -> FileName
-ps2fn ps = FN $ decodeWhite $ decodeLocale ps
-
-{-# INLINE sp2fn #-}
-sp2fn :: SubPath -> FileName
-sp2fn = fp2fn . toFilePath
+-- | Interpret an 'AnchoredPath' as relative the current working
+-- directory. Intended for IO operations in the file system.
+-- Use with care!
+realPath :: AnchoredPath -> FilePath
+realPath = anchorPath ""
 
 -- | 'encodeWhite' translates whitespace in filenames to a darcs-specific
 --   format (numerical representation according to 'ord' surrounded by
@@ -181,94 +165,6 @@ decodeWhite cs_ = go cs_ [] False
            _ -> error "malformed filename"
        go (c:cs) acc modified = go cs (c:acc) modified
 
-encodeWhiteName :: Name -> B.ByteString
-encodeWhiteName = encodeLocale . encodeWhite . decodeLocale . unName
-
-decodeWhiteName :: B.ByteString -> Name
-decodeWhiteName = Name . encodeLocale . decodeWhite . decodeLocale
-
-ownName :: FileName -> FileName
-ownName (FN f) =  case breakLast '/' f of Nothing -> FN f
-                                          Just (_,f') -> FN f'
-superName :: FileName -> FileName
-superName fn = case normPath fn of
-                FN f -> case breakLast '/' f of
-                        Nothing -> FN "."
-                        Just (d,_) -> FN d
-breakOnDir :: FileName -> Maybe (FileName,FileName)
-breakOnDir (FN p) = case breakFirst '/' p of
-                      Nothing -> Nothing
-                      Just (d,f) | d == "." -> breakOnDir $ FN f
-                                 | otherwise -> Just (FN d, FN f)
-
--- | convert a path string into a sequence of directories strings
---   "/", "." and ".." are generally interpreted as expected.
---   Behaviour with too many '..' is to leave them.
---
---   Examples:
---     Splitting:
---       "aa/bb/cc"       -> ["aa","bb","cc"]
---     Ignoring "." and extra "/":
---       "aa/./bb"        -> ["aa","bb"]
---       "aa//bb"         -> ["aa","bb"]
---       "/aa/bb/"        -> ["aa","bb"]
---     Handling "..":
---       "aa/../bb/cc"    -> ["bb","cc"]
---       "aa/bb/../../cc" -> ["cc"]
---       "aa/../bb/../cc" -> ["cc"]
---       "../cc"          -> ["..","cc"]
-normPath :: FileName -> FileName
-normPath (FN p) = FN $ norm p
-
-norm :: String -> String
-norm ('.':'/':s) = norm s
-norm ('/':s)     = norm s
-norm "."         = ""
-norm s = go s [] False
- where go "" _   False = s           -- no modification
-       go "" acc True  = reverse acc
-       go ('/':r)         acc _ | sep r = go r acc True
-       go ('/':'.':r)     acc _ | sep r = go r acc True
-       go ('/':'.':'.':r) acc _ | sep r = go r (doDotDot acc) True
-       go (c:s') acc changed = go s' (c:acc) changed
-       -- remove last path or add "/.." if impossible
-       doDotDot ""                       = ".."
-       doDotDot acc@('.':'.':r) | sep r  = '.':'.':'/':acc
-       doDotDot acc = let a' = dropWhile (/='/') acc in -- eat dir
-                       if null a' then "" else tail a'
-       -- check if is a path separator
-       sep ('/':_) = True
-       sep []      = True -- end of string is considered separator
-       sep _       = False
-
-breakFirst :: Char -> String -> Maybe (String,String)
-breakFirst c = bf []
-    where bf a (r:rs) | r == c = Just (reverse a,rs)
-                      | otherwise = bf (r:a) rs
-          bf _ [] = Nothing
-breakLast :: Char -> String -> Maybe (String,String)
-breakLast c l = case breakFirst c (reverse l) of
-                Nothing -> Nothing
-                Just (a,b) -> Just (reverse b, reverse a)
-
-isParentOrEqOf :: FileName -> FileName -> Bool
-isParentOrEqOf fn1 fn2 = case stripPrefix (fn2fp fn1) (fn2fp fn2) of
-    Just ('/' : _) -> True
-    Just [] -> True
-    _ -> False
-
-movedirfilename :: FileName -> FileName -> FileName -> FileName
-movedirfilename old new name =
-    if name' == old'
-        then new
-        else case stripPrefix old' name' of
-            Just rest@('/':_) -> fp2fn $ "./" ++ new' ++ rest
-            _ -> name
-        where old' = fn2fp $ normPath old
-              new' = fn2fp $ normPath new
-              name' = fn2fp $ normPath name
-
-
 class FilePathOrURL a where
   toPath :: a -> String
 
@@ -298,11 +194,6 @@ instance FilePathOrURL AbsoluteOrRemotePath where
   toPath (AbsP a) = toPath a
   toPath (RmtP r) = r
 
-instance FilePathOrURL FileName where
-  toPath = fn2fp
-instance FilePathLike FileName where
-  toFilePath = fn2fp
-
 instance FilePathLike AbsolutePath where
   toFilePath (AbsolutePath x) = x
 instance FilePathLike SubPath where
@@ -326,14 +217,10 @@ makeSubPathOf (AbsolutePath p1) (AbsolutePath p2) =
     else Nothing
 
 simpleSubPath :: FilePath -> Maybe SubPath
-simpleSubPath x | null x = bug "simpleSubPath called with empty path"
+simpleSubPath x | null x = error "simpleSubPath called with empty path"
                 | isRelative x = Just $ SubPath $ FilePath.normalise $ pathToPosix x
                 | otherwise = Nothing
 
-isSubPathOf :: SubPath -> SubPath -> Bool
-isSubPathOf (SubPath p1) (SubPath p2) =
-    p1 == "" || p1 == p2 || (p1 ++ "/") `isPrefixOf` p2
-
 -- | Ensure directory exists and is not a symbolic link.
 doesDirectoryReallyExist :: FilePath -> IO Bool
 doesDirectoryReallyExist f = do
@@ -401,10 +288,6 @@ x /- ('/':r) = x /- r
 simpleClean :: String -> String
 simpleClean = normSlashes . reverse . dropWhile (=='/') . reverse . pathToPosix
 
--- | The root directory as an absolute path.
-rootDirectory :: AbsolutePath
-rootDirectory = AbsolutePath "/"
-
 makeAbsoluteOrStd :: AbsolutePath -> String -> AbsolutePathOrStd
 makeAbsoluteOrStd _ "-" = APStd
 makeAbsoluteOrStd a p = AP $ makeAbsolute a p
@@ -505,13 +388,7 @@ setCurrentDirectory = System.Directory.setCurrentDirectory . toFilePath
     accepts both path separators, and repositories always use the UNIX
     separator anyway.
 -}
-isMaliciousPath :: String -> Bool
-isMaliciousPath fp =
-    not (isExplicitlyRelative fp) || isGenerallyMalicious fp
 
--- | Warning : this is less rigorous than isMaliciousPath
---   but it's to allow for subpath representations that
---   don't start with ./
 isMaliciousSubPath :: String -> Bool
 isMaliciousSubPath fp =
     not (FilePath.isRelative fp) || isGenerallyMalicious fp
@@ -522,29 +399,6 @@ isGenerallyMalicious fp =
  where
     contains_any a b = not . null $ intersect a b
 
-
-isExplicitlyRelative :: String -> Bool
-isExplicitlyRelative ('.':'/':_) = True  -- begins with "./"
-isExplicitlyRelative _ = False
-
--- | Construct a filter from a list of AnchoredPaths, that will accept any path
--- that is either a parent or a child of any of the listed paths, and discard
--- everything else.
-filterPaths :: [AnchoredPath]
-            -> AnchoredPath
-            -> t
-            -> Bool
-filterPaths files p _ = any (\x -> x `isPrefix` p || p `isPrefix` x) files
-
-
--- | Same as 'filterPath', but for ordinary 'FilePath's (as opposed to
--- AnchoredPath).
-filterFilePaths :: [FilePath]
-                -> AnchoredPath
-                -> t
-                -> Bool
-filterFilePaths = filterPaths . map floatPath
-
 -- | Iteratively tries find first non-existing path generated by
 -- buildName, it feeds to buildName the number starting with -1.  When
 -- it generates non-existing path and it isn't first, it displays the
@@ -563,23 +417,19 @@ getUniquePathName talkative buildMsg buildName = go (-1)
        else go $ i+1
     where thename = buildName i
 
--- | Transform a SubPath into an AnchoredPath.
-floatSubPath :: SubPath -> AnchoredPath
-floatSubPath = floatPath . fn2fp . sp2fn
- 
 -------------------------------
 -- AnchoredPath utilities
 --
 
-newtype Name = Name { unName :: B.ByteString } deriving (Eq, Show, Ord)
+newtype Name = Name { unName :: B.ByteString } deriving (Binary, Eq, Show, Ord)
 
 -- | This is a type of "sane" file paths. These are always canonic in the sense
 -- that there are no stray slashes, no ".." components and similar. They are
 -- usually used to refer to a location within a Tree, but a relative filesystem
 -- path works just as well. These are either constructed from individual name
 -- components (using "appendPath", "catPaths" and "makeName"), or converted
--- from a FilePath ("floatPath" -- but take care when doing that) or .
-newtype AnchoredPath = AnchoredPath [Name] deriving (Eq, Show, Ord)
+-- from a FilePath ("floatPath" -- but take care when doing that).
+newtype AnchoredPath = AnchoredPath [Name] deriving (Binary, Eq, Show, Ord)
 
 -- | Check whether a path is a prefix of another path.
 isPrefix :: AnchoredPath -> AnchoredPath -> Bool
@@ -587,81 +437,151 @@ isPrefix :: AnchoredPath -> AnchoredPath -> Bool
 
 -- | Append an element to the end of a path.
 appendPath :: AnchoredPath -> Name -> AnchoredPath
-appendPath (AnchoredPath p) n =
-    case n of
-      (Name s) | B.null s -> AnchoredPath p
-               | s == BC.pack "." -> AnchoredPath p
-               | otherwise -> AnchoredPath $ p ++ [n]
+appendPath (AnchoredPath p) n = AnchoredPath $ p ++ [n]
 
 -- | Catenate two paths together. Not very safe, but sometimes useful
 -- (e.g. when you are representing paths relative to a different point than a
 -- Tree root).
 catPaths :: AnchoredPath -> AnchoredPath -> AnchoredPath
-catPaths (AnchoredPath p) (AnchoredPath n) = AnchoredPath $ p ++ n
+catPaths (AnchoredPath p) (AnchoredPath n) = AnchoredPath (p ++ n)
 
 -- | Get parent (path) of a given path. foo/bar/baz -> foo/bar
-parent :: AnchoredPath -> AnchoredPath
-parent (AnchoredPath x) = AnchoredPath (init x)
+parent :: AnchoredPath -> Maybe AnchoredPath
+parent (AnchoredPath []) = Nothing
+parent (AnchoredPath x) = Just (AnchoredPath (init x))
 
--- | List all parents of a given path. foo/bar/baz -> [foo, foo/bar]
+-- | List all parents of a given path. foo/bar/baz -> [.,foo, foo/bar]
 parents :: AnchoredPath -> [AnchoredPath]
-parents (AnchoredPath x) = map AnchoredPath . init . inits $ x
+parents (AnchoredPath []) = [] -- root has no parents
+parents (AnchoredPath xs) = map AnchoredPath $ inits $ init xs
+
+-- | If the patch is under a directory, split into Right of the first component
+-- (which must be a directory name) and the rest of teh path. Otherwise
+-- return Left of the single component.
+-- This function is *undefined* on the root path (which has no components).
+breakOnDir :: AnchoredPath -> Either Name (Name, AnchoredPath)
+breakOnDir (AnchoredPath []) = error "breakOnDir called on root"
+breakOnDir (AnchoredPath (n:[])) = Left n
+breakOnDir (AnchoredPath (n:ns)) = Right (n, AnchoredPath ns)
 
 -- | Take a "root" directory and an anchored path and produce a full
 -- 'FilePath'. Moreover, you can use @anchorPath \"\"@ to get a relative
 -- 'FilePath'.
 anchorPath :: FilePath -> AnchoredPath -> FilePath
-anchorPath dir p = dir </> decodeLocale (flatten p)
+anchorPath dir p = dir FilePath.</> decodeLocale (flatten p)
 {-# INLINE anchorPath #-}
 
+name2fp :: Name -> FilePath
+name2fp (Name ps) = decodeLocale ps
+
+-- FIXME returning "." for the root is wrong
 flatten :: AnchoredPath -> BC.ByteString
 flatten (AnchoredPath []) = BC.singleton '.'
-flatten (AnchoredPath p) = BC.intercalate (BC.singleton '/')
-                                           [ n | (Name n) <- p ]
+flatten (AnchoredPath p) = BC.intercalate (BC.singleton '/') [n | (Name n) <- p]
 
-makeName :: String -> Name
-makeName ".." = error ".. is not a valid AnchoredPath component name"
-makeName n | '/' `elem` n = error "/ may not occur in a valid AnchoredPath component name"
-           | otherwise = Name $ encodeLocale n
+-- | Make a 'Name' from a 'String'. If the input 'String'
+-- is invalid, that is, "", ".", "..", or contains a '/', return 'Left'
+-- with an error message.
+makeName :: String -> Either String Name
+makeName = rawMakeName . encodeLocale
 
--- | Take a relative FilePath and turn it into an AnchoredPath. The operation
--- is (relatively) unsafe. Basically, by using floatPath, you are testifying
--- that the argument is a path relative to some common root -- i.e. the root of
--- the associated "Tree" object. Also, there are certain invariants about
--- AnchoredPath that this function tries hard to preserve, but probably cannot
--- guarantee (i.e. this is a best-effort thing). You should sanitize any
+-- | Make a 'Name' from a 'String'. If the input 'String'
+-- is invalid, that is, "", ".", "..", or contains a '/', call error.
+internalMakeName :: String -> Name
+internalMakeName = either error id . rawMakeName . encodeLocale
+
+-- | Take a relative FilePath and turn it into an AnchoredPath. This is a
+-- partial function. Basically, by using floatPath, you are testifying that the
+-- argument is a path relative to some common root -- i.e. the root of the
+-- associated "Tree" object. In particular, the input path may not contain any
+-- ocurrences of "." or ".." after normalising. You should sanitize any
 -- FilePaths before you declare them "good" by converting into AnchoredPath
--- (using this function).
+-- (using this function), especially if the FilePath come from any external
+-- source (command line, file, environment, network, etc)
 floatPath :: FilePath -> AnchoredPath
-floatPath = make . splitDirectories . normalise . dropTrailingPathSeparator
-  where make ["."] = AnchoredPath []
-        make x = AnchoredPath $ map (Name . encodeLocale) x
-
+floatPath =
+    AnchoredPath . map internalMakeName . filter sensible .
+    splitDirectories . normalise . dropTrailingPathSeparator
+  where
+    sensible s = s `notElem` ["", "."]
 
 anchoredRoot :: AnchoredPath
 anchoredRoot = AnchoredPath []
 
--- | Take a prefix path, the changed prefix path, and a path to change.
--- Assumes the prefix path is a valid prefix. If prefix is wrong return
--- AnchoredPath [].
-replacePrefixPath :: AnchoredPath -> AnchoredPath -> AnchoredPath -> AnchoredPath
-replacePrefixPath (AnchoredPath []) b c = catPaths b c
-replacePrefixPath (AnchoredPath (r:p)) b (AnchoredPath (r':p'))
-    | r == r' = replacePrefixPath (AnchoredPath p) b (AnchoredPath p')
-    | otherwise = AnchoredPath []
-replacePrefixPath _ _ _ = AnchoredPath []
+-- | A view on 'AnchoredPath's.
+parentChild :: AnchoredPath -> Maybe (AnchoredPath, Name)
+parentChild (AnchoredPath []) = Nothing
+parentChild (AnchoredPath xs) = Just (AnchoredPath (init xs), last xs)
 
--- | Append a String to the last Name of an AnchoredPath.
-appendToName :: AnchoredPath -> String -> AnchoredPath
-appendToName (AnchoredPath p) s = AnchoredPath (init p++[Name finalname])
-    where suffix = encodeLocale s
-          finalname | suffix `elem` (BC.tails lastname) = lastname
-                    | otherwise = BC.append lastname suffix
-          lastname = case last p of
-                        Name name -> name
+-- | Replace the second arg's parent with the first arg.
+replaceParent :: AnchoredPath -> AnchoredPath -> Maybe AnchoredPath
+replaceParent (AnchoredPath xs) p =
+  case parentChild p of
+    Nothing -> Nothing
+    Just (_,x) -> Just (AnchoredPath (xs ++ [x]))
 
-unsafeMakeName :: B.ByteString -> Name
-unsafeMakeName = Name
+-- | Make a 'Name' from a 'B.ByteString'.
+rawMakeName :: B.ByteString -> Either String Name
+rawMakeName s
+  | isBadName s =
+      Left $ "'"++decodeLocale s++"' is not a valid AnchoredPath component name"
+  | otherwise = Right (Name s)
+
+isBadName :: B.ByteString -> Bool
+isBadName n = hasPathSeparator n || n `elem` forbiddenNames
+
+-- It would be nice if we could add BC.pack "_darcs" to the list, however
+-- "_darcs" could be a valid file or dir name if not inside the top level
+-- directory.
+forbiddenNames :: [B.ByteString]
+forbiddenNames = [BC.empty, BC.pack ".", BC.pack ".."]
+
+hasPathSeparator :: B.ByteString -> Bool
+hasPathSeparator = BC.elem '/'
 
 eqAnycase :: Name -> Name -> Bool
 eqAnycase (Name a) (Name b) = BC.map toLower a == BC.map toLower b
+
+encodeWhiteName :: Name -> B.ByteString
+encodeWhiteName = encodeLocale . encodeWhite . decodeLocale . unName
+
+data CorruptPatch = CorruptPatch String deriving (Eq, Typeable)
+instance Exception CorruptPatch
+instance Show CorruptPatch where show (CorruptPatch s) = s
+
+decodeWhiteName :: B.ByteString -> Name
+decodeWhiteName =
+  either (throw . CorruptPatch) id .
+  rawMakeName . encodeLocale . decodeWhite . decodeLocale
+
+-- | The effect of renaming on paths.
+-- The first argument is the old path, the second is the new path,
+-- and the third is the possibly affected path we are interested in.
+movedirfilename :: AnchoredPath -> AnchoredPath -> AnchoredPath -> AnchoredPath
+movedirfilename (AnchoredPath old) newp@(AnchoredPath new) orig@(AnchoredPath path) =
+  case stripPrefix old path of
+    Just [] -> newp -- optimization to avoid allocation in this case
+    Just rest -> AnchoredPath (new ++ rest)
+    Nothing -> orig -- old is not a prefix => no change
+
+-- | Construct a filter from a list of AnchoredPaths, that will accept any path
+-- that is either a parent or a child of any of the listed paths, and discard
+-- everything else.
+filterPaths :: [AnchoredPath] -> AnchoredPath -> t -> Bool
+filterPaths files p _ = any (\x -> x `isPrefix` p || p `isPrefix` x) files
+
+
+-- | Transform a SubPath into an AnchoredPath.
+floatSubPath :: SubPath -> AnchoredPath
+floatSubPath = floatPath . toFilePath
+
+-- | Is the given path in (or equal to) the _darcs metadata directory?
+inDarcsdir :: AnchoredPath -> Bool
+inDarcsdir (AnchoredPath (x:_)) | x == darcsdirName = True
+inDarcsdir _ = False
+
+darcsdirName :: Name
+darcsdirName = internalMakeName darcsdir
+
+isRoot :: AnchoredPath -> Bool
+isRoot (AnchoredPath xs) = null xs
