@@ -1,51 +1,24 @@
--- | A 'Document' is at heart 'ShowS' from the prelude
+-- | Darcs pretty printing library
 --
--- Essentially, if you give a Doc a string it'll print out whatever it
--- wants followed by that string. So @text "foo"@ makes the Doc that
--- prints @"foo"@ followed by its argument. The combinator names are taken
--- from 'Text.PrettyPrint.HughesPJ', although the behaviour of the two libraries is
--- slightly different.
---
--- The advantage of Printer over simple string appending/concatenating is
--- that the appends end up associating to the right, e.g.:
---
--- >   (text "foo" <> text "bar") <> (text "baz" <> text "quux") ""
--- > = \s -> (text "foo" <> text "bar") ((text "baz" <> text "quux") s) ""
--- > = (text "foo" <> text "bar") ((text "baz" <> text "quux") "")
--- > = (\s -> (text "foo") (text "bar" s)) ((text "baz" <> text "quux") "")
--- > = text "foo" (text "bar" ((text "baz" <> text "quux") ""))
--- > = (\s -> "foo" ++ s) (text "bar" ((text "baz" <> text "quux") ""))
--- > = "foo" ++ (text "bar" ((text "baz" <> text "quux") ""))
--- > = "foo" ++ ("bar" ++ ((text "baz" <> text "quux") ""))
--- > = "foo" ++ ("bar" ++ ((\s -> text "baz" (text "quux" s)) ""))
--- > = "foo" ++ ("bar" ++ (text "baz" (text "quux" "")))
--- > = "foo" ++ ("bar" ++ ("baz" ++ (text "quux" "")))
--- > = "foo" ++ ("bar" ++ ("baz" ++ ("quux" ++ "")))
---
--- The Empty alternative comes in because you want
---
--- > text "a" $$ vcat xs $$ text "b"
---
--- '$$' means above, 'vcat' is the list version of '$$'
--- (to be @\"a\\nb\"@ when @xs@  is @[]@), but without the concept of an
--- Empty Document each @$$@ would add a @'\n'@ and you'd end up with
--- @\"a\\n\\nb\"@.
--- Note that @Empty \/= text \"\"@ (the latter would cause two
--- @'\\n'@).
+-- The combinator names are taken from 'Text.PrettyPrint.HughesPJ', although
+-- the behaviour of the two libraries is slightly different.
 --
 -- This code was made generic in the element type by Juliusz Chroboczek.
 module Darcs.Util.Printer
     (
     -- * 'Doc' type and structural combinators
       Doc(Doc,unDoc)
-    , empty, (<>), (<?>), (<+>), ($$), vcat, vsep, hcat, hsep
+    , empty, (<>), (<?>), (<+>), ($$), ($+$), vcat, vsep, hcat, hsep
     , minus, newline, plus, space, backslash, lparen, rparen
-    , parens
+    , parens, sentence
     -- * Constructing 'Doc's
     , text
     , hiddenText
     , invisibleText
     , wrapText, quoted
+    , formatText
+    , formatWords
+    , pathlist
     , userchunk, packedString
     , prefix
     , hiddenPrefix
@@ -76,18 +49,15 @@ module Darcs.Util.Printer
     , hPutDocWith, hPutDocLnWith, putDocWith, putDocLnWith
     , hPutDocCompr
     , debugDocLn
-    , ePutDocLn
-    , errorDoc
     -- * TODO: It is unclear what is unsafe about these constructors
     , unsafeText, unsafeBoth, unsafeBothText, unsafeChar
     , unsafePackedString
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Data.String ( IsString(..) )
-import System.IO ( Handle, stdout, stderr )
+import System.IO ( Handle, stdout )
 import qualified Data.ByteString as B ( ByteString, hPut, concat )
 import qualified Data.ByteString.Char8 as BC ( singleton )
 
@@ -140,9 +110,16 @@ rparen = unsafeBoth ")" (BC.singleton ')')
 parens :: Doc -> Doc
 parens d = lparen <> d <> rparen
 
--- | Fail with a stack trace and the given 'Doc' as error message.
-errorDoc :: Doc -> a
-errorDoc x = error $ renderString x
+-- | Turn a 'Doc' into a sentence. This appends a ".".
+sentence :: Doc -> Doc
+sentence = (<> text ".")
+
+-- | Format a list of 'FilePath's as quoted text. It deliberately refuses to
+-- use English.andClauses but rather separates the quoted strings only with a
+-- space, because this makes it usable for copy and paste e.g. as arguments to
+-- another shell command.
+pathlist :: [FilePath] -> Doc
+pathlist paths = hsep (map quoted paths)
 
 -- | 'putDocWith' puts a 'Doc' on stdout using the given printer.
 putDocWith :: Printers -> Doc -> IO ()
@@ -162,17 +139,11 @@ putDoc = hPutDoc stdout
 putDocLn :: Doc -> IO ()
 putDocLn = hPutDocLn stdout
 
--- | 'eputDocLn' puts a 'Doc', followed by a newline to stderr using
--- 'simplePrinters'. Like putDocLn, it encodes with the user's locale.
--- This function is the recommended way to output messages that should
--- be visible to users on the console, but cannot (or should not) be
--- silenced even when --quiet is in effect.
-ePutDocLn :: Doc -> IO ()
-ePutDocLn = hPutDocLn stderr
-
 -- | 'hputDocWith' puts a 'Doc' on the given handle using the given printer.
 hPutDocWith :: Printers -> Handle -> Doc -> IO ()
-hPutDocWith prs h d = hPrintPrintables h (renderWith (prs h) d)
+hPutDocWith prs h d = do
+  p <- prs h
+  hPrintPrintables h (renderWith p d)
 
 -- | 'hputDocLnWith' puts a 'Doc', followed by a newline on the given
 -- handle using the given printer.
@@ -221,7 +192,7 @@ instance IsString Doc where
 -- hanlde, and the current prefix of the document.
 data St = St { printers :: !Printers',
                currentPrefix :: !([Printable] -> [Printable]) }
-type Printers = Handle -> Printers'
+type Printers = Handle -> IO Printers'
 
 -- | A set of printers to print different types of text to a handle.
 data Printers' = Printers {colorP :: !(Color -> Printer),
@@ -391,12 +362,23 @@ colorText c = mkColorPrintable c . S
 -- | @'wrapText' n s@ is a 'Doc' representing @s@ line-wrapped at 'n' characters
 wrapText :: Int -> String -> Doc
 wrapText n s =
-    vcat . map text . reverse $ "" : foldl add_to_line [] (words s)
+    vcat . map text . reverse $ foldl add_to_line [] (words s)
   where add_to_line [] a = [a]
         add_to_line ("":d) a = a:d
         add_to_line (l:ls) new | length l + length new > n = new:l:ls
         add_to_line (l:ls) new = (l ++ " " ++ new):ls
 
+-- | Given a list of 'String's representing the words of a paragraph, format
+-- the paragraphs using 'wrapText' and separate them with an empty line.
+formatText :: Int -> [String] -> Doc
+formatText w = vsep . map (wrapText w)
+
+-- | A variant of 'wrapText' that takes a list of strings as input.
+-- Useful when @{-# LANGUAGE CPP #-}@ makes it impossible to use multiline
+-- string literals.
+formatWords :: [String] -> Doc
+formatWords = wrapText 80 . unwords
+
 -- | Creates a 'Doc' from any 'Printable'.
 printable :: Printable -> Doc
 printable x = Doc $ \st -> defP (printers st) x st
@@ -419,7 +401,7 @@ userchunkPrintable x = Doc $ \st -> userchunkP (printers st) x st
 -- | 'simplePrinters' is a 'Printers' which uses the set 'simplePriners\'' on any
 -- handle.
 simplePrinters :: Printers
-simplePrinters _ = simplePrinters'
+simplePrinters _ = return simplePrinters'
 
 -- | A set of default printers suitable for any handle. Does not use color.
 simplePrinters' :: Printers'
@@ -447,7 +429,7 @@ invisiblePrinter _ = unDoc empty
 
 infixr 6 `append`
 infixr 6 <+>
-infixr 5 `vplus`
+infixr 5 $+$
 infixr 5 $$
 
 -- | The empty 'Doc'
@@ -506,9 +488,9 @@ Doc a $$ Doc b =
                         where pf = currentPrefix st
                               sf = lineColorS $ printers st
 
--- | @vplus a b@ is @a@ above @b@ with an empty line in between if both are non-empty
-vplus :: Doc -> Doc -> Doc
-Doc a `vplus` Doc b =
+-- | @a '$+$' b@ is @a@ above @b@ with an empty line in between if both are non-empty
+($+$) :: Doc -> Doc -> Doc
+Doc a $+$ Doc b =
    Doc $ \st -> case a st of
                 Empty -> b st
                 Document af ->
@@ -524,7 +506,7 @@ vcat = foldr ($$) empty
 
 -- | Pile 'Doc's vertically, with a blank line in between
 vsep :: [Doc] -> Doc
-vsep = foldr vplus empty
+vsep = foldr ($+$) empty
 
 -- | Concatenate 'Doc's horizontally
 hcat :: [Doc] -> Doc
