@@ -1,10 +1,10 @@
 {-# LANGUAGE CPP #-}
 module Darcs.Util.Printer.Color
-    ( showDoc, errorDoc, traceDoc, assertDoc, fancyPrinters
+    ( unsafeRenderStringColored, traceDoc, debugDoc, fancyPrinters
     , environmentHelpColor, environmentHelpEscape, environmentHelpEscapeWhite
+    , ePutDocLn
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import Darcs.Util.Printer
@@ -12,7 +12,9 @@ import Darcs.Util.Printer
     , invisiblePrinter, (<?>), Doc(Doc,unDoc), unsafeBothText, simplePrinter, hcat
     , unsafeText, unsafePackedString
     , renderStringWith, prefix
+    , hPutDocLnWith
     )
+import Darcs.Util.Global ( whenDebugMode, putTiming )
 
 import Debug.Trace ( trace )
 import Data.Char ( isAscii, isPrint, isSpace, isControl, ord, chr )
@@ -32,18 +34,25 @@ dollar, cr :: Doc
 dollar = unsafeBothText "$"
 cr     = unsafeBothText "\r"
 
-errorDoc :: Doc -> a
-errorDoc = error . showDoc
+-- | 'eputDocLn' puts a 'Doc', followed by a newline to stderr using
+-- 'fancyPrinters'. Like putDocLn, it encodes with the user's locale.
+-- This function is the recommended way to output messages that should
+-- be visible to users on the console, but cannot (or should not) be
+-- silenced even when --quiet is in effect.
+ePutDocLn :: Doc -> IO ()
+ePutDocLn = hPutDocLnWith fancyPrinters stderr
+
+debugDoc :: Doc -> IO ()
+debugDoc m = whenDebugMode $ do
+  putTiming
+  hPutDocLnWith fancyPrinters stderr m
 
 traceDoc :: Doc -> a -> a
-traceDoc d = trace (showDoc d)
+traceDoc = trace . unsafeRenderStringColored
 
-assertDoc :: Maybe Doc -> a -> a
-assertDoc Nothing x = x
-assertDoc (Just e) _ = errorDoc e
+unsafeRenderStringColored :: Doc -> String
+unsafeRenderStringColored = renderStringWith (unsafePerformIO (fancyPrinters stderr))
 
-showDoc :: Doc -> String
-showDoc = renderStringWith (fancyPrinters stderr)
 
 -- | The 'Policy' type is a record containing the variables which control
 -- how 'Doc's will be rendered on some output.
@@ -60,12 +69,11 @@ data Policy = Policy { poColor :: Bool    -- ^ overall use of color
                      , poSpace :: Bool    -- ^ escape spaces (used with poTrailing)
                      }
 
-{-# NOINLINE getPolicy #-}
 -- | 'getPolicy' returns a suitable policy for a given handle.
 -- The policy is chosen according to environment variables, and to the
 -- type of terminal which the handle represents
-getPolicy :: Handle -> Policy
-getPolicy handle = unsafePerformIO $
+getPolicy :: Handle -> IO Policy
+getPolicy handle =
  do isTerminal <- hIsTerminalDevice handle
     nColors <- if isTerminal then getTermNColors else return 0
 
@@ -83,14 +91,13 @@ getPolicy handle = unsafePerformIO $
     envDontColor         <- getEnvBool "DARCS_DONT_COLOR"
     envAlwaysColor       <- getEnvBool "DARCS_ALWAYS_COLOR"
     envAlternativeColor  <- getEnvBool "DARCS_ALTERNATIVE_COLOR"
-    envDoColorLines    <- getEnvBool "DARCS_DO_COLOR_LINES"
 
     let haveColor = envAlwaysColor || (isTerminal && (nColors > 4))
         doColor   = not envDontColor && haveColor
 
     return Policy { poColor    = doColor,
                     poEscape   = not envDontEscapeAnything,
-                    poLineColor= doColor && envDoColorLines,
+                    poLineColor= doColor && not envAlternativeColor,
                     poIsprint  = envDontEscapeIsprint || envUseIsprint,
                     po8bit     = not envEscape8bit,
                     poNoEscX   = envDontEscapeExtra,
@@ -127,15 +134,17 @@ getTermNColors = return (-1)
 -- | @'fancyPrinters' h@ returns a set of printers suitable for outputting
 -- to @h@
 fancyPrinters :: Printers
-fancyPrinters h = let policy = getPolicy h in
-                      Printers { colorP = colorPrinter policy,
-                             invisibleP = invisiblePrinter,
-                             hiddenP = colorPrinter policy Green,
-                             userchunkP = userchunkPrinter policy,
-                             defP       = escapePrinter policy,
-                             lineColorT = lineColorTrans policy,
-                             lineColorS = lineColorSuffix policy
-                           }
+fancyPrinters h = do
+  policy <- getPolicy h
+  return Printers {
+    colorP = colorPrinter policy,
+    invisibleP = invisiblePrinter,
+    hiddenP = colorPrinter policy Green,
+    userchunkP = userchunkPrinter policy,
+    defP       = escapePrinter policy,
+    lineColorT = lineColorTrans policy,
+    lineColorS = lineColorSuffix policy
+  }
 
 -- | @'lineColorTrans' policy@ tries to color a Doc, according to policy po.
 -- That is, if @policy@ has @poLineColor@ set, then colors the line, otherwise
@@ -309,17 +318,17 @@ makeInvert = withColor "\x1B[07m"
 
 environmentHelpColor :: ([String], [String])
 environmentHelpColor = (["DARCS_DONT_COLOR", "DARCS_ALWAYS_COLOR",
-                         "DARCS_ALTERNATIVE_COLOR", "DARCS_DO_COLOR_LINES"],[
+                         "DARCS_ALTERNATIVE_COLOR"],[
   "If the terminal understands ANSI color escape sequences, darcs will",
-  "highlight certain keywords and delimiters when printing patches. This",
-  "can be turned off by setting the environment variable DARCS_DONT_COLOR",
-  "to 1. If you use a pager that happens to understand ANSI colors, like",
+  "highlight certain keywords and delimiters when printing patches, and",
+  "also print hunk lines in color according to whether they are removed",
+  "or added. This can be turned off by setting the environment variable",
+  "DARCS_DONT_COLOR to 1.",
+  "If you use a pager that happens to understand ANSI colors, like",
   "`less -R`, darcs can be forced always to highlight the output by setting",
   "DARCS_ALWAYS_COLOR to 1. If you can't see colors you can set",
   "DARCS_ALTERNATIVE_COLOR to 1, and darcs will use ANSI codes for bold",
-  "and reverse video instead of colors. In addition, there is an",
-  "extra-colorful mode, which is not enabled by default, which can be",
-  "activated with DARCS_DO_COLOR_LINES"])
+  "and reverse video instead of colors."])
 
 environmentHelpEscapeWhite :: ([String], [String])
 environmentHelpEscapeWhite = ([ "DARCS_DONT_ESCAPE_TRAILING_SPACES",
