@@ -5,4 +5,4 @@ module Darcs.Util.Ratified
     , hGetContents
     ) where
 
-import System.IO( hGetContents )
+import System.IO ( hGetContents, readFile )
