@@ -15,7 +15,7 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, DeriveDataTypeable #-}
+{-# LANGUAGE CPP #-}
 
 module Darcs.Util.SignalHandler
     ( withSignalsHandled, withSignalsBlocked,
@@ -23,7 +23,6 @@ module Darcs.Util.SignalHandler
       tryNonSignal, stdoutIsAPipe
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 import System.IO.Error ( isUserError, ioeGetErrorString, ioeGetFileName )
@@ -53,13 +52,13 @@ stdoutIsAPipe
             return (isNamedPipe stat))
         (\(_ :: IOException) -> return False)
 
-withSignalsHandled :: IO a -> IO a
 newtype SignalException = SignalException Signal deriving (Show, Typeable)
 
 instance Exception SignalException where
    toException = SomeException
    fromException (SomeException e) = cast e
 
+withSignalsHandled :: IO a -> IO a
 withSignalsHandled job = do
     thid <- myThreadId
     mapM_ (ih thid) [sigINT, sigHUP, sigABRT, sigTERM, sigPIPE]
@@ -71,16 +70,16 @@ withSignalsHandled job = do
                      | s == sigTERM = ew s "TERM"
                      | s == sigPIPE = exitWith $ ExitFailure 1
                      | otherwise = ew s "Unhandled signal!"
-          ew sig s = do hPutStrLn stderr $ "withSignalsHandled: " ++ s
+          ew sig s = do hPutStrLn stderr s
                         resethandler sig
                         raiseSignal sig -- ensure that our caller knows how we died
                         exitWith $ ExitFailure 1
           die_with_string e | "STDOUT" `isPrefixOf` e =
                 do is_pipe <- stdoutIsAPipe
                    unless is_pipe $
-                        hPutStrLn stderr $ "\ndarcs failed:  "++drop 6 e
+                        hPutStrLn stderr $ drop 6 e
                    exitWith $ ExitFailure 2
-          die_with_string e = do hPutStrLn stderr $ "\ndarcs failed:  "++e
+          die_with_string e = do hPutStrLn stderr e
                                  exitWith $ ExitFailure 2
 #ifdef WIN32
           job' thid =
