@@ -1,7 +1,7 @@
 --  Copyright (C) 2009-2011 Petr Rockai
 --
 --  BSD3
-{-# LANGUAGE ScopedTypeVariables, MultiParamTypeClasses, FlexibleInstances, BangPatterns #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
 
 -- | The abstract representation of a Tree and useful abstract utilities to
 -- handle those.
@@ -20,6 +20,7 @@ module Darcs.Util.Tree
     , items, list, listImmediate, treeHash
     , lookup, find, findFile, findTree, itemHash, itemType
     , zipCommonFiles, zipFiles, zipTrees, diffTrees
+    , explodePath, explodePaths
 
     -- * Files (Blobs).
     , readBlob
@@ -29,10 +30,14 @@ module Darcs.Util.Tree
 
     -- * Manipulating trees.
     , modifyTree, updateTree, partiallyUpdateTree, updateSubtrees, overlay
-    , addMissingHashes ) where
+    , addMissingHashes
+
+    -- * Properties
+    , prop_explodePath
+    ) where
 
-import Prelude ()
 import Darcs.Prelude hiding ( filter )
+import qualified Prelude ( filter )
 
 import Control.Exception( catch, IOException )
 import Darcs.Util.Path
@@ -155,6 +160,17 @@ list t_ = paths t_ (AnchoredPath [])
                     concat [ paths subt (appendPath p subn)
                              | (subn, SubTree subt) <- listImmediate t ]
 
+-- | Like 'explodePath' but for multiple paths.
+explodePaths :: Tree IO -> [AnchoredPath] -> [AnchoredPath]
+explodePaths tree paths = concatMap (explodePath tree) paths
+
+-- | All paths in the tree that that have the given path as prefix.
+--
+-- prop> explodePath t p == Prelude.filter (p `isPrefix`) (map fst (list t))
+explodePath :: Tree m -> AnchoredPath -> [AnchoredPath]
+explodePath tree path =
+  path : maybe [] (map (catPaths path . fst) . list) (findTree tree path)
+
 expandUpdate :: (Monad m) => (AnchoredPath -> Tree m -> m (Tree m)) -> Tree m -> m (Tree m)
 expandUpdate update t_ = go (AnchoredPath []) t_
     where go path t = do
@@ -261,7 +277,7 @@ restrict guide tree = filter accept tree
                 (Just (SubTree _), Stub _ _) -> True
                 (Just (File _), File _) -> True
                 (Just (Stub _ _), _) ->
-                    bug "*sulk* Go away, you, you precondition violator!"
+                    error "*sulk* Go away, you, you precondition violator!"
                 (_, _) -> False
 
 -- | Read a Blob into a Lazy ByteString. Might be backed by an mmap, use with
@@ -322,7 +338,7 @@ diffTrees left right =
         subtree :: TreeItem m -> m (Tree m)
         subtree (Stub x _) = x
         subtree (SubTree x) = return x
-        subtree (File _) = bug "diffTrees tried to descend a File as a subtree"
+        subtree (File _) = error "diffTrees tried to descend a File as a subtree"
         maybeUnfold (Stub x _) = SubTree `fmap` (x >>= expand)
         maybeUnfold (SubTree x) = SubTree `fmap` expand x
         maybeUnfold i = return i
@@ -352,7 +368,7 @@ diffTrees left right =
                              do l' <- maybeUnfold l
                                 r' <- maybeUnfold r
                                 return (n, Just l', Just r')
-                     _ -> bug "n lookups failed"
+                     _ -> error "n lookups failed"
                    | n <- immediateN left' `union` immediateN right' ]
           let is_l = [ (n, l) | (n, Just l, _) <- is ]
               is_r = [ (n, r) | (n, _, Just r) <- is ]
@@ -386,14 +402,14 @@ modifyTree t_ p_ i_ = snd $ go t_ p_ i_
                     Just (Stub s _) -> (False, Stub (do x <- s
                                                         return $! snd $! subtree x) NoHash)
                     Nothing -> (False, SubTree $! snd $! subtree emptyTree)
-                    _ -> bug $ "Modify tree at " ++ show path
+                    _ -> error $ "Modify tree at " ++ show path
 
         go _ (AnchoredPath []) (Just (Stub _ _)) =
-            bug $ "descending in modifyTree, case = (Just (Stub _ _)), path = " ++ show p_
+            error $ "descending in modifyTree, case = (Just (Stub _ _)), path = " ++ show p_
         go _ (AnchoredPath []) (Just (File _)) =
-            bug $ "descending in modifyTree, case = (Just (File _)), path = " ++ show p_
+            error $ "descending in modifyTree, case = (Just (File _)), path = " ++ show p_
         go _ (AnchoredPath []) Nothing =
-            bug $ "descending in modifyTree, case = Nothing, path = " ++ show p_
+            error $ "descending in modifyTree, case = Nothing, path = " ++ show p_
 
 countmap :: forall a k. M.Map k a -> Int
 countmap = M.foldr (\_ i -> i + 1) 0
@@ -404,7 +420,7 @@ updateSubtrees fun t =
             , treeHash = NoHash }
   where update (k, SubTree s) = (k, SubTree $ updateSubtrees fun s)
         update (k, File f) = (k, File f)
-        update (_, Stub _ _) = bug "Stubs not supported in updateTreePostorder"
+        update (_, Stub _ _) = error "Stubs not supported in updateTreePostorder"
 
 -- | Does /not/ expand the tree.
 updateTree :: (Monad m) => (TreeItem m -> m (TreeItem m)) -> Tree m -> m (Tree m)
@@ -420,7 +436,7 @@ partiallyUpdateTree fun predi t' = go (AnchoredPath []) t'
                                        , treeHash = NoHash }
           case subtree of
             SubTree t'' -> return t''
-            _ -> bug "function passed to partiallyUpdateTree didn't changed SubTree to something else"
+            _ -> error "function passed to partiallyUpdateTree changed SubTree to something else"
         maybeupdate path (k, item) = if predi (path `appendPath` k) item
           then update (path `appendPath` k) (k, item)
           else return (k, item)
@@ -446,7 +462,7 @@ overlay base over = Tree { items = M.fromList immediate
                                                                    b' <- b
                                                                    return $ overlay b' o') NoHash
                     (Just x, _) -> x
-                    (_, _) -> bug $ "Unexpected case in overlay at get " ++ show n ++ "."
+                    (_, _) -> error $ "Unexpected case in overlay at get " ++ show n ++ "."
 
 addMissingHashes :: (Monad m) => (TreeItem m -> m Hash) -> Tree m -> m (Tree m)
 addMissingHashes make = updateTree update -- use partiallyUpdateTree here
@@ -468,3 +484,9 @@ isSub :: TreeItem m -> Bool
 isSub (File _) = False
 isSub _ = True
 
+-- Properties
+
+-- | Specification of 'explodePath'
+prop_explodePath :: Tree m -> AnchoredPath -> Bool
+prop_explodePath t p =
+  explodePath t p == Prelude.filter (isPrefix p) (map fst (list t))
