@@ -25,14 +25,16 @@ module Darcs.Util.Tree.Plain
     , writePlainTree
     ) where
 
+import Control.Monad ( forM )
 import Data.Maybe( catMaybes )
 import qualified Data.ByteString.Lazy as BL
 import System.FilePath( (</>) )
-import System.Directory( getDirectoryContents
-                       , createDirectoryIfMissing )
+import System.Directory ( listDirectory, createDirectoryIfMissing )
 import System.Posix.Files
     ( getSymbolicLinkStatus, isDirectory, isRegularFile, FileStatus )
 
+import Darcs.Prelude
+
 import Darcs.Util.Path
 import Darcs.Util.File ( withCurrentDirectory )
 import Darcs.Util.ByteString ( readSegment )
@@ -43,17 +45,17 @@ import Darcs.Util.Tree( Tree(), TreeItem(..)
 
 readPlainDir :: FilePath -> IO [(FilePath, FileStatus)]
 readPlainDir dir =
-    withCurrentDirectory dir $ do
-      items <- getDirectoryContents "."
-      sequence [ do st <- getSymbolicLinkStatus s
-                    return (s, st)
-                 | s <- items, s `notElem` [ ".", ".." ] ]
+  withCurrentDirectory dir $ do
+    items <- listDirectory "."
+    forM items $ \s -> do
+      st <- getSymbolicLinkStatus s
+      return (s, st)
 
 readPlainTree :: FilePath -> IO (Tree IO)
 readPlainTree dir = do
   items <- readPlainDir dir
   let subs = catMaybes [
-       let name = makeName name'
+       let name = either error id $ makeName name'
         in case status of
              _ | isDirectory status -> Just (name, Stub (readPlainTree (dir </> name')) NoHash)
              _ | isRegularFile status -> Just (name, File $ Blob (readBlob' name') NoHash)
