@@ -9,9 +9,7 @@
 -- Portability : portable
 
 module Darcs.Util.Workaround
-    (
-      renameFile
-    , setExecutable
+    ( setExecutable
     , getCurrentDirectory
     , installHandler
     , raiseSignal
@@ -25,22 +23,17 @@ module Darcs.Util.Workaround
     , sigPIPE
     ) where
 
-import Prelude ()
 import Darcs.Prelude
 
 #ifdef WIN32
 
-import Control.Monad ( unless )
-import qualified System.Directory ( renameFile, getCurrentDirectory, removeFile )
-import Control.Exception ( catch, IOException )
-import qualified Control.Exception ( mask )
-import qualified System.IO.Error ( isDoesNotExistError, ioError )
+import qualified System.Directory ( getCurrentDirectory )
 
 #else
 
 import System.Posix.Signals(installHandler, raiseSignal, Handler(..), Signal,
                             sigINT, sigHUP, sigABRT, sigALRM, sigTERM, sigPIPE)
-import System.Directory ( renameFile, getCurrentDirectory )
+import System.Directory ( getCurrentDirectory )
 import System.Posix.Files (fileMode,getFileStatus, setFileMode,
                            setFileCreationMask,
                            ownerReadMode, ownerWriteMode, ownerExecuteMode,
@@ -92,22 +85,6 @@ sigALRM :: Signal
 sigALRM = 0
 
 
--- | System.Directory.renameFile incorrectly fails when the new file already
--- exists.  This code works around that bug at the cost of losing atomic
--- writes.
-renameFile :: FilePath
-           -> FilePath
-           -> IO ()
-renameFile old new = Control.Exception.mask $ \_ ->
-   System.Directory.renameFile old new
-   `catch` \(_ :: IOException) ->
-   do System.Directory.removeFile new
-        `catch`
-         (\e -> unless (System.IO.Error.isDoesNotExistError e) $
-                    System.IO.Error.ioError e)
-      System.Directory.renameFile old new
-
-
 setExecutable :: FilePath
               -> Bool
               -> IO ()
