@@ -47,7 +47,7 @@ darcs init temp1
 cd temp1
 not darcs add a/ 2> err
 cat err
-grep 'File a does not exist!' err
+grep 'does not exist' err
 cd ..
 rm -rf temp1
 
