@@ -30,7 +30,6 @@
 # issue1427: apply gzipped bundles
 
 rm -rf temp1 temp2
-
 darcs init temp1
 darcs init temp2
 
@@ -60,11 +59,10 @@ darcs apply < ../temp1/funpatch.gz
 cd ..
 cmp temp1/bar temp2/bar
 
-rm -rf temp1 temp2
-
 ## issue2017 - apply should gracefully handle tag missing
 ## from context (complain, not crash)
 
+rm -rf R* S* T*
 darcs init R
 cd R
 echo 'Example content.' > f
@@ -83,7 +81,7 @@ darcs send -o foo.dpatch -a
 cd ..
 not darcs apply --repo R0 T0/foo.dpatch > log 2>&1
 not grep bug log
-grep missing log
+grep "Cannot find tag" log
 
 # variant 1 - tag in shared context
 darcs clone R R1
@@ -106,7 +104,7 @@ cd ..
 # the test: can't apply this due to incorrect context
 not darcs apply --repo R1 T1/foo.dpatch > log 2>&1
 not grep 'bug' log
-grep missing log
+grep "Cannot find tag" log
 
 # variant 2 - tag created after the fact
 darcs clone R  R2
@@ -121,14 +119,13 @@ cd ..
 darcs tag '2'  --repo R2  # only tag after
 not darcs apply --repo R2 T2/foo.dpatch > log 2>&1
 not grep 'bug' log
-grep missing log
-
-rm -rf R* S* T*
+grep "Cannot find tag" log
 
 # issue1921
 # Attempting to apply a patch which depends on a missing tag should not cause
 # darcs to die.
 
+rm -rf R* S* T*
 darcs init R
 cd R
 
@@ -164,8 +161,7 @@ cd ../S
 not darcs apply ../patch.dpatch &> apply_output.txt
 
 # A best-attempt at ensuring darcs warns about the missing tag:
-grep "tagged file2 tag" apply_output.txt
-grep "FATAL: Cannot apply this bundle. We are missing the above patches." apply_output.txt
+grep "Cannot find tag file2" apply_output.txt
 cd ..
 rm -rf R S
 
@@ -198,11 +194,10 @@ not darcs apply --repo R R/x.dpatch 2>&1 | tee log
 not grep '^  \* d' log # does not complain about an unrelated patch
     grep '^  \* y' log # complains about the offending one instead
 
-rm -rf R S
-
 ## Test that apply --skip-conflicts filters the conflicts
 ## appropriately.
 
+rm -rf R S
 darcs init R
 cd R
 echo 'foo' > foo
@@ -222,10 +217,10 @@ cd ../S
 darcs apply --skip-conflicts applyme.dpatch
 test `darcs log --count` -eq 3
 cd ..
-rm -rf R S
 
 # issue2193 - "darcs apply --test runs the test twice.
 
+rm -rf R S
 darcs init R
 darcs clone R S
 
