@@ -14,6 +14,9 @@
 # test fails for these obsolete formats:
 skip-formats darcs-1 darcs-2
 
+# skip if time executable is not found
+/usr/bin/env time --help 2> /dev/null || exit 200
+
 num_conflicts=40
 
 rm -rf RA RB
