@@ -27,6 +27,6 @@ darcs record -am 'first to third'
 cd ..
 
 cd temp1
-darcs pull -a ../temp2 | grep conflict
+darcs pull -a ../temp2 2>&1 | grep conflict
 grep third a
 cd ..
