@@ -14,9 +14,8 @@ ln -s temp1 temp2
 cd temp2
 darcs init
 touch a b
-IFS='' # annoying hack for cygwin and hspwd below
-DIR=`hspwd`
-darcs add ${DIR}/../temp1/a # should work, just to contrast with the case below
-darcs add ${DIR}/b          # this is the case we are testing for
+DIR=`pwd`
+darcs add "${DIR}/../temp1/a" # should work, just to contrast with the case below
+darcs add "${DIR}/b"          # this is the case we are testing for
 cd ..
 rm -rf temp1 temp2
