@@ -40,6 +40,6 @@ darcs rec --ignore-times -am 'patch Z'
 
 darcs obliterate --dry-run --patch 'patch Y' | not grep 'patch Z'
 
-echo 'yy' | darcs amend-rec --ask-deps
+echo 'yYyY' | tr '[A-Z]' '[a-z]' | darcs amend --ask-deps
 
 darcs obliterate --dry-run --patch 'patch Y' | grep 'patch Z'
