@@ -25,7 +25,7 @@
 
 . ./lib
 
-cacheDir=$HOME/.darcs/cache
+cacheDir=$HOME/.cache/darcs
 
 rm -rf R S
 darcs init  --repo R
