@@ -25,7 +25,7 @@
 ## SOFTWARE.
 
 # this test is not relevant for other than darcs 2 repositories
-grep darcs-2 $HOME/.darcs/defaults || exit 200
+only-format darcs-2
 
 . lib
 
