@@ -56,8 +56,8 @@ ln -s ../non-recorded-dir ./non-recorded-dir/loop               # relative symli
 ln -s "`pwd`"/non-recorded-dir ./non-recorded-dir/loop2           # absolute symlink
 darcs w -l >log 2>&1                                            # should not loop
 darcs rec -alm "added ./non-recorded-dir" >>log 2>&1            # should not loop
-darcs changes -s --patch="added ./non-recorded-dir" >>log 2>&1  # should report only dir, not symlink
-not grep -vE "(^ *$|^\+|[0-9]:[0-9][0-9]:[0-9]|./non-recorded-dir)" log
+darcs changes -s --patches="added ./non-recorded-dir" >>log 2>&1  # should report only dir, not symlink
+not grep -vE "(^patch|^Author|^ *$|^\+|[0-9]:[0-9][0-9]:[0-9]|./non-recorded-dir)" log
 
 # Case 2: looping symlink to recorded dir
 mkdir recorded-dir
@@ -84,8 +84,8 @@ ln -s ./non-recorded-dir2 link
 ln -s "`pwd`"/non-recorded-dir2 ./link2
 darcs w -l >log 2>&1                                             # should report only "non-recorded-dir2"
 darcs rec -alm "added ./non-recorded-dir2" >>log 2>&1            # should add only dir, not symlink
-darcs changes -s --patch="added ./non-recorded-dir2" >>log 2>&1  # should report only dir, not symlink
-not grep -vE "(^ *$|^\+|[0-9]:[0-9][0-9]:[0-9]|./non-recorded-dir2)" log
+darcs changes -s --patches="added ./non-recorded-dir2" >>log 2>&1  # should report only dir, not symlink
+not grep -vE "(^patch|^Author|^ *$|^\+|[0-9]:[0-9][0-9]:[0-9]|./non-recorded-dir2)" log
 rm link link2
 
 # Case 5: non-looping symlink to recorded dir
@@ -118,8 +118,8 @@ ln -s ./non-recorded-file ./link
 ln -s "`pwd`"/non-recorded-file ./link2
 darcs w -l >log 2>&1                                             # should report only "non-recorded-file"
 darcs rec -alm "added ./non-recorded-file" >>log 2>&1            # should add only file, not symlink
-darcs changes -s --patch="added ./non-recorded-file" >>log 2>&1  # should report only file, not symlink
-not grep -vE "(^ *$|^\+|[0-9]:[0-9][0-9]:[0-9]|./non-recorded-file)" log
+darcs changes -s --patches="added ./non-recorded-file" >>log 2>&1  # should report only file, not symlink
+not grep -vE "(^patch|^Author|^ *$|^\+|[0-9]:[0-9][0-9]:[0-9]|./non-recorded-file)" log
 rm link link2
 
 # Case 9: symlink to recorded file
