@@ -26,12 +26,16 @@
 
 . lib                           # Load some portability helpers.
 rm -rf minimal-darcs-2.4
-tar zx < $TESTDATA/minimal-darcs-2_4.tgz
+unpack_testdata minimal-darcs-2_4
 
 cd minimal-darcs-2.4
 darcs check
 darcs setpref test false
 echo 'hi' > README
-not darcs record -a -m argh --test
+not darcs record -a -m argh --test 2> errlog 1> outlog
+# check that we are really doing the pristine conversion...
+grep 'one-time conversion of pristine' errlog
+# ...and not fail for some other reason
+grep "Test failed" outlog
 darcs check
 cd ..
