@@ -24,5 +24,5 @@
 ## SOFTWARE.
 
 . lib                  # Load some portability helpers.
-gunzip -c $TESTDATA/badrepo.tgz | tar xf -
+unpack_testdata badrepo
 not darcs get badrepo
