@@ -1,6 +1,6 @@
 #!/bin/sh
 
-set -ev
+. ./lib
 
 rm -rf temp1 temp2
 mkdir temp1 temp2
