@@ -2,9 +2,6 @@
 
 . ./lib
 
-# this test fails in the darcs 1 format
-skip-formats darcs-1
-
 mkdir temp1
 cd temp1
 darcs init
