@@ -23,8 +23,8 @@
 
 . lib
 
-# this test fails for darcs-1 repos
-skip-formats darcs-1
+# this tests a property of darcs-2 repos
+only-format darcs-2
 
 darcs init --repo R
 darcs init --repo S
