@@ -2,8 +2,8 @@
 
 . ./lib
 
-rm -rf temp1 temp2
-mkdir temp1 temp2
+rm -rf temp1
+mkdir temp1
 cd temp1
 
 # simple add and move
@@ -71,7 +71,15 @@ mv foo foo2
 echo 'yyy' | darcs amend-record -p add_file --look-for-moves
 darcs wh --look-for-moves --look-for-adds >log 2>&1
 grep -vE "(^ *$|^\+|No changes!)" log
-darcs log -v --machine -p add_file | grep "] addfile ./foo2"
+darcs log -v --machine -p add_file >log 2>&1
+# 'darcs log --machine' internally calls 'showPatch ForStorage'
+# which is why we need to distinguish between darcs-3 and earlier here.
+# This behavior of 'darcs log' is questionable at best.
+if test "$format" = darcs-3; then
+  grep -A1 "] hash 1 " log | grep "addfile ./foo2"
+else
+  grep "] addfile ./foo2" log
+fi
 rm -rf *
 
 # add, move, add same name and amend-record
