@@ -9,6 +9,7 @@ test_setup() {
   shift
   paths=$@
 
+  rm -rf R$num
   darcs init R$num
   cd R$num
 
@@ -50,10 +51,7 @@ diff out.actual out.expected
 test_setup 3 new
 # remove the line about What's new in: new
 sed -i '1d' out.actual
-# NOTE: the move is NOT detected in this case
-# We might want to change that...
-sed '1d' out.expected | sed 's/new/old/' > out.expected3
-diff out.actual out.expected3
+diff out.actual out.expected
 
 # same but only for old and new
 
