@@ -125,7 +125,7 @@ rm -rf temp1 temp2
 
 # issue662, which triggered:
 #  darcs failed:  Error applying hunk to file ./t.t
-#  Error applying patch to the working directory.
+#  Error applying patch to the working tree.
 
 darcs init temp1
 cd temp1
@@ -140,7 +140,8 @@ darcs clone temp1 temp2
 cd temp2
 darcs obliterate --last 1 --all
 echo 'content: local change'> t.t
-darcs pull -a ../temp1
+# this is now recognized as a conflict with working:
+echo y | darcs pull -a ../temp1
 darcs w -s
 darcs revert -a
 cd ..
@@ -315,7 +316,13 @@ darcs init temp2
 cd temp2
 echo yny | darcs pull --set-default ../temp1
 rm foo
-darcs pull -a
+# for darcs-1 and darcs-3 this pull conflicts with unrecorded changes
+# (which is correct, whereas darcs-2 is buggy)
+if test "$format" != "darcs-2"; then
+  echo y | darcs pull -a
+else
+  darcs pull -a
+fi
 cd ..
 
 rm -rf temp1 temp2
