@@ -16,14 +16,17 @@ darcs init
 touch titi
 darcs add titi
 darcs record -am titi
-cat > _darcs/format <<EOF
-hashed|gobbledygook
-darcs-2
-EOF
+sed -i 's/hashed/hashed|gobbledygook/' _darcs/format
 cd ..
 
+corrupt_format_file() {
+  # mimic format file corruption as in issue2650
+  sed -i 's/gobbledygook/Unknown format: gobbledygook/' future/_darcs/format
+}
+
 # check the rules for reading and writing
 
+test_garbage() {
 ## garbage repo: we don't understand anything
 rm -rf temp1
 not darcs get garbage temp1 2> log
@@ -65,14 +68,23 @@ grep -i "read repository.*unknown format" log
 # issue2650
 not grep 'Unknown format' _darcs/format
 cd ..
+}
+
+test_garbage
+
+# corrupt once
+corrupt_format_file
+test_garbage
+# corrupt multiple times
+corrupt_format_file
+corrupt_format_file
+test_garbage
 
 
 ## future repo: we don't understand one
 #  alternative of a line of format
-#  only look at future vs darcs2
-
-skip-formats darcs-1
 
+test_future () {
 # get future repo: ok
 # --to-match is needed because of bug###
 rm -rf temp1
@@ -121,3 +133,14 @@ grep -i "write repository.*unknown format" log
 # issue2650
 not grep 'Unknown format' _darcs/format
 cd ..
+}
+
+test_future
+
+# corrupt once
+corrupt_format_file
+test_future
+# corrupt multiple times
+corrupt_format_file
+corrupt_format_file
+test_future
