@@ -2,9 +2,6 @@
 . ./lib
 
 # The argument quoting in the test script below breaks on Windows paths
-# Actually sending email from Windows is very unlikely anyway, so just
-# skip this test.
-abort_windows
 
 DARCS_EDITOR=echo
 export DARCS_EDITOR
