@@ -25,10 +25,10 @@ cd ../temp1
 echo C > foo
 darcs record -a -A author -m CC
 
-darcs pull -a ../temp2
-darcs pull -a ../temp3
+darcs pull -a ../temp2 --allow-conflicts
+darcs pull -a ../temp3 --allow-conflicts
 
 cd ../temp2
-darcs pull -a ../temp3
-darcs pull -a ../temp1
+darcs pull -a ../temp3 --allow-conflicts
+darcs pull -a ../temp1 --allow-conflicts
 rm -rf temp1 temp2 temp3
