@@ -3,7 +3,7 @@ echo
 echo Checking that each command expects each option only once
 echo
 
-set -ev
+. ./lib
 
 if echo $OS | grep -i windows; then
     echo Noone knows how to handle newlines under cygwin, so we skip this test
